/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.iwidget.Container");

/*
 * TODOs
 */
// * From Ed: Should we get our own copy of Open Ajax Hub?

/*
 * Imports 
 */
// TODO: MP: Verify if requires are needed
dojo.require("dojo.number");
dojo.require("dijit.Menu");
dojo.require("dojo.dnd.Moveable");
dojo.require("dojox.uuid.generateTimeBasedUuid");
dojo.require("dojox.uuid");
dojo.require("dojox.data.CsvStore");
dojo.require("dojo.cookie");
dojo.require("net.jazz.ajax.contextRoot");

// Missing require from enabler.core
dojo.require("dojo.back");

dojo.require("dojo.i18n"); 
dojo.requireLocalization("jazz.iwidget.internal", "Messages");

(function() {

var messages = dojo.i18n.getLocalization("jazz.iwidget.internal", "Messages");
var contextRoot = net.jazz.ajax.contextRoot();
var mmModulePath = contextRoot + "/web/jazz.iwidget/internal/mm";

/*
 * Constants 
 */
var LoadStatus = {
	NOT_LOADED: 0,
	LOADING: 1,
	LOADED: 2
};

dojo.registerModulePath("com.ibm.mm", mmModulePath);
	
/*
 * Class
 */
dojo.declare("jazz.iwidget.Container", null, {

	_loadCallbacks: [],
	_status :  LoadStatus.NOT_LOADED,
	
	isLoaded : function ()
	{
    	return (this._status == LoadStatus.LOADED);
 	},
	
	/**
	 * this will load enabler asynchronously and call the call back when it is complete
	 */
	load : function(callback)
	{
		if (this._status == LoadStatus.LOADED) 
		{
			callback();
			return;
		}
		else 
		{
			this._loadCallbacks.push(callback);
		}
		
		if (this._status == LoadStatus.LOADING) return;
		
		this._status = LoadStatus.LOADING;
		
		// TODO: MP: Verify if this is needed and the proper values
		// TODO: MP: Is this inherited somewhere? API to set this? Why global?
		// Global Enabler Core variable: ibmConfig
		ibmConfig = jazz.iwidget.DEFAULT_IBM_CONFIG;
		
		var openAjaxUri;
		var enablerUri;
		
		if (djConfig.isDebug) {
			// Debug mode
			enablerUri = dojo.moduleUrl("jazz.iwidget", "internal/mm/enabler/enabler.core.js.uncompressed.js");
			openAjaxUri = dojo.moduleUrl("jazz.iwidget", "internal/mm/openajaxhub/OpenAjaxManagedHub-all.js.uncompressed.js");
	    } else {
	    	enablerUri = dojo.moduleUrl("jazz.iwidget", "internal/mm/enabler/enabler.core.js");
	    	openAjaxUri = dojo.moduleUrl("jazz.iwidget", "internal/mm/openajaxhub/OpenAjaxManagedHub-all.js");
	    }
		
		this.loadScriptAsync(openAjaxUri, dojo.hitch(this, function() {
			// Note: Enabler Core must be loaded after Open Ajax Hub
			this.loadScriptAsync(enablerUri, dojo.hitch(this, "onEnablerLoad"));
		}));
	},
	
	loadScriptAsync: function(scriptUrl, callback) {
		
		if (djConfig.isDebug) {
			scriptUrl = scriptUrl + "?debug=true";
		} 
		var element = dojo.doc.createElement("script");
		element.type = "text/javascript";
		element.src = scriptUrl;
		element.charset = "utf-8";
		element.onloadDone = false;
		element.onload = function()
		{
			if (!element.onloadDone) 
			{
				element.onloadDone = true;
				callback && callback();
			}
		};
		element.onreadystatechange = function()
		{
			if (("loaded" === element.readyState || "complete" === element.readyState) && !element.onloadDone) 
			{
				element.onload();
			}
		};
		dojo.doc.getElementsByTagName("head")[0].appendChild(element);
	},	
	
	onEnablerLoad: function() {
		// HACK
		// iWidget error callback override
		com.ibm.mm.iwidget.RenderController.prototype.handleWidgetInfoRetrieved = this.handleWidgetInfoRetrieved;

		this._status = LoadStatus.LOADED;
		dojo.forEach(this._loadCallbacks, function(cb)
		{
			cb();
		});
		this._loadCallbacks = [];
	},
	
	createIWidget: function(parentNode, widgetDefXml, parameters) {

		var factory = dojo.getObject("com.ibm.mashups.iwidget.model.Factory");
		if (!factory) {
			// TODO: MP: Return object to indicate error and handle error appropriately
			console.error("ERROR: Unable to load: 'com.ibm.mashups.iwidget.model.Factory'.");
			return null;
		}
		
		var widgetModel = factory.getGlobalWidgetModel();
		if (!widgetModel) {
			// TODO: MP: Return object to indicate error and handle error appropriately
			console.error("iWidget Enabler is not loaded!");
			return null;
		}
		
		var widgetId = "ns_" + dojox.uuid.generateTimeBasedUuid().replace(/\-/g, "");
		
		var rootSpan = dojo.doc.createElement("span");
		dojo.addClass(rootSpan, "mm_iWidget");
		rootSpan.setAttribute("id", widgetId);
		rootSpan.setAttribute("issystemwidget", "false");
		dojo.attr(rootSpan, "internalWidget", "true");
		parentNode.appendChild(rootSpan);
		
		
		var defTag = document.createElement("a");
		dojo.addClass(defTag, "mm_Definition");
		defTag.setAttribute("href", widgetDefXml);
		rootSpan.appendChild(defTag);
		
		if (parameters) 
		{
			var paramSpan = document.createElement("span");
			dojo.addClass(paramSpan, "mm_ItemSet");
			paramSpan.setAttribute("title", "attributes");
			
			for (var name in parameters) 
			{
				var item = document.createElement("a");
				dojo.addClass(item, "mm_Item");
				item.setAttribute("href", "#" + name);
				item.innerHTML = parameters[name];
				paramSpan.appendChild(item);
			}
			rootSpan.appendChild(paramSpan);
		}
		var iWidget = widgetModel.createWidget(rootSpan);
		widgetModel.renderWidget(iWidget);
		return iWidget;
	},
	
    handleWidgetInfoRetrieved: function(data, statuscode, xhr) {
        // logging support
        var LOG_METHOD = "handleWidgetInfoRetrieved";
        var bIsLoggable = this.LOGGER.isLoggable(this.LOG_LEVEL);
        if (bIsLoggable) {
            this.LOGGER.entering(LOG_METHOD, arguments);
        }
        var wTitle;
        if (statuscode == "200" || statuscode == "0") {
            this.widgetwrapper.setIWidgetDefinition(data); //save widget definition
            this.widgetDef = data;
        }
        else {
            if (!this._isWidgetInstalled()) {
                //handle widget not installed
                this._handleInlineWidgetNotInstalledMessage(this.iwStr.WIDGET_NOT_INSTALLED, this.iwStr.WIDGET_NOT_INSTALLED_SUB, data.message);
            }
            else {
            	// ORIGINAL CODE
                //handle error
                //wTitle = unescape(this.widgetwrapper.getIWidgetInstance().getIDescriptorItems().getItemValue("title", dojo.locale));
                //if (!wTitle) {
                //    wTitle = this.widgetXMLUrl;
                //}
                //this._handleInlineMessage("error", dojo.string.substitute(this.iwMessages.E_IWIDGETDEF_NOTAVAILABLE_1, [wTitle]), data.message);
                
                // HACK: Start --------------------------------->
    			if ((statuscode === 403) && 
    					xhr && 
    					xhr.getResponseHeader("X-net-jazz-ajax-proxy")) {
    				this._handleInlineMessage("error", dojo.string.substitute(messages.errorLoadingIWidgetWhitelist, [statuscode, this.widgetXMLUrl]));
    			} else {
                	this._handleInlineMessage("error", dojo.string.substitute(messages.errorLoadingIWidget, [statuscode, this.widgetXMLUrl]));
    			}                
                // HACK: End ----------------------------------->
                
            }
            return;
        }
        var isSandboxed = this.isSandboxed();
        if (dojo.isIE < 7 && isSandboxed) {
            wTitle = unescape(this.widgetwrapper.getIWidgetInstance().getIDescriptorItems().getItemValue("title", dojo.locale));
            if (!wTitle) {
                wTitle = this.widgetXMLUrl;
            }
            this._handleInlineMessage("error", dojo.string.substitute(this.iwMessages.E_NODISPLAY_UNSECUREWIDGET_1, [wTitle]));
            return;
        }
        this._doRender(isSandboxed);
        // exit trace
        if (bIsLoggable) {
            this.LOGGER.exiting(LOG_METHOD);
        }
    }	
	
});

/*
 * Static Methods
 */
dojo.mixin(jazz.iwidget, {

	DEFAULT_IBM_CONFIG: {
		"com.ibm.mashups.proxy.url": contextRoot + "/proxy",
		"com.ibm.mashups.hub.url": "/mashuphub",
		"com.ibm.mashups.contextroot": "/mum",
		"com.ibm.mashups.contextroot.builder": "/builder",
		"com.ibm.mashups.contextroot.enabler": "/enabler",
		"com.ibm.mashups.contenthandler.private": "/mycontenthandler",
		"com.ibm.mashups.contenthandler.public": "/contenthandler",
		"com.ibm.mashups.theme.defaultThemeId": "defaultTheme",
		"isDebug": "false",
		"allowPublishLogging": "true",
		"allowPublishTracing": "false",
		"additionalServices": "null",
		"popupConsole": "false",
		"pageSourceReadOnly": "true",
		"autoAcceptSharedGroupName": "default",
		"userIdKey": "uid",
		"groupCNKey": "cn",
		"tunnel.mode": "false",
		"available.locales": "tr",
		"default.locale": "en",
		"com.ibm.mashups.multipart.enabled": "false",
		"com.ibm.mashups.multipart.correlatehosts": "false",
			
		xhrTunneling: false,
		loadServices: false,
		availableLocales: "de,en,es,fr,it,he,ja,ko,pt-br,zh,zh-tw,ROOT",
			
		sessionId: null,
		hubURL: "/mashuphub",
		locale: "en-us",
		fontSizeEnlarged: false,
		mappingFunction: null,
		extensions: [],
		namespaces: {
			base: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base",
			model: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements",
			ext: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions",
			opensearch: "http://a9.com/-/spec/opensearch/1.1/"
		},
			
		servicedocument: '<?xml version="1.0" encoding="UTF-8"?><app:service xmlns:app="http://www.w3.org/2007/app" xmlns:atom="http://www.w3.org/2005/Atom" xmlns:xhtml="http://www.w3.org/1999/xhtml"><app:workspace><atom:title>Models</atom:title><app:collection href="http://localhost:8080/mum/contenthandler?uri=nm:collection"><atom:title>navigation</atom:title><app:categories><atom:category term="navigation"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection><app:collection href="http://localhost:8080/mum/contenthandler?uri=snm:collection"><atom:title>shared-navigation</atom:title><app:categories><atom:category term="shared-navigation"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection><app:collection href="http://localhost:8080/mum/contenthandler?uri=fragment:collection"><atom:title>content</atom:title><app:categories><atom:category term="content"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection><app:collection href="http://localhost:8080/mum/contenthandler?uri=catalog:collection"><atom:title>catalog</atom:title><app:categories><atom:category term="catalog"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection><app:collection href="http://localhost:8080/mum/contenthandler?uri=community:collection"><atom:title>community</atom:title><app:categories><atom:category term="community"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection><app:collection href="http://localhost:8080/mum/contenthandler?uri=theme:collection"><atom:title>theme</atom:title><app:categories><atom:category term="theme"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection><app:collection href="http://localhost:8080/mum/contenthandler?uri=um:collection"><atom:title>user</atom:title><app:categories><atom:category term="user"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection><app:collection href="http://localhost:8080/mum/contenthandler?uri=resource:collection"><atom:title>resource</atom:title><app:categories><atom:category term="resource"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection><app:collection href="http://localhost:8080/mum/contenthandler?uri=widget:collection"><atom:title>widget</atom:title><app:categories><atom:category term="widget"/><atom:category term="enabler-ns-base" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"/><atom:category term="enabler-ns-model" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements"/><atom:category term="enabler-ns-ext" scheme="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions"/></app:categories></app:collection></app:workspace></app:service>'
	},	
	
	// From Ed:
	// Not sure I like using a singleton, but neither do I like having to keep up with
	// a container pointer especially since the iWidget container is also a singleton.
	
	getContainer: function() {
		if (!this._widgetContainer) {
			this._widgetContainer = new jazz.iwidget.Container();
		}
		return this._widgetContainer;
	}
	
});

})();
