/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.ui.MenuPopup");

dojo.require("dijit._Widget");
dojo.require("jazz.util.moduleResource");
dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "MenuPopupMessages");

var messages,
	_currentOpen = null, 
	_currentOver = null, 
	_deferred = null;

dojo.declare("jazz.ui.MenuPopup", [dijit._Widget], {

	align: "left",
	around: null,
	cacheMenu: false,
	caretSrc: "",
	caretSrcDirty: "",
	caretClass: "jazz-ui-MenuPopup-caret",
	caretClassDirty: "jazz-ui-MenuPopup-dirtyCaret",
	destroyOnClose: false,
	group: "",
	injectCaret: false,
	label: "",
	menuClass: "",
	menuProvider: null,
	openClass: "menu-open",
	openOnHover: false,
	openOnCaretClick: false,
	
	openDelay: 300,
	closeDelay: 800,
	closeBufferPx: 5,
	
	_menuOpen: false,
	
	constructor: function() {
		 if (!messages) messages = dojo.i18n.getLocalization("jazz.ui", "MenuPopupMessages");
	},

	postCreate: function() {
		dojo.addClass(this.domNode, "jazz-ui-MenuPopup");
		if (this.group || this.openOnHover) this.connect(this.domNode, "onmouseover", this._onMouseOver);
		this.connect(this.domNode, "onclick", this._onClickHandler);
		this.connect(this.domNode, "onkeypress", this._handleKeypress);	
		this.connect(dijit.popup, "open", function (args) {
			if (args.popup === this._currentMenu)
				this._popupArgs = args;
		});
		
		if (this.injectCaret && (this.caretSrc||this.caretClass)){
			this._caret = dojo.create("img", {
				"class": "caret" + (this.caretClass ? " " + this.caretClass : ""),
				src: jazz.util.moduleResource(this.caretSrc)||this._blankGif,
				alt: ""
			}, this.focusNode||this.domNode);
			if (this.openOnCaretClick) {
				this.connect(this._caret, "onclick", this._onCaretClick);
			}
		}
		
		if (!this.around) {
			this.around = this.domNode;
		}
		dijit.setWaiState(this.focusNode||this.domNode, "haspopup", "true");
		dijit.setWaiState(this.focusNode||this.domNode, "label", (this.label||this.domNode.title||"")+messages.dropDownMenu);
	},
	
	open: function() {
		if (_currentOpen) {
			if (_currentOpen == this) {  // menu is already open.
				if (!this.openOnHover) this.close(false);
				return;
			} else {
				_currentOpen.close(false);	
			}
		}
		_currentOver = this;
		if ( this._mouseMoveEvent ) {
			this.disconnect(this._mouseMoveEvent);
			this._mouseMoveEvent = null;
		}
		
		if (this._menuOpen) {
			this._currentMenu.onOpen();
			return;
		}
		if (_deferred) {
			_deferred.cancel();
		}
		if (this.cacheMenu && this._currentMenu) {
			var menu = this._currentMenu;  //there's a menu from last time. Re-use it.
		} else {
			try {
				menu = this.menuProvider(this);
				if (menu instanceof dojo.Deferred) {
					menu.addCallback(dojo.hitch(this,this._deferredOpen));
					this._deferred = _deferred = menu;
					this.domNode.style.cursor = "progress";
					return;
				}
			} catch(e) {
				console.error(e);
			}
			if (!menu) return;
			this._currentMenu = menu;
			dojo.addClass(menu.domNode, "jazz-ui-MenuPopup-menu");
			dojo.addClass(menu.domNode, this.menuClass);
			dijit.setWaiState(menu.domNode, "labelledby", this.id);
			dijit.setWaiRole(menu.domNode, "menu");
		}
		var exec = dojo.hitch(this, this.close);
		this.domNode.focus();
		dijit.popup.open({
			popup: menu, 
			parent: this, 
			around: this.around, 
			orient: (this.align == "left") ? ["below","above"] : ["below-alt","above-alt"],
			onExecute: exec,
			onCancel: exec
		});
		if (menu.onReflow) this._reflow = this.connect(menu, "onReflow", this._onResize);
		if (!this._resizeHandlers) {
			this._resizeHandlers = [
				this.connect(window, "onresize", this._onResize),
				this.connect(window, "onscroll", this._onResize)
			];
		}
		this._menuOpen = true;
		_currentOpen = this;
		dojo.addClass(this.around,this.openClass);
		if (menu.startup) menu.startup();
		dijit.setWaiState(this.focusNode||this.domNode, "expanded", "true");
	},
	destroy: function() {
		this.close(false);
		delete this.around;
		this.inherited(arguments);
	},
	_onBlur: function() {
		this.close(false);
		this.inherited(arguments);
	},
	close: function(focus) {
		var currentMenu = this._currentMenu;
		if (currentMenu && this._menuOpen) {
			if (focus !== false)
				dijit.focus(this.focusNode||this.domNode);
			if (currentMenu.domNode) {
				dijit.popup.close(currentMenu);
			}
			if (this._resizeHandlers) {
				while(this._resizeHandlers.length) {
					this.disconnect(this._resizeHandlers.pop());
				}
				delete this._resizeHandlers;
			}
			dijit.setWaiState(this.focusNode||this.domNode, "expanded", "false");
			this._menuOpen = false;
			if (this.destroyOnClose) {
				if (currentMenu.domNode && currentMenu.destroyRecursive) {
					currentMenu.destroyRecursive();
				}
				this._currentMenu = null;
			}
			dojo.removeClass(this.around, this.openClass);
			this._mouseMoveEvent && this.disconnect(this._mouseMoveEvent);
			this._reflow && this.disconnect(this._reflow);
			clearTimeout(this._closeTimeout);
			clearTimeout(this._overTimeout);
			this._mouseMoveEvent = null;
			this._closeTimeout = null;
			this._overTimeout = null;
			this._reflow = null;
			_currentOpen = null;
		}
	},

	setDirty: function(dirty) {
		if (!this.injectCaret) {
			console.warn("[MenuPopup] setDirty not supported when injectCaret is false");
			return;
		}
		if (dirty && !this._caretWrapper) {
			this._caretWrapper = dojo.create("span", {
				"class": "caret-wrapper j-mirror",
				title: messages.unsaved
			}, this.domNode);
			dojo.place(this._caret,this._caretWrapper);
			this._caret.src = jazz.util.moduleResource(this.caretSrcDirty)||this._blankGif;
			this._caret.className = "caret" + (this.caretClassDirty ? " " + this.caretClassDirty : "");
		} else if (!dirty && this._caretWrapper) {
			this._caret.src = jazz.util.moduleResource(this.caretSrc)||this._blankGif;
			this._caret.className = "caret" + (this.caretClass ? " " + this.caretClass : "");
			dojo.place(this._caret,this.focusNode||this.domNode);
			this.domNode.removeChild(this._caretWrapper);
			delete this._caretWrapper;
		}
	},
	
	// private functions

	_onMouseOver: function(event) {
		dojo.stopEvent(event);
		if (_currentOpen) {
			if (_currentOpen == this || _currentOpen.group !== this.group) {
				return;
			}
			_currentOpen.close(false);
			this.open();
			
			//prevent accidental closing of the menu when clicking on a recently opened menu anchor.
			this._preventClose = true;
			setTimeout(dojo.hitch(this, function(){delete this._preventClose}),500);
		} else if (this.openOnHover && !this._overTimeout) {
			_currentOver = this;
			this._overTimeout = setTimeout(dojo.hitch(this,this._deferredOpen),this.openDelay);
		}
		if (this.openOnHover) {
			if (!this._mouseMoveEvent) this._mouseMoveEvent = this.connect(dojo.doc, "onmousemove", this._mouseMove);
		}
	},
	
	_mouseMove: function(event) {
		var stayOpen = false;
		if (this._menuOpen) {
			var pc = dojo.position(this._currentMenu.domNode),
				mouseX = event.clientX,
				mouseY = event.clientY,
				BUFFER = this.closeBufferPx;
			if (pc.x - BUFFER <= mouseX && pc.x + pc.w + BUFFER >= mouseX && pc.y - BUFFER <= mouseY && pc.y + pc.h + BUFFER >= mouseY) {
				stayOpen = true;
			}
		}
		if ((this._menuOpen && stayOpen) || dojo.isDescendant(event.target, this.around)) {
			//stay open
			clearTimeout(this._closeTimeout);
			this._closeTimeout = null;
			return;	
		} else {
			//close
			if (this._menuOpen) {
				if (dojo.isFunction(this._currentMenu.canDismiss)){
					if (this._currentMenu.canDismiss() === false) {
						//the popup doesn't want to be dismissed yet.
						return;
					}
				}
				// wait delay and kill popup.
				if (!this._closeTimeout) {
					this._closeTimeout = setTimeout(dojo.hitch(this,this._deferredKill), this.closeDelay);	
				}
			} else {
				this.disconnect(this._mouseMoveEvent);
				this._mouseMoveEvent = null;
				clearTimeout(this._overTimeout);
				this._overTimeout = null;
			}
		}		
	},
	
	_deferredKill: function() {
		this.disconnect(this._mouseMoveEvent);
		this._mouseMoveEvent = null;
		this.close();
	},
	
	_onCaretClick: function(event) {
		this._onClickHandler(event, true);
	},
	
	_onClickHandler: function(event, caretClick) {
		if (!this.openOnCaretClick || caretClick) {
			if (event) dojo.stopEvent(event);
			if (this._preventClose) return;
			this.open();
		} else {
			if (this._preventClose) return;
			this.close(false);
			clearTimeout(this._overTimeout);
			this._overTimeout = null;
		}
	},
	
	_deferredOpen: function(event) {
		clearTimeout(this._overTimeout);
		this._overTimeout = null;
		this.domNode.style.cursor = "";
		if (_currentOver && _currentOver != this) {
			// the timeout fired here, but the mouse is over another menu
			if (_deferred && _deferred == this._deferred) {
				_deferred = null;
			}
			_currentOver.open(); // open the other menu
		} else {
			this.open();
		}
		_currentOver = null;
	},
	_handleKeypress: function(e) {
		var k = dojo.keys;
		if (e.charCode == k.SPACE) e.charOrCode = k.SPACE;
		switch (e.charOrCode) {
			case k.UP_ARROW:
			case k.DOWN_ARROW:
			case k.ENTER:
			case k.SPACE:
				if (!this._menuOpen) {
					this.open();
				} else if (e.charOrCode == k.ENTER || e.charOrCode == k.SPACE) {
					this.close();
				} 
				dojo.stopEvent(e);
				break;
		}
		switch (e.charOrCode) {
			case k.UP_ARROW:
				this._currentMenu.focus("end");
				break;
			case k.DOWN_ARROW:
				this._currentMenu.focus("start");
				break;
			case k.TAB:
				if (this._menuOpen) {
					this._currentMenu.focus();
					e.preventDefault();
				}
				break;
			case k.ESCAPE:
			case k.LEFT_ARROW:
			case k.RIGHT_ARROW:
				if (this._menuOpen) {
					this.close();
					dojo.stopEvent(e);
				}
				break;
		}
	},
	_onResize: function(e){
		var menu = this._currentMenu;
		if (menu && menu.domNode && this._menuOpen) {
			var args = this._popupArgs;
			dijit.placeOnScreenAroundElement(args.popup.domNode.parentNode, args.around, args.orient, menu.orient ? dojo.hitch(menu,menu.orient) : null);
		}
	}
});
jazz.ui.MenuPopup.closeAll = function() {
	if (_currentOpen) _currentOpen.close();
};
})();