/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.adapter.internal.OpenSocialViewletSite");

dojo.require("com.ibm.team.dashboard.web.ui.ViewletSite");
dojo.require("com.ibm.team.dashboard.web.ui.internal.MementoAdapter");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.JPIMappingRegistry");

(function(){

var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var ViewletSite = com.ibm.team.dashboard.web.ui.ViewletSite;
var MementoAdapter = com.ibm.team.dashboard.web.ui.internal.MementoAdapter;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var JPIMappingRegistry = com.ibm.team.dashboard.web.ui.internal.util.JPIMappingRegistry;

dojo.declare("com.ibm.team.dashboard.web.adapter.internal.OpenSocialViewletSite",
	ViewletSite, {
		
	_isLoading: false,
	
	constructor: function(args) {
		dojo.mixin(this, args);
		this.prefs = new gadgets.Prefs();
	},
	
	/*
	 * Adds a menu item to the viewlet menu.
	 * 
	 * @param label the text shown for the menu item.
	 * @param icon (optional) either an image URL or a css class name specifying
	 * 		a background-image property for the menu item's icon, or null for none.
	 * 		The use of css class is deprecated as it does not work with iframed
	 * 		viewlets (because the css is not available outside the iframe).
	 * @param callback the function to call when the user activates the menu item. The scope
	 * 		(i.e. 'this') will be the viewlet adding the menu item.
	 */
	addMenuItem: function(/* string */ label, /* string */ icon, /* function */ callback) {
		// Not supported in the OpenSocial spec
	},
	
	/*
	 * Returns true or false based on if the current server license level allows viewing of the
	 * indicated dashboard type.
	 * 
	 * @param scope a number representing one of the constants for the scope of the dashboard
	 * @since 0.6.1
	 */
	/* boolean */ canViewDashboard: function(/* number */ scope) {
	},
	
	/*
	 * Signals to the viewlet container that the viewlet is done loading.
	 * Viewlets should call this.
	 */
	doneRefresh: function() {
		this.adapter._doResize();
		this._isRefreshing = false;
	},
	
	_isRefreshing: function() {
		return this._isRefreshing;
	}, 
	
	/*
	 * Can be called by viewlets to programatically enter edit mode.
	 */
	editContents: function() {
		this.adapter.handleEditContent();
	},
	
	/*
	 * Reports a serious error which may impact the normal operation of the
	 * dashboard or other viewlets (e.g. connection failure, server down).
	 * 
	 * @param error An Error object representing the problem.
	 * @since 0.7
	 */
	error: function(/* Error */ error) {
	},
	
	/*
	 * Returns the action area of the container, a DOM element that is part of
	 * the viewlet trim where specialized actions can be added while
	 * in edit mode.
	 */
	getActionArea: function() {
		return this.actionArea;
	},
	
	/*
	 * Returns the jazz.ui.FilterBox for the viewlet. Will return null if the
	 * viewlet is not flagged as "filterable" in its definition.
	 */
	getFilterBox: function() {
		return this.adapter.getFilterBox();
	},
	
	/*
 	 * Returns the height of the viewlet's content area
 	 */		
	getHeight: function() {
		return gadgets.window.getViewportDimensions().height;
	},
	
	/*
	 * Sets the secondary viewlet title.
	 */
	getSecondaryTitle: function() {
		// Not supported in the OpenSocial spec
	},
	
	/*
	 * Returns the viewlet's main title.
	 */
	getTitle: function() {
		var savedTitle = this.getPreference("_viewletTitle");
		if (!savedTitle || savedTitle == "") {
			return net.jazz.ajax._viewletDefinition.name;
		}
		return savedTitle;
	},
	
	/*
 	 * Returns the width of the viewlet's content area
 	 */		
	getWidth: function() {
		return gadgets.window.getViewportDimensions().width;
	},
	
	getScope: function() {
		var scope = this.getPreference("_scope");
		switch (net.jazz.ajax._viewletDefinition.appliesTo) {
			case Constants.SCOPE_APPLIES_TO_TEAMS:
				return DashboardConstants.SCOPE_TEAM_AREA;
			case Constants.SCOPE_APPLIES_TO_CONTRIBUTORS:
				return DashboardConstants.SCOPE_CONTRIBUTOR;
			case Constants.SCOPE_APPLIES_TO_PROJECTS:
				return DashboardConstants.SCOPE_PROJECT_AREA;
			case Constants.SCOPE_APPLIES_TO_PROCESS:
				if (scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
					return DashboardConstants.SCOPE_PROJECT_AREA;
				}
		}
		return scope;
	},
	
	getScopeItem: function() {
		// ignore the viewlet helper if the viewlet is not scope sensitive, in this case always defer to the dashboard
		if (net.jazz.ajax._viewletDefinition.scopeSensitive) {
			var itemId = this.getPreference("_scopeItem");
			if (itemId && itemId != "") {
				var scopeItem = {itemId: itemId};
				var name = ItemNameRegistry.getName(itemId);
				if (name) {
					scopeItem.name = name;
				}
				return scopeItem;
			}
		}
		
		// if the viewlet is not local, return null since dashboard scopes do not apply, viewlets must handle this case
		return null;
		
	},
	
	getContributorState: function() {
		return this.getViewletState();
	},
	
	getViewletState: function() {
		if (!this._viewletMemento) {
			var obj = {}
			var mementoStr = this.getPreference("_viewletContentMemento");
			if (typeof(mementoStr) != "string" || mementoStr == "") {
				obj = this.defaultState;
			} else {
				obj = dojo.fromJson(mementoStr);
			}
			this._viewletMemento = new MementoAdapter({
				obj: obj,
				onStateChange: dojo.hitch(this, function() {
					this.setPreference("_viewletContentMemento", 
						Util.toJson(this._viewletMemento.getState()));
				})
			});
		}
		return this._viewletMemento;
	},
	
	getScopeSubteams: function() {
		var value = this.getPreference("_scopeSubteams");
		if (typeof value === "boolean") {
			return value;
		}
		return (net.jazz.ajax._viewletDefinition.scopeSubteams == "yes-and-check");
	},
	
	/*
	 * Returns whether or not the viewlet is currently active, i.e. not hidden
	 * in a background UI element.
	 */
	isActive: function() {
		return true;
	},
	
	/*
	 * Returns whether or not the dashboard is currently editable.
	 */
	isDashboardEditing: function() {
		// Not supported by OpenSocial? Always return true
		return true;
	},
	
	/*
 	 * Indicates whether the viewlet is loading or not
 	 */	
	isLoading: function() {
		return _isLoading;
	},	
	
	/*
	 * Signals that the user is navigating away from the current page. This is
	 * used to prompt the user about unsaved changes.
	 */
	navigateAwayEvent: function(eventObj, workbenchNavigation) {
		// No implementation - the ViewletSiteAdapter doesn't seem to do much with this,
		// so I guess this adapter doesn't have to either
		return true;
	},
	
	/*
	 * Refreshes the viewlet.
	 */
	refresh: function() {
		this.viewlet && this.viewlet.refresh();
		this._isRefreshing = true;
	},
	
	/*
	 * Enables or disables automatic periodic refreshing of viewlets. Viewlets
	 * may wish to temporarily disable automatic refresh if it disrupts the
	 * normal user flow when, for example, the user is reading a hover tooltip or
	 * editing content. Normal auto-refresh will resume as soon as it is re-enabled.
	 */
	setAutoRefresh: function(/* boolean */ value) {
		if (net.jazz.ajax._viewletDefinition.refreshInterval > 0) {
			if (value && !this._refreshTimeout) {
				var timeSinceLastRefresh = new Date().getTime() - this._lastRefreshTime;
				var timeUntilNextRefresh = (net.jazz.ajax._viewletDefinition.refreshInterval * 1000) - timeSinceLastRefresh;
				// always wait at least 5 sec before an auto-refresh to avoid disruptions
				if (timeUntilNextRefresh < 5000) {
					timeUntilNextRefresh = 5000;
				}
				// resume auto-refresh
				this._refreshTimeout = setTimeout(dojo.hitch(this.viewlet, "refresh"), timeUntilNextRefresh);
			} else if (!value && this._refreshTimeout) {
				// suspend auto-refresh
				if (this._refreshTimeout) {
					clearTimeout(this._refreshTimeout);
					delete this._refreshTimeout;
				}
			}
		}
	},
	
	/*
	 * Sets the viewlet icon to the image pointed to by the given URL.
	 * 
	 * @param url the URL of the image file.
	 */
	setIcon: function(/* string */ url) {
		// Do nothing - open social spec doesn't need to support individual icons for gadgets (TODO verify)
	},
	
	/*
	 * Called to set loading state of viewlet.  Displays progress icon when loading.
	 */
	setLoading: function(loadingStatus) {
		isLoading = loadingStatus;
	},
	
	/*
	 * Sets the viewlet's secondary title.
	 */
	setSecondaryTitle: function(/* string */ title) {
		// Do nothing - open social spec doesn't need to support the notion of a 'secondary title' (TODO verify)
	},
	
	/*
	 * Called by viewlets to update the dynamic title. Some viewlets only know the title
	 * when the settings area is closed. These viewlets should indicate that the title
	 * needs to be persisted by setting the persist flag to true. Note that this results
	 * in a dashboard modification and will make the Dashboard dirty, thus it should
	 * only be done in tandem with another action that dirties the dashboard, such as
	 * changing the viewlet settings.
	 */
	setTitle: function(/* string */ title, /* boolean */ persist) {
		this.setPreference("_viewletTitle", title);
		gadgets.window.setTitle(title);
	},
	
	/*
	 * Can be called by viewlets to programatically open or close the settings area.
	 */
	toggleSettings: function() {
		this.adapter._handleSettings();
	},
	
	/* string */ getPreference: function(/* string */ id) {
		
		//HACK ALERT - some of the viewlets definitions have preferences with the '-' character in the name. 
		//OpenSocial spec disallows this - simply convert it into a lower case
		if (id.indexOf("-", 0) != -1) {
			id = id.replace(/-/g,"_");
		}

		var result;
		if (id == "_viewletTitle"
			|| id == "_viewletContentMemento"
			|| id == "_scope"
			|| id == "_scopeItem") {
			/* These preferences are injected and are not a part of the viewlet definition */
			result = this.prefs.getString(id);
		} else if (id == "_scopeSubteams") {
			return this.prefs.getBool(id);
		} else {
			var viewletPrefs = net.jazz.ajax._viewletDefinition.preferenceDefinitions;
			if (viewletPrefs) {
				for (var key in viewletPrefs) {
					var pref = viewletPrefs[key];
					if (id == pref.id) {
						var prefType = pref.mappedPrefType;
						//Valid values are "string", "hidden", "bool", "list", and "number".
						if (prefType == "string" 
							|| prefType == "hidden"
							|| prefType == "enum") {
							result = this.prefs.getString(id);
						} else if (prefType == "bool") {
							return this.prefs.getBool(id);
						} else if (prefType == "list") {
							result = this.prefs.getArray(id);
						} else if (prefType == "number") {
							result = this.prefs.getInt(id);
						}
					}
				}
			} else {
				// Viewlet doesn't have any defined preferences. This may be one of the implicit
				// scope preferences. Try to get a string and return it
				result = this.prefs.getString(id);
			}
		}
		
		if (result) {
			// WORKAROUD ALERT:
			result = decodeURIComponent(result);
		}
		
		return result;	
	},
	/*
	 * Sets the preference with the given id.
	 */
	setPreference: function(/* string */ id, /* string */ value) {
		if (id) {
			this.prefs.set(id, encodeURIComponent(value));
		}
	},
	
	/*
	 * Converts the array of JPIs into an array of matching URLs in the same order,
	 * and returns them in a deferred.
	 */
	/* dojo.Deferred */ jpisToURLs: function(/* string[] */ jpis) {
		return JPIMappingRegistry.jpisToURLs(jpis);
	},
	
	/*
	 * Converts the array of URLs into an array of matching JPIs in the same order,
	 * and returns them in a deferred.
	 */
	/* dojo.Deferred */ urlsToJPIs: function(/* string[] */ urls) {
		return JPIMappingRegistry.urlsToJPIs(urls);
	}
});
	
})();
