/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.adapter.internal.WindowlessDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.underlay");

(function() {

var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var underlay = jazz.ui.underlay;


dojo.declare("com.ibm.team.dashboard.web.adapter.internal.WindowlessDialog", [dijit._Widget, dijit._Templated], {
	
	templateString: "<div dojoAttachPoint='_dialogContent' name='com-ibm-team-dashboard-web-adapter-internal-WindowlessDialog' class='com-ibm-team-dashboard-web-adapter-internal-WindowlessDialog'></div>",
	
	/**
	 * DOM node for contents of the dialog.
	 * @type DomNode
	 * @required
	 */
	contentNode: null,
	
	/**
	 * DOM node that should get focus when the dialog opens
	 * @type DomNode
	 */
	focusNode: null,
	
	/**
	 * Specify the width (eg. "400px", "40%"). This is the outer width of the StyledBox (including trim).
	 * Zero and empty string result in "width: auto" (not recommended if {@link jazz.ui.Dialog.center} is true 
	 * due to inconsistent behavior in IE7 of position: absolute nodes). 
	 * @type String
	 * @default "40%"
	 */
	width: "40%",
	
	/**
	 * Specify the height. This is the inner height of the content area of the StyledBox (not including trim).
	 * Zero and empty string result in "height: auto".
	 * @type String
	 * @default ""
	 */
	height: "",
	
	/**
	 * Specify the title of the box. 
	 * @type String
	 * @default ""
	 */
	primaryTitle: "",
	
	/**
	 * Specify the secondary title of the box. Appears beside primary title, in a lighter color 
	 * @type String
	 * @default ""
	 */
	secondaryTitle: "",
	
	/**
	 * Specify the trim style of the StyledBox. Valid values are available under {@link jazz.ui.StyledBox.TrimStyles}.
	 * @type String
	 * @default BLUE_ALPHA_SHADOW
	 */
	trim: TrimStyles.BLUE_ALPHA_SHADOW,
	
	/**
	 * Specify whether the StyledBox can be closed by the user. 
	 * @type Boolean
	 * @default true
	 */
	closable: true,
	
	/**
	 * Specify whether the header color should get darker on hover. 
	 * @type Boolean
	 * @default false
	 */
	headerHoverState: false,
	
	/**
	 * Specify whether the header should default to hover state color. This is a slightly darker rendering for the header color.
	 * If true, {@link jazz.ui.StyledBox#headerHoverState} is ignored.
	 * @type Boolean
	 * @default true
	 */
	headerDefaultToHoverState: true,
	
	/**
	 * Specify whether the Dialog should have a solid background color. 
	 * @type Boolean
	 * @default false
	 */
	showBackground: false,
	
	/**
	 * Specify whether the Dialog should be modal (i.e. should the user be able to interact with stuff behind the dialog?) 
	 * @type Boolean
	 * @default true
	 */
	modal: true,
	
	/**
	 * Specify whether the Dialog can be moved with drag and drop. 
	 * @type Boolean
	 * @default true
	 */
	moveable: true,	
	
	/**
	 * Specify whether the Dialog should be positioned in the center of the window.  If false, the
	 * consumer is responsible for appending the dialog to the DOM. 
	 * @type Boolean
	 * @default true
	 */
	center: true,
	
	/**
	 * Specify whether the escape key should close the dialog. 
	 * @type Boolean
	 * @default true
	 */
	closeOnEsc: true,

	/**
	 * Specify whether the clicking outside the dialog should close the dialog. 
	 * @type Boolean
	 * @default false
	 */
	closeOnBlur: false,
	
	/**
	 * Specify the path to a 16x16 icon to appear in the header of the box to the left of the title. 
	 * @type String
	 * @default ""
	 */
	iconPath: "",
	
	/**
	 * Specify whether the dialog should destroy the widget passed in 'contentNode' on close (if it's a widget) 
	 * @type Boolean
	 * @default false
	 */
	destroyContents: false,
	
	/**
	 * Customize the arguments that are passed to jazz.ui.underlay when the dialog opens. Only used if {@link jazz.ui.Dialog#modal} is true. 
	 * @type Object
	 * @default null
	 */
	underlayArgs: null,
	
	/**
	 * A function to be called when the user asks to close the dialog. This function can return boolean false to veto the close. 
	 * @type Object
	 * @default null
	 */	
	onClose: null,
	
	/**
	 * A function to be called when the dialog is opened. 
	 * @type Object
	 * @default null
	 */
	onOpen: null,
	
	updatePosition: function(event) {
		// NO-OP
	},
	
	close: function(event) {
		underlay.hide();
		document.body.removeChild(this.domNode);
		this.onClose && this.onClose();
	},
	
	postCreate: function() {
		// Call onOpen if there is one...
		this.onOpen && this.onOpen();
		
		// Show the underlay to hide the existing content
		underlay.show({
			zIndex: 998, 
			animate: false,
			appendTo: this._dialogContent,
			stopEvents: true,
			opacity: 1, 
			color: "white"
		 });
		
		// Append the content node and display it in the document body
		this.contentNode.style.zIndex = 999;			 
		this._dialogContent.appendChild(this.contentNode);
		document.body.appendChild(this.domNode);
	}

});

})();