/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.client.internal.ParameterRegistry"); //$NON-NLS-1$

dojo.require("jazz.core.RegistryFactory"); //$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$

(function() { // module container

var RegistryFactory = jazz.core.RegistryFactory;
var loader = jazz.core.loader;

com.ibm.team.reports.web.client.internal.ParameterRegistry = {
	
	// ---- API ------------------------------------------------------------------------------------------------------------	
		
	getContributedParameterHandler: function(parameterType) {
		var candidates = [];
		for (var key in this._contributedHandlers) {
			if (this._matches(key, parameterType)) {
				candidates.push({key: key, value: this._contributedHandlers[key]});
			}
		}
		if (candidates.length == 0) {
			// No matches. Return the default handler, if one exists.
			return this._defaultHandler;
		}
		if (candidates.length == 1) {
			return candidates[0].value;
		}
		// Sort by length, longest first
		var sortFunction = function(a, b) {
			return b.key.length - a.key.length;
		};
		candidates.sort(sortFunction);
		
		for (var i = 0; i < candidates.length; i++) {
			var obj = dojo.getObject(candidates[i].value);
			if (obj.accept(parameterType)) {
				return candidates[i].value;
			}
		}
		// No matches. Return the default handler, if one exists.
		return this._defaultHandler;
	},
	
	/**
	 * Returns a dojo.Deferred that will call back when the registry has been initialized
	 */
	init: function() {
		if (this._contributedHandlers === null) {
			this._loadContributedParameterHandlers();
		}
		
		if (this._loadingHandlersDeferred){
			return this._loadingHandlersDeferred;
		} else {
			//handlers have been loaded already; just return a "blank" deferred
			var deferred = new dojo.Deferred();
			deferred.callback();
			return deferred;
		}
	},	

	// ---- implementation -------------------------------------------------------------------------------------------------

	_contributedHandlers: null,
	_defaultHandler: null,
	_loadingHandlersDeferred: null,

	_matches: function(extension, parameterType) {
		var exp = "^";
		if (extension.indexOf("*") == (extension.length - 1)) {
			var strippedExtension = extension.substring(0, extension.indexOf("*"));
			exp += strippedExtension;
		} else {
			exp += extension;
			exp += "$";
		}
		var regexp = new RegExp(exp);
		
		return regexp.test(parameterType);
	},
	
	_loadContributedParameterHandlers: function() {
		var extensionPoint = RegistryFactory.getRegistry().getExtensionPoint("com.ibm.team.reports.web.parameter"); //$NON-NLS-1$

		var contributedHandlers = [];
		var defaultHandler = null;
		
		var modulesToLoad = [];
		
		dojo.forEach(extensionPoint.getExtensions(), function(extension) {
			dojo.forEach(extension.getConfigurationElements(), function(configElement) {
				if (configElement.getAttribute("default") == "true") {
					defaultHandler = configElement.getAttribute("widgetId");
					modulesToLoad.push(defaultHandler);
				} else {
					var widgetId = configElement.getAttribute("widgetId");
					contributedHandlers[configElement.getAttribute("parameterType")] = widgetId;
					modulesToLoad.push(widgetId);
				}
			});
		});
		
		this._contributedHandlers = contributedHandlers;
		this._defaultHandler = defaultHandler;

		//we will load all the widget modules proactively, instead of on-demand, due to problems with 
		//synchronous module loading
		this._loadingHandlersDeferred = new dojo.Deferred();
		var self = this;
		
		try {
			loader.batch_load_async(modulesToLoad, function(){
				self._loadingHandlersDeferred.callback();	
				delete self._loadingHandlersDeferred;
			});
		} catch (e) {
			console.log(e);
		}

	},

	__sentinel: null // terminates this class definition
	
};

})();