/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.dnd.DraggableManager");

dojo.require("jazz.ui.underlay");

(function() {
	/*
	 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
	 * future with minimal or no warning.
	 * 
	 * @since 0.7
	 */
	jazz.ui.dnd.DraggableManager = function(){
		this.avatar = null;
		this.containers = new Array();
		this.fromContainer = null;
		this.overContainer = null;
		this.widget = null;
		this.isDragging = false;
		this.paused = false;
		this.onMouseMoveEvent = null;
		this.events = [];
		this.targets = new Array();
		this.maxRow = 0;  //set during drag operations in constrainAvatar = "A" containers
		this._uniqueId = 0;
	};
	
	dojo.extend(jazz.ui.dnd.DraggableManager, {
		destroy: function() {
			dojo.disconnect(this.onMouseMoveEvent);
			jazz.ui.dnd._draggableManager = null;
		},
		onMouseMoveInitial: function(e) {
			var difX = Math.abs(e.pageX - this.initialMouseCoords.x);
			var difY = Math.abs(e.pageY - this.initialMouseCoords.y);
			if (!this.isDragging && (difX >= 4 || difY >= 4)) {
				this.startDrag();
				this.onMouseMove(e);
			} else {
				this.dragStopEvent(e);
			}
		},
		getUniqueId: function() {
			var id = this._uniqueId;
			this._uniqueId++;
			return "draggable"+id;
		},
		onMouseMove: function(e){
			if (this.paused) return;
			this.dragStopEvent(e);
			var x = e.pageX;
			var y = e.pageY;
			this._updateAvatarPosition(x, y);
			var containers = this.currentContainers;
			var bestGuess = null;
			var fc = this.fromContainer;
			for (var i = 0; i < containers.length; i++) {
				var container = containers[i];
				var coords = container.coords;
				if (x < coords.x)
					continue; //if the mouseX coord is to the left of the box
				if (x > coords.x && y < coords.y)
					continue; //if the mouseY coord is above the box
				if ((x >= coords.x && x <= coords.x + coords.w) || (!container.isTarget && fc.constrainAvatar && (fc.constrainAvatar == "Y" || fc.constrainAvatar == "A"))) {
					if ((y >= coords.y && y <= coords.y + coords.h) || (!container.isTarget && fc.constrainAvatar && (fc.constrainAvatar == "X" || fc.constrainAvatar == "A"))) {
						this._notifyContainer(container,e);
						//move the current container to the top of the array
						containers.splice(i,1);
						containers.splice(0,0,container);
						return;
					} else if (y >= coords.y && !container.isTarget) {
						bestGuess = container;
					}
				}
			}
			if (bestGuess) {
				this._notifyContainer(bestGuess,e);
				return;
			}
			if (this.atEnd || this.atStart) {
				fc.onMouseMove(e);
			}
			this._notifyContainer({
				container: this.fromContainer,
				index: this.widget.index
			},e);
		},
		_notifyContainer: function(c,e) {
			var container = c.container;
			if (this.overContainer && container != this.overContainer) {
				container.clearPlaceholder();
			}
			if (this.overContainer && this.overContainer.node) {
				dojo.removeClass(this.overContainer.node, "dndOver");
			}
			this.overContainer = container;
			if (c.isTarget) {
				dojo.addClass(container.node, "dndOver");
			} else {
				container.onMouseMove(e,c.index);
			}
		},
		_updateAvatarPosition: function (pageX, pageY) {
			var a = this.avatar;
			if (a){
				var x = pageX - this.initialMouseCoords.offsetX;
				var y = pageY - this.initialMouseCoords.offsetY;
				if (this.fromContainer.relativeTo) {
					var rt = dojo.position(this.fromContainer.relativeTo,true);
					x = x - rt.x;
					y = y - rt.y;
				}
				if (!this.fromContainer.constrainAvatar) {  //if the avatar should follow the mouse (i.e. not constrained)
					dojo.marginBox(a.node, {l: x, t: y});
				} else {
					var coords = this.containerCoords;
					var widgetCoords = this.widgetCoords;
					
					if (this.fromContainer.constrainAvatar == "A") {
						if (!this.yStart) {
							this.yStart = coords.y;
						}
						this.maxRow = Math.round(this.overContainer.node.parentNode.clientHeight / widgetCoords.h) - 1;
						var row = Math.round(((y) - (coords.y)) / widgetCoords.h);
						if (row >= 0 && row <= this.maxRow) {
							this.yStart = coords.y + row * widgetCoords.h;
							if (dojo.isIE) this.yStart = this.yStart - widgetCoords.marginTop;
						}
						if (x < coords.x) {  // if mouse is to the left of the constrained area
							dojo.marginBox(a.node, {l: coords.x, t: this.yStart});
							this.atStart = true;
							this.atEnd = false;
							this.atRowEnd = false;
						} else if (x > coords.x + coords.w - widgetCoords.w - 8) {  //if mouse is to the right of the constrained area
							dojo.marginBox(a.node, {l: coords.x + coords.w - widgetCoords.w - 8, t: this.yStart});
							this.atStart = false;
							
							if (this.maxRow == row)
								this.atEnd = true;
							else {
								this.atEnd = false;
							}
							this.atRowEnd = true;
						} else { //if mouse is inside constrained area
							dojo.marginBox(a.node, {l: x, t: this.yStart});
							
							var node = this.overContainer.widgets[this.overContainer.widgets.length-1].domNode;
							var lastItemCoords = dojo.marginBox(node);
							lastItemCoords.x = dojo.position(node, true);
							if (row == this.maxRow && (x > coords.x + coords.w - widgetCoords.w - 10 || x > lastItemCoords.x + lastItemCoords.w)) {
								if (row == this.maxRow)
									this.atEnd = true;
								else 
									this.atEnd = false;
								this.atRowEnd = true;
								this.atStart = false;
							} else if (x < coords.x + 18) {
								this.atEnd = false;
								this.atRowEnd = false;
								this.atStart = true;
							} else {
								this.atEnd = false;
								this.atRowEnd = false;
								this.atStart = false;
							}
						}
					} else if (this.fromContainer.constrainAvatar == "X") {
						if (!this.yStart) {
							this.yStart = coords.y + widgetCoords.marginTop;
						}
						if (x < coords.x) {
							dojo.marginBox(a.node, {l: coords.x, t: this.yStart});
						} else {
							dojo.marginBox(a.node, {l: x, t: this.yStart});
						}
					} else if (this.fromContainer.constrainAvatar == "Y") {
						if (!this.xStart) {
							this.xStart = coords.x + widgetCoords.marginLeft;
						}
						if (y < coords.y) {
							dojo.marginBox(a.node, {l: this.xStart, t: coords.y});	
						} else {
							dojo.marginBox(a.node, {l: this.xStart, t: y});
						}
					}
				}
			}	
		},
		initDrag: function(container, widget, event){
			if (this.onMouseMoveEvent) return;
			
			this.widget = widget;
			this.fromContainer = this.overContainer = container;
			
			var coords = dojo.marginBox(widget.domNode);
			var coords2 = dojo.position(widget.domNode, true);
			coords.x = coords2.x;
			coords.y = coords2.y;
			coords.marginTop = dojo.style(widget.domNode,"marginTop");
			coords.marginLeft = dojo.style(widget.domNode,"marginLeft");
			coords.marginBottom = dojo.style(widget.domNode,"marginBottom");
			coords.marginRight = dojo.style(widget.domNode,"marginRight");
			this.widgetCoords = coords;
			
			this.initialMouseCoords = {
				x: event.pageX,
				y: event.pageY,
				offsetX: event.pageX - coords.x + coords.marginLeft,
				offsetY: event.pageY - coords.y + coords.marginTop
			};
			
			this.currentContainers = this._getValidContainers(widget.type);
			
			this.events.push(dojo.connect(document,"ondragstart", this, this.dragStopEvent));
			this.events.push(dojo.connect(document,"onselectstart", this, this.dragStopEvent));
			
			this.onMouseMoveEvent = dojo.connect(dojo.doc, "onmousemove", this, "onMouseMoveInitial");
		},
		_getValidContainers: function (type) {
			var validContainers = new Array();
			var widget = this.widget;
			var containers = this.containers;
			for (var i = 0; i < containers.length; i++) {
				var container = containers[i];
				if (container && !container._disabled && dojo.isDescendant(container.node, document.body) && dojo.some(container.accept, function(type) { return type == widget.type; })) {
					var coords = dojo.position(container.node, true);
					coords.w = container.node.offsetWidth;
					coords.h = container.node.offsetHeight;
					validContainers.push({
						container: container,
						coords: coords,
						isTarget: false
					});
				}
			}
			
			var targets = this.targets;
			for (i = 0; i < targets.length; i++) {
				var target = targets[i];
				if (target && !target._disabled && dojo.isDescendant(target.node, document.body) == true && dojo.some(target.accept, function(type) { return type == widget.type; })) {
					var coords = dojo.position(target.node, true);
					coords.w = target.node.offsetWidth;
					coords.h = target.node.offsetHeight;
					validContainers.push({
						container: target,
						coords: coords,
						isTarget: true
					});
				}
			}
			validContainers.sort(dojo.hitch(this,this._compareContainers));
			return validContainers;
		},
		_compareContainers: function(a, b) {
			if (a.coords.x < b.coords.x) 
				return 1;
			else if (a.coords.x > b.coords.x)
				return -1;
			else {
				if (a.coords.y < b.coords.y) 
					return 1;
				else if (a.coords.y > b.coords.y) 
					return -1;
			}
			return 0;
		},		
		startDrag: function(){
			dojo.disconnect(this.onMouseMoveEvent);
			if (this.fromContainer.onStartDrag) {
				dojo.hitch(this.fromContainer, this.fromContainer.onStartDrag)();
			}
			jazz.ui.underlay.show({opacity: 0, zIndex: 2000});
			if (this.overContainer) {
				if (this.overContainer != this.fromContainer) {
					if (this.overContainer.node)
						dojo.removeClass(this.overContainer.node,"dndOver");
					this.overContainer = this.fromContainer;
				}
			} else if (this.fromContainer) {
				this.overContainer = this.fromContainer;
			}
			this.isDragging = true;
			var de = document.documentElement;
			this._scrollMaxX = de.scrollWidth - de.clientWidth;
			this._scrollMaxY = de.scrollHeight - de.clientHeight;
			this.containerCoords = dojo.marginBox(this.fromContainer.node);
			var pos = dojo.position(this.fromContainer.node, true);
			this.containerCoords.x = pos.x;
			this.containerCoords.y = pos.y;
			if (this.fromContainer.useAvatar === false) {
				this.widget.node = this.widget.domNode;
				this.avatar = this.widget;
				this.avatar.node.style.position = "absolute";
				this.avatar.node.style.width = this.widgetCoords.w - this.widgetCoords.marginLeft - this.widgetCoords.marginRight + "px";
				this.avatar.node.style.zIndex = "1999";
				dojo.addClass(this.avatar.node, "jazz-ui-dnd-avatar");
				if (this.fromContainer.avatarClass) {
					dojo.addClass(this.avatar.node, this.fromContainer.avatarClass);
				}
			} else {
				this.avatar = new jazz.ui.dnd.Avatar(this, this.fromContainer.avatarClass);
				dojo.body().appendChild(this.avatar.node);
			}
			this.onMouseMoveEvent = dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove");
		},
		stopDrag: function(){
			dojo.disconnect(this.onMouseMoveEvent);
			this.onMouseMoveEvent = null;
			jazz.ui.underlay.hide();
			if (!this.overContainer.isSource) {
				dojo.removeClass(this.overContainer.node,"dndOver");
			}
			if (this.avatar) {
				if (this.fromContainer.useAvatar === false) {
					var aStyle = this.avatar.node.style;
					aStyle.position = aStyle.height = aStyle.width = aStyle.left = aStyle.top = aStyle.zIndex = "";
					dojo.removeClass(this.avatar.node, "jazz-ui-dnd-avatar");
					if (this.fromContainer.avatarClass) {
						dojo.removeClass(this.avatar.node,this.fromContainer.avatarClass);							
					}
					this.avatar.node = null;
				} else {
					this.avatar.destroy();
				}
				this.avatar = null;
			}
			this.fromContainer = null;
			this.overContainer = null;
			this.widget = null;
			setTimeout(dojo.hitch(this,function() {
				this.isDragging = false;
			}),0);
			while(this.events.length) {
				dojo.disconnect(this.events.pop());
			}
			this.yStart = null;
			this.xStart = null;
			this.widgetCoords = null;
			this.containerCoords = null;
			this.atEnd = false;
			this.atStart = false;
			this.paused = false;
			this.currentContainers = null;
			this.initialMouseCoords = null;
		},
		stopForInput: function() {
			this.paused = true;
		},
		dragStopEvent: function(event) {  //If viewlet is dragging, cancel text select and bubbling.
			if (event && (this.isDragging || this.fromContainer)) {
				dojo.stopEvent(event);
			}
		}
	});
	
	jazz.ui.dnd._draggableManager = null;
	
	jazz.ui.dnd.draggableManager = function(){
		if(!jazz.ui.dnd._draggableManager){
			jazz.ui.dnd._draggableManager = new jazz.ui.dnd.DraggableManager();
		}
		return jazz.ui.dnd._draggableManager;
	};
})();