/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.IFrameWidgetContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.xml.parser");

// Inclusion of this class automatically applies Jazz authentication logic 
// to responses received through jazz.client.xhr 
dojo.require("jazz.app.auth");

dojo.require("jazz.util.binding");
dojo.require("jazz.client.xhr");
dojo.require("jazz.ensemble.internal.util.bindings");

dojo.require("jazz.ensemble.internal.WidgetFactory");
dojo.require("jazz.ensemble.WidgetContext");

dojo.require("jazz.ui.MessageBox");

// TODO: Make a shared utility in common components?
dojo.require("jazz.ensemble.internal.catalog.ErrorUtility");

dojo.requireLocalization("jazz.ensemble.internal", "IFrameWidgetContainerMessages");

(function() {

/*
 * Aliases 
 */	
var WidgetFactory = jazz.ensemble.internal.WidgetFactory;
var WidgetContext = jazz.ensemble.WidgetContext;
var binding = jazz.util.binding;
var bindings = jazz.ensemble.internal.util.bindings;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
var ErrorUtility = jazz.ensemble.internal.catalog.ErrorUtility;

/*
 * Constants
 */
var RESIZE_POLL_INTERVAL = 1000; // 1 s
var DELIMETER = "~";
var CONTRIBUTOR_SERVICE_PATH = "/service/com.ibm.team.repository.common.internal.IContributorRestService/currentContributor";

/*
 * 
 */
dojo.declare("jazz.ensemble.internal.IFrameWidgetContainer",
	[dijit._Widget, dijit._Templated, WidgetContext], {
	
	// TODO: Styled box?
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/templates/IFrameWidgetContainer.html"),
	
	_id: window.name,

	_frontsideURL: null,
	_parentURL: null,
	_widgetState: null,
	_widget: null,
	_handler: null,
	_navigateTopicHandle: null,
	
	_onResizeTimeout: null,
	_height: 0,
	
	_static: function() {}, 	
	
	
	/*
 	 * @param Object args
 	 * 
 	 * @return IFrameWidgetContainer
 	 */
	constructor: function(args) {
		
		// Messages //--------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "IFrameWidgetContainerMessages");
		
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		
		// Instance Variables //----------------------------------------------->

		this._messageBox = null;
		this._name = "[container]: " + this._id;
		
		// Initialization //--------------------------------------------------->

		this._init();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initStatics: function() {
		this._static.COMMAND_STATUS = "1";
		this._static.COMMAND_RESIZE = "2";
		this._static.COMMAND_SETAUTOHEIGHT = "3";
		this._static.COMMAND_SETWIDGETSTATE = "4";
		this._static.COMMAND_START = "5";
		this._static.COMMAND_NAVIGATE = "6";
		
		this._static.STATUS_READY = "1";
		this._static.STATUS_OK = "2";
		this._static.STATUS_ERROR = "3";
		
		this._static.MIN_WIDGET_HEIGHT = 34; // Pixels
		
		this._static.NAVIGATE_TOPIC = "/jazz.ensemble.iframe.viewlet.container/navigate";
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_init: function() {
		ErrorUtility.logInfo(new Error("CREATE"), this._name);	
		
		this._initStatics();
		this._addPostMessageListener();
		this._loadFrontSideURL();
		this._loadParentURL();
		this._setBaseTarget();
		this._addNavigateTopicSubscription();

		var deferred = this._setAuthenticatedContributor();
		deferred.addCallback(dojo.hitch(this, function(response) {
			this._sendPostMessageData(this._static.COMMAND_STATUS, this._static.STATUS_READY);
			return response; 
		}));			

	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initAutoResizeFeature: function() {
		// Start polling interval
		this._onResize();
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_setBaseTarget: function() {
		
		var base = dojo.doc.getElementsByTagName("base")[0];
		// TODO: Check if base is defined

		// Can't set href otherwise code loading fails 
		//base.href = context._parentURL;
		base.target = "_top";
		
		//var head = dojo.doc.getElementsByTagName("head")[0];
		// TODO: Validate head is defined
		//dojo.create("base", { href: this._parentURL, target: "_top" }, head);
	},	
	
	/*
 	 * @param String command
 	 * @param String data
 	 * 
 	 * @return void
 	 */		
	_sendPostMessageData: function(command, data) {
		var parentWindow = window.parent;
		var result = this._id + DELIMETER + command + DELIMETER + data;
		parentWindow.postMessage(result, this._parentURL);
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadFrontSideURL: function() {
		// e.g. IFrame URL
		// https://localhost:9443/jazz/_ajax-modules/jazz.ensemble.iframe.viewlet.container?debug=true#http%3A%2F%2Flocalhost
		var url = window.location.href;
		// e.g. Front-side URL
		// https://localhost:9443/jazz
		var result = url.match(/^(.+?)\/_ajax-modules/);
		if ((result instanceof Array) &&
				(result.length === 2)) {
			this._frontsideURL = result[1];
		} else {
			ErrorUtility.logError(new Error("Failed to load the front-side URL."), this._name);
		}		
	},		

	/*
	 * @param void
	 * 
	 * @return void
	 */
	_setAuthenticatedContributor: function() {
		
		var deferred = this._xhrGetCurrentContributor(false);

		deferred.addCallback(dojo.hitch(this, function(response) {
			if (!response) {
				ErrorUtility.logError(new Error("Failed to set the authenticated contributor. Invalid response."), this._name);
				return response; 
			}
			// Unwrap SOAP envelope
			var contributor = dojo.getObject("soapenv:Body.response.returnValue.value", false, response);
			if (!contributor) {
				ErrorUtility.logError(new Error("Failed to set the authenticated contributor. Missing return value."), this._name);
				return response; 
			}
			// Required to support com.ibm.team.repository.web.client.session API for viewlets within the iframe widget container.
			// Since, no initialization data is available in the embedded jazz widget context and the session is not initialized.
			// e.g. Session.init(initData);	
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", contributor);
		
			return response; 
		}));			
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			// Fallback: Define the variable to prevent the checkNotNull exception from being thrown in Repository
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", {});

			ErrorUtility.logError(new Error("Failed to set the authenticated contributor. Exception encountered."), this._name);
			ErrorUtility.logError(errorObject, this._name);
			return errorObject; 
		}));	
		
		return deferred;
	},		
	
	/*
	 * @param Boolean passive
	 * 
	 * @return dojo.Deferred
	 */
	_xhrGetCurrentContributor: function(passive) {
		var url = this._frontsideURL + CONTRIBUTOR_SERVICE_PATH; 
		var args = {
			url: url,
			handleAs: "json",
			headers: {
				"Accept": "text/json"
			}
		};
		// Skip OAuth for passive requests
		if (passive) {
			args.skipInterceptors = ["jazz.oauth"];
		}		
		return jazz.client.xhrGet(args);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadParentURL: function() {
		// TODO: Check parent url - if null critical error?
		// Get the parent url from the hash
		var hash = location.hash; 
		if ((typeof(hash) !== 'string') ||
				(hash.length < 2)) {
			this._showStatusMessage(STATUS_TYPES.ERROR, 
					this.messages.errWidgetInit,
					this.messages.errWidgetInit_explanation,
					this.messages.errWidgetInit_useraction);
			return;
		}
		// Remove #
		var url = hash.substring(1);
		this._parentURL = decodeURIComponent(url);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_addPostMessageListener: function() {
		this._handler = this.connect(window, "onmessage", dojo.hitch(this, this._receivePostMessage));	
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_addNavigateTopicSubscription: function() {
		this._navigateTopicHandle = dojo.subscribe(this._static.NAVIGATE_TOPIC, dojo.hitch(this, function(url) {
			this._sendPostMessageData(this._static.COMMAND_NAVIGATE, url);
		}));
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_removePostMessageListener: function() {
		this.disconnect(this._handler);	
	},		
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_receivePostMessage: function(event) {
		
		if (!event) {
			ErrorUtility.logError(new Error("Command failed. The post message event is missing."), this._name);
			return false;
		} else if (!event.source) {
			ErrorUtility.logError(new Error("Command failed. The post message event source cannot be determined."), this._name);
			return false;
		} else if (event.origin !== this._parentURL) {
			ErrorUtility.logError(new Error("Command failed. The post message event origin does not match the server origin."), this._name);
			return false;
		}
		
		var result = event.data.split(DELIMETER);
		// TODO: Validate result
		var command = result[0];
		var data = result[1];		

		// TODO: Yield UI thread with timeout?
		
		if (command === this._static.COMMAND_SETAUTOHEIGHT) {
			ErrorUtility.logInfo(new Error("COMMAND:SETAUTOHEIGHT"), this._name);
			this._initAutoResizeFeature();
		} else if (command === this._static.COMMAND_SETWIDGETSTATE) {
			ErrorUtility.logInfo(new Error("COMMAND:SETWIDGETSTATE"), this._name);
			this._processWidgetState(data);
		} else if (command === this._static.COMMAND_START) {
			ErrorUtility.logInfo(new Error("COMMAND:COMMAND_START"), this._name);
			this._createWidget();
		} else {
			ErrorUtility.logError(new Error("Unrecognized command received: " + command), this._name);
			return false;
		}
		
		return true;
	},		
	
	_processWidgetState: function(data) {
		var xml = this._unescapeWidgetState(data);
		var widgetElement = this._getWidgetXMLElement(xml);
		var widget = binding.bind(widgetElement, bindings.Widget);
		this._widgetState = widget;
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_createWidget: function() {
		var deferred = WidgetFactory.createWidget(this._widgetState, this, true);
		deferred.addCallback(dojo.hitch(this, this._createContents));
		deferred.addErrback(dojo.hitch(this, this._showError));	
	},		

	/*
 	 * @param Widget widget
 	 * 
 	 * @return void
 	 */	
	_createContents: function(widget) {
		
		this._widget = widget;
		
		// Remove loading text
		dojo.removeClass(this._contentArea, "loading-text");
		// Add widget
		dojo.place(widget.domNode, this._contentArea, "only");
		// Start widget
		widget.startup();
		
		// TODO: Required?
		this._sendPostMessageData(this._static.COMMAND_STATUS, this._static.STATUS_OK);

		return widget;
	},
	
	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */	
	_showError: function(errorObject) {

		ErrorUtility.logError(errorObject, this._name);
		this._showStatusMessage(STATUS_TYPES.ERROR, 
				this.messages.errWidgetLoad,
				this.messages.errWidgetLoad_explanation,
				this.messages.errWidgetLoad_useraction);
		
		// TODO: Required?
		this._sendPostMessageData(this._static.COMMAND_STATUS, this._static.STATUS_ERROR);
	
	},
	
	_getWidgetXMLElement: function(xml) {
		var xmlDocument = dojox.xml.parser.parse(xml);
		return xmlDocument.documentElement;
	},		
	
	_unescapeWidgetState: function(state) {
		// Unescape XML String
		// <, >, &, ~
		// Note: Tilde (~) is used as a delimeter.
		var result = state.replace(/&amp;/gm, "&").replace(/&lt;/gm, "<").replace(/&gt;/gm, ">").replace(/&tilde;/gm, "~");
		return result;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_onResize: function() {
		var computedHeight = this._getHeight();
		if (computedHeight < this._static.MIN_WIDGET_HEIGHT) {
			computedHeight = this._static.MIN_WIDGET_HEIGHT;
		} 
		if (computedHeight !== this._height) {
			ErrorUtility.logInfo(new Error("RESIZE: " + this._height + " -> " + computedHeight), this._name);
			this._height = computedHeight;
			// TODO: Validate height?
			this._sendPostMessageData(this._static.COMMAND_RESIZE, computedHeight);
		}
		clearTimeout(this._onResizeTimeout);
		// Poll
		this._onResizeTimeout = setTimeout(dojo.hitch(this, this._onResize), RESIZE_POLL_INTERVAL);
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return Number
 	 */	
	_getHeight: function() {
		if (!document.body) { 
			return 0;
		}
		var position = dojo.position(document.body);
		if (!position) {
			return 0;
		}
		return Math.floor(position.h);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return MessageBox
 	 */		
	_getMessageBox: function() {
		if (!this._messageBox) {
			this._messageBox = new MessageBox().placeAt(this._messageBoxSite);
		}
		return this._messageBox;
	},	
	
	/*
 	 * @param Number type
 	 * @param String summary
 	 * @param String explanation
 	 * @param String useraction
 	 * 
 	 * @return void
 	 */		
	_showStatusMessage: function(type, summary, explanation, useraction) {

		var args = {
			type: type,
			summary: summary,
			closable: true
		};
		
		if (explanation && 
				useraction) {
			args.details = [
			    {
					message: explanation
				}, 
				{
					message: useraction
				}
			];
		}
		
		var messageBox = this._getMessageBox();
		messageBox.showMessage(args);
		
	},	
	
	// Widget Context Methods ------------------------------------------------------------------------>
	
	/*
	 * Gets the widget's 16x16 icon URL, or undefined if there is none.
	 * 
	 * @return {string} url the 16x16 icon url
	 */
	getIcon: function() {
		if (!this._icon) {
			this._icon = this._widgetState.getIcon();
		}
		// TEST
		// ErrorUtility.logInfo(new Error("WidgetContext.getIcon()"), this._name);
		return this._icon;
	},	
	
	/*
	 * Returns the memento, which is a string containing arbitrary persisted data
	 * for the widget. Provided by the container if exists. Can be set via setMemento.
	 * 
	 * @return {string} the widget memento
	 */
	getMemento: function() {
		if (!this._memento) {
			this._memento = this._widgetState.getMemento();
		}
		// TEST
		// ErrorUtility.logInfo(new Error("WidgetContext.getMemento()"), this._name);
		return this._memento;
	},	
	
	/*
	 * Returns an object with string properties representing preference key-value
	 * pairs. Provided by the container. Can be set via setPreferences.
	 * 
	 * @return {object} the widget preferences
	 */
	getPreferences: function() {
		if (!this._preferences) {
			var preferences = this._widgetState.getPreferences();
			// Not a real Array...
			if (preferences && 
					preferences.length) {
				this._preferences = {};
				for (var index = 0; index < preferences.length; index++) {
					var preference = preferences[index];
					if (!preference) {
						ErrorUtility.logError(new Error("Undefined preference encountered at index: " + index), this._name);
						continue;
					}
					this._preferences[preference.getName()] = preference.getValue();
				}
			}
		}
		// TEST
		// ErrorUtility.logInfo(new Error("WidgetContext.getPreferences()"), this._name);
		return this._preferences;
	},	
	
	/**
	 * Returns the widget container's current secondary title.
	 * 
	 * @return {string} the current widget's secondary title
	 */	
	getSecondaryTitle: function() {
		// TEST
		// ErrorUtility.logInfo(new Error("WidgetContext.getSecondaryTitle()"), this._name);
		return "";
	},
	
	/**
	 * Returns the widget container's current title.
	 * 
	 * @return {string} the current widget title
	 */	
	getTitle: function() {
		if (!this._title) {
			this._title = this._widgetState.getTitle();
		}
		// TEST
		// ErrorUtility.logInfo(new Error("WidgetContext.getTitle()"), this._name);
		return this._title;
	},
	
	/**
	 * Returns the widget container's toolbar, where additional toolbar items can
	 * be added, or undefined if the container has no toolbar.
	 * 
	 * @return {jazz.ui.SimpleToolbar} the container's toolbar
	 */
	getToolbar: function() {
		// TEST
		// ErrorUtility.logInfo(new Error("WidgetContext.getToolbar()"), this._name);
		return null;
	},	
	
	/*
	 * Returns whether or not the widget state is shared among multiple users or not.
	 * Widgets may use this information to decide whether or not to allow user-
	 * specific personalization of the state.
	 * 
	 * @return {boolean} whether or not the widget state is shared or not
	 */
	isShared: function() {
		// TEST
		// ErrorUtility.logInfo(new Error("WidgetContext.isShared()"), this._name);
		// TODO: Should be false?
		return true;
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		ErrorUtility.logInfo(new Error("DESTROY"), this._name);
		
		if (this._messageBox) {
			this._messageBox.destroy();
			delete this._messageBox;
		}
		
		if (this._onResizeTimeout) {
			clearTimeout(this._onResizeTimeout);
			delete this._onResizeTimeout;
		}		
		
		if (this._navigateTopicHandle) {
			dojo.unsubscribe(this._navigateTopicHandle);
		}
		
		// TODO: Destroy widget
		
		this.inherited(arguments);
	}

});

})();
