/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.ui.internal.PopupMenu");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

var _currentOpen = null;

dojo.declare("jazz.ui.internal.PopupMenu", [dijit._Widget, dijit._Templated], {

	align: "left",
	group: "",
	menu: null,
	
	createMenu: function(/* this widget */) {
	},
	
	isOpen: function() {
		return _currentOpen === this;
	},
	
	toggleOpen: function() {
		if (this.isOpen())
			this.close();
		else
			this.open();
	},
	
	open: function() {
		if (this.isOpen())
			return;
		this.createMenu(this);
		var menu = this.menu;

		dijit.setWaiState(menu.domNode, "labelledby", this.id);
		
		var close = dojo.hitch(this, this.close);
		this.domNode.focus();
		dijit.popup.open(this._popupArgs = {
			popup: menu,
			parent: this,
			around: this.domNode,
			orient: (this.align == "right") ? {"BR":"TR","TR":"BR"}: {"BL":"TL","TL":"BL"},
			onExecute: close,
			onCancel: close
		});
		
		if (menu.onReflow)
			this._reflow = this.connect(menu, "onReflow", this._onResize);
		this._onresize = this.connect(window, "onresize", this._onResize);
		this._onscroll = this.connect(window, "onscroll", this._onResize);
		_currentOpen = this;
		dojo.addClass(this.domNode, "menu-open");
		if (menu.startup)
			menu.startup();
		dijit.setWaiState(this.domNode, "expanded", "true");
	},
	
	destroy: function() {
		this.close();
		this.inherited(arguments);
	},
	
	onBlur: function() {
		this.close();
	},
	
	close: function() {
		if (!this.isOpen())
			return;
		var menu = this.menu;
		dijit.popup.close(menu);
		this.disconnect(this._onscroll);
		this.disconnect(this._onresize);
		this.disconnect(this._reflow);
		delete this._onscroll;
		delete this._onresize;
		delete this._reflow;
		
		dojo.removeClass(this.domNode, "menu-open");
		dijit.setWaiState(this.domNode, "expanded", "false");
		menu.destroyRecursive();
		delete this.menu;
		_currentOpen = null;
	},

	// private functions

	_createMenu: function() {
		this.onProvideMenu(this);
		return this.menu;
	},

	_onMouseOver: function(event) {
		if (!this.group)
			return;
		if (_currentOpen && _currentOpen.group === this.group && _currentOpen !== this) {
			var self = this;
			self.open();
			self._preventClose = true;
			setTimeout(function() {
						delete self._preventClose;
					},500);
		}
	},
	
	_handleDijitClick: function(event) {
		event.preventDefault();
		if (this._preventClose)
			return;
		this.toggleOpen();
	},
	
	_handleKeypress: function(e) {
		return;
		var k = dojo.keys;
		if (e.charCode == k.SPACE) e.charOrCode = k.SPACE;
		switch (e.charOrCode) {
			case k.UP_ARROW:
			case k.DOWN_ARROW:
			case k.ENTER:
			case k.SPACE:
				if (!this.isOpen()) {
					this.open();
				} else if (e.charOrCode == k.ENTER || e.charOrCode == k.SPACE) {
					this.close();
				} 
				dojo.stopEvent(e);
				break;
		}
		switch (e.charOrCode) {
			case k.UP_ARROW:
				this._currentMenu.focus("end");
				break;
			case k.DOWN_ARROW:
				this._currentMenu.focus("start");
				break;
			case k.TAB:
				if (this.isOpen()) {
					this._currentMenu.focus();
					e.preventDefault();
				}
				break;
			case k.ESCAPE:
			case k.LEFT_ARROW:
			case k.RIGHT_ARROW:
				if (this.isOpen()) {
					this.close();
					dojo.stopEvent(e);
				}
				break;
		}
	},
	_onResize: function(e){
		var menu = this.menu;
		if (this.isOpen()) {
			var args = this._popupArgs;
			dijit.placeOnScreenAroundElement(args.popup.domNode.parentNode, args.around, args.orient, menu.orient ? dojo.hitch(menu,menu.orient) : null);
		}
	}
});
})();