/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.catalog.WidgetCodeSnippetDialog");

/*
 * Imports 
 */
dojo.require("dojo.i18n");

dojo.require("jazz.ui.Dialog");
dojo.require("dijit.form.SimpleTextarea");

dojo.requireLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

(function() {

/*
* Aliases 
*/
var Dialog = jazz.ui.Dialog;

/*
 * Constants 
 */

/*
 * Class
 */
dojo.declare("jazz.ensemble.internal.catalog.WidgetCodeSnippetDialog",
		[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->
	
	codeSnippet: "",
	
	// ------------------------------------------------------------------------>

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/catalog/templates/WidgetCodeSnippetDialog.html"),
	widgetsInTemplate: true,
	
	/*
 	 * @param Object args
 	 * 
 	 * @return WidgetCodeSnippetDialog
 	 */
	constructor: function(args) {
	
		// Messages //--------------------------------------------------------->

		this.msg = dojo.i18n.getLocalization("jazz.ensemble.internal", "WidgetCatalogMessages");

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._dialog = null;
		
		// Initialization //--------------------------------------------------->
		
		this._initUI();
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initUI: function() {
		// Disable spell-checking feature in FF
		this._textArea.domNode.spellcheck = false;	
		this._textArea.domNode.style.width = "558px";		
			
		if (this.codeSnippet) {
			this._textArea.set('value', this.codeSnippet);
		}
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	show: function() {
		this._dialog = new Dialog({
			contentNode: this.domNode,
			primaryTitle: this.msg.snippetDialogTitle,
			width: "600px",
			destroyContents: true
		});		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	hide: function() {
		if (this._dialog) {
			this._dialog.close();
		}		
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOnClickOKButton: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		
		this.hide();
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {

		// Dialog
		if (this._dialog) {
			this._dialog.destroy();
			delete this._dialog;
		}

		this.inherited(arguments);
	}

});

})();
