/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.wizards.ExportWidgetWizard");

/*
 * Imports 
 */
dojo.require("dojo.i18n");

dojo.require("jazz.ui.Wizard");

dojo.require("jazz.ensemble.internal.wizards.ExportWidgetWizardStep1");
dojo.require("jazz.ensemble.internal.wizards.ExportWidgetWizardStep2");

dojo.requireLocalization("jazz.ensemble.internal", "Messages");

(function() {

/*
* Aliases 
*/
var Wizard = jazz.ui.Wizard;
var ExportWidgetWizardStep1 = jazz.ensemble.internal.wizards.ExportWidgetWizardStep1;
var ExportWidgetWizardStep2 = jazz.ensemble.internal.wizards.ExportWidgetWizardStep2;


/*
 * Constants 
 */


/*
 * Class
 */
dojo.declare("jazz.ensemble.internal.wizards.ExportWidgetWizard",
		[dijit._Widget], {

	// Properties //----------------------------------------------------------->

	state: null,
	width: 0,  // auto
	height: 0, // auto
	
	// ------------------------------------------------------------------------>
	
	/*
 	 * @param Object args
 	 * 
 	 * @return ExportWidgetWizard
 	 */
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "Messages");
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._wizard = null;
		this._step1 = null;
		this._step2 = null;
		
		// Initialization //--------------------------------------------------->

		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	show: function() {
		// Create step 1
		this._step1 = new ExportWidgetWizardStep1({
			context: this
		});
		// Create step 2
		this._step2 = new ExportWidgetWizardStep2({
			context: this
		});
		// Create wizard
		this._wizard = new Wizard({
			steps: [this._step1, this._step2],
			primaryTitle: this.messages.wizardsExportWidget,
			width: "600px",
			stepListLayout: "horizontal"
		});		
		// TODO: Call hide on wizard close?
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	hide: function() {
		if (this._wizard) {
			this._wizard.close();
		}
		this._uninitializeWizard();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_uninitializeWizard: function() {
		if (this._step1) {
			this._step1.destroy();
			delete this._step1;
		}
		if (this._step2) {
			this._step2.destroy();
			delete this._step2;
		}	
		if (this._wizard) {
			this._wizard.destroy();
			delete this._wizard;
		}
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {

		this._uninitializeWizard();
		
		this.inherited(arguments);
	}

});

})();
