/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.AllTeamAreasPage");

dojo.require("com.ibm.team.dashboard.web.ui.Dashboard");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardConfiguration");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dojo.hash");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.core.loader");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var Dashboard = com.ibm.team.dashboard.web.ui.Dashboard;
var DashboardConfiguration = com.ibm.team.dashboard.web.ui.internal.DashboardConfiguration;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var messages;

var ACTION_VIEW_ALL_DASHBOARDS = DashboardConstants.ACTION_VIEW_ALL_DASHBOARDS;
var PARAM_ACTION = "action";
var VAR_PROJECT_NAME = "${projectName}";
var PATH_PREFIX = "/web/projects/" + VAR_PROJECT_NAME;

var DEFAULT_ALL_DASHBOARDS_HREF = PATH_PREFIX + "#" + PARAM_ACTION + "=" + ACTION_VIEW_ALL_DASHBOARDS;

dojo.declare("com.ibm.team.dashboard.web.ui.AllTeamAreasPage",
		[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllTeamAreasPage.html"),
	widgetsInTemplate: true,
	
	constructor: function() {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	postCreate: function() {
		this.subscribe("/jazz/dashboard/initMenu", this._initMenu);
		var workbench = PlatformUI.getWorkbench();
		// register actions only if default URLs are being used
		var actionRegistry = workbench.getActionRegistry();
		if (DashboardConfiguration.hrefs.allProjectDashboards === DEFAULT_ALL_DASHBOARDS_HREF) {
			actionRegistry.registerAction(ACTION_VIEW_ALL_DASHBOARDS, this, "_handleViewAllDashboard");
		}
	},
	
	destroy: function() {
		this._destroyDashboard();
		this.inherited(arguments);
	},
	
	/**
	 * Shows the hierarchy of all dashboards for this project.
	 */
	showAllDashboards: function() {
		if (this._allDashboards) return;
		this._destroyDashboard();
		jazz.core.loader.load_async("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas", dojo.hitch(this,function() {
			this._allDashboards = new com.ibm.team.dashboard.web.ui.internal.AllTeamAreas();
			this._setContent(this._allDashboards);
			this._updateBrowserTitle();
		}));
	},
	
	_destroyDashboard: function() {
		if (this._allDashboards) {
			this._allDashboards.destroy();
			delete this._allDashboards;
		}
		window.scrollTo(0,0);
	},
	
	_handleViewAllDashboard: function() {
		this.showAllDashboards();
	},
	
	_initMenu: function(menu) {
		if (typeof unsaved === "object") {
			menu.onDirtyStateChange(unsaved, true);
			unsaved = null;
		}
	},
		
	_setContent: function(/* Widget */ widget) {
		widget.placeAt(this.content, "only");
		this._contentWidget = widget;
	},
	
	_updateBrowserTitle: function() {
		if (this._allDashboards) {
			title = messages.allTeamAreasTitle;
		} else {
			title = messages.dashboardsGenericTitle;
		}
		title = dojo.string.substitute(messages.projectDashboardTitle, [title]);
		PlatformUI.getWorkbench().setTitle(title);
	}
});

})();