/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.ProjectDashboardPage");

dojo.require("com.ibm.team.dashboard.web.ui.Dashboard");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.ProjectDashboard");
dojo.require("com.ibm.team.dashboard.web.ui.TeamDashboard");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardConfiguration");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dojo.hash");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.app.context");
dojo.require("jazz.app.viewHistory");
dojo.require("jazz.core.loader");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.util.html");

dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var BootstrapProperties = net.jazz.ajax.BootstrapProperties;
var Dashboard = com.ibm.team.dashboard.web.ui.Dashboard;
var DashboardConfiguration = com.ibm.team.dashboard.web.ui.internal.DashboardConfiguration;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ProjectDashboard = com.ibm.team.dashboard.web.ui.ProjectDashboard;
var TeamDashboard = com.ibm.team.dashboard.web.ui.TeamDashboard;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var messages;
var unsaved = true;

var ACTION_VIEW_DASHBOARD = DashboardConstants.ACTION_VIEW_DASHBOARD;
var ACTION_VIEW_ALL_DASHBOARDS = DashboardConstants.ACTION_VIEW_ALL_DASHBOARDS;
var DASHBOARD_TYPE = "com.ibm.team.dashboard";
var PARAM_ACTION = "action";
var PARAM_TEAM = "team";
var VAR_PROJECT_NAME = "${projectName}";
var VAR_TEAM_PATH = "${team}";
var PATH_PREFIX = "/web/projects/" + VAR_PROJECT_NAME;

var DEFAULT_ALL_DASHBOARDS_HREF = PATH_PREFIX + "#" + PARAM_ACTION + "=" + ACTION_VIEW_ALL_DASHBOARDS;
var DEFAULT_PROJECT_DASHBOARD_HREF = PATH_PREFIX + "#" + PARAM_ACTION + "=" + ACTION_VIEW_DASHBOARD;
var DEFAULT_TEAM_DASHBOARD_HREF = PATH_PREFIX + "#" + PARAM_ACTION + "=" + ACTION_VIEW_DASHBOARD + "&" + PARAM_TEAM + "=" + VAR_TEAM_PATH;

/**
 * The project dashboard page, which displays project and team dashboards for applications, and
 * provides navigation logic including breadcrumbs and all project dashboards.
 * 
 * The page can be consumed as-is if default dashboard hrefs are desired. If custom hrefs are
 * needed, the page should be subclassed and the href properties overridden. If custom hrefs are
 * used, the subclass must also register its own URL handlers and call the methods to show the
 * team and project dashboards accordingly.
 * 
 * This page is meant to be used in conjunction with the ProjectDashboardComponent, and will
 * automatically coordinate to show recently viewed and current selection.
 * 
 * @since 4.0.1
 */
dojo.declare("com.ibm.team.dashboard.web.ui.ProjectDashboardPage",
		[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ProjectDashboardPage.html"),
	widgetsInTemplate: true,
	
	constructor: function() {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	postCreate: function() {
		this.subscribe("/jazz/dashboard/initMenu", this._initMenu);
		var workbench = PlatformUI.getWorkbench();
		// register actions only if default URLs are being used
		var actionRegistry = workbench.getActionRegistry();
		if (DashboardConfiguration.hrefs.projectDashboard === DEFAULT_PROJECT_DASHBOARD_HREF ||
				DashboardConfiguration.hrefs.teamDashboard === DEFAULT_TEAM_DASHBOARD_HREF) {
			actionRegistry.registerAction(ACTION_VIEW_DASHBOARD, this, "_handleViewDashboard");
		}
		if (DashboardConfiguration.hrefs.allProjectDashboards === DEFAULT_ALL_DASHBOARDS_HREF) {
			actionRegistry.registerAction(ACTION_VIEW_ALL_DASHBOARDS, this, "_handleViewAllDashboard");
		}
		this.navigateAwayHandle = workbench.registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
	},
	
	destroy: function() {
		this._destroyDashboard();
		this.inherited(arguments);
	},
	
	/**
	 * Shows the hierarchy of all dashboards for this project.
	 */
	showAllDashboards: function() {
		if (this._allDashboards) return;
		this._destroyDashboard();
		jazz.core.loader.load_async("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas", dojo.hitch(this,function() {
			this._allDashboards = new com.ibm.team.dashboard.web.ui.internal.AllTeamAreas();
			this._setContent(this._allDashboards);
			this._updateBrowserTitle();
		}));
	},
	
	/**
	 * Shows the project dashboard for the current project.
	 * 
	 * @param {string || number} page the string id or index of the widget page to show.
	 */
	showProjectDashboard: function(page) {
		this._showDashboard({tab: page});
	},
	
	/**
	 * Shows the specified team dashboard.
	 * 
	 * @param {string} team a slash-separated path string containing the team hierarchy from a
	 * 		top-level team in the project to the target team. Each path segment is a team name
	 * 		(unencoded). e.g. "Root Team/Child Team".
	 * @param {string || number} page the string id or index of the widget page to show.
	 */
	showTeamDashboard: function(team, page) {
		this._showDashboard({team: team, tab: page});
	},
	
	_addViewHistoryEntry: function() {
		var dashboard = this._dashboard;
		var projectName = dashboard.projectArea;
		var uri, iconFile;
		if (dashboard instanceof ProjectDashboard) {
			uri = DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, projectName, null, projectName, BootstrapProperties.FRONTSIDE_URL, true);
		}
		else {
			uri = DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, dashboard.team.split("/"), null, projectName, BootstrapProperties.FRONTSIDE_URL, true);
		}
		var query = {
			application: "%MY_APPLICATION%",
			type: DASHBOARD_TYPE,
			project: projectName
		};
		var entry = {
			label: jazz.util.html.escape(dashboard.getTitle()),
			uri: uri,
			iconUri: dashboard.iconUrl
		};
		jazz.app.viewHistory.put(query, entry);
		dojo.publish("/jazz/dashboard/view", [{
			label: entry.label,
			href: entry.uri,
			uri: entry.uri,
			icon: dashboard.iconUrl
		}]);
	},
	
	_destroyDashboard: function() {
		if (this._dashboard) {
			var len = this._connectHandles.length;
			for (var i=0;i<len;++i) {
				this.disconnect(this._connectHandles[i]);
			}
			delete this._connectHandles;
			dojo.publish("/jazz/dashboard/dirty", [this._dashboard.getBundle(), false]);
			PlatformUI.getWorkbench().setPageDirty("com.ibm.team.dashboard", false);
			this._dashboard.destroy();
			delete this._dashboard;
			this.breadcrumbs.domNode.style.display = "none";
		}
		if (this._allDashboards) {
			this._allDashboards.destroy();
			delete this._allDashboards;
		}
		window.scrollTo(0,0);
	},
	
	_handleNavigate: function(urlObj) {
		var winLoc = new dojo._Url(window.location.href);
		if ((!urlObj.host || !urlObj.scheme || (urlObj.host === winLoc.host && urlObj.scheme === winLoc.scheme)) &&   	// if the target is to the same domain and
				urlObj.fragment && ((!urlObj.host && !urlObj.path) || (urlObj.path && urlObj.path === winLoc.path)) &&	// the target is in the web ui and
				this._dashboard instanceof Dashboard) { 																// we have a dashboard showing
				
			if (urlObj.fragment.indexOf("com.ibm.team.dashboard") === -1) {												// if the target is not in dashboards
				// if we are leaving dashboards, allow the transition and notify the dashboard
				if (winLoc.fragment.indexOf("com.ibm.team.dashboard") !== -1)
					this._dashboard.setActive(false);		
			} else if (!this._isDashboardShowing(dojo.queryToObject(urlObj.fragment)) && 		// if the target is in dashboards and is not the current dashboard and
					this._dashboard.isUnsaved()) {																		// the dashboard has unsaved changes	
				var doNavigate = this._dashboard.promptNavigateAway();
				if (!doNavigate) {
					dojo.hash(this._currentDashboardHash);
					return true;
				}
			}
		}
		return false;
	},
	
	_handleViewAllDashboard: function() {
		this.showAllDashboards();
	},
	
	_handleViewDashboard: function(args) {
		if (!args[PARAM_ACTION]) {
			// default action; redirect to specific hash
			// leave the rest of the URL, it may contain necessary URL params (e.g. DM)
			var projectName = jazz.app.context.get().name;
			var href = DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, projectName);
			dojo.hash(href.split("#")[1], true);
			return;
		}
		// dispatch request
		var team = args[PARAM_TEAM];
		if (team) {
			this.showTeamDashboard(args.team, args.tab);
		}
		else {
			this.showProjectDashboard(args.tab);
		}
	},
	
	_initMenu: function(menu) {
		if (typeof unsaved === "object") {
			menu.onDirtyStateChange(unsaved, true);
			unsaved = null;
		}
	},
	
	_isDashboardShowing: function(args) {
		var dashboard = this._dashboard;
		if (!dashboard) return false;
		var projectName = jazz.app.context.get().name;
		if (args[PARAM_TEAM]) {
			return (dashboard instanceof TeamDashboard) && (dashboard.team === args[PARAM_TEAM]);
		}
		else {
			return (dashboard instanceof ProjectDashboard) && (dashboard.projectName === projectName);
		}
	},
	
	_onDirtyStateChange: function(isDirty) {
		PlatformUI.getWorkbench().setPageDirty("com.ibm.team.dashboard", isDirty);
		var bundle = this._dashboard.getBundle();
		if (unsaved) {
			if (isDirty) unsaved = bundle;
			else if (unsaved.dashboardUrl === bundle.dashboardUrl) unsaved = true;
		}
		dojo.publish("/jazz/dashboard/dirty", [bundle, isDirty]);
	},
	
	_onLoad: function() {
		this._updateBrowserTitle();
		this._addViewHistoryEntry();
	},
	
	_onPageChange: function(/* string */ pageId) {
		// update url with current page id
		var hash = dojo.hash();
		var params = hash ? hash.split("&") : [];
		var found = false;
		for (var i=0;i<params.length;++i) {
			if (params[i].indexOf("tab=") === 0) {
				// update existing tab param
				params[i] = "tab=" + pageId;
				found = true;
				break;
			}
		}
		if (!found) {
			// not found - add it
			params.push("tab=" + pageId);
		}
		dojo.hash(params.join("&"));
	},
	
	_onTitleChange: function(/* string */ newTitle) {
		dojo.publish("/jazz/dashboard/rename", [this._dashboard.url, newTitle]);
		this._updateBrowserTitle();
	},
	
	_setContent: function(/* Widget */ widget) {
		widget.placeAt(this.content, "only");
		this._contentWidget = widget;
	},
	
	_showBreadcrumbs: function(/* Dashboard */ dashboard) {
		var breadcrumbs = this.breadcrumbs;
		var projectName = dashboard.projectName;
		var segments = breadcrumbs.segments = [{
			label: dojo.string.substitute(messages.allProjectDash,[projectName]),
			href: Util.getAllProjectDashboardsHref(projectName)
		}];
		if (dashboard instanceof TeamDashboard) {
			segments.push({
				label: projectName,
				href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, projectName)
			});
			var parts = dashboard.team.split("/");
			var path = [];
			for (var i=0;i<parts.length-1;++i) {
				path.push(parts[i]);
				segments.push({
					label: parts[i],
					href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, path)
				});
			}
		}
		breadcrumbs.refresh();
		breadcrumbs.domNode.style.display = "";
	},
	
	_showDashboard: function(args) {
		this._currentDashboardHash = dojo.hash();
		
		// check if already showing
		if (this._isDashboardShowing(args)) {
			// same dashboard, show requested page (or first page if none requested)
			this._dashboard.showPage(args.tab || 0);
		} else {
			// different dashboard; destroy old one
			this._destroyDashboard();
			var dashboard;
			var projectName = jazz.app.context.get().name;
			var team = args[PARAM_TEAM];
			if (team) {
				dashboard = new TeamDashboard({
					page: args.tab,
					projectName: projectName,
					team: team
				});
			}
			else {
				dashboard = new ProjectDashboard({
					projectName: projectName,
					page: args.tab
				});
			}
			this._dashboard = dashboard;
			this._showBreadcrumbs(dashboard);
			this._connectHandles = [
				this.connect(this._dashboard, "onDirtyStateChange", this._onDirtyStateChange),
				this.connect(this._dashboard, "onLoad", this._onLoad),
				this.connect(this._dashboard, "onPageChange", this._onPageChange),
				this.connect(this._dashboard, "onTitleChange", this._onTitleChange)
			];
			this._setContent(this._dashboard);
		}
	},
	
	_updateBrowserTitle: function() {
		var contentWidget = this._contentWidget;
		if (contentWidget && contentWidget instanceof Dashboard) {
			var title = contentWidget.getTitle(true);
		} else if (this._allDashboards) {
			title = messages.allTeamAreasTitle;
		} else {
			title = messages.dashboardsGenericTitle;
		}
		title = dojo.string.substitute(messages.projectDashboardTitle, [title]);
		PlatformUI.getWorkbench().setTitle(title);
	}
});

})();