/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.AllReportsTab");

dojo.require("dijit._BidiSupport");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.reports.web.ui.internal.MoveReport");
dojo.require("com.ibm.team.reports.cognos.web.ui.internal.UploadCognosReport");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");
dojo.require("com.ibm.team.reports.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.app.i18n");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {

var session = com.ibm.team.repository.web.client.session;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var ResourceLink = jazz.ui.ResourceLink;

var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;
var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var BASE_PADDING = 9;
var INDENT = 38;

var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var REPORTIMG = ImagePool.REPORT_OBJ;
var FOLDERIMG = ImagePool.REPORTS_OBJ;

var FOLDERTYPE = "folder";
var QUERYTYPE = "query";
var OPEN = "twistie-open";
var CLOSED = "twistie-closed";

var AllReportsTab = dojo.declare("com.ibm.team.reports.web.ui.internal.AllReportsTab",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/AllReportsTab.html"),
	_pendingDeleteReportNode: null,
	_pendingDeleteFolderNode: null,
	_pendingDeleteFolderChildrenNodes: null,
	_childMap: {},
	_folderMap: {},
	editFolderAction: "#action=com.ibm.team.reports.editFolder",
	runReportAction: "#action=com.ibm.team.reports.viewQuery",
	editReportAction: "#action=com.ibm.team.reports.editReport",
	editTemplateAction: "#action=com.ibm.team.reports.editTemplate",
	createReportAction: "#action=com.ibm.team.reports.newReport",
	heading: null,
	
	constructor: function(args) {
		dojo.mixin(this, args);
		this.messages = messages;
		this.heading = messages.sharedReports;
		if(args.shared == "false") this.heading = messages.myReports;
		this._controller = args.controller;
	},
	
	postCreate: function() {
		if (!this.textDir)
			this.textDir = jazz.app.i18n.getPreferredBaseTextDir();
		this.metanav = new com.ibm.team.reports.web.ui.internal.MetaNav({AllReportsTab: this}, this.metanav);
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this._table = new StyledTable({columns: [{label: messages.report, width: "40%", sortable: false}, {label: messages.descriptioncolumnheader, width: "45%", sortable: false}, {label: messages.actions, width: "15%", className: "action-column", sortable: false}], trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, showHover: true});
		this._resultsArea.appendChild(this._table.domNode);
		this._populateHeader();
		this._runNewQuery();
		this._updateActions();
		dojo.connect(this._folderFilterSelect, "onchange",this, this._runNewQueryForView);
		dojo.connect(this.collapseAll, "onclick", this, this._collapseAll);
		dojo.connect(this.expandAll, "onclick", this, this._expandAll);
	},
	
	_populateHeader: function(){
		this.metanav.setTitle(StringUtils.escapeAndCheckForNull(this.heading));
		var toolbar = new jazz.ui.SimpleToolbar({});
		this.metanav._toolbar = toolbar;
		this.metanav._toolbarDiv.appendChild(toolbar.domNode);
		toolbar.addButton(this, this._newReport, {showLabel: true, imgSrc: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/create-report_obj.gif"), label: messages.createReport, requiresSelection:false});
		toolbar.addButton(this, this._newFolder, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/new-folder-obj.gif"), label: messages.createFolder, requiresSelection:false});
		toolbar.addButton(this, this._importCognosReport, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/import-base-report_co.gif"), label: messages.importReport, requiresSelection:false});
		toolbar.addButton(this, this._runNewQuery, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/refresh.gif"), label: messages.refresh, requiresSelection:false});		
		toolbar.addButton(this, this._deleteItems, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/delete.gif"), label: messages.deleteReportMsg, requiresSelection:true});
		toolbar.addButton(this, this._moveItems, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/movecopy.gif"), label: messages.moveReport, requiresSelection:true});
		
		if(this.shared == "true") this.metanav.createContextSensitiveHelp(ReportsConstants.CSH_SHARED_REPORTS);
		else this.metanav.createContextSensitiveHelp(ReportsConstants.CSH_MY_REPORTS);
		
		if(this.displayBreadcrumbs) {
			this.metanav.populateReportsBreadcrumbs();
		} else {
			this.metanav.handle.style.height = "42px";
			this.metanav.handle.style.minHeight = "42px";
		}
	},	
		
	_runNewQueryForView: function(event){
		var select = this._folderFilterSelect;
		var folderUUID = select.options[select.selectedIndex].value;
		if(folderUUID){
			if(folderUUID == "_allFolders"){
				this._runNewQuery();
			}else{
				this._renderChildFoldersForParent(folderUUID);
			}
		}
	},
	
	_newFolder: function() {
		var isShared = (this.shared == "true") ? true : false;
		document.location.href = this.editFolderAction + "&shared=" + isShared;
	},

	_newReport: function() {
		document.location.href = this.createReportAction;
	},
		
	_runNewQuery: function() {
		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		//Signal the loading to clear the folder filter
		//We have to clear it or refreshing will add duplicates to the filter combo box
		this._clearFilter = true;
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var responseHandler = new ServiceResponseHandler(
				this,
				"_renderRootFolders",
				"_remoteError");
		ReportsClient.getSystemRootFolderDescriptors(responseHandler);
	},
	
	/* Deprecated */
	_runQuery: function() {
		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var serviceObject = {
			self: this,
			id: this._nextQueryId,
			success: function(results) {
				if (this.self._lastQueryId && this.self._lastQueryId > this.id)
					return;
				this.self._lastQueryId = this.id;
				this.self._renderResults(results);
				PlatformUI.getWorkbench().hideMessageArea();
			},
			failure: function(error) {
				// TODO
				//dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		var args = {
			// Project Area will be added by ReportsClient
    		sortByName: true
    	};
    	if (this._currentFilter) {
    		args.filter = this._currentFilter;
    	}
    	if (this.shared == "true") {
    		args.flags = ReportsClient.EXCLUDE_MICRO | ReportsClient.SHARED;
    	} else {
    		args.flags = ReportsClient.EXCLUDE_MICRO | ReportsClient.PRIVATE;
    		args.ownerUUID = session.getAuthenticatedContributor().itemId;
    	}
		ReportsClient.getQueryDescriptors(srh, args);
		this._nextQueryId++;
	},
	
	_renderRootFolders: function(results) {
		// Don't actually create table nodes for the root "My Reports" or "Shared Reports".
		// Just do the recursive call.
		if (results.length != 2) {
			// Something went wrong
			return;
		}
		var index = (this.shared == "true") ? 1 : 0;
		this._rootFolder = results[index];
		this._renderChildFoldersForParent(this._rootFolder.folderUUID);
	},
	
	_renderChildFoldersForParent: function(folderUUID){
		//Clear the filter box if the clear flag is set
		if (this._clearFilter)
		{
			var len = this._folderFilterSelect.options.length;
			for (var i = len-1; i > 0; i--)
				this._folderFilterSelect.remove(i);
			this._clearFilter = false;
		}
		
		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		this._table.clearBody();
		this._updateActions();
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var args = {
			// Project Area will be added by ReportsClient
			// Owner is intentionally empty
    		parentUUID: folderUUID,
    		flags: ReportsClient.ALL,
    		sortByName: true
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_addChildFolders",
				"_remoteError");
		ReportsClient.getFolderDescriptors(responseHandler, args);
		var args2 = {
			// Project Area will be added by ReportsClient
			// Owner is intentionally empty
    		folderUUID: folderUUID,
    		flags: ReportsClient.EXCLUDE_MICRO,
    		sortByName: true
    	};
    	if (this._currentFilter) {
    		args2.filter = this._currentFilter;
    	}
		var responseHandler2 = new ServiceResponseHandler(
				this,
				"_addChildQueries",
				"_remoteError");
		ReportsClient.getQueryDescriptors(responseHandler2, args2);
	},
	
	_addChildFolders: function(results) {
		PlatformUI.getWorkbench().hideMessageArea();
		//How many filters are already in the list
		var filterEntryCount = this._folderFilterSelect.options.length;
		if (results != null) {
			for (var i = 0; i < results.length; i++) {
				var folder = results[i];
				this._folderFilterSelect.options[filterEntryCount + i] = new Option(StringUtils.escapeAndCheckForNull(folder.name), folder.folderUUID);
				this._childMap[folder.folderUUID] = new Array();
				// Locate the parent to insert it under
				if (folder.parentUUID == this._rootFolder.folderUUID || folder.parentUUID == null) {
					this._table.tbody.appendChild(this._createFolderRow(folder, 9));
				} else {
					var parent = document.getElementById(folder.parentUUID);
					if (parent == null) {
						// Do nothing, shouldn't happen.
					} else {
						this._childMap[folder.parentUUID].push(folder.folderUUID);	//MFV added folders to the child map to handle click on parent folder checkbox
						this._insertAfter(this._createFolderRow(folder, parent._indent + INDENT), parent);
					}
				}
			}
			// Recurse
			for (var i = 0; i < results.length; i++) {
				var folder = results[i];
				var args = {
					// Project Area will be added by ReportsClient
					// Owner is intentionally empty
	        		parentUUID: folder.folderUUID,
	        		flags: ReportsClient.ALL,
	        		sortByName: true
	        	};
				var responseHandler = new ServiceResponseHandler(
						this,
						"_addChildFolders",
						"_remoteError");
				ReportsClient.getFolderDescriptors(responseHandler, args);
				var args2 = {
					// Project Area will be added by ReportsClient
					// Owner is intentionally empty
	        		folderUUID: folder.folderUUID,
	        		flags: ReportsClient.EXCLUDE_MICRO,
		    		sortByName: true
	        	};
		    	if (this._currentFilter) {
		    		args2.filter = this._currentFilter;
		    	}
				var responseHandler2 = new ServiceResponseHandler(
						this,
						"_addChildQueries",
						"_remoteError");
				ReportsClient.getQueryDescriptors(responseHandler2, args2);
			}
		}
	},
	
	_addChildQueries: function(results) {
		PlatformUI.getWorkbench().hideMessageArea();
		if (results != null) {
			for (var i = 0; i < results.length; i++) {
				var query = results[i];
				if (this._childMap[query.folderUUID] == null)
					this._childMap[query.folderUUID] = new Array();
				this._childMap[query.folderUUID].push(query.queryUUID);
				// Locate the parent to insert it under
				if (query.folderUUID == this._rootFolder.folderUUID) {
					this._table.tbody.appendChild(this._createQueryRow(query, 9));
				} else {
					var parentFolder = document.getElementById(query.folderUUID);
					if (parentFolder == null) {
						// Couldn't find the folder, so must be a root folder
						this._table.tbody.appendChild(this._createQueryRow(query, 9));
					} else {						
						this._insertAfter(this._createQueryRow(query, parentFolder._indent + INDENT), this._getQueryInsertionPoint(query.folderUUID));
						this._updateContainingFolderCounts(1, query.folderUUID);
					}
				}
			}
		}
		this._updateReportCounters();
	},
	
	_getQueryInsertionPoint: function(folderUUID) {
		var folderTRNode = document.getElementById(folderUUID);
		var candidate = folderTRNode;
		var nextCandidate = candidate.nextSibling;
		while (true) {
			if (nextCandidate == null) {
				return candidate;
			}
			if (nextCandidate._indent <= folderTRNode._indent || nextCandidate._rowType == FOLDERTYPE){   
				return candidate;
			}
			candidate = nextCandidate;
			nextCandidate = nextCandidate.nextSibling;
		} 
	},
	
	_renderResults: function(results) {
		this._table.clearBody();
		if (results && results.length > 0) {
			for (var i = 0; i < results.length; i++) {
				this._table.tbody.appendChild(this._createQueryRow(results[i]));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	_createQueryRow: function(report, padding) {
		var parentFolder = document.getElementById(report.folderUUID);
		
		var row = document.createElement("tr");
		row._indent = padding;
		row._rowType = QUERYTYPE;
		row._name = report.name;
		row.id = report.queryUUID;
		row.folderID = report.folderUUID;   //Save this so we can find the folder row easily
		
		if(parentFolder && !this._folderMap[report.folderUUID]) {
			row.style.display = "none";
		}

		var td = document.createElement("td");
		//If a formatter for the action was provided by a client use that to create the href
		//Otherwise use the team default.
		var hRef = this.runReportAction + "&name=" + encodeURIComponent(report.name) + "&queryUUID=" + report.queryUUID + "&shared=" + report.shared;
		
		
		//Create the checkbox
		var cb = document.createElement("input");
		cb.style.paddingRight = "5px";
		cb.type = "checkbox";
		cb.checked = false;
		this.connect(cb, "onclick", this._handleQueryCheckBoxClick);

		//The report image
		var img = document.createElement("img");
		img.src = REPORTIMG;
		img.classname = "row-image";
		img.style.paddingLeft = "5px";
		img.style.paddingRight= "5px";		

		//Create the anchor element for the report
		var anchor = document.createElement("a");
		anchor.appendChild(document.createTextNode(" " + StringUtils.escapeAndCheckForNull(report.name)));
		anchor.href = hRef;
		this.applyTextDir(anchor, report.name);
		td.style.paddingLeft = padding + "px";
		td.className="reportName";
		
		//append the checkbox and anchor to the table data
		td.appendChild(cb);
		td.appendChild(img);
		td.appendChild(anchor);
		
		//append the td to the row
		row.appendChild(td);
		row.id = report.queryUUID;
		row.folderID = report.folderUUID;   //Save this so we can find the folder row easily
		
		//Create the report description
		td = document.createElement("td");
		td.classname = "reportName";
		td.innerHTML = StringUtils.escapeAndCheckForNull(report.description);
		this.applyTextDir(td, report.description);
		row.appendChild(td);

		//Create the actions
		td = document.createElement("td");
		td.className = "action-column";
		span = document.createElement("span");
		span.className = "actionBar";
		var toolbar = new com.ibm.team.reports.web.ui.internal.common.Toolbar({});
		toolbar.addButton(this, dojo.hitch(this, "_editReport", report), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/edit.gif"), label: messages.edit});
		toolbar.addButton(this, dojo.hitch(this, "_deleteReport", report), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/delete.gif"), label: messages.deleteReportMsg});
		toolbar.addButton(this, dojo.hitch(this, "_moveReport", report), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/movecopy.gif"), label: messages.moveReport});
		span.appendChild(toolbar.domNode);
		td.appendChild(span);
		
		row.appendChild(td);
		
		return row;
	},	
	
	/**
	 * Update the count of report contained in the given folder, and recursively in all of its parent
	 * folders.
	 * 
	 * param adjustInterval - integer; amount by which to increase or decrease count.  Use a negative value to decrease  
	 */
	_updateContainingFolderCounts: function(adjustInterval, startFolderUUID) {
		var parentFolder = document.getElementById(startFolderUUID);
		while (parentFolder) {
			parentFolder._numReports = parentFolder._numReports + adjustInterval;
			if(parentFolder.folderID != null) {
				parentFolder = document.getElementById(parentFolder.folderID);
			} else {
				parentFolder = null;
			}
		}		
	},
	
	/**
	 * Updates the display of report counts in each folder, based on the report numbers that have already
	 * been calculated.
	 */
	_updateReportCounters: function() {
		var trs = document.getElementsByTagName("tr");
		for(var i = 0; i < trs.length; i++) {
			if(trs[i]._numReports != null) {
				var span = trs[i].children[0].children[3];
				span.innerHTML = " (" + trs[i]._numReports + ")";
			}
		}
	},
	
	_collapseOrExpandElements: function(folderName) {
		var twistie = document.getElementById(folderName + "twistie");
		if(twistie==null)
			return;
			
		if(this._folderMap[folderName]) {
			twistie.src = ImagePool.TWISTIE_CLOSED;
			this._collapse(folderName);
		}
		else {
			twistie.src = ImagePool.TWISTIE_OPEN;
			this._expand(folderName);
		}
	},
	
	_collapse: function(folderName) {
		var children = this._childMap[folderName];
		for(var i = 0; i < children.length; i++) {
			var tr = document.getElementById(children[i]);
			tr.style.display = "none";
			if (this._folderMap[children[i]]) { //child is a expanded folder; collapse recursively
				this._collapseOrExpandElements(children[i]);
			}
		}
		this._folderMap[folderName] = false;
	},
	
	_expand: function(folderName) {
		var children = this._childMap[folderName];
		for(var i = 0; i < children.length; i++) {
			var tr = document.getElementById(children[i]);
			if (tr.style.removeProperty) { //method not supported on IE 7,8
				tr.style.removeProperty("display");
			} else if (tr.style.removeAttribute) { //method supported on IE 7, 8
				tr.style.removeAttribute("display");
			}
			this._folderMap[folderName] = true;
		}
	},
	
	_collapseAll: function(event) {		
		event.preventDefault();
		
		for (var item in this._childMap) {
			if (this._childMap.hasOwnProperty(item)) {
				var twistie = document.getElementById(item + "twistie");
				if(twistie) {
					twistie.src = ImagePool.TWISTIE_CLOSED;
					this._collapse(item);
				}
			}
		}
	},
	
	_expandAll: function(event) {
		event.preventDefault();
		
		for (var item in this._childMap) {
			if (this._childMap.hasOwnProperty(item)) {
				var twistie = document.getElementById(item + "twistie");
				if(twistie && !this._folderMap[item]) {
					twistie.src = ImagePool.TWISTIE_OPEN;
			    	this._expand(item);
				}
			}
		}
	},
	
	_createFolderRow: function(folder, padding) {
		var row = document.createElement("tr");
		row.setAttribute("id", folder.folderUUID);
		row._indent = padding;
		row._rowType = FOLDERTYPE;
		row._name = folder.name;
		row._numReports = 0;
		
		if(folder.parentUUID != null) {
			row.folderID = folder.parentUUID;
			if (!this._folderMap[folder.parentUUID]) {
				//if the parent folder is collapsed, hide this folder
				row.style.display = "none";
			}
		}		

		td = document.createElement("td");
		td.style.paddingLeft = padding + "px";
		td.className="folderName";
		
		if(!this._folderMap[folder.folderUUID]) {
			this._folderMap[folder.folderUUID] = false;
		} 
		
		//Create twistie
		var twistie = document.createElement("img");
		if(this._folderMap[folder.folderUUID]) {
			twistie.src = ImagePool.TWISTIE_OPEN;
		} else {
			twistie.src = ImagePool.TWISTIE_CLOSED;
		}
		twistie.style.cursor="pointer";
		twistie.style.position = "relative";
		twistie.style.top = "-2px";
		twistie.id = folder.folderUUID + "twistie";
		twistie.style.paddingRight = "5px";
		
		//wrap the img in an anchor tab so that it is accessible (similar to how toolbar works)
		var anchor = document.createElement("a");
		anchor.href = "#";
		dijit.setWaiRole(anchor, "button");
		this.connect(anchor, "onclick", function(evt) { evt.preventDefault(); this._collapseOrExpandElements(folder.folderUUID);} );
		anchor.appendChild(twistie);
		td.appendChild(anchor);
		
		//Create the checkbox
		var cb = document.createElement("input");
		cb.type = "checkbox";
		cb.checked = false;
		cb.style.paddingRight= "5px";
		cb.folderID = folder.folderUUID;
		this.connect(cb, "onclick", this._handleFolderCheckBoxClick);
		td.appendChild(cb);

		//Folder image	
		var img = document.createElement("img");
		img.src = FOLDERIMG;
		img.classname = "row-image";
		img.style.paddingLeft = "5px";
		img.style.paddingRight= "5px";		

		//Create the text node to hold the folder name
		var textNode = document.createTextNode(" " + StringUtils.escapeAndCheckForNull(folder.name));
		
		var span = document.createElement("span");
		span.style.fontWeight = "normal";
		span.style.color = "grey";

		td.appendChild(img);
		td.appendChild(textNode);
		td.appendChild(span);

		row.appendChild(td);

		//Create the description
		td = document.createElement("td");
		td.innerHTML = StringUtils.escapeAndCheckForNull(folder.description);
		td.classname = "folderName";
		row.appendChild(td);

		//Create the actions
		td = document.createElement("td");
		td.className = "action-column";
		span = document.createElement("span");
		span.className = "actionBar";
		var toolbar = new com.ibm.team.reports.web.ui.internal.common.Toolbar({});
		toolbar.addButton(this, dojo.hitch(this, "_createNewFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/new-folder-obj.gif"), label: messages.createFolder});
		toolbar.addButton(this, dojo.hitch(this, "_editFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/edit.gif"), label: messages.edit});
		toolbar.addButton(this, dojo.hitch(this, "_deleteFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/delete.gif"), label: messages.deleteMsg});
		toolbar.addButton(this, dojo.hitch(this, "_moveFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/movecopy.gif"), label: messages.moveFolder});
		span.appendChild(toolbar.domNode);
		td.appendChild(span);
		
		row.appendChild(td);
		
		return row;
	},
	
	_createNewFolder: function(folder) {
		document.location.href = this.editFolderAction + "&parentUUID=" + folder.folderUUID + "&processAreaUUID=" + folder.processAreaUUID + "&shared=" + folder.shared;
	},
	
	_editReport: function(report)
	{
		var location = this.editReportAction + "&queryUUID=" + report.queryUUID + "&editTemplateAction=" + this.editTemplateAction;  //HTML.escape(this.editTemplateAction);
		document.location.href = location;		
	},
	
	_editFolder: function(folder) {
		document.location.href = this.editFolderAction + "&folderUUID=" + folder.folderUUID;
	},
	
	_deleteFolder: function(folder) {
		if (!confirm(messages.confirmDelFolder)) {
			return;
		}
		var args = {
    		folderUUID: folder.folderUUID
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_folderDeleted",
				"_remoteError");
		var rowNode = dojo.query("tr[id=" + folder.folderUUID + "]");
		if(rowNode.length > 0){
			rowNode = rowNode[0];
			this._pendingDeleteFolderNode = rowNode;
		}
		//find all of the child nodes
		this._pendingDeleteFolderChildrenNodes = new Array();
		this._addAllChildReportsToFolderDelete(folder.folderUUID);
		ReportsClient.postDeleteFolderDescriptor(responseHandler, args);
	},
	
	_addAllChildReportsToFolderDelete: function(uuid){
		var folderChildren = this._childMap[uuid];
		if(folderChildren){
			for(var i = 0; i < folderChildren.length; i++){
				var child = dojo.query("tr[id=" + folderChildren[i] + "]");
				if(child.length > 0){
					this._pendingDeleteFolderChildrenNodes.push(child[0]);
					this._addAllChildReportsToFolderDelete(child[0].id);
				}
			}
		}
	},
	
	_folderDeleted: function() {
		if(this._pendingDeleteFolderNode && this._pendingDeleteFolderNode != null){
			var reportsDeleted = this._pendingDeleteFolderNode._numReports;
			var parentFolderUUID = this._pendingDeleteFolderNode.folderID;
			this._table.tbody.removeChild(this._pendingDeleteFolderNode);
			this._pendingDeleteFolderNode = null;
		}
		if(this._pendingDeleteFolderChildrenNodes && this._pendingDeleteFolderChildrenNodes != null){
			for(var i = 0; i < this._pendingDeleteFolderChildrenNodes.length; i++){
				this._table.tbody.removeChild(this._pendingDeleteFolderChildrenNodes[i]);
			}
			this._pendingDeleteFolderChildrenNodes = null;
		}
		
		if (parentFolderUUID) {
			this._updateContainingFolderCounts(reportsDeleted * -1, parentFolderUUID);
		}
		
		this._updateReportCounters();
		this._updateActions();
	},
	
	_deleteReport: function(report) {
		if (!confirm(messages.confirmDelReport)) {
			return;
		}
		var args = {
    		queryUUID: report.queryUUID
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_queryDeleted",
				"_remoteError");
		var rowNode = dojo.query("tr[id=" + report.queryUUID + "]");
		if(rowNode.length > 0){
			rowNode = rowNode[0];
			this._pendingDeleteReportNode = rowNode;
		}
		ReportsClient.postDeleteQueryDescriptor(responseHandler, args);
	},
	
	_queryDeleted: function() {
		if(this._pendingDeleteReportNode && this._pendingDeleteReportNode != null){
			var parentFolder = this._pendingDeleteReportNode.folderID;
			
			this._updateContainingFolderCounts(-1, parentFolder);
			this._table.tbody.removeChild(this._pendingDeleteReportNode);
			this._pendingDeleteReportNode = null;
		}

		this._updateReportCounters();
		this._updateActions();
	},
	
	_moveReport: function(report)
	{
		var queryUUIDs = [report.queryUUID];
		var queryNames = [report.name];
		var args = {
			queryUUIDs: queryUUIDs,
			queryNames: queryNames,
			parentUUID:report.folderUUID != null ? report.folderUUID : "",
			
			okCallback: dojo.hitch(this,this._okDialogCallback),
			cancelCallback: dojo.hitch(this,this._cancelDialogCallback)
		}
		var moveReport = new com.ibm.team.reports.web.ui.internal.MoveReport(args);
		
		this._dialog = new jazz.ui.Dialog({
			contentNode: moveReport.mainDiv,
			primaryTitle: messages.moveReport,
			width: "400px",
			isClosable: true
		});
	},

	_moveFolder: function(folder)
	{
		var folderUUIDs = [folder.folderUUID];
		var folderNames = [folder.name];
		var args = {
			folderUUIDs:folderUUIDs,
			folderNames: folderNames,
			parentUUID:folder.parentUUID != null ? folder.parentUUID : "",

			okCallback: dojo.hitch(this,this._okDialogCallback),
			cancelCallback: dojo.hitch(this,this._cancelDialogCallback)
		}
		var moveReport = new com.ibm.team.reports.web.ui.internal.MoveReport(args);
		
		this._dialog = new jazz.ui.Dialog({
			contentNode: moveReport.mainDiv,
			primaryTitle: messages.moveFolder,
			width: "300px",
			isClosable: true
		});
	},
	
	_okDialogCallback: function(args)
	{
		this._dialog.close();
		this._dialog.destroy();
		this._runNewQuery();
		
	},
	
	_cancelDialogCallback: function(args)
	{
		this._dialog.close();
		this._dialog.destroy();
	},

	
	_createNoneMessage: function() {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.innerHTML = StringUtils.escapeAndCheckForNull(messages.allReportsNone);
		row.appendChild(td);
		return row;
	},
	
	_doFilter: function(filter) {
		this._currentFilter = filter;
		this._isRelatedFilterStale = true;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
		if (!this._runQueryTimeout) {
			this._runQueryTimeout = setTimeout(dojo.hitch(this,this._runNewQueryForView,0,true),500);
		}
	},
	
	_importCognosReport: function()
	{
		new com.ibm.team.reports.cognos.web.ui.internal.UploadCognosReport( {reportsView: this } );  
	},
	
	refreshCallback: function()
	{
		this._runNewQuery();		
	},
	
	_remoteError: function(error) {
		PlatformUI.getWorkbench().showMessage(error.message);		
	},
	
	_insertAfter: function(newNode, referenceNode) {
	    referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
	},
	
	_trim: function(string) {
		return this._trimLeft(this._trimRight(string));
	},
	_trimLeft: function(string) {
		return string.replace(new RegExp("^[\\s]+", "g"), "");
	},
	_trimRight: function(string) {
		return string.replace(new RegExp("[\\s]+$", "g"), "");
	},

	_handleQueryCheckBoxClick: function(args)
	{
		var checkedState = args.currentTarget.checked;

		//if we unchecked a report then make sure the folder is unchecked also
		if (checkedState == false)
			this._setRowCheckedState(args.currentTarget.parentNode.parentNode.folderID, false);

		this._updateActions();
	},
	_handleFolderCheckBoxClick: function(args)
	{
		var checkedState = args.currentTarget.checked;
		var folderID = args.currentTarget.folderID;
		this._handleFolderCheck(folderID, checkedState);
		this._updateActions();
	},
	
	_handleFolderCheck: function(folderID, checkedState)
	{
		var childIDs = this._childMap[folderID];
		for (var i = 0; i < childIDs.length; i++)
		{
			var childID = childIDs[i];
			this._setRowCheckedState(childID, checkedState)
			if (this._childMap[childID] != null)   //Recurse child nodes
				this._handleFolderCheck(childID, checkedState);
		}
	},
	
	_updateActions: function()
	{
		var rowsChecked = this._getCheckedCount() > 0;
		var buttonCounter = 0;
		var button = null;
		while((button = this.metanav._toolbar.getItem(buttonCounter++)) != null)
		{
			if (button.args.requiresSelection)
			{
				if (rowsChecked)
					this.metanav._toolbar.enableButton(button);
				else
					this.metanav._toolbar.disableButton(button);
			}			
		}
	},
	
	_getCheckboxFromTR: function(trNode)
	{				
		var td = trNode.children[0]; //the first cell in the row
		
		//location of the checkbox depends on whether this is a folder or a query row
		if (td.children.length === 3) {
			return td.children[0];
		} else if (td.children.length === 4) {
			return td.children[1];
		} else {
			return null;
		}
	},
	
	_isRowChecked: function(trNode)
	{
		return this._getCheckboxFromTR(trNode).checked;
	},
	
	_setRowCheckedState: function(rowID, checkedState)
	{
		var tr = document.getElementById(rowID);
		var cb = this._getCheckboxFromTR(tr);
		var oldCheckedState = cb.checked; 
		cb.checked = checkedState;
	},
	
	_deleteItems: function()
	{
		var items = this._getCheckedItems(true);
		if (items.length <= 0)
			return;
			
		var queryItemIDs = [];
		var folderItemIDs = [];

		for (var i = 0; i < items.length; i++)
		{
			var item = items[i];
			if (item.rowType == QUERYTYPE)
				queryItemIDs.push(item.id);
			else if (item.rowType == FOLDERTYPE)
				folderItemIDs.push(item.id);
		}

		var confirmMsg = "";
		if (queryItemIDs.length > 0 && folderItemIDs.length > 0)
			confirmMsg = dojo.string.substitute(messages.confirmDelReportsAndFolders, [queryItemIDs.length, folderItemIDs.length]);
		else if (queryItemIDs.length > 0)
			confirmMsg = dojo.string.substitute(messages.confirmDelReports, [queryItemIDs.length]);
		else
			confirmMsg = dojo.string.substitute(messages.confirmDelFolders, [folderItemIDs.length]);
		
		if (!confirm(confirmMsg)) {
			return;
		}

		this._deletedTypes = 0;
		//Delete the queries if any
		if (queryItemIDs.length > 0) 
		{
			this._deletedTypes++;
			var args = {
    			queryUUIDs: queryItemIDs
    		};
			var responseHandler = new ServiceResponseHandler(this, "_queriesDeleted", "_remoteError");
			ReportsClient.postDeleteQueryDescriptors(responseHandler, args);
		}
		
		if (folderItemIDs.length > 0) 
		{
			this._deletedTypes++;
			var folderArgs = {
				folderUUIDs: folderItemIDs
			}
			//Delete the folders
			var responseHandler = new ServiceResponseHandler(this, "_queriesDeleted", "_remoteError");
			ReportsClient.postDeleteFolderDescriptors(responseHandler, folderArgs);
		}		
	},

	_moveItems: function()
	{
		var items = this._getCheckedItems(true);
		if (items.length <= 0)
			return;
			
		var queryItemIDs = [];
		var queryNames = [];
		var folderItemIDs = [];
		var folderNames = [];

		for (var i = 0; i < items.length; i++)
		{
			var item = items[i];
			if (item.rowType == QUERYTYPE) 
			{
				queryItemIDs.push(item.id);
				queryNames.push(item.name);
			}
			else if (item.rowType == FOLDERTYPE)
			{
				folderItemIDs.push(item.id);
				folderNames.push(item.name);
			} 
		}
		
		var args = {
			folderUUIDs: folderItemIDs,
			folderNames: folderNames,
			queryUUIDs: queryItemIDs,
			queryNames: queryNames,
			
			parentUUID: null,

			okCallback: dojo.hitch(this,this._okDialogCallback),
			cancelCallback: dojo.hitch(this,this._cancelDialogCallback)
		}
		var moveReport = new com.ibm.team.reports.web.ui.internal.MoveReport(args);
		
		this._dialog = new jazz.ui.Dialog({
			contentNode: moveReport.mainDiv,
			primaryTitle: messages.moveReport,
			width: "300px",
			isClosable: true
		});
		
		
	},
	_queriesDeleted: function(args)
	{
		this._deletedTypes--;
		if (this._deletedTypes <= 0)
			this._runNewQuery();
	},
	
	_getCheckedItems: function(excludeChildrenOfSelectedFolders)
	{
		var retVal = [];
		var body = this._table.tbody;
		var tableRows = body.children;
		for (var i = 0; i < tableRows.length; i++)
		{
			var row = tableRows[i];
			if (this._isRowChecked(row))
			{
				var selectedItem = {};
				selectedItem.rowType = row._rowType;
				selectedItem.id = row.id;
				selectedItem.name = row._name;
				retVal.push(selectedItem);
				//If we want to exclude children of selected folders
				if (excludeChildrenOfSelectedFolders && row._rowType == FOLDERTYPE)
				{
					//skip all the subsequent rows until the next sibling
					var indent = row._indent;
					while (i+1 < tableRows.length && tableRows[i+1]._indent > indent)
						i++;
				}
			}
		}
		
		return retVal;
	},
		
	_getCheckedCount: function()
	{
		var retVal = 0
		var body = this._table.tbody;
		var tableRows = body.children;
		for (var i = 0; i < tableRows.length; i++)
		{
			var row = tableRows[i];
			if (this._isRowChecked(row))
				retVal++;
		}
		
		return retVal;
	}
	
	
});

})();
