/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.AllSharedPage");

dojo.require("com.ibm.team.dashboard.web.ui.internal.AllShared");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojo.hash");

dojo.require("jazz.ensemble.internal.client.RestClient");
dojo.require("jazz.app.context");
dojo.require("jazz.app.events");
dojo.require("jazz.core.loader");



dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.contextRoot");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var AllShared = com.ibm.team.dashboard.web.ui.internal.AllShared;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var RestClient = jazz.ensemble.internal.client.RestClient;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var PersonalDashboardClient = com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient;
var messages;
var unsaved = true;
var RESOURCE_PATH = "/all";
var APP_PATH = PlatformUI.getWorkbench().getAppPath();


dojo.declare("com.ibm.team.dashboard.web.ui.internal.AllSharedPage",
		[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllSharedPage.html"),
	widgetsInTemplate: true,
	
	constructor: function() {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	postCreate: function() {
		
		this.subscribe("/jazz/dashboard/initMenu", this._initMenu);
		var workbench = PlatformUI.getWorkbench();
		var resourceRegistry = workbench.getResourceRegistry();
		resourceRegistry.registerHandlerForPattern(RESOURCE_PATH, this, "viewAllShared");
		var actionRegistry = workbench.getActionRegistry();
		actionRegistry.registerAction("com.ibm.team.dashboard.viewAllShared", this, "viewAllShared");
		
	},
	
	destroy: function() {
		this.destroyDashboard();
		this.inherited(arguments);
	},
	
	destroyDashboard: function() {
		if (this._allDashboards) {
			this._allDashboards.destroy();
			delete this._allDashboards;
		}
		window.scrollTo(0,0);
	},
	
	_initMenu: function(menu) {
		if (typeof unsaved === "object") {
			menu.onDirtyStateChange(unsaved, true);
			unsaved = null;
		}
	},
	
	_getInfoFromUrl: function(){
		var url = location.href;
		var urlObj = new dojo._Url(url),
			args = {},
			hash = urlObj.fragment,
			path = urlObj.path;
		if (hash && hash.length > 0) {
			if (path.indexOf(hash, path.length - hash.length) >= 0) { //if path ends with hash
				//strip off the hash and continue
				dojo.hash("",true);
			}
			path = APP_PATH + urlObj.fragment;
		}
		
		// ends with /all?
		if (path.indexOf("/all") == (path.length - 4)) {
			return "all";
		}
		
		// default dashboard?
		if (path.search(/\/dashboards\/?$/) != -1) {
			return null;
		}
		return args;
	},
	
	viewAllShared: function() {
		var args = this._getInfoFromUrl();
		if (args) {
			this._showAllShared(args);
		}
	},
	
	_showAllShared: function(args) {
		if (args == "all") {
			this._showAllPersonalDashboards();
			return;
		} 
		// check if already showing
		if (this._dashboard) {
			if (this._dashboard.url != args.url) {
				// different dashboard; destroy old one
				this.destroyDashboard();
			} else {
				// same dashboard - pass the current tab, if specified.
				this._dashboard.showTab(args.tab);
			}
		} else {
			this.destroyDashboard();
		}
		var context = jazz.app.context.get();
		context.dashboard = args.url;
		jazz.app.context.set(context);
		if (this._title) {
			// "touch" the title so that it doesn't revert between navigations
			PlatformUI.getWorkbench().setTitle(this._title);
		}	
		
	},
	
	_showAllPersonalDashboards: function() {
		if (this._allDashboards) return;
		this.destroyDashboard();
		jazz.core.loader.load_async("com.ibm.team.dashboard.web.ui.internal.AllShared", dojo.hitch(this, function() {
			this._allDashboards = new com.ibm.team.dashboard.web.ui.internal.AllShared();
			this._setContent(this._allDashboards);
			this._updateBrowserTitle();
			var context = jazz.app.context.get();
			context.dashboard = Util.getContextPath(true) + "/dashboards/all";
			jazz.app.context.set(context);
		}));
	},
	
	_setContent: function(/* Widget */ widget) {
		widget.placeAt(this.domNode, "only");
		this._contentWidget = widget;
	},
	
	_updateBrowserTitle: function() {
		var contentWidget = this._contentWidget;
		var pageTemplate = dijit.byId("jazz_ui_PageTemplate_0");
		if (this._allDashboards) {
				title = messages.allPersonalTitle;
		} 
		pageTemplate.set("bannerTitle", title);
		title = dojo.string.substitute(messages.personalDashboardTitle, [title]);
		this._title = title;
		PlatformUI.getWorkbench().setTitle(title);
	}
});

})();