/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.cognos.web.ui.internal.CognosReportsBrowseDialog");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");

dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Tree");
dojo.require("jazz.ui.tree.IconItem");
dojo.require("jazz.ui.Dialog");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.Tree");
dojo.require("dojox.data.dom");
dojo.require("dojo.data.ItemFileReadStore");

dojo.requireLocalization("com.ibm.team.reports.cognos.web.ui.internal", "messages");

(function() {

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var CognosReportsClient = com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;

dojo.declare("com.ibm.team.reports.cognos.web.ui.internal.CognosReportsBrowseDialog", [dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.reports.cognos.web", "ui/internal/templates/CognosReportsBrowseDialog.html"),
	widgetsInTemplate: true,
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.cognos.web.ui.internal", "messages");
		this.elementsToBeDisplayed = args.elementsToBeDisplayed;
		this.labelOK = this.messages.labelOK;
		this.labelCancel = this.messages.labelCancel;
		this._templateMap = {};
	},
	
	postCreate: function() {
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			modal: true,
			primaryTitle: this.messages.labelReportBrowse,
			width: "460px",
			height: "270px",
			isClosable: true
		});
		this.connect(this._okButton, "onclick", this._onOk);
		this.connect(this._cancelButton, "onclick", this._onCancel);
		this.textLoading = document.createTextNode("Loading...");
		this._treeContainer.appendChild(this.textLoading);
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		//fetch jts url
		var srh = new ServiceResponseHandler(this, "_handleFetchJTSUrl", "_handleFetchJTSUrlError");
		ServerStatusClient.getJazzTeamServerURI(srh);
	},
	
	_handleFetchJTSUrl : function(response){
		if(response == null){
			response = net.jazz.ajax._contextRoot + "/";
		}
		this.url = response + "service/com.ibm.team.reports." +
		  "service.cognos.internal.ICognosReportService/";
		this._fetchDescriptors();
	},
	
	_handleFetchJTSUrlError : function(){
		this.url = net.jazz.ajax._contextRoot + "/service/com.ibm.team.reports." +
		  "service.cognos.internal.ICognosReportService/";
		this._fetchDescriptors();
	},
	
	/*
	 * We need to wait for the fetch JTS url call to return so we can properly set the url to the reports
	 */
	_fetchDescriptors : function(){
		var self = this;
		if (this.elementsToBeDisplayed == "ReportTemplate"){
			jazz.client.xhrGet
			({
			  url: this.url,
			  headers: this.headers,
			  load: function(response, ioArgs){
			  	var reportStore = new dojo.data.ItemFileReadStore({data: response});
				var reportTree = new dijit.Tree({           
					store: reportStore,
					query: {type: 'content'},
					labelAttr: "name",
					typeAttr: "type",
					onClick: dojo.hitch(self, function(item, node){
						if(node.item.type == "report"){
							self.reportPath = node.item.path;
							self.node = node;
							self.reportId = node.item.reportId;
							self._okButton.disabled = false;
						}
					})}, dojo.byId("_treeContainer"));
				self.reportTree = reportTree;
			//self._tree = tree;
			self._treeContainer.removeChild(self.textLoading);
			self._treeContainer.appendChild(reportTree.domNode);
			},error: dojo.hitch(self, function(){
				self._treeContainer.removeChild(self.textLoading);
				self._treeContainer.appendChild(document.createTextNode(self.messages.errorCommunicating));
			}),
			  handleAs: "json"
			});
		} else {
			var context = this;
			// Create the service object
			var serviceObject = {
				success: function(reports) {
					if (reports == null) {
						alert(context.messages.errorNoReports);
						reports = [];
					}
					context._createTree(reports);
				},
				failure: function(errorObject) {
					alert(this.messages.errorCantCommunicate);
				}
			};
			// Create the handler
			var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
			CognosReportsClient.getCognosReportDescriptors(handler, {});
		}
	},
	
	uninitialize: function() {
		if (this.reportTree){
			this.reportTree.destroy();
		}
		if (this.dialog){
			this.dialog.destroy();
		}
	},
	
	_createTree: function(reports){
		var reportStore = this._createTemplateDataStore(reports);
		this._createTemplateTree(reportStore);
	},
	
	_createTemplateTree: function(response) {
		var self = this;
		var projectAreaMap = {};
		this._templateTreeWidget = new jazz.ui.Tree();
		for(var i = 0; i < response.items.length; i++){
			var item = response.items[i];
			//reportIds are only unique to a project
			this._templateMap[item.projectAreaUUID + item.reportId] = item;
			if(projectAreaMap[item.projectAreaUUID]){
				projectAreaMap[item.projectAreaUUID].push(item.projectAreaUUID + item.reportId);
			}else{
				projectAreaMap[item.projectAreaUUID] = new Array();
				projectAreaMap[item.projectAreaUUID].push(item.projectAreaUUID + item.reportId);
			}
		}
		for(var project in projectAreaMap){
			var serviceObject = {
				success: function(projectArea) {
					var node = new jazz.ui.tree.IconItem({label: projectArea.name, iconClass: "iconTemplateGroup"});
					self._templateTreeWidget.addChild(node);
					var reportIds  = projectAreaMap[projectArea.itemId];
					var children = new Array(reportIds.length);
					for(var i = 0; i < reportIds.length; i++) {
						children[i] = self._templateMap[reportIds[i]];
					}
					children.sort(function(a, b) {
						if (a.name < b.name) {
							return -1;
						} else if (b.name < a.name) {
							return 1;
						}
						return 0;
					});
					
					for(var i = 0; i < children.length; i++){
						var item = children[i];
						node.addChild(new jazz.ui.tree.IconItem({label: item.name, iconClass: "iconTemplate", id: item.projectAreaUUID + item.reportId}));
					}
				},
				failure: function(errorObject) {
					
				}
			};
			// Create the handler
			var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
			ProcessClient.getProjectAreaByUUID({uuid: project}, handler);
		}
		this._templateTreeWidget.onSelectionChanged = function(items){
			if(items && items.length > 0 && self._templateMap[items[0].id]){
				var item = items[0];
				self._okButton.disabled = false;
				self._loadReport(self._templateMap[item.id]);
			}else{
				self._okButton.disabled = true;
			}
		};
		this._templateTreeWidget.onDblClick = function(){
			var items = self._templateTreeWidget.getSelection();
			if(items && items.length > 0){
				var item = items[0];
				if(self._templateMap[items[0].id]){
					self._okButton.disabled = false;
					self._loadReport(self._templateMap[item.id]);
					self._onOk(item, null);
				}else{
					item.toggleExpanded();
					self._okButton.disalbed = true;
				}
			}
		};

		this._treeContainer.removeChild(this.textLoading);
		this._treeContainer.appendChild(this._templateTreeWidget.domNode);
	},
	
	_loadReport: function(item){
		this.reportUUID = item.reportUUID;
		this.cognosReportName = item.name;
		this.reportProjectAreaUUID = item.projectAreaUUID;
	},
		
	_onOk: function(item, node) {
		this.close();
		this.onReportSelected();
	},
	
	_onCancel: function() {
		this.close();
		this._destroyTemplateTree();
	},

	close: function() {
		this.dialog.close();
		this._destroyTemplateTree();
	},
	
	_destroyTemplateTree: function() {
		if(this._templateTreeWidget){
			var widgets = dijit.findWidgets(this._templateTreeWidget.domNode);
			dojo.forEach(widgets, function(w) {
			    w.destroyRecursive(true);
			});
		}
	},
	
	_createTemplateDataStore: function(reports) {
			var store = new Object();
			store.label = "name";
			store.identifier = "name";
			var items = new Array();
			store.items = reports;
			return store;
	},
	
	//to be connected to for users to be notified when a report has been selected
	onReportSelected: function() {
	}
});

})();