/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ConfigureDataWarehouseConnection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportsManagementMessages");

(function() {

var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var MessageBox = jazz.ui.MessageBox;

var DataWarehouseConnection = dojo.declare("com.ibm.team.reports.web.ui.internal.ConfigureDataWarehouseConnection", [ConfigUtility, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ConfigureDataWarehouseConnection.html"),
	_allServices: {
		"com.ibm.team.datawarehouse.service.internal.RemoteDatabaseService": "dataWarehouseTableHeading",
		"com.ibm.team.datawarehouse.service.internal.MetaDataService" : "internalDataWarehouseTableHeading",
		"com.ibm.team.datawarehouse.service.internal.RemoteSnapshotManagerTask" : "remoteSnapshotManagerTaskHeading"
	},
	_propertySortOrder: [
		"db.type",
		"db.vendor",
		"db.jdbc.location",
		"db.jdbc.password",
		"db.jdbc.pool.size",
		"db.jdbc.pool.wait",
		"db.j2ee.dataSource"
	],

	_csPageHelpWidgets: null,
	csHelpNode: null,
	// widget initialization
	
	constructor: function() {
		//the super class already initializes its own "messages" - so give this a different name
		this.reportMessages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportsManagementMessages");
		this._csPageHelpWidgets= [];
	},

	postCreate: function () {
		com.ibm.team.reports.web.ui.internal.ConfigureDataWarehouseConnection.superclass.postCreate.apply(this);
		this._initializeContent();
		this._saveButton.disabled = true;
		this._createSaveHandler(this._saveButton);
		this._createContextSensitiveHelp();
		
		this.header = new MessageBox();
		this.headerArea.appendChild(this.header.domNode);
	},

	_initializeContent: function () {
		this._services = new Array();
		for (var serviceName in this._allServices) {
			var curService = new ServerPropertyService({
				"serviceInfo": {
					"implementationClassName": serviceName
				},
				"enableEdits": true,
				"serviceHeaderClass": ["header1"],
				"propertiesRowClass": [],
				"propertySortOrder": this._propertySortOrder,
				"customTitle": this.reportMessages[this._allServices[serviceName]],
				"displayIn": this._configContainer
			});
			this._services.push(curService);
			this.connect(curService, "_markChanged", function () {
				this._enableSaveIfValid(this._saveButton);
			});
			this.connect(curService, "_checkUnchanged", function () {
				var change = false;
				for (var j=0; j < this._services.length; j++) {
					if (this._services[j]._changed) {
						change = true;
						break;
					}
				}
				if (!change) {
					this._saveButton.disabled = true;
				}
			});
		}
	},
	
	_createContextSensitiveHelp: function() {
		dojo.empty(this.csHelpNode);
		this._createCSHWidget(DataWarehouseConnection.CSH_DATA_WAREHOUSE_CONNECTION);
	},

	_createCSHWidget: function(uaKey) {
		var parms = {};
		parms.uaKey = uaKey;
		var pageHelp = new PageHoverHelp(parms);
		this.csHelpNode.appendChild(pageHelp.domNode);
		this._csPageHelpWidgets.push(pageHelp);
	},
	
	_getChanges: function () {
		var changes = {
			"changes": [],
			"properties": []
		};
		for (var i=0; i < this._services.length; i++) {
			var obj = this._services[i]._getChanges();
			changes["changes"] = changes["changes"].concat(obj["changes"]);
			changes["properties"] = changes["properties"].concat(obj["properties"]);
		}
		return changes;
	},
	
	_destroyServices: function() {
		for (var i = 0; i < this._services.length; i++) {
			this._services[i].destroyRecursive();
		}
		this._services = [];
	},
	
	uninitialize: function(){
		this._destroyServices();
		
		this.inherited(arguments);
	}
});
DataWarehouseConnection.CSH_DATA_WAREHOUSE_CONNECTION = "com.ibm.jazz.reports.web.cshelp.data_warehouse_connection";

})();