/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2014. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.DeployTemplatesViewer");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");
dojo.require("com.ibm.team.reports.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.util.html");
dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var MAX_MESSAGE_LENGTH = 500;
var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;
var APPLICATION_OPTION = "APPLICATION";
var SEPARATOR_OPTION = "SEPARATOR_OPTION";

dojo.declare("com.ibm.team.reports.web.ui.internal.DeployTemplatesViewer", [dijit._Widget, dijit._Templated], {
		/*
		 * Attach points in template:
		 * sourceSelector - the select box for the source of the deploy
		 * templateSelector - the select list containing the templates
		 * spanSelected - the span showing how many templates are selected
		 * targetLink - the link indicating the selected target project/team
		 * deployButton
		 */
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/DeployTemplatesViewer.html"),
		
		_projectAreaUUID: null, /*The current project area context*/
		_targetProcessAreaUUID: null, /*The process area where the templates should be deployed*/
		_selectedTemplateIds: null, /*Array of selected templates IDs*/
		_sourceId: null, /*The ID of the source for reports - either from the application, or another project area*/
		_sourceToReportMap: null, /*Cache of source Ids to reports for deploying*/
						
		constructor: function(props) {
			dojo.mixin(this, props);
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		},
		
		postCreate: function() {
			this.metanav = new com.ibm.team.reports.web.ui.internal.MetaNav({DeployTemplatesViewer: this}, this.metanav);
			this.metanav.setTitle(this.messages.deployNewTemplates);
			this.typeIcon = document.createElement("img");
			this.typeIcon.style.paddingTop = "3px";
			this.typeIcon.src = ImagePool.REPORTTEMPLATE_OBJ;
			this.typeIcon.className = "typeIcon";
			this.metanav.insertTypeIcon(this.typeIcon);
			
			this._selectedTemplateIds = [];
			this._sourceToReportMap = {};
			
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			this.deployButton.disabled = true;
			
			if (this.displayBreadcrumbs) { 
				this.metanav.populateResourcesBreadcrumbs();
			}
			
			dojo.connect(this.templateSelector, "onchange",this, this._templateSelectionChanged);
			dojo.connect(this.sourceSelector, "onchange", this, this._sourceSelectionChanged);
			dojo.connect(this.targetLink, "onclick", this, this._processAreaLinkSelected);
			dojo.connect(this.deployButton, "onclick", this, this._deploySelected);			
			
			//initial target is the current project area
			this._projectAreaUUID = jazz.app.context.get().itemId;
			this._targetProcessAreaUUID = this._projectAreaUUID;
			this.targetLink.appendChild(document.createTextNode(jazz.app.context.get().name));
			
			var srh = new ServiceResponseHandler(this, "_initializeSourceSelector", "_remoteError");
			var getProjectsArgs = {
				hideArchivedProjects: true
			};

			ProcessClient.getAllProjectAreas(srh, getProjectsArgs);
		},
					
		_initializeSourceSelector: function(projects) {
			var applicationOption = document.createElement("option");
			applicationOption.appendChild(document.createTextNode(this.messages.applicationSourceOption));
			applicationOption.value=APPLICATION_OPTION;
			applicationOption.selected = true; //this is the default selection
			this.sourceSelector.appendChild(applicationOption);
			
			if (projects.length > 1) { //if there's only one, then it's just the current project
				var headerOption = document.createElement("option");
				headerOption.appendChild(document.createTextNode("---"+ this.messages.projectHeadingOption));
				headerOption.value=SEPARATOR_OPTION;
				headerOption.disabled = true;
				this.sourceSelector.appendChild(headerOption);
				
				projects.sort(function(a, b) {
					if (a.name < b.name) {
						return -1;
					} else if (b.name < a.name) {
						return 1;
					}
					return 0;
				});
			}
			
			for (var i = 0; i<projects.length; i++) {
				var project = projects[i];
				if (project.itemId != this._projectAreaUUID) {
					var option = document.createElement("option");
					option.appendChild(document.createTextNode(project.name));
					option.value=project.itemId;
					this.sourceSelector.appendChild(option);
				}
			}
			
			//trigger loading report templates for the default selection
			this._sourceSelectionChanged();
		},
		
		/**
		 * Called when the selection changes in the sources list.
		 */
		_sourceSelectionChanged: function() {
			this.clearValidationMessage();
			var selected = this.sourceSelector.options[this.sourceSelector.selectedIndex];
			
			if (selected.value === SEPARATOR_OPTION) {
				return; //should not happen, since this option is disabled, but do a check just in case
			}
					
			this._sourceId = selected.value;			

			//remove the current children of the template selector
			while(this.templateSelector.hasChildNodes()) {
				this.templateSelector.removeChild(this.templateSelector.childNodes[0]);
			}

			this.deployButton.disabled = true;
			this._selectedTemplateIds = [];

			//check to see if we've cached the report templates for this source
			if (this._sourceToReportMap[this._sourceId]) {
				this._setReportTemplates(this._sourceToReportMap[this._sourceId]);
			} else {
				var responseHandler = new ServiceResponseHandler(
						this,
						"_setReportTemplates",
						"_remoteError");
				var args = {
						processAreaUUID: this._targetProcessAreaUUID	
				};			

				if (this._sourceId == APPLICATION_OPTION) {
					ReportsClient.getMissingSharedReportDescriptors(responseHandler, args);
				} else {
					args.sourceProjectAreaUUID = this._sourceId;
					ReportsClient.getMissingSharedReportDescriptors(responseHandler, args);
				}
			}
		},	
		
		/**
		 * Called when the selection changes in the templates list.  Updates the selected template
		 * IDs, and the num selected text
		 */
		_templateSelectionChanged: function() {
			this.clearValidationMessage();
			var options = this.templateSelector.options;
			this._selectedTemplateIds = [];
			if(options != null){
				for(var i = 0; i < options.length; i++) {
					if(options[i].selected) {
						this._selectedTemplateIds.push(options[i].value);
					}
				}
			}
			
			var selected = this._selectedTemplateIds.length;
			if (selected > 0) {
				this.deployButton.disabled = false;
			} else {
				this.deployButton.disabled = true;
			}
			
			this.spanSelected.innerHTML = dojo.string.substitute(
					this.messages.numSelected, 
					[selected, this.templateSelector.options.length]);			
		},
		
		/**
		 * Called when the target change link is clicked.
		 */
		_processAreaLinkSelected: function(e) {
			e.preventDefault();

			var self = this;
			var params = {
				projects: true,
				teams: true,
				allowsMultiple: false,
				selection: self._targetProcessAreaUUID,
				filterProjectArea: self._projectAreaUUID,
				onOk: function(selectedItem, action) {
					self._setTarget(selectedItem);
				}
			};
			new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);			
		},
		
		/**
		 * Sets the deploy target to the given process area.
		 */
		_setTarget: function(targetProcessArea) {
			this._targetProcessAreaUUID = targetProcessArea.itemId;
			this.targetLink.innerHTML = jazz.util.html.escape(targetProcessArea.name);			
		},

		_setReportTemplates: function(reportTemplates) {
			//save the list in the cache 
			this._sourceToReportMap[this._sourceId] = reportTemplates;			
			
			// Let's sort them alphabetically first
			var sortFunc = function(a,b) {
				return a.name.localeCompare(b.name);
			}
			if (reportTemplates) {
				reportTemplates.sort(sortFunc);
			}			
			
			if (!reportTemplates || reportTemplates.length == 0) {
				this.setHeaderMessage({
					message: this.messages.noTemplatesFound,
					showOnlyMessage: true,
					severity: 4
				});
				return;
			}
			
			//Populate the selection list
			for (var i = 0; i < reportTemplates.length; i++) {
				var option = document.createElement("option");
				option.text = reportTemplates[i].name;
				option.value = reportTemplates[i].reportId;
				this.templateSelector.options.add(option);
			}
			this.spanSelected.innerHTML = dojo.string.substitute(
					this.messages.numSelected, 
					[0, reportTemplates.length]);
		},
		
		_deploySelected: function(e) {
			if (this._selectedTemplateIds.length > 0) {	
				this.deployButton.disabled = true;
				
				var handler = new ServiceResponseHandler(
						this,
						"_templatesCreated",
						"_remoteError");
				this.setHeaderMessage({
					message: this.messages.deployingNewTemplates,
					showOnlyMessage: true,
					severity: 1
				});
				var args = {
					reportIds: this._selectedTemplateIds,
					processAreaUUID: this._targetProcessAreaUUID
				};
				
				if (this._sourceId !== APPLICATION_OPTION) {
					args.sourceProjectAreaUUID = this._sourceId;
				}
				
				ReportsClient.postSaveMissingReportDescriptors(handler, args);
			}
		},
		
		/**
		 * Called if templates were deployed successfully.
		 */
		_templatesCreated: function() {
			this.clearValidationMessage();
			var summary = dojo.string.substitute(this.messages.deploySuccess, [this.targetLink.innerHTML]) + ":";
			
			var messageDiv = document.createElement("div");
			messageDiv.appendChild(document.createTextNode(summary));
			
			//append the name(s) of the deployed reports
			var options = this.templateSelector.options;
			for (var i = 0; i < options.length; i++) {
				if (options[i].selected) {
					messageDiv.appendChild(document.createElement("br"));
					messageDiv.appendChild(document.createTextNode(options[i].text));
				}				
			}
			
			messageDiv.appendChild(document.createElement("br"));
			
			//add link to the report resource view
			var reportResourcesLink = document.createElement("a");
			reportResourcesLink.href = "#action=com.ibm.team.reports.viewAllReportTemplates";
			reportResourcesLink.appendChild(document.createTextNode(this.messages.reportResourceLinkText));
			messageDiv.appendChild(reportResourcesLink);				
			
			this.messageBox.showMessage({
				summary: messageDiv,
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				closable: true
			});
			
			//reset the selections to remove the deployed templates from the list
			this._sourceToReportMap[this._sourceId] = null;
			this._sourceSelectionChanged();
		},
						
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this.templateSelector, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this.setHeaderMessage({
				message: error.message,
				showOnlyMessage: true,
				severity: 8
			});
		},
		
		// Status message area
		setHeaderMessage: function(message) {
			//TODO would it be better to replace all this functionality with a jazz.ui.MessageBox?
			this.clearValidationMessage();
			if (!message) {
				return;
			}
			
			if (!message.message || message.message.length == 0) {
				message.message = this.messages.errorUnhelpful;
			} else {
				if(message.message.lastIndexOf(".") != (message.message.length-1)) {
					message.message += ".";
				}
			}
			
            if (!message.details) {
                message.details = "";
            } else {
                if (message.details.lastIndexOf(".") != (message.details.length - 1)) {
                    message.details += ".";
                }
            }
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            } else {
            	if (message.showOnlyMessage == true) {
            		messageText = message.message
            	} else {
                	messageText = message.message + " " + message.details;            		
            	}
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this.metanav.validationMessage.__jazzMessage = message;
            if (message.severity==8) {
				this.typeIcon.src = ImagePool.ERROR;
                this.metanav.validationMessageSpan.className = "validationMessageError";
            } else {
				if (message.severity==4) {
					this.typeIcon.src = ImagePool.WARNING;
					this.metanav.validationMessageSpan.className = "validationMessageWarning";
				} else {
					this.typeIcon.src = ImagePool.INFO;
					this.metanav.validationMessageSpan.className = "validationMessageInfo";
				}
            }
            this.metanav.validationMessage.title = message.details || message.message;
			this.metanav.validationMessage.innerHTML = messageText;
			window.scrollTo(0,0); // show message
		},

		clearValidationMessage: function(force) {
			this.typeIcon.src = ImagePool.REPORTTEMPLATE_OBJ;
			this.metanav.validationMessageSpan.className = "validationMessageEmpty";
		}
	}
);

})();