/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.ExternalContentViewlet");

/*
 * Imports
 */
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");

dojo.require("jazz.client.xhr");
dojo.require("jazz.util.i18n");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal",
                         "ExternalContentMessages");

(function() {

var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var Util = com.ibm.team.repository.web.ui.internal.Util;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.ExternalContentViewlet",
             Viewlet, {
	/*
	 * Templates
	 */
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web",
	                             "ui/internal/templates/ExternalContentViewlet.html"),

	/*
	 * @param Object args
	 *
	 * @return StaticViewlet
	 */
	constructor: function(args) {
		// Messages //------------------------------------------------------------->
		this.messages = dojo.i18n.getLocalization(
			"com.ibm.team.dashboard.viewlets.web.ui.internal", "ExternalContentMessages");
	},

	/*
 	 * @param void
 	 * @return void
 	 * 
 	 * @overrides super
 	 */		
	init: function() {
		// Work-around to force a call to getTitleHyperlinkURL
		this.getSite().setTitle(this.getSite().getTitle(), false);
	},	

	/*
	 * @param void
	 *
	 * @return void
	 * @overrides super
	 */
	refresh: function() {
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			this._refreshTimeout = null;
		}
		if (this.getPreference("refreshInterval")) {
			this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh),
			                                  this.getPreference("refreshInterval")*1000);
		}
		this._loadDisplayArea();
	},

	/*
	 * @param Bool active is the viewlet active
	 *
	 * @return void
	 * @overrides super
	 */
	setActive: function(active) {
		if (active) {
			if (this.getPreference("refreshInterval")) {
				if (this._refreshTimeout) {
					clearTimeout(this._refreshTimeout);
					this._refreshTimeout = null;
				}
				this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh),
				                                  this.getPreference("refreshInterval")*1000);
			}
		} else if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			this._refreshTimeout = null;
		}
	},

	settingsChanged: function(oldSettings, newSettings) {
		var newUri = newSettings.preferences.externalUri;
		if (!this._checkValidUrl(newUri)) {
			this.setPreference("externalUri", "");
			this._invalidUriMessage(newUri);
			// Work-around to force a call to getTitleHyperlinkURL
			this.getSite().setTitle(this.getSite().getTitle(), false);
			return;
		}
		// Work-around to force a call to getTitleHyperlinkURL
		this.getSite().setTitle(this.getSite().getTitle(), false);
		this.getSite().refresh();
	},

	/*
	 * @param void
	 * @return string
	 * @overrides super
	 */
	getTitleHyperlinkURL: function() {
		if (this.getPreference("externalUri") === "") {
			return null;
		} else {
			return this.getPreference("externalUri");
		}
	},

	/*
 	 * Called right after a viewlet zoom operation is completed
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postZoom: function() {
		var iframeNode = this._frameArea.firstChild;
		if (!iframeNode) {
			// No iframe
			return;
		}
		var userSpecifiedHeight = this.getPreference("frameHeight");
		if (this.isZoomed()) {
			// Zoomed
			var newHeight = Math.floor(this._getDocumentHeight() * 0.7);
			if (newHeight > userSpecifiedHeight) {
				iframeNode.height = newHeight;
			}
		} else {
			// Not Zoomed
			var currentHeight = iframeNode.height;
			if (currentHeight !== userSpecifiedHeight) {
				iframeNode.height = userSpecifiedHeight;
			}
		}
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return Number
 	 */	
	_getDocumentHeight: function() {
		if (!document.body) { 
			return 0;
		}
		var position = dojo.position(document.body);
		if (!position) {
			return 0;
		}
		return Math.floor(position.h);
	},		
	
	_invalidUriMessage: function(uri) {
		dojo.empty(this._displayMessagesArea);
		var span = dojo.create('span', {}, this._displayMessagesArea, 'only');
		span.innerHTML = this._getOperationalErrorText(
				dojo.string.substitute(this.messages.invalidUrl, [uri]), 
				this.messages.invalidUrl_explanation, 
				this.messages.invalidUrl_useraction);

		this._updateVisibility(this._displayMessagesArea, true);
		this._updateVisibility(this._frameArea, false);
	},

	/*
 	 * @param String message
 	 * @param String explanation
 	 * @param String useraction
 	 * 
 	 * @return String
 	 */	
	_getOperationalErrorText: function(message, explanation, useraction) {
		var text = message + " " +
			explanation + " " +
			useraction;
		return text;
	},		
	
	_maybeShowMessage: function() {
		dojo.empty(this._displayMessagesArea);
		if (this.getSite().isDashboardEditing()) {
			var span = dojo.create('span', {});
			var nodes = jazz.util.i18n.addMessage(span, this.messages.addUrl);
			this.connect(nodes[0], "onclick", dojo.hitch(this, function(event) {
				if (event) {
					dojo.stopEvent(event);
				}
				this.getSite().toggleSettings();
			}));
			dojo.place(span, this._displayMessagesArea, "only");
		} else {
			// Don't offer to change the settings if the user doesn't have edit
			// permissions
			this._displayMessagesArea.innerHTML = this.messages.noUrl;
		}
		this._updateVisibility(this._displayMessagesArea, true);
		this._updateVisibility(this._frameArea, false);
	},

	_checkValidUrl: function(url) {
		// Using our own method instead of dojox.validate.isUrl because it has
		// false negatives
		if (this._isValidUrl(url)) {
			return true;
		}
		return false;
	},

	_isValidUrl: function(url) {
		// Regexp taken from RFC 3986 Appendix A
		return url.match(new RegExp('^(http(s)?:)(//([^/?#]*))([^?#]*)(\\?([^#]*))?(#(.*))?'));
	},

	_loadDisplayArea: function() {
		if (this.getPreference("externalUri") && this.getPreference("externalUri") !== "") {
			this._updateVisibility(this._displayMessagesArea, false);
			this._updateVisibility(this._frameArea, true);
			dojo.create("iframe", {
				'src': this.getPreference("externalUri"),
				'height': this.getPreference("frameHeight"),
				'frameBorder': 0
			}, this._frameArea, "only");
		} else {
			this._maybeShowMessage();
		}
	},

	/*
	 * @param Element node
	 * @param boolean show
	 *
	 * @return void
	 */
	_updateVisibility: function(node, show) {
		if (show) {
			node.style.display = "block";
		} else {
			node.style.display = "none";
		}
	}
});

})();
