/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.JobMonitor");

dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("jazz.client.xhr");

var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;

dojo.declare("com.ibm.team.repository.web.ui.internal.JobMonitor", null, {
	
	_minDelay: 1500,
	
	constructor: function(args) {
		if(!args.uri) {
			throw new Error("Missing required property: " + uri);
		}
		this._lastRequestTime = null;
		this._listeners = [];
		this._uri = args.uri;
		this._checkStatus();

		this._considerFinishedIfSubTasksFinished = (args.considerFinishedIfSubTasksFinished === undefined) ? false : args.considerFinishedIfSubTasksFinished;
	},
	
	addListener: function(f) {
		if(!dojo.isFunction(f)) {
			throw new Error("Arg 1 must be a function");
		}
		this._listeners.push(f);
	},
	
	_checkStatus: function() {
		this._lastRequestTime = (new Date()).getTime();
		//console.warn("---------- NEW REQUEST (Time: " + this.lastRequestTime + ") ----------");
		
		var headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		headers[HttpConstants.ACCEPT] = HttpConstants.CT_TEXT_JSON;
		var success = dojo.hitch(this, this._handleResponse);
		jazz.client.xhrGet({
			url: this._uri,
			handleAs: "json",
			headers: headers,
			load: function(response) {
				success(response);
			},
			error: function(errObject) {
				console.error(errObject);
			}
		});

	},
	
	_handleResponse: function(job) {
		if(job.state !== "FINISHED" && (!this._considerFinishedIfSubTasksFinished || !this._allSubTasksFinished(job))) {
			var timeSinceLastRequest = (new Date()).getTime() - this._lastRequestTime;
			//console.warn("timeSinceLastRequest: " + timeSinceLastRequest);
			var delay = timeSinceLastRequest > this._minDelay ? 0 : this._minDelay - timeSinceLastRequest;
			//console.warn("delay: " + delay);
			setTimeout(dojo.hitch(this, this._checkStatus), delay);
		}
		dojo.forEach(this._listeners, function(listener) {
			listener(job);
		});
	},
	
	_allSubTasksFinished: function(job) {
		if ("RUNNING" === job.state) {			
			var allFinished = true;
			dojo.forEach(job.steps, function(step) {
				allFinished = allFinished && "FINISHED" === step.state;
			});
			if (allFinished) {
				job.state = "FINISHED";
			}
			return allFinished;
		}
		return false;
	}
});

})();