/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.MembersViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.PagedTableViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("jazz.util.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MembersClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.util.html");
dojo.require("net.jazz.ajax.dataServerRoot");
dojo.require("jazz.app.context");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MembersMessages");

(function() {

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var HoverPopup = jazz.ui.HoverPopup;
var ResourceLink = jazz.ui.ResourceLink;
var HTML = jazz.util.html;
var I18nUtil = jazz.util.i18n;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var MemberInfoClient = com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient;	
var MembersClient = com.ibm.team.dashboard.viewlets.web.client.internal.MembersClient;
var PagedTableViewlet = com.ibm.team.dashboard.web.ui.PagedTableViewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

var SERVICE_PHOTO_ID = "com.ibm.team.dashboard.viewlets.service.internal.members.IMemberPhotoService";
var PHOTO_LIST = "PhotoList";
var PREFERENCE_PRESENTATION = "Presentation";

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MembersMessages");

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.MembersViewlet",
	PagedTableViewlet, {
	
	contentClass: "com-ibm-team-dashboard-viewlets-web-ui-internal-MembersViewlet",

	init: function() {
		this._updateTitle();
		this._widgets = new Array();
		this.inherited(arguments);
	},
	
	refresh: function() {
		delete this._totalResults;
		delete this._token;
		this._destroyWidgets();
		this.inherited(arguments);
		this._setTableLayout();
	},
	
	getResultCount: function() {
		return (this._totalResults || this._totalResults === 0) ? this._totalResults : -1;
	},
	
	getNoResultsMessage: function() {
		if (!this.getScopeItem()) {
			return this._createNoScopeMsg();
		}
		if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA)
			return document.createTextNode(messages.noMembersTeam);
		return document.createTextNode(messages.noMembersProject);
	},
	
	getItemsPerPage: function() {
		return this.getPreference("items");
	},
	
	fetch: function(srh, start, count, entries, details){
		// handle Auto(none) scope when no team areas
		if (!this.getScopeItem()) {
			this._totalResults = 0;
			this._totalResultsChanged();
			this.getSite().setSecondaryTitle("");
			srh.doCallback();
			return;
		}
		var serviceObject = {
			self: this,
			success: function(page) {
				if (page && page.totalResults > 0) {
					if (this.self._totalResults != page.totalResults) {
						this.self._totalResults = page.totalResults;
						this.self._totalResultsChanged();
					}
					this.self._token = page.token;
					if (page.members) {
						var members = page.members;
						var result = new Object();
						if (entries)
							result.entries = members;

						srh.doCallback(result);
					}
				} else {
					this.self._totalResults = 0;
					this.self._totalResultsChanged();
					srh.doCallback();
				}
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		var items = this.getPreference("items");
		switch(this.getScope()) {
			case DashboardConstants.SCOPE_TEAM_AREA:
				MembersClient.getMembersForTeamArea(this.getScopeItem().itemId, start, count, this._token ? this._token : "", false, srh2, this.getServiceTracker());
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				MembersClient.getMembersForProjectArea(this.getScopeItem().itemId, start, count, this._token ? this._token : "", false, srh2, this.getServiceTracker());
				break;
			default:
				console.error(new Error("Invalid dashboard scope encountered: " + this.getScope()));
				break;
		}
	},

	/* Node[] */ renderEntry: function(member, memberDetails, /* boolean */ zoomed) {
		var presentation = this.getPreference(PREFERENCE_PRESENTATION);
		var columns = [];
		var href = URLUtility.getContributorHref(member.itemId, this.getWebURL());
		
		if (presentation === PHOTO_LIST) {
			// PhotoList
		    
		    // Photo
		    var photoCell = document.createElement("td");
		    photoCell.className = "photoTD";
		    var photoLink = dojo.create("a", { 
		    	href: href
		    }, photoCell, "only");
		    var imgSrc = Util.getServiceURL(SERVICE_PHOTO_ID, null, {userId: member.userId});
		    dojo.create("img", { 
		    	src: imgSrc,
		    	width: 40,
		    	height: 40,
		    	alt: messages.photo
		    }, photoLink, "only");
			
		    // Name
		    var nameDiv = document.createElement("div");
		    nameDiv.className = "nameDiv";
		    var nameLink = dojo.create("a", { 
		    	href: href
		    }, nameDiv, "only");
		    dojo.place(document.createTextNode(member.name), nameLink, "only");
		    
		    // Email
		    var emailDiv = document.createElement("div");
		    dojo.addClass(emailDiv, "emailDiv");
		    
		    var emailIcon = document.createElement("div");
		    dojo.addClass(emailIcon, "emailIcon");
		    var emailLink = document.createElement("a");
		    emailLink.href = "mailto:" + member.email;
		    emailLink.innerHTML = member.email;
		    emailDiv.appendChild(emailIcon);
		    emailDiv.appendChild(emailLink);
		    
		    var nameCell = document.createElement("td");
		    nameCell.className = "nameTD";
		    nameCell.appendChild(nameDiv);
		    nameCell.appendChild(emailDiv);
		    
		    columns.push(photoCell);
		    columns.push(nameCell);
		} else {
			// DetailedList
			var node = document.createElement("a");
			node.appendChild(document.createTextNode(member.name));
			var rl = new ResourceLink({
				uri: (net.jazz.ajax.dataServerRoot() || this.getWebURL()) + "/users/" + encodeURIComponent(member.userId),
				presentationUri: href,
				lazyFetch: true
			}, node);
			this._widgets.push(rl);
		    var nameCell2 = document.createElement("td");
			nameCell2.className = "nameCell";
			nameCell2.appendChild(node);
			columns.push(nameCell2);
		
			var roleCell = document.createElement("td");
			roleCell.className = "roleCell";
			
			var roles = member.roles;
			
			for (var i = 0; roles && i < roles.length; i++) {
				var roleDTO = roles[i];
				var roleSpan = document.createElement("span");
				roleSpan.appendChild(document.createTextNode(roleDTO.label));
				
				tooltip = new HoverPopup({
					around: roleSpan,
					persist: false,
					createContents: dojo.hitch(this,this._populateRoleTooltip)});
				tooltip.data = roleDTO.id;
				this._widgets.push(tooltip);
				
				roleCell.appendChild(roleSpan);
				if (i != roles.length - 1)
					roleCell.appendChild(document.createTextNode(", "));
			}
			columns.push(roleCell);
			
		}
		return columns;
	},
	
	_setTableLayout: function() {
		if (!this.pagedTable) {
			return;
		}
		var table = this.pagedTable.domNode;
		if (!table) {
			return;
		}
		var presentation = this.getPreference(PREFERENCE_PRESENTATION);
		if (presentation === PHOTO_LIST) {
			dojo.addClass(table, "table-layout-fixed");
		} else {
			dojo.removeClass(table, "table-layout-fixed");
		}
	},	
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var nodes = I18nUtil.addMessage(editSpan, messages.notScopedEdit);
			this.connect(nodes[0], "onclick", this._handleToggleSettings);
			msgSpan.appendChild(editSpan);
		}
		return msgSpan;
	},
	
	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},
	
	_populateRoleTooltip: function(tip) {
		var div = document.createElement("div");
		div.innerHTML = "Loading...";
		div.className = "com-ibm-team-dashboard-viewlets-web-ui-internal-MembersViewlet-TooltipLoading";
		var serviceObj = {
			node: div,
			success: function(/* string */ description) {
				this.node.className = "com-ibm-team-process-web-viewlets-ui-internal-MembersViewlet-Tooltip";
				if (description)
					this.node.innerHTML = HTML.escape(description);
				else
					this.node.innerHTML = messages.noDescription;
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		MemberInfoClient.getRoleDescription(tip.data, this.getScope(), this.getScopeItem().itemId, srh, this.getServiceTracker());
		return div;
	},
	
	_totalResultsChanged: function() {
		this.updateTotalPageFooter();
		this.getSite().setSecondaryTitle("(" + this._totalResults + ")");
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var updateTitle = newSettings.userTitle != oldSettings.userTitle;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			this.reset(true);
		} else if (newSettings.preferences.items != oldSettings.preferences.items) {
			this.reset(false);
		} else if (newSettings.preferences.Presentation != oldSettings.preferences.Presentation) {
			this.reset(false);
		}
		if (updateTitle) {
			this._updateTitle();
		}
	},
	
	reset: function(/* boolean */ refresh) {
		if (!refresh) {
			this._setTableLayout();
		}
		this.inherited(arguments);
	},	
	
	_updateTitle: function() {
		this.getSite().setTitle((this.getScopeItem() && this.getScopeItem().name) ? dojo.string.substitute(messages.titleScoped, [this.getScopeItem().name]) : messages.title);
	},
	
	getTitleHyperlinkURL: function() {
		var scope = this.getScope();
		var context = this.getScopeItem();
		var url;

		if (!context ||
				!context.itemId) {
			// No team area or project area selected
			return null;
		}
			
		if (scope === DashboardConstants.SCOPE_TEAM_AREA) {
			// Team Area
			// https://localhost:9443/jazz/admin#action=com.ibm.team.process.editTeamArea&itemId=_Tj9RkDulEeCAZ5rY1YKmyQ&projectAreaItemId=_WOqPMDujEeCIDJ7vsnU5zQ
			var projectArea = this._getProjectArea();
			if (!projectArea ||
					!projectArea.itemId) {
				return null;
			}
			url = this.getWebURL() +
					"/admin#action=com.ibm.team.process.editTeamArea&itemId=" + 
					context.itemId +
					"&projectAreaItemId=" + 
					projectArea.itemId;
		} else {
			// Project Area
			// https://localhost:9443/jazz/admin#action=com.ibm.team.process.editProjectArea&itemId=_dS6j0DtEEd-CltVayS3d1Q
			url = this.getWebURL() +
					"/admin#action=com.ibm.team.process.editProjectArea&itemId=" + 
					context.itemId;
		}
		
		return url;
	},	

	_getProjectArea: function() {
		var site = this.getSite();
		if (!site.isRemote || site.isRemote()) {
			// Remote viewlet
			// TODO: Get from another source (e.g REST service) or add to viewlet scope API
			return null;
		}
		// Local viewlet
		return jazz.app.context.get();
	},	
	
	onDestroy: function() {
		this._destroyWidgets();
	},
	
	_destroyWidgets: function() {
		while (this._widgets.length > 0)
			this._widgets.pop().destroy();
	}

});

})();
