/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.MetaNav");
dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.Dialog");

dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");
dojo.require("com.ibm.team.reports.web.util.ReportsUtil");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.app.ua.PageHoverHelp");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {

var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;	
var ReportsUtil = com.ibm.team.reports.web.util.ReportsUtil;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
var PageHoverHelp= jazz.app.ua.PageHoverHelp;
var messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

dojo.declare("com.ibm.team.reports.web.ui.internal.MetaNav",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/MetaNav.html"),
	
	widgetsInTemplate: true,
	
	constructor: function(args) {
		this.loading = messages.loading;
		this.messages = messages;
	},
	
	setTitle: function(title) {
		// consumers of setTitle are responsible for escaping the parameter themselves
		this.heading.innerHTML = title;
		dojo.removeClass(this.heading, "loading");
	},

	populateReportsBreadcrumbs: function(shared) {
		this.breadcrumbs.segments = [];
		this.breadcrumbs.segments.push({ label: messages.reports, href: ReportsUtil.getReportsWelcomeHref()});
		
		if(shared=="true"){
			this.breadcrumbs.segments.push({ label: messages.sharedReports, href: ReportsUtil.getReportsListHref(shared)});
		} else if(shared=="false"){
			this.breadcrumbs.segments.push({ label: messages.myReports, href: ReportsUtil.getReportsListHref(shared)});
		}
		this.breadcrumbs.refresh();
	},
	
	populateResourcesBreadcrumbs: function() {
		this.breadcrumbs.segments = [];
		this.breadcrumbs.segments.push({ label: messages.reports, href: ReportsUtil.getReportsWelcomeHref()});
		this.breadcrumbs.segments.push({ label: messages.listOfResources, href: ReportsUtil.getResourcesListHref()});	
		this.breadcrumbs.refresh();
	},
	
	_handleHeaderMessageClick: function(e) {
		e.preventDefault();
		if (!e || !e.target || !e.target.__jazzMessage) return;

		var message = e.target.__jazzMessage;
		var text = message.message;
		if (message.details) {
			text += "\n\n";
			text += message.details;
		}
		alert(text);
	},
	
	clearValidationMessage: function(force) {
		this.validationMessageSpan.className = "validationMessageEmpty";
	},
	
	insertTypeIcon: function(typeIcon){
		this.leftHeadingWrapper.insertBefore(typeIcon, this.heading);
	},
	
	clearMessage: function () {
		this.messageBox.hideMessage();
	},
	
	displayErrorMessage: function (msg) {
		this.displayMessage(msg, STATUS_TYPES.ERROR);
	},
	
	displayMessage: function (msg, type) {
		this.messageBox.showMessage({ 
		   type: type,
		   summary: msg,
		   closable: true
		});
	},
	
	createContextSensitiveHelp: function(uaKey) {
		dojo.empty(this.csHelpNode);
		if (this.pageHelp) {
			this.pageHelp.destroy();
		}
		var parms= {};
		parms.uaKey= uaKey;
		this.pageHelp= new PageHoverHelp(parms);
		this.csHelpNode.appendChild(this.pageHelp.domNode);
	}, 
	
	destroy: function() {
		if (this.pageHelp) {
			this.pageHelp.destroy();
		}
		this.inherited(arguments);
	}	
});

})();
