/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.PersonalDashboard");

dojo.require("com.ibm.team.dashboard.web.ui.Dashboard");

dojo.require("jazz.core.loader");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.util.binding");

var binding = jazz.util.binding;
var Dashboard = com.ibm.team.dashboard.web.ui.Dashboard;
var MessageBox = jazz.ui.MessageBox;

var DASHBOARD_BINDING = {
	namespaces: {
		jdb: "http://jazz.net/xmlns/prod/jazz/dashboard/1.0/"
	},
	getShowWelcome: ["boolean", "jdb:showWelcome", "1"],
	setShowWelcome: ["boolean", "jdb:showWelcome", "1"]
};
var DASHBOARD_MSG_GLOBAL = "com.ibm.team.dashboard.web.ui.internal.dashboardMsg";
var WELCOME_MSG_CLASS = "com.ibm.team.dashboard.web.ui.internal.util.WelcomeMsg";

dojo.declare("com.ibm.team.dashboard.web.ui.internal.PersonalDashboard", [Dashboard], {
	
	// required
	url: null,
	
	// optional, with defaults
	allowDelete: true,
	allowSetDefault: true,
	allowSharing: true,
	showBreadcrumbs: true,
	
	// internal
	isPersonal: true,
	
	destroy: function() {
		this._destroyWelcomeMsg();
		this.inherited(arguments);
	},
	
	_destroyWelcomeMsg: function() {
		if (this._welcomeMsg) {
			this._welcomeMsg.destroyRecursive();
			delete this._welcomeMsg;
		}
	},
	
	onLoad: function() {
		this.inherited(arguments);
		/*
		 * Check if there is a message to show for the next dashboard. This is done using
		 * a global variable since the page may not exist yet by the time the creator
		 * (e.g. a wizard) has sent the user to the new dashboard's URL.
		 */
		var msg = dojo.getObject(DASHBOARD_MSG_GLOBAL);
		if (msg) {
			// consume the msg
			dojo.setObject(DASHBOARD_MSG_GLOBAL, null);
			this._showMessage(msg, true);
		}
		else {
			// or, show welcome message if applicable
			var doc = this.getResource();
			if (doc) {
				this._resource = binding.bind(doc.documentElement, DASHBOARD_BINDING);
				if (this._resource.getShowWelcome()) {
					this._showWelcomeMsg(this.url);
				}
			}
		}
	},
	
	_showWelcomeMsg: function(/* string */ dashboardUrl) {
		jazz.core.loader.load_async(WELCOME_MSG_CLASS, dojo.hitch(this, function() {
			this._destroyWelcomeMsg();
			var welcomeMsg = this._welcomeMsg = new dojo.getObject(WELCOME_MSG_CLASS)({
				dashboardUrl: dashboardUrl,
				onFinish: dojo.hitch(this, function() {
					this._hideMessage();
					this.refresh();
				})
			});
			this._showMessage({
				type: MessageBox.STATUS_TYPES.OK,
				summary: welcomeMsg.domNode,
				closable: true,
				onClose: dojo.hitch(this, function() {
					this._resource.setShowWelcome(false);
					this.resourceChanged();
				})
			}, false);
		}));
	}
});

})();
