/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2014. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportTemplateEditor");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");
dojo.require("com.ibm.team.reports.web.ui.internal.ResourceDialog");
dojo.require("com.ibm.team.reports.web.util.ReportsUtil");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.reports.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");

dojo.require("jazz.app.context");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.util.html");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.InlineEditBox");
dojo.require("dojo.io.iframe");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var FILE_UPLOAD_RESPONSE = "File Upload Response:";
var FILE_UPLOAD_ERROR = "File Upload Error:";
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var RESOURCE_SERVICE_URI = TeamServerClient._getServicePrefix() + "com.ibm.team.reports.service.internal.IReportResourceService";
var MAX_MESSAGE_LENGTH = 500;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

(function() {

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var ReportsUtil = com.ibm.team.reports.web.util.ReportsUtil;

var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;

var _currentBindRequest;
var _currentUploadFrameNumber = 100;
var _showSavedMessage = false;

/**
 * Widget to create a new report query
 */
var ReportTemplateEditor = dojo.declare("com.ibm.team.reports.web.ui.internal.ReportTemplateEditor", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportTemplateEditor.html"),
		
		/* src to icon add */
		__addIconSrc: null,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			this.dirty = false;
			this.__addIconSrc = dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/add.gif");
		},
				
		postCreate: function() {
			this.metanav = new com.ibm.team.reports.web.ui.internal.MetaNav({ReportTemplateEditor: this}, this.metanav);
			this._reportUUID = this.reportUUID;
			
			this.typeIcon = document.createElement("img");
			this.typeIcon.src = ImagePool.REPORTTEMPLATE_OBJ;
			this.typeIcon.className = "typeIcon";
			this.typeIcon.style.paddingTop = "3px";
			this.metanav.insertTypeIcon(this.typeIcon);
			
			this._populateHeader();

			// The report template. We may not have one initially.
			this._reportTemplate = null;
			
			this._editableResources = [];
			if (this._reportUUID) {
				var responseHandler = new ServiceResponseHandler(
						this,
						"_setReportTemplate",
						"_remoteError");
				ReportsClient.getReportDescriptor(responseHandler, {reportUUID: this._reportUUID});
			} else {
				PlatformUI.getWorkbench().setTitle(this.messages.newReportTemplate);
				this._setEmptyReportTemplate();
			}
			
			var self = this;
			this.connect(this._processAreaAnchor, "onclick", this._handleProcessAreaClick);
			this.connect(this._uploadForm, "onsubmit", "_handleUploadSubmit");			
			this.connect(this.fileLocation, "onchange", "_handleDesignChange");
			this.connect(this.__newResourceAction, "onclick", dojo.hitch(this, function(event) {
				dojo.stopEvent(event);
				self._handleNewResource();
			}));
			this.connect(this.restoreInitialContents, "onclick", this._restoreInitialContentsHandler);
			this.connect(this.setAsPlanningReport, "onsubmit", this._setAsPlanningReportHandler);
			
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			if (_showSavedMessage == true) {
				this._showStatusMessage(this.messages.templateSavedSuccessfully);
				_showSavedMessage = false;
			}
			
			if (!this.navigateAwayHandle) {
				this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
			}
			if (!this.onBeforeUnloadHandle) {
				this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleOnBeforeUnload), /*priority*/ 10000);
			}

		},
		
		_populateHeader: function(){
			this.metanav.setTitle(this.messages.reportTemplate);
			
			this._cancelButton = document.createElement("button");
			this._cancelButton.className = "Button";
			this._cancelButton.innerHTML = this.messages.cancel;
			
			this._saveButton = document.createElement("button");
			this._saveButton.className = "Button";
			this._saveButton.disabled = "true";
			this._saveButton.innerHTML = this.messages.save;
			
			dojo.connect(this._cancelButton, "onclick", this, this._handleCancel);
			dojo.connect(this._saveButton, "onclick", this, this._handleSave);
			
			this.metanav.commands.appendChild(this._cancelButton);
			this.metanav.commands.appendChild(this._saveButton);
			
			this.metanav.createContextSensitiveHelp(ReportsConstants.CSH_REPORT_TEMPLATE_EDITOR);
			if(this.displayBreadcrumbs) this.metanav.populateResourcesBreadcrumbs();
			else {
				this.metanav.handle.style.height = "62px";
				this.metanav.handle.style.minHeight = "62px";
			}
		},	
		
		/**
		 * Callback function to populate the page with the given report template.
		 */
		_setReportTemplate: function(template) {
			this._reportTemplate = template;
			this.result = template;

			// Set the widgets up
			this._nameInput.value = template.name;
			PlatformUI.getWorkbench().setTitle(template.name);
			if (template.description) {
				this._descriptionText.value = template.description;
			}
			if (template.reportId) {
				this._idInput.value = template.reportId;
			}
			if (template.processAreaUUID) {
				// Make a request for the process area name, which we'll fill in later
				var responseHandler = new ServiceResponseHandler(
					this,
					"_setProcessArea",
					"_remoteError");
				// Try team area first; if that fails look for project area
				var args = { uuid: template.processAreaUUID };
				ReportsClient.getProcessAreaByUUID(responseHandler, args);
			}
			
			if (this._sourceText.firstChild) {
				this._sourceText.removeChild(this._sourceText.firstChild);
			}
						
			if (template.source) {				
				if (template.localContents) {
					dojo.style(this._restoreSection, "display", "inline");
				}
				
				if (template.source === ReportsConstants.APPLICATION_SOURCE) {
					this._sourceText.appendChild(document.createTextNode(this.messages.applicationSourceOption));
				} else {
					//the source is a project area UUID - make a call for its name to fill in
					var responseHandler = new ServiceResponseHandler(
							this,
							"_setSourceProjectArea",
							"_remoteError");
						var args = { uuid: template.source };
						ReportsClient.getProcessAreaByUUID(responseHandler, args);
				}
			} else {
				this._sourceText.appendChild(document.createTextNode(this.messages.localSourceText));
			}
			
			if (this._table) {
				this._table.destroy();
				this._table = null;
			}
			this._table = new jazz.ui.StyledTable({columns: [{label: this.messages.resource, width: "90%", sortable: false}, {label: this.messages.actions, width: "10%", sortable: false, className: "action-column"}],
									trim: jazz.ui.styledTable.TrimStyles.GOLD,
									sortBy: 0});
			this._resourceTable.appendChild(this._table.domNode);
			
			if (template.resources) {
				// Make a copy for saving
				template.oldResources = [];
				for (var i = 0; i < template.resources.length; i++) {
					template.oldResources[i] = template.resources[i];
				}
								
				for (var i = 0; i < template.resources.length; i++) {
					this._table.tbody.appendChild(this._createRow(template.resources[i]));
				}
				// Fields for keeping track of uploaded content
	        	this.resourcesLocation = this._createArray(template.resources.length, "");
	        	this.resourcesSize = this._createArray(template.resources.length, 0);
	        	this.resourcesContentType = this._createArray(template.resources.length, "");
	        	this.resourcesChecksum = this._createArray(template.resources.length, 0);
			} else {
				template.resources = [];
				template.oldResources = [];
	        
				// Fields for keeping track of uploaded content
		        this.resourcesLocation = [];
		        this.resourcesSize = [];
				this.resourcesContentType = [];
				this.resourcesChecksum = [];
			}
			this._sharedCheckbox.checked = template.shared;
			this._supportsCachingCheckbox.checked = template.supportsDataCaching;
			this._flushPendingRequests();
		},
		
		_createArray: function(size, initialValue) {
			var array = new Array(size);
			for (var i = 0; i < array.length; i++) {
				array[i] = initialValue;
			}
			return array;
		},
			
		/**
		 * Populate the page with no initial report template.
		 */
		_setEmptyReportTemplate: function() {
			this._reportTemplate = new Object();
			// Assume we want the project area for sharing
			var currentProjectArea = jazz.app.context.get();
			this._reportTemplate.processAreaUUID = currentProjectArea.itemId;
			this._reportTemplate.projectAreaUUID = currentProjectArea.itemId;
			if (this._reportTemplate.processAreaUUID) {
				// Make a request for the process area name, which we'll fill in later
				var responseHandler = new ServiceResponseHandler(
					this,
					"_setProcessArea",
					"_remoteError");
				// Try team area first; if that fails look for project area
				var args = { uuid: this._reportTemplate.processAreaUUID };
				ReportsClient.getProcessAreaByUUID(responseHandler, args);
			}
			this._reportTemplate.resources = [];
			this._reportTemplate.oldResources = [];
			// Fields for keeping track of uploaded content
        	this.resourcesLocation = [];
        	this.resourcesSize = [];
        	this.resourcesContentType = [];
        	this.resourcesChecksum = [];
        	// Create an empty table
			if (this._table) {
				this._table.destroy();
				this._table = null;
			}
			
			this._sourceText.appendChild(document.createTextNode(this.messages.localSourceText));
			this._table = new jazz.ui.StyledTable({columns: [{label: this.messages.resource, width: "80%", sortable: false}, {label: this.messages.actions, width: "20%", sortable: false}],
									trim: jazz.ui.styledTable.TrimStyles.GOLD,
									sortBy: 0});
			this._resourceTable.appendChild(this._table.domNode);
        	
		},
					
		_createRow: function(resourceName) {
			var row = document.createElement("tr");
			var td = document.createElement("td");
			var div = document.createElement("div");
			div.innerHTML = resourceName;
			var self = this;
	        this._editableResources[resourceName] = new dijit.InlineEditBox({title: this.messages.clickToEdit, autoSave: true, 
	        	width: "200px", noValueIndicator: "?", 
	        	onChange: dojo.hitch(this, function(newName) {
					self._handleResourceNameChange(resourceName, newName);
				})}, div);
			this._editableResources[resourceName].startup();
			td.appendChild(div);
			td.className="reportName";
			row.appendChild(td);
			
			td = document.createElement("td");
			td.className = "action-column";
			var toolbar = new com.ibm.team.reports.web.ui.internal.common.Toolbar({});
			toolbar.addButton(this, dojo.hitch(this, "_uploadResource", resourceName), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/upload-content_co.gif"), label: this.messages.uploadNewContent});
			toolbar.addButton(this, dojo.hitch(this, "_deleteResource", resourceName, row), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/delete.gif"), label: this.messages.deleteMsg});
			td.appendChild(toolbar.domNode);
			
			row.appendChild(td);
			return row;
		},
		
		_handleResourceNameChange: function(oldName, newName) {
			var index;
			if (this._reportTemplate.resources.indexOf) {
				index = this._reportTemplate.resources.indexOf(oldName);
			} else {
				//"indexOf" not supported in IE8, so do it manually
				index = -1;
				for (var i = 0; i<this._reportTemplate.resources.length; i++) {
					if (this._reportTemplate.resources[i] === oldName) {
						index = i;
						break;
					}
				}
			}
			this._reportTemplate.resources[index] = newName;
			this._setDirty(true);
		},
		
		_deleteResource: function(resourceName, row) {
			var resources = this._reportTemplate.resources;
			var index = resources.indexOf(resourceName);
			if (index == -1) {
				// Should never happen
				return;
			}
			this._reportTemplate.resources.splice(index, 1);
			// Also remove it from the oldResources table
			this._reportTemplate.oldResources.splice(index, 1);
			this._table.tbody.removeChild(row);
			// Remove any info about uploaded content for this resource
        	this.resourcesLocation.splice(index, 1);
        	this.resourcesSize.splice(index, 1);
        	this.resourcesContentType.splice(index, 1);
        	this.resourcesChecksum.splice(index, 1);
			this._setDirty(true);
		},
		
		_handleNewResource: function() {
			var self = this;
			var handleOk = dojo.hitch(this, function() {
				var resourceFileLocation = self.resourceDialog.getFileLocation();
				// Store the values for later.
	        	self.resourcesLocation.push(resourceFileLocation.savedLocation);
	        	self.resourcesSize.push(resourceFileLocation.savedFileSize);
	        	self.resourcesContentType.push(resourceFileLocation.savedContentType);
	        	self.resourcesChecksum.push(resourceFileLocation.savedChecksum);
	        	self._reportTemplate.oldResources.push("");
	        	var localFileLocation = resourceFileLocation.value;
	        	var splitLocation = dojo.isIE ? resourceFileLocation.value.split("\\") : resourceFileLocation.value.split("/");
	        	var localFileName = splitLocation[splitLocation.length - 1];
	        	self._reportTemplate.resources.push(localFileName);
				self._table.tbody.appendChild(this._createRow(localFileName));
				self._setDirty(true);
			});		
			this.resourceDialog = new com.ibm.team.reports.web.ui.internal.ResourceDialog({onOk: handleOk});
		},
		
		_uploadResource: function(resourceName) {
			var self = this;
			var handleOk = dojo.hitch(this, function() {
				var resourceFileLocation = self.resourceDialog.getFileLocation();
				// Store the values for later.
				var index = self._reportTemplate.resources.indexOf(resourceName);
	        	self.resourcesLocation[index] = resourceFileLocation.savedLocation;
	        	self.resourcesSize[index] = resourceFileLocation.savedFileSize;
	        	self.resourcesContentType[index] = resourceFileLocation.savedContentType;
	        	self.resourcesChecksum[index] = resourceFileLocation.savedChecksum;

				self._setDirty(true);
			})		
			this.resourceDialog = new com.ibm.team.reports.web.ui.internal.ResourceDialog({onOk: handleOk});
		},
		
		_setProcessArea: function(result) {
			this._processAreaAnchor.innerHTML = result.name;
		},
		
		_setSourceProjectArea: function(result) {
			this._sourceText.appendChild(document.createTextNode(result.name));
		},
		
		_handleSave: function() {
			this._clearStatusMessage();
	        var args = {
		        name: this._getName()
	        }
	        if (this._reportUUID) {
	        	args.reportUUID = this._reportUUID;
	        }
	        var description = this._getDescription();
	        if (description && description.length > 0) {
	        	args.description = description;
	        }
	        var reportId = this._getIdentifier();
	        if (reportId && reportId.length > 0) {
	        	args.reportId = reportId;
	        }
	        args.shared = this._getShared();
	        args.supportsDataCaching = this._getSupportsDataCaching();
	        args.processAreaUUID = this._reportTemplate.processAreaUUID;
	        
	        if (this.fileLocation.savedLocation && (this.fileLocation.savedLocation != "")) {
	        	args.designLocation = this.fileLocation.savedLocation;
	        	args.designSize = this.fileLocation.savedFileSize;
	        	args.designContentType = this.fileLocation.savedContentType;
	        	args.designChecksum = this.fileLocation.savedChecksum;
	        	args.designFileExtension = this.fileLocation.savedFileExtension;
	        }

	        args.resources = this._reportTemplate.resources;
	        args.oldResources = this._reportTemplate.oldResources;
	        
	        args.resourcesLocation = this.resourcesLocation;
	        args.resourcesSize = this.resourcesSize;
			args.resourcesContentType = this.resourcesContentType;
			args.resourcesChecksum = this.resourcesChecksum;

	        var responseHandler = new ServiceResponseHandler(
					this,
					"_saveComplete",
					"_remoteError");
			ReportsClient.postSaveReportDescriptor(responseHandler, args);
		},
		
		_handleCancel: function() {
			if (this.dirty) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
					this.destroy();
					document.location.href = "#action=com.ibm.team.reports.viewAllReportTemplates";
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.destroy();
				document.location.href = "#action=com.ibm.team.reports.viewAllReportTemplates";
				return false;
			}
			
		},
		
		_handleNavigateAway: function(url) {
			var currentLocation = document.location.href;
			var badLink = currentLocation.substring(0, currentLocation.indexOf("#")+1);
			if (url == badLink) {
				return true;
			}
			
			if (this.dirty) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
					this.destroy();
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.destroy();
				return false;
			}
		},
		
		destroy: function() {
			if (this.navigateAwayHandle) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				this.navigateAwayHandle = null;
			}
			if (this.onBeforeUnloadHandle) {
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				this.onBeforeUnloadHandle = null;
			}
		},
		
		_handleOnBeforeUnload: function() {
			return this.dirty ? this.messages.unsavedChanges + "\n\n" : false;
		},
		
		_getName: function() {
			return this._nameInput.value;
		},
		
		_getDescription: function() {
			return this._descriptionText.value;
		},
		
		_getIdentifier: function() {
			return this._idInput.value;
		},
		
		_getShared: function() {
			return this._sharedCheckbox.checked;
		},
		
		_getSupportsDataCaching: function() {
			return this._supportsCachingCheckbox.checked;
		},
		
		_saveComplete: function(args) {
			this._showStatusMessage(this.messages.templateSavedSuccessfully);
			this.setHeaderMessage(null); //clear header message, if it is showing
			var newTemplate = true;
			if (this._reportUUID) {
				newTemplate = false;
			}
			this._reportUUID = args;
			if(args.toLowerCase() != "error") {
				if (newTemplate == true) {
					_showSavedMessage = true;
					this.destroy();
				}
				document.location.href = "#action=com.ibm.team.reports.editTemplate&reportUUID=" + args;
			}
			var responseHandler = new ServiceResponseHandler(
					this,
					"_setReportTemplate",
					"_remoteError");
			ReportsClient.getReportDescriptor(responseHandler, {reportUUID: this._reportUUID});
			this._setDirty(false);	
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this.setHeaderMessage({
				message: error.message,
				showOnlyMessage: true,
				severity: 8
			});
		},
		
		_handleNameChange: function() {
			this._setDirty(true);
		},
		
		_handleDescriptionChange: function() {
			this._setDirty(true);
		},
		
		_handleIdChange: function() {
			this._setDirty(true);
		},
		
		_handleCheckboxChange: function() {
			this._setDirty(true);
		},
		
		_handleProcessAreaClick: function(e) {
			e.preventDefault();

			var self = this;
			var params = {
				projects: true,
				teams: true,
				allowsMultiple: false,
				selection: self._reportTemplate.processAreaUUID,
				filterProjectArea: self._reportTemplate.projectAreaUUID,
				onOk: function(selectedItem, action) {
					self._onProcessOk(selectedItem);
				}
			};
			new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);
		},
		
		_handleDesignChange: function () {
			if (this.fileLocation.value != "") {			
				this._handleUploadSubmit(this.fileLocation.value);
				this.fakeFileLocation.value = this.fileLocation.value.replace("C:\\fakepath\\", "");
				//dojo.toggleClass(this.fileLocation, "highlightChange", true);
				// TODO: How does this work with dirty state?
				//this.setDirtyOverviewPage(true);
			}
		},
		
		_handleUploadSubmit: function(fullPath) {
			var self = this;
			if (this._currentBindRequest != null) {
				this._currentBindRequest.cancel();
				this._currentBindRequest  = null;
			}
			
			/*
			 * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the 
			 * next one breaks. Resetting the iframe fixes the problem
			 * Unfortunately this code reaches into iframe private variables.
			 */
			if (dojo.io.iframe._frame) {
				document.body.removeChild(dojo.io.iframe._frame);				
			}
			var frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
			dojo.io.iframe._iframeName = frameName;
			dojo.io.iframe._frame = null;
			
			//disable "save" while upload operation is in progress
			var prevSaveState = this._saveButton.disabled;
			this._saveButton.disabled = true;
			if (this.controller) {
				this.controller.startOperation(this.messages.uploadingFile);				
			}

			this._currentBindRequest = dojo.io.iframe.send({
				form: this._uploadForm,
				url: RESOURCE_SERVICE_URI,
				content: {
						mimetype: "text/html",
						fileFields: "fileLocation"
					},
				handleAs: "html",
				handle: function(response, ioArgs) {					
					if (self.controller) {
						self.controller.finishOperation();
					}			
					if (response instanceof Error){
						if (response.data-dojo-type && response.data-dojo-type == "cancel") {
							self._saveButton.disabled = prevSaveState;
							return; // this is a cancelled request, ignore it
						}
						self._currentBindRequest = null; // no need to cancel it any more
						this.setHeaderMessage({
							message: self.messages.invalidFileName,
							detail: response.message,
							showOnlyMessage: true,
							severity: 8
						});
					} else {
						var responseMsg = response.body.innerHTML;
						if (responseMsg.indexOf(FILE_UPLOAD_RESPONSE) == 0) {
							var data = responseMsg.substring(FILE_UPLOAD_RESPONSE.length);
							var dataArray = data.split(",");
							// Make sure we have success
							if (dataArray.length == 4) {
								var designLocation = dataArray[0];
								var designSize = dataArray[1];
								var designContentType = dataArray[2];
								var designChecksum = dataArray[3];
								self.fileLocation.savedLocation = designLocation;
								self.fileLocation.savedFileSize = designSize;
								self.fileLocation.savedContentType = designContentType;
								self.fileLocation.savedChecksum = designChecksum;
								var split = self.fileLocation.value.split('.');
								self.fileLocation.savedFileExtension = split[split.length - 1];
								// Just to be helpful, set name and id fields if they aren't already
								var oldName = self._getName();
								var name = self._extractFilename(self.fileLocation.value);
								if (!oldName || oldName.length == 0) {
									self._nameInput.value = name;
								}
								var oldId = self._getIdentifier();
								if (!oldId || oldId.length == 0) {
									self._idInput.value = self._nameToId(name);
								}
								var oldDescription = self._getDescription();
								if (!oldDescription || oldDescription.length == 0) {
									self._descriptionText.value = dojo.string.substitute(self.messages.reportDescription, [name]);
								}
							} else {
								// Means failure.  Set the location to null so that it will not be saved
								self.fileLocation.savedLocation = null;
								self.setHeaderMessage({
									message: data,
									showOnlyMessage: true,
									severity: 8
								});
							}
						}
					}
					
					self._saveButton.disabled = prevSaveState;
					self._setDirty(true);
				}		  
			});
		},
		
		_onProcessOk: function(item) {
			this._reportTemplate.processAreaUUID = item.itemId;
			this._processAreaAnchor.innerHTML = item.name;
			this._setDirty(true);
		},
		
		_setDirty: function(isDirty) {
			if (isDirty) {
				this.metanav.heading.innerHTML = this.messages.reportTemplate + "*";
				this.metanav.heading.style.color = "#A83300";
				this._saveButton.disabled = false;
			} else {
				this.metanav.heading.innerHTML = this.messages.reportTemplate;
				this.metanav.heading.style.color = "#000000";
				this._saveButton.disabled = true;
			}
	
			this.dirty = isDirty;
		},
		
		_nameToId: function(name) {
			var tokens = name.split(" ");
			var buffer = new Array();
			for (var i = 0; i < tokens.length; i++) {
				var token = tokens[i];
				var c = token[0];
				if (c.toUpperCase() != c) {
					c = c.toUpperCase();
					token = c + token.substring(1);
				}
				buffer.push(token);
			}
			return "report." + (buffer.join(""));
		},
		
		_extractFilename: function(path) {
			var splitLocation;
			if (path.indexOf("\\" > -1)){
				splitLocation = path.split("\\");
			}
			else {
				splitLocation = path.split("/");
			}
	       	var localFileName = splitLocation[splitLocation.length - 1];
			var splitFileName = localFileName.split(".");
			return splitFileName[0];
		},
		
		setHeaderMessage: function(message) {
			this.clearValidationMessage();
			if (!message) {
				return;
			}
			
			if (!message.message || message.message.length == 0) {
				message.message = this.messages.errorUnhelpful;
			} else {
				if(message.message.lastIndexOf(".") != (message.message.length-1)) {
					message.message += ".";
				}
			}
			
            if (!message.details) {
                message.details = "";
            } else {
                if (message.details.lastIndexOf(".") != (message.details.length - 1)) {
                    message.details += ".";
                }
            }
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            } else {
            	if (message.showOnlyMessage == true) {
            		messageText = message.message
            	} else {
                	messageText = message.message + " " + message.details;            		
            	}
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this.metanav.validationMessage.__jazzMessage = message;
            if (message.severity==8) {
				this.typeIcon.src = ImagePool.ERROR;
                this.metanav.validationMessageSpan.className = "validationMessageError";
            } else {
				if (message.severity==4) {
					this.typeIcon.src = ImagePool.WARNING;
					this.metanav.validationMessageSpan.className = "validationMessageWarning";
				} else {
					this.typeIcon.src = ImagePool.INFO;
					this.metanav.validationMessageSpan.className = "validationMessageInfo";
				}
            }
            this.metanav.validationMessage.title = message.details || message.message;
            dojox.data.dom.replaceChildren(this.metanav.validationMessage, document.createTextNode(messageText));
			window.scrollTo(0,0); // show message
		},
		
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if (!e || !e.target || !e.target.__jazzMessage) return;

			var message = e.target.__jazzMessage;
			var text = message.message;
			if (message.details) {
				text += "\n\n";
				text += message.details;
			}
			alert(text);
		},
		
		clearValidationMessage: function(force) {
			this.metanav.validationMessageSpan.className = "validationMessageEmpty";
			this.typeIcon.src = ImagePool.REPORTTEMPLATE_OBJ; //also reset icon
		},
		
		_restoreInitialContentsHandler: function(e) {
			e.preventDefault();
			if (!this._reportTemplate || !this._reportTemplate.reportUUID) {
				this.setHeaderMessage({
					message: this.messages.cannotRestoreContents,
					showOnlyMessage: true,
					severity: 8
				});
				return;
			}
			var handler = new ServiceResponseHandler(
				this,
				"_contentsRestored",
				"_remoteError");
			var currentProjectArea = jazz.app.context.get();
			var args = {
				projectAreaUUID: currentProjectArea.itemId,
				reportUUID: this._reportTemplate.reportUUID
			};
			ReportsClient.postResetReportDescriptor(handler, args);
		},

		_contentsRestored: function() {
			this.setHeaderMessage({
				message: this.messages.contentsRestored,
				showOnlyMessage: true,
				severity: 1
			});
			
			this._reportTemplate.localContents = false;
			dojo.style(this._restoreSection, "display", "none");
		},

		_setAsPlanningReportHandler: function(e) {
			e.preventDefault();
			if (!this._reportTemplate || !this._reportTemplate.reportUUID) {
				this.setHeaderMessage({
					message: this.messages.cannotSetAsPlanningReport,
					showOnlyMessage: true,
					severity: 8
				});
				return;
			}
			var handler = new ServiceResponseHandler(
				this,
				"_defaultReportSet",
				"_remoteError");
			var currentProjectArea = jazz.app.context.get();
			var args = {
				projectAreaUUID: currentProjectArea.itemId,
				reportUUID: this._reportTemplate.reportUUID
			};
			ReportsClient.postSetDefaultReportDescriptor(handler, args);
		},
		_defaultReportSet: function() {
			this.setHeaderMessage({
				message: this.messages.defaultReportSet,
				showOnlyMessage: true,
				severity: 1
			});
		},
		
		_showStatusMessage: function(message) {
			this._clearStatusMessage();
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.showMessage(status);
		},
		
		_clearStatusMessage: function() {
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.hideMessage();
		},
		
		_flushPendingRequests: function() {
			if (this._pendingInfoRequests) {
				var info = this._getInfo();
				for (var i = 0; i < this._pendingInfoRequests.length; i++) {
					this._pendingInfoRequests[i].doCallback(info);
				}
			}
			this._pendingInfoRequests = null;
		},
		
		getArtifactInfo: function(srh) {
			if (!this.result) {
				if (!this._pendingInfoRequests) {
					this._pendingInfoRequests = [];
				}
				this._pendingInfoRequests.push(srh);
			}
			else {
				srh.doCallback(this._getInfo());
			}
		},
		
		_getInfo: function() {
			if (!this.result) {
				return null;
			}
			return {
				type: ReportsConstants.TEMPLATE_TYPE_ID,
				itemId: this._reportUUID,
				title: this._getName(),
				iconUrl: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/reporttemplate-obj.gif"),
				href: ReportsUtil.getReportTemplateHref(this._reportUUID),
				url: ReportsUtil.getReportUri(ReportsConstants.TEMPLATE_TYPE_ID,this._getName(),this._reportUUID)
			};
			
		},
		

	}
);

})();