/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportsComponentMenu");

dojo.require("jazz.app._ComponentMenu");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.RecentlyViewed");
dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");



dojo.require("jazz.util.html");
dojo.require("jazz.util.format");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var RecentlyViewed = com.ibm.team.reports.web.ui.internal.RecentlyViewed;
var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;


var REPORTS_WELCOME_URI = "#action=com.ibm.team.reports.defaultAction";
var MY_REPORTS_URI = "#action=com.ibm.team.reports.viewAllReports&shared=false";
var SHARED_REPORTS_URI = "#action=com.ibm.team.reports.viewAllReports&shared=true";
var REPORT_TEMPLATES_URI = "#action=com.ibm.team.reports.viewAllReportTemplates";
var NEW_REPORT_URI = "#action=com.ibm.team.reports.newReport";
var RUNNING_REPORTS_URI = "#action=com.ibm.team.reports.viewRunningReports";
var ADDITIONAL_TASKS = "#action=com.ibm.team.reports.showAdditionalTasks";

dojo.declare("com.ibm.team.reports.web.ui.internal.ReportsComponentMenu", jazz.app._ComponentMenu, {

	postCreate: function() {
		dojo.subscribe(ReportsConstants.EVENT_REPORTS_VIEW_CHANGED, this, "refreshMenu");
		this.refreshMenu();
		RecentlyViewed.init();
	},
	
	refreshMenu: function() {
		this.clear();
		this._index = 0;
		this._index = this._addQueries(this._index);
		this._index = this._addRecentlyViewed(this._index);
	},
	
	_addQueries: function(index) {
		var srh = new ServiceResponseHandler(this, "_storeCustomReportingConfigured", "_handleError");
		ServerStatusClient.getServiceConfiguration(srh, "com.ibm.team.reports.service.cognos.internal.CognosReportService");
		
		this.addItemByArgs({
			label: messages.welcomeToReports,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/report-obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: REPORTS_WELCOME_URI,
			insertIndex: index++
		});
		this.addTitle(messages.browse);
		this.addItemByArgs({
			label: messages.myReports,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/reports-obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: MY_REPORTS_URI,
			insertIndex: index++
		});
		this.addItemByArgs({
			label: messages.sharedReports,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/reports-obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: SHARED_REPORTS_URI,
			insertIndex: index++
		});
		this.addItemByArgs({
			label: messages.reportTemplates,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/report-template-group_obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: REPORT_TEMPLATES_URI,
			insertIndex: index++
		});
		this.addItemByArgs({
			label: messages.runningReports,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/runningreport_obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,
			href: RUNNING_REPORTS_URI,
			insertIndex: index++			
		});
				
		this.addTitle(messages.create);
		this.addItemByArgs({
			label: messages.createReportFromTemplate,
			icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/create-report_obj.gif"),
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: NEW_REPORT_URI,
			insertIndex: index++
		});
		
		index+=2;
		
		return index;
	},
	
	_addRecentlyViewed: function(index) {
		var count = RecentlyViewed.getArtifactsCount();		
		if(!count || count < 1) {
			return index;
		}
		var titleItem = {};
		titleItem.wrapHint= 2;
		titleItem.insertIndex= index++;
		var countString = count == 10 ? dojo.string.substitute(messages.maximum, [count]) : count;
		this.addTitle("<b>" + messages.recentlyViewed + "</b> (" + countString + ")", titleItem);
		
		var artifacts = RecentlyViewed.getArtifacts();
		for (var i = 0; i < artifacts.length; i++) {
			var info = artifacts[i];
			var menuItem = {};
			menuItem.icon = info.iconUrl;
			menuItem.href = info.href;
			menuItem.label = jazz.util.html.escape(info.title);
			this.addItemByArgs(menuItem, index++);			
		}
		return index;
	},
	
	// Called by the super class. It will put a black dot in front of the
	// returned item to denote it as "selected"
	getSelected: function() {
		var items = this.getItems();
		var href = location.href;
		if (href.search(REPORTS_WELCOME_URI) != -1) {
			return items[0];
		} else if (href.search(MY_REPORTS_URI) != -1) {
			return items[2];
		} else if (href.search(SHARED_REPORTS_URI) != -1) {
			return items[3];
		} else if (href.search(REPORT_TEMPLATES_URI) != -1) {
			return items[4];
		} else if (href.search(NEW_REPORT_URI) != -1) {
			return items[8];
		} else if (href.search(RUNNING_REPORTS_URI) != -1) {
			return items[5];
		} 
		else {
			return null;
		}
	},
	
	_storeCustomReportingConfigured: function(response) {
		for(var i = 0; i < response.properties.length; i++){
			if(response.properties[i].name == "cognos.hostAndPort"){
				if(response.properties[i].overrideValue && response.properties[i].overrideValue.length > 0){
					this.addTitle(messages.customizeReport);
					this.addItemByArgs({
						label: messages.performAdditional,
						icon: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/report-obj.gif"),
						wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
						href: ADDITIONAL_TASKS
					});
					return;
				}
			}
		}
	},
	
	_handleError: function() {
	}
});
})();