/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportsView");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.core.loader");

dojo.require("com.ibm.team.reports.web.ui.internal.AllReportsTab");
dojo.require("com.ibm.team.reports.web.ui.internal.AllReportTemplatesTab");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportsController");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportViewer");
dojo.require("com.ibm.team.reports.web.ui.internal.NewReportEditor");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportsWelcome");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportsAdditionalTasks");
dojo.require("com.ibm.team.reports.web.ui.internal.FolderEditor");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportEditor");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportCustomTemplateEditor");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportTemplateEditor");
dojo.require("com.ibm.team.reports.web.ui.internal.DeployTemplatesViewer");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportJobStatusPage");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.DOMUtils");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.client.internal.AdminPageRegistry");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.reports.web.ui.internal.RecentlyViewed");


dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

var session	= com.ibm.team.repository.web.client.session;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var AdminPageRegistry = com.ibm.team.reports.web.client.internal.AdminPageRegistry;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ActionRegistry = null; // Initialize later
var newReportURI = "#action=com.ibm.team.reports.newReport";
var loader = jazz.core.loader;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var removeChildren = com.ibm.team.reports.web.ui.internal.utils.DOMUtils.removeChildren;
var RecentlyViewed = com.ibm.team.reports.web.ui.internal.RecentlyViewed;

/**
 * Main perspective view for the Reports perspective.
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.ReportsView", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportsView.html"),
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		},
		
		displayBreadcrumbs: false,

		postCreate: function() {
			com.ibm.team.reports.web.ui.internal.ReportsView.superclass.postCreate.apply(this);

		 	this.controller = new com.ibm.team.reports.web.ui.internal.ReportsController(this);
		 	
			var srh = new ServiceResponseHandler(this, "_storeHasWorkItems", "_handleHasWorkItemsError");
			ServerStatusClient.postStatusForServiceList(srh, {serviceNames: ["com.ibm.team.reports.workitem.service.internal.IReportWorkItemRestService"]});
		 	
			srh = new ServiceResponseHandler(this, "_storeCustomReportingConfigured", "_handleCustomReportingConfiguredError");
			ServerStatusClient.getServiceConfiguration(srh, "com.ibm.team.reports.service.cognos.internal.CognosReportService");
		 	
			srh = new ServiceResponseHandler(this, "_handleShowArchived", "_handleShowArchivedError");
			ReportsClient.getShowArchived(srh);
			
		 	// Lazy initialize ActionRegistry
		 	ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		 	
			ActionRegistry.registerAction("com.ibm.team.reports.defaultAction", this.getController(), "defaultAction");
			ActionRegistry.registerAction("com.ibm.team.reports.viewQuery", this.getController(), "viewQuery");
			ActionRegistry.registerAction("com.ibm.team.reports.viewReport", this.getController(), "viewReport");
			ActionRegistry.registerAction("com.ibm.team.reports.getMyQueries", this.getController(), "getMyQueries");
			ActionRegistry.registerAction("com.ibm.team.reports.getSharedQueries", this.getController(), "getSharedQueries");
			ActionRegistry.registerAction("com.ibm.team.reports.admin", this.getController(), "admin");
		 	ActionRegistry.registerAction("com.ibm.team.reports.viewWorkItems", this.getController(), "viewWorkItems");
		 	ActionRegistry.registerAction("com.ibm.team.reports.newReport", this.getController(), "newReport");
		 	ActionRegistry.registerAction("com.ibm.team.reports.editTemplate", this.getController(), "editTemplate");
		 	ActionRegistry.registerAction("com.ibm.team.reports.editCustomTemplate", this.getController(), "editCustomTemplate");
		 	ActionRegistry.registerAction("com.ibm.team.reports.editFolder", this.getController(), "editFolder");
		 	ActionRegistry.registerAction("com.ibm.team.reports.editReport", this.getController(), "editReport");
		 	ActionRegistry.registerAction("com.ibm.team.reports.deployNewTemplates", this.getController(), "deployNewTemplates");
		 	ActionRegistry.registerAction("com.ibm.team.reports.showAdditionalTasks", this.getController(), "showAdditionalTasks");
		 	ActionRegistry.registerAction("com.ibm.team.reports.viewAllReports", this, "viewAllReports");
		 	ActionRegistry.registerAction("com.ibm.team.reports.viewAllReportTemplates", this, "viewAllReportTemplates");
		 	ActionRegistry.registerAction("com.ibm.team.reports.authorCustomReport", this, "authorCustomReport");
		 	ActionRegistry.registerAction("com.ibm.team.reports.reportStudio", this, "reportStudio");
		 	ActionRegistry.registerAction("com.ibm.team.reports.queryStudio", this, "queryStudio");
		 	ActionRegistry.registerAction("com.ibm.team.reports.viewRunningReports", this, "viewRunningReports");
		 	
		    this._canSeeReports = false;
		    this._showWelcome = false;
		    // The number of asnyc calls below we need to wait for
		    this._numberOfPendingResponses = 1;
			var responseHandler = new ServiceResponseHandler(
					this,
					"_handleCanSeeReports",
					"_remoteError");
			ReportsClient.getCanSeeReports(responseHandler);
			
			dojo.subscribe("/jazz/reports/includeArchived", this,
					function(include) {
						if (!this._reportViewer){
							return;
						}
						// Check if the report viewer is still displaying
						if (!dojo.isDescendant(this._reportViewer.domNode, document.body)) {
							return;
						}
						if (this.controller){
							this.controller.reload();
						}
					});
			if (this.showBreadcrumbs) this.displayBreadcrumbs = true;
		},
		
		_handleShowArchived: function(response) {
			this.controller.setIncludeArchived(response == "true");
		},
		
		_handleShowArchivedError: function() {
		},
		
		_storeHasWorkItems: function(response) {
			response = response[0];
			this.hasWorkItems = response.code == "OK" ? true : false;
		},
		
		_handleHasWorkItemsError: function() {
			this.hasWorkItems = false;
		},
		
		_storeCustomReportingConfigured: function(response) {
			var proxyUrl, url;
			for(var i = 0; i < response.properties.length; i++){
				if (response.properties[i].name == "cognos.hostAndPort"){
					if (response.properties[i].overrideValue && response.properties[i].overrideValue.length > 0){
						url = response.properties[i].overrideValue;
					}
				} else if (response.properties[i].name == "cognos.proxy.hostAndPort") {
					if (response.properties[i].overrideValue && response.properties[i].overrideValue.length > 0){
						proxyUrl = response.properties[i].overrideValue;
					}
				}
			}
			
			if (url) {
				this.customReportingConfigured = true;
				if (proxyUrl) {
					this.customReportingUrl = proxyUrl;
				} else {
					this.customReportingUrl = url;
				}
			} else {
				this.customReportingConfigured = false;
			}
		},
		
		_handleCustomReportingConfiguredError: function() {
			this.customReportingConfigured = false;
		},
		
		/**
		 * Framework API for hyperlinks and back/forward.
		 */
		getController: function() {
			return this.controller;
		},
		
		_handleCanSeeReports: function(result) {
			if (result == "true") {
				this._canSeeReports = true;
			}
			this._numberOfPendingResponses--;
			if (this._numberOfPendingResponses > 0) {
				return;
			}
			this._initializeNavigation();		
		},
		
		_initializeNavigation: function() {
			if (!this._canSeeReports) {
				return;
			}
			
			if (this._showWelcome) {
				this.showWelcome();
			}
		},
		
		// show methods to be called from ReportsController
		
		showQuery: function(args) {
			if (this._reportViewer) {
				this._reportViewer.destroy();
			}
			if (args.name) {
				PlatformUI.getWorkbench().setTitle(this.messages.reportColon + args.name);
			} else {
				PlatformUI.getWorkbench().setTitle(this.messages.report);
			}
			this._reportViewer = new com.ibm.team.reports.web.ui.internal.ReportViewer({queryUUID: args.queryUUID, controller: this.controller, reportTitle: args.name, shared: args.shared, displayBreadcrumbs: this.displayBreadcrumbs}, null);
			this._setMainContent(this._reportViewer.domNode);
			RecentlyViewed.pageVisited(this._reportViewer);
			this._reportViewer.startup();

		}, 
		
		showReport: function(args) {
			if (this._reportViewer) {
				this._reportViewer.destroy();
			}
			var projectAreaUUID = "";
			if (args.projectAreaUUID) {
				projectAreaUUID = args.projectAreaUUID;
			}
			var queryUUID = "";
			if (args.queryUUID) {
				queryUUID = args.queryUUID;
			}
			PlatformUI.getWorkbench().setTitle(this.messages.report);
			this._reportViewer = new com.ibm.team.reports.web.ui.internal.ReportViewer
				({projectAreaUUID: projectAreaUUID, 
				  reportUUID: args.reportUUID, 
				  queryUUID: queryUUID, 
				  parameterName: args.parameterName, 
				  parameterValue: args.parameterValue, 
				  controller: this.controller});
			this._setMainContent(this._reportViewer.domNode);
			RecentlyViewed.pageVisited(this._reportViewer);
			this._reportViewer.startup();
		}, 
		
		showWelcome: function() {
			if (this._numberOfPendingResponses > 0) {
				this._showWelcome = true;
				return;
			}
			if (!this._canSeeReports) {
				return;
			}
			this._showWelcome = false;
			var welcomePageWidget = new com.ibm.team.reports.web.ui.internal.ReportsWelcome({rrdiConfigured: this.customReportingConfigured});
			PlatformUI.getWorkbench().setTitle(this.messages.welcomeToReports);
			this._setMainContent(welcomePageWidget.domNode);
		},
		
		showAdditionalTasks: function() {
			var additionalTasksPage = new com.ibm.team.reports.web.ui.internal.ReportsAdditionalTasks({rrdiConfigured: this.customReportingConfigured});
			PlatformUI.getWorkbench().setTitle(this.messages.additionalReporting);
			this._setMainContent(additionalTasksPage.domNode);
		},
		
		reportStudio: function() {
			if (!this.customReportingConfigured) {
				return;
			}
			
			var baseUrl = this._processCustomReportingUrl(this.customReportingUrl);
			
			//ui.gateway is a required parameter when opening the report studio
			var gateway = baseUrl;
			window.open(baseUrl + '?b_action=xts.run&m=portal/launch.xts&ui.tool=ReportStudio&ui.gateway='+encodeURIComponent(gateway));
			
			//switch the action back to the default in case the user hits refresh
			document.location.href = "#action=com.ibm.team.reports.showAdditionalTasks";
		},
		
		queryStudio: function() {
			if (!this.customReportingConfigured) {
				return;
			}			
			var baseUrl = this._processCustomReportingUrl(this.customReportingUrl);
			window.open(baseUrl + '?b_action=xts.run&launch.openJSStudioInFrame=true&m=portal/launch.xts&ui.action=new&ui.tool=QueryStudio');
			
			//switch the action back to the default in case the user hits refresh
			document.location.href = "#action=com.ibm.team.reports.showAdditionalTasks";
		},
		
		newReport: function(args) {
			PlatformUI.getWorkbench().setTitle(this.messages.newReport);
			var newReportArgs = {controller: this.controller, displayBreadcrumbs: this.displayBreadcrumbs};
			if (args && args.reportUUID) {
				newReportArgs.reportUUID = args.reportUUID;
			}
			var newReportEditor = new com.ibm.team.reports.web.ui.internal.NewReportEditor(newReportArgs);
			this._setMainContent(newReportEditor.domNode);
			RecentlyViewed.pageVisited(newReportEditor);
		},
		
		admin: function(args) {
			var pages = AdminPageRegistry.getContributedPages();
			for (var i = 0; i < pages.length; i++) {
				var page = pages[i];
				if (page.id == args.id) {
					PlatformUI.getWorkbench().setTitle(page.name);
					if (dojo.exists(page.widgetId)) {
						this._showWidget(page.widgetId);
					} else {
						loader.load_async(page.widgetId, dojo.hitch(this, function() {
							this._showWidget(page.widgetId);
						}));
					}
					break;
				}
			}
		},
		
		_showWidget: function(widgetId) {
			var ctor = dojo.getObject(widgetId);
			var adminViewer = new ctor({controller: this.controller});
			this._setMainContent(adminViewer.domNode);
		},
		
		_setMainContent: function(domNode) {
			removeChildren(this._appContainer);
			this._appContainer.appendChild(domNode);
		},
		
		viewWorkItems: function(args) {
			if (this.hasWorkItems) {
				loader.load_async("com.ibm.team.reports.workitem.web.ui.internal.WorkItemsNavigator", 
					dojo.hitch(this, function() {
						eval("com.ibm.team.reports.workitem.web.ui.internal.WorkItemsNavigator").viewWorkItems(args);
					}));
			}
		},
		
		editTemplate: function(args) {
			var editTemplateArgs = {controller: this.controller, displayBreadcrumbs: this.displayBreadcrumbs};
			if (args && args.reportUUID) {
				editTemplateArgs.reportUUID = args.reportUUID;
			}
			var reportTemplateEditor = new com.ibm.team.reports.web.ui.internal.ReportTemplateEditor(editTemplateArgs);
			this._setMainContent(reportTemplateEditor.domNode);
			RecentlyViewed.pageVisited(reportTemplateEditor);
		},
		
		editCustomTemplate: function(args) {
			PlatformUI.getWorkbench().setTitle(this.messages.newCustomTemplate);
			var editTemplateArgs = {controller: this.controller, displayBreadcrumbs: this.displayBreadcrumbs};
			if (args && args.reportUUID) {
				editTemplateArgs.reportUUID = args.reportUUID;
			}
			var reportTemplateEditor = new com.ibm.team.reports.web.ui.internal.ReportCustomTemplateEditor(editTemplateArgs);
			this._setMainContent(reportTemplateEditor.domNode);
		},
		editFolder: function(args) {
			args.controller = this.controller;
			if (args.shared == "true") {
				args.shared = true;
			} else {
				args.shared = false;
			}
			var folderEditor = new com.ibm.team.reports.web.ui.internal.FolderEditor(args);
			this._setMainContent(folderEditor.domNode);
		},

		editReport: function(args) {
			args.controller = this.controller;
			var reportEditor = new com.ibm.team.reports.web.ui.internal.ReportEditor(args);
			this._setMainContent(reportEditor.domNode);
		},
		
		deployNewTemplates: function(args) {
			// Load the deployNewTemplates page
			var deployTemplatesViewer = new com.ibm.team.reports.web.ui.internal.DeployTemplatesViewer({processAreaUUID: args.processAreaUUID, controller: this.controller, displayBreadcrumbs: this.displayBreadcrumbs});
			PlatformUI.getWorkbench().setTitle(this.messages.deployNewTemplates);
			this._setMainContent(deployTemplatesViewer.domNode);
		},
		
		viewAllReports: function(args) {
			var allReports = new com.ibm.team.reports.web.ui.internal.AllReportsTab({reportsView: this, shared: args.shared, controller: this.controller, displayBreadcrumbs: this.displayBreadcrumbs});
			var pageTitle = null;
			
			if (args.shared === "true") {
				pageTitle = this.messages.sharedReports;
			} else {
				pageTitle = this.messages.myReports;
			}
			PlatformUI.getWorkbench().setTitle(pageTitle);
			this._setMainContent(allReports.domNode);
		},
		
		viewRunningReports: function(args) {
			var runningReports = new com.ibm.team.reports.web.ui.internal.ReportJobStatusPage({isAdminView: false, showToolbar:true});
			PlatformUI.getWorkbench().setTitle(this.messages.runningReports);
			this._setMainContent(runningReports.domNode);
		},
		
		viewAllReportTemplates: function(args) {
			var allReportTemplates = new com.ibm.team.reports.web.ui.internal.AllReportTemplatesTab({reportsView: this, shared: args.shared, displayBreadcrumbs: this.displayBreadcrumbs});
			PlatformUI.getWorkbench().setTitle(this.messages.reportTemplates);
			this._setMainContent(allReportTemplates.domNode);
		},
		
		authorCustomReport: function(args){
			if (args != null && args.customReportingUrl != null) {
				this.customReportingConfigured = true;
				this.customReportingUrl = args.customReportingUrl;
			}
			window.open(this.customReportingUrl);
			//switch the action back to the default in case the user hits refresh
			document.location.href = "#action=com.ibm.team.reports.showAdditionalTasks";
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._appContainer, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			alert(error.message);
		},
		
		/**
		 * Process the custom reporting URL to return the base URL of the reporting tool. 
		 * 
		 * Note that calling this method does not change the value of the customReportingUrl variable.
		 */
		_processCustomReportingUrl: function(url) {
			//strip off the trailing slash, if present
			if (url.lastIndexOf("/") === url.length -1) {
				url = url.substr(0, url.length -1);
			}
			
			//for backwards compatibility with pre-4.0.5 - add "servlet/dispatch" to the URL if it contains
			//only one path segment (e.g., /reporting)
			//javascript/dojo doesn't have good built-in URL parsing; count "/" in the URL instead
			var index = url.lastIndexOf("/");
			var count = 0;
			while (index != -1) {
				count++;
				index = url.lastIndexOf("/", index-1);
			}						

			if (count === 3) {
				url = url + "/servlet/dispatch";
			}

			return url;
		}
	}
);

function removeAction(actionId){
	var foundNode = null;
	dojo.query('a.section-entry').forEach(function(node, index, arr){
		if (node.__jazzActionId == actionId)
		{
			foundNode = node;
		}		
	});
	if (foundNode) {
  		foundNode.parentNode.removeChild(foundNode);
 	}
}

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

function removeAction(actionId){
	var foundNode = null;
	dojo.query('a.section-entry').forEach(function(node, index, arr){
		if (node.__jazzActionId == actionId)
		{
			foundNode = node;
		}		
	});
	if (foundNode) {
  		foundNode.parentNode.removeChild(foundNode);
 	}
}

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

})();