/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ResourceDialog");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.Tree");
dojo.require("dojox.data.dom");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.io.iframe");

dojo.require("jazz.app.context");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {

var FILE_UPLOAD_RESPONSE = "File Upload Response:";
var FILE_UPLOAD_ERROR = "File Upload Error:";
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var RESOURCE_SERVICE_URI = TeamServerClient._getServicePrefix() + "com.ibm.team.reports.service.internal.IReportResourceService";
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;

var _currentBindRequest;
var _currentUploadFrameNumber = 100;

dojo.declare("com.ibm.team.reports.web.ui.internal.ResourceDialog", [dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ResourceDialog.html"),
	widgetsInTemplate: true,
	
	constructor: function(args) {
		this.onOk = args.onOk;
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
	},
	
	postCreate: function() {
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			modal: true,
			primaryTitle: this.messages.resource,
			width: "375px",
			isClosable: true
		});
		this.connect(this._okButton, "onclick", this._onOk);
		this.connect(this._cancelButton, "onclick", this._onCancel);
		this.connect(this.fileLocation, "onchange", "_handleResourceChange");
		this.messageBox = new jazz.ui.MessageBox();
		this._messageBoxContainer.appendChild(this.messageBox.domNode);
		this._cancelButton.focus();
	},
	
	close: function() {
		this.dialog.close();
	},
	
	_onOk: function() {
		var self = this;
		if (this._currentBindRequest != null) {
			this._currentBindRequest.cancel();
			this._currentBindRequest  = null;
		}
		
		this._currentBindRequest = dojo.io.iframe.send({
			form: this._uploadForm,
			url: RESOURCE_SERVICE_URI,
			content: {
					mimetype: "text/html",
					fileFields: "fileLocation"
				},
			handleAs: "html",
			handle: function(response, ioArgs) {
				if (response instanceof Error){
					if (response.data-dojo-type && response.data-dojo-type == "cancel") {
						return; // this is a cancelled request, ignore it
					}
					self._currentBindRequest = null; // no need to cancel it any more
					self._showErrorMessage(self.messages.invalidFileName);
					console.debug(dojo.string.substitute(self.messages.errorUploadingFile, [ response.message ]));
				} else {
					var responseMsg = response.body.innerHTML;
					if (responseMsg.indexOf(FILE_UPLOAD_RESPONSE) == 0) {
						var data = responseMsg.substring(FILE_UPLOAD_RESPONSE.length);
						var dataArray = data.split(",");
						// Make sure we have a success
						if (dataArray.length == 4) {
							var resourceLocation = dataArray[0];
							var resourceSize = dataArray[1];
							var resourceContentType = dataArray[2];
							var resourceChecksum = dataArray[3];
							self.fileLocation.savedLocation = resourceLocation;
							self.fileLocation.savedFileSize = resourceSize;
							self.fileLocation.savedContentType = resourceContentType;
							self.fileLocation.savedChecksum = resourceChecksum;
							var split = self.fileLocation.value.split('.');
							self.fileLocation.savedFileExtension = split[split.length - 1];
							//self.removePhotoFlag = false;
							self.dialog.close();
							if (self.onOk) {
								self.onOk();
							}
						} else {
							// Means failure.  Set the location to null so that it will not be saved
							self.fileLocation.savedLocation = null;
							self._showErrorMessage(dojo.string.substitute(self.messages.errorUploadingFile, [ responseMsg ]));
							//dojo.addClass(self.photoErr, "inlineError");
							//self.photoErr.innerHTML = data;
							//self.displayErrorMessage(data);
						}
					}
				}
			}		  
		});
		
	},
	
	getFileLocation: function() {
		return this.fileLocation;
	},
	
	_onCancel: function() {
		this.dialog.close();
	},
	
	_resourceUploaded: function() {
		this.dialog.close();
	},
	
	_remoteError: function() {
		this.dialog.close();
	},

	_handleResourceChange: function () {
		this._updateEnablement();
	},

	_updateEnablement: function() {
		if (this.fileLocation.value != null && this.fileLocation.value != "") {
			this._okButton.disabled = false;
		} else {
			this._okButton.disabled = true;
		}
	},

	_handleError: function(e) {
		alert(e.message);
	},
	
	_showErrorMessage: function(message) {
		var status = {
			type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary : message,
			closable : false
		};
		dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
		this.messageBox.showMessage(status);
	}
});

})();