/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection"); //$NON-NLS-1$

	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit.MenuItem"); //$NON-NLS-1$
	
	dojo.require("jazz.app.ua.TextHoverHelp"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$

	dojo.require("dojo.i18n"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	var sectionUUID = 0;
	
	var TEXT_TERM_CLASS = "com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection._TextTerm"; //$NON-NLS-1$
	var COUNTER_TERM_CLASS = "com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection._CounterTerm"; //$NON-NLS-1$

	/**
	 * Widget that allows the custom naming convention of a group of iterations
	 * to be configured.
	 * 
	 * Callbacks namingConventionChanged() revertToDefaultNamingConvention()
	 * that are passed to the constructor are invoked to notify consumers
	 * that the convention has changed and that the user wishes to revert to
	 * the default convention respectively.
	 * 
	 * Consumers invoke functions getNamingConventionSummary() and
	 * generateName() to make use of the custom naming convention.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection", dijit._Widget, {  //$NON-NLS-1$
		
		/**
		 * @param {Object} args
		 * 
		 * Besides the arguments supported by dijit._Widget, the following
		 * properties are required:
		 * 
		 * iterationName	The name of an iteration, as entered into the
		 * 					row in the Iteration Structure table. Used
		 * 					to initialize the terms to match the default
		 * 					naming convention.
		 * 
		 * originalIterationName	The default name of an iteration, as it was
		 * 							initialized to when iteration was created.
		 * 							If a non-empty name cannot be generated for
		 * 							any reason, this string will be returned
		 * 							from generateName.
		 * 
		 * namingConventionChanged	A function that is to be invoked whenever
		 * 							the naming convention has been modified.
		 * 
		 * revertToDefaultNamingConvention	A function is to be invoked
		 * 									when the default naming convention
		 * 									is to be used instead of the one
		 * 									configured in this section.
		 * 
		 * createElement
		 * createMenuButton
		 * createToolbarButton
		 * createTextBox
		 * createNumberSpinner
		 * createCheckbox			Helper functions to create DOM elements and
		 * 							widgets. Allows the embedding widget to
		 * 							customize the controls within the naming
		 * 							convention UI, eg. to apply custom styles or
		 * 							behavior.
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._terms = [];
			
			this._termWidgetUUID = 0;
			
			this._sectionUUID = sectionUUID++;
		},
		
		postCreate: function() {
			this.inherited(arguments);
			
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-CreateIterationsDialogCustomNamingConventionSection"); //$NON-NLS-1$
		},
		
		startup: function() {
			this._termLabelWidth = (UIHelper.getMaxStringDisplayWidthInEms([
				this.messages.CreateIterationsDialogCustomNamingConventionSection_6,
				this.messages.CreateIterationsDialogCustomNamingConventionSection_7
			], this.domNode) + .5) + "em"; //$NON-NLS-1$
			
			//Create header: a heading label and menu button to add terms
			
			var headerDiv = this.createElement("div", {'class': "headerDiv j-h2"}, this.domNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			headerDiv.appendChild(this.createElement("img", { //$NON-NLS-1$
				'class': "headerTitleIcon", //$NON-NLS-1$ //$NON-NLS-2$
				'src': dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/naming_convention_obj.gif") //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}, headerDiv));
			headerDiv.appendChild(document.createTextNode(this.messages.CreateIterationsDialogCustomNamingConventionSection_1));
			
			var menuButtonDiv = this.createElement("div", {'class': "addMenu"}, headerDiv); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._addMenu = this.createMenuButton({
				label: this.messages.CreateIterationsDialogCustomNamingConventionSection_2,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/add.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				menuProvider: dojo.hitch(this, this._addTermMenuProvider),				
				align: this.isLeftToRight() ? "right" : "left", //$NON-NLS-1$ //$NON-NLS-2$
				destroyOnClose: true
			});
			menuButtonDiv.appendChild(this._addMenu.domNode);
			
			this.createElement("div", {'class': "clear"}, this.domNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			//Create table for terms, and the initial term rows, a text term
			//with the name plus a space and a counter term 
			
			this._termsTable = this.createElement("table", {'class': "termsTable"}, this.domNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			this._addTerm(TEXT_TERM_CLASS, { //$NON-NLS-1$
				value: this.iterationName
			});
			
			this._addTerm(COUNTER_TERM_CLASS, {}); //$NON-NLS-1$
			
			//Select all text in the text term's text box
			dijit.selectInputText(this._terms[0]._textBox.textbox);
		},
		
		_internalNamingConventionChanged: function() {
			if (this._terms.length === 0) {
				this.revertToDefaultNamingConvention();
			} else {
				this.namingConventionChanged();
			}
		},
		
		_addTermMenuProvider: function() {
			var menu = new jazz.ui.Menu();
			dojo.addClass(menu.domNode, "com-ibm-team-process-web-ui-internal-OperationBehaviorEditor-addConfigurationDropDown"); //$NON-NLS-1$
			
			//Add text term
			menu.addChild(new dijit.MenuItem({
				label: this.messages.CreateIterationsDialogCustomNamingConventionSection_3,
				onClick: dojo.hitch(this, this._addTerm, TEXT_TERM_CLASS, {value: ""}, true) //$NON-NLS-1$
			}));
			
			//Add counter term
			menu.addChild(new dijit.MenuItem({
				label: this.messages.CreateIterationsDialogCustomNamingConventionSection_4,
				onClick: dojo.hitch(this, this._addTerm, COUNTER_TERM_CLASS, {}, true) //$NON-NLS-1$
			}));
			
			menu.startup();
			
			return menu;
		},
		
		_addTerm: function(termClass, termArgs, notifyNamingConventionChanged) {
			var termId = this._termWidgetUUID++;
			
			var termConstructor = dojo.getObject(termClass);
			termArgs.namingConventionChanged = dojo.hitch(this, this._internalNamingConventionChanged);
			termArgs.sectionUUID = this._sectionUUID;
			termArgs.namingConventionSection = this;
			termArgs.labelWidth = this._termLabelWidth;
			termArgs.onRemove = dojo.hitch(this, function(theId) { //$NON-NLS-1$
				for (var i = 0; i < this._terms.length; i++) {
					if (this._terms[i]._id === theId) {
						this._terms.splice(i, 1);
						dojo.destroy(this._termsTable.childNodes[i]);
						break;
					}
				}
				this._internalNamingConventionChanged();
			}, termId);
			var termWidget = new termConstructor(termArgs);
			termWidget._id = termId;
			
			this._terms.push(termWidget); //$NON-NLS-1$
			
			termWidget.createControls(this.createElement("tr", {}, this._termsTable)); //$NON-NLS-1$
			termWidget.getFocusElement().focus();
			
			if (true === notifyNamingConventionChanged) {
				this._internalNamingConventionChanged();
			}
		},
		
		/**
		 * Returns a human-readable summary of the naming convention, as a
		 * String.
		 */
		getNamingConventionSummary: function() {
			var summary = ""; //$NON-NLS-1$
			dojo.forEach(this._terms, function(termWidget) {
				summary += termWidget.getSummaryText();
			});
			
			//See comment for ctor parameter originalIterationName 
			if (summary.length === 0) {
				summary = this.originalIterationName;
			}
			
			return summary;
		},		
				
		/**
		 * Given a number of iterations, which "batch" of iterations
		 * is being processed and the index of the iteration being processed,
		 * returns the calculated name of the iteration.
		 * 
		 * Parameter <batchNum> is only consulted when the naming convention
		 * includes counters that are continuous. For the following
		 * structure, generateName would be invoked six times, (once  for each
		 * child) with the arguments noted:
		 * 
		 * Parent A
		 *     Child 1 --> generateName(0, 3, 0)
		 *     Child 2 --> generateName(1, 3, 0)
		 *     Child 3 --> generateName(2, 3, 0)
		 * Parent B
		 *     Child 1 --> generateName(0, 3, 1)
		 *     Child 2 --> generateName(1, 3, 1)
		 *     Child 3 --> generateName(2, 3, 1)
		 * 
		 * If the naming convention was a text term "child " followed by
		 * a counter that starts at 1 followed by a continuous counter that
		 * starts at 0, the results would be:
		 * 
		 * Parent A
		 *     child 1 0
		 *     child 2 1
		 *     child 3 2
		 * Parent B
		 *     child 1 3
		 *     child 2 4
		 *     child 3 5
		 * 
		 * @param {integer} itemIndex [0, *)
		 * @param {integer} numIterations [1, *)
		 * @param {integer} batchNum  [0, *)
		 */
		generateName: function(itemIndex, numIterations, batchNum) {
			var name = ""; //$NON-NLS-1$
			dojo.forEach(this._terms, function(termWidget) {
				name += termWidget.generateName(itemIndex, numIterations, batchNum);
			});
			
			//Instead of returning a zero-length name that would be problematic
			//for the preview to render, return the original iteration name in
			//the rare case where the terms add up to an empty string
			if (name.length === 0) {
				name = this.originalIterationName;
			}
			
			return name;
		},		
		
		uninitialize: function() {
			this.inherited(arguments);
			
			dojo.forEach(this._terms, function(term) {
				term.cleanUpControls();
			});
			delete this._terms;
			
			this._addMenu.destroyRecursive();
			delete this._addMenu;
		}
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection._AbstractTerm", [], { //$NON-NLS-1$
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._namingConventionSection = args.namingConventionSection;
			this._namingConventionChanged = args.namingConventionChanged;
			this._sectionUUID = args.sectionUUID;
			this._labelWidth = args.labelWidth;
			this._onRemove = args.onRemove;
		},
		
		createControls: function(row) {
			//Create a cell for the label and 'Remove' button, and let sub-type
			//fill in the middle cell.
			
			var td = this._namingConventionSection.createElement("td", {'class': "termLabelCell j-label"}, row); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			td.appendChild(document.createTextNode(this._termLabel));
			dojo.style(td, "width", this._labelWidth); //$NON-NLS-1$
			
			this._primCreateControls(this._namingConventionSection.createElement("td", {}, row)); //$NON-NLS-1$
			
			td = this._namingConventionSection.createElement("td", {'class': "removeButtonCell"}, row); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._removeButton = this._namingConventionSection.createToolbarButton({
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/remove.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title: this.messages.CreateIterationsDialogCustomNamingConventionSection_5,
				onClick: dojo.hitch(this, function(event) {
					dojo.stopEvent(event);
					this.cleanUpControls();
					this._onRemove();
				})
			});
			dojo.addClass(this._removeButton.domNode, "removeButton"); //$NON-NLS-1$
			td.appendChild(this._removeButton.domNode);
		},
		
		cleanUpControls: function() {
			this._removeButton.destroyRecursive();
			delete this._removeButton;
		}
	});
	
	dojo.declare(TEXT_TERM_CLASS, [com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection._AbstractTerm], { //$NON-NLS-1$
		constructor: function(args) {
			this._termLabel = this.messages.CreateIterationsDialogCustomNamingConventionSection_6;
			this._value = args.value || ""; //$NON-NLS-1$
		},
		
		_primCreateControls: function(td) {
			this._textBox = this._namingConventionSection.createTextBox({
				value: this._value
			}, dojo.hitch(this, function(value) { //$NON-NLS-1$
				this._value = value;
				this._namingConventionChanged();
			}));
			dojo.addClass(this._textBox.domNode, "textTermTextBox"); //$NON-NLS-1$
			td.appendChild(this._textBox.domNode);
		},
		
		getSummaryText: function() {
			return this.generateName();
		},
		
		generateName: function() {
			return this._value;
		},
		
		getFocusElement: function() {
			return this._textBox.focusNode;
		},
		
		cleanUpControls: function() {
			this._textBox.destroyRecursive();
			delete this._textBox;
			
			this.inherited(arguments);
		}
	});
	
	dojo.declare(COUNTER_TERM_CLASS, [com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection._AbstractTerm], { //$NON-NLS-1$
		constructor: function() {
			this._termLabel = this.messages.CreateIterationsDialogCustomNamingConventionSection_7;
			
			this._fromValue = 1;
			this._byValue = 1;
			this._isContinuous = false;
		},
		
		_primCreateControls: function(td) {
			var fromSpan = this._namingConventionSection.createElement("span", {'class': "counterTermElement"}, td); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._namingConventionSection.createElement("span", {'class': "j-label counterTermLabel"}, fromSpan).appendChild(document.createTextNode(this.messages.CreateIterationsDialogCustomNamingConventionSection_8)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._fromSpinner = this._namingConventionSection.createNumberSpinner({
				value: this._fromValue,
				intermediateChanges: true,
				required: true
			}, dojo.hitch(this, function(value) {
				if (!isNaN(value)) {
					this._fromValue = value;
					this._namingConventionChanged();
				}
			}));
			dojo.addClass(this._fromSpinner.domNode, "counterTermNumberSpinner"); //$NON-NLS-1$
			fromSpan.appendChild(this._fromSpinner.domNode);
			
			var bySpan = this._namingConventionSection.createElement("span", {'class': "counterTermElement"}, td); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._namingConventionSection.createElement("span", {'class': "j-label counterTermLabel"}, bySpan).appendChild(document.createTextNode(this.messages.CreateIterationsDialogCustomNamingConventionSection_9)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._bySpinner = this._namingConventionSection.createNumberSpinner({
				value: this._byValue,
				intermediateChanges: true,
				required: true
			}, dojo.hitch(this, function(value) {
				if (!isNaN(value)) {
					this._byValue = value;
					this._namingConventionChanged();
				}
			}));
			dojo.addClass(this._bySpinner.domNode, "counterTermNumberSpinner"); //$NON-NLS-1$
			bySpan.appendChild(this._bySpinner.domNode);
			
			var isContinuousCheckboxId = "CreateIterationsDialogCustomNamingConventionSection._CounterTerm" + sectionUUID + "." + this._id; //$NON-NLS-1$ //$NON-NLS-2$
			var isContinuousCheckbox = this._namingConventionSection.createCheckbox(
				isContinuousCheckboxId,
				this.messages.CreateIterationsDialogCustomNamingConventionSection_10,
				this._isContinuous,
				dojo.hitch(this, function(isChecked) {
					this._isContinuous = isChecked;
					this._namingConventionChanged();
				})
			);
			var isContinuousCheckboxSpan = this._namingConventionSection.createElement("span", {}, td); //$NON-NLS-1$
			isContinuousCheckboxSpan.appendChild(isContinuousCheckbox);
			
			this._continuousUA = new jazz.app.ua.TextHoverHelp({
				contentNode: isContinuousCheckbox,
				helpContentString: this.messages.CreateIterationsDialogCustomNamingConventionSection_11
			});
		},
		
		getSummaryText: function() {
			return this.messages.CreateIterationsDialogCustomNamingConventionSection_numericTerm;
		},
		
		generateName: function(itemIndex, numIterations, batchNum) {
			var countWithinBatch = this._fromValue + itemIndex * this._byValue; 
			if (this._isContinuous) {
				return (batchNum * numIterations * this._byValue) + countWithinBatch;
			} else {
				return countWithinBatch;
			}
		},
		
		getFocusElement: function() {
			return this._fromSpinner.focusNode;
		},
		
		cleanUpControls: function() {
			this._fromSpinner.destroyRecursive();
			delete this._fromSpinner;
			
			this._bySpinner.destroyRecursive();
			delete this._bySpinner;
			
			this._continuousUA.destroyRecursive();
			delete this._continuousUA;
			
			this.inherited(arguments);
		}
	});
})();
