/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.HistoryTable"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	
	dojo.require("dijit._Templated"); //$NON-NLS-1$	
	dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
	dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("dojo.fx"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	
	dojo.require("jazz.app.i18n"); //$NON-NLS-1$
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	
	dojo.require("net.jazz.ajax.contextRoot"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	var PROJECT_AREAS_SERVICE = "/process/project-areas/"; //$NON-NLS-1$
	var TEAM_AREAS_SERVICE = "/team-areas/"; //$NON-NLS-1$
	var HISTORY_SERVICE = "/history"; //$NON-NLS-1$
	var PAGE_SIZE_PARAMETER = "?pageSize="; //$NON-NLS-1$
	var PAGE_SIZE = 25;
	var START_TIME_PARAMETER = "&startTime="; //$NON-NLS-1$
	var EXPLAIN_EMPTY_PARAMETER = "explainEmptyHistory=true"; //$NON-NLS-1$
	var EXPLAIN_ALL_PARAMETER = "explainAllHistory=true"; //$NON-NLS-1$
	
	// XML parsing constants
	var HISTORY = "history"; //$NON-NLS-1$
	var PREVIOUS_PAGE = "previous-page-url"; //$NON-NLS-1$
	var NEXT_PAGE = "next-page-url"; //$NON-NLS-1$
	var HISTORY_ENTRY = "history-entry"; //$NON-NLS-1$
	var HISTORY_DETAIL = "history-detail"; //$NON-NLS-1$
	var DATE = "date"; //$NON-NLS-1$
	var USERNAME = "user-name"; //$NON-NLS-1$
	var DETAIL_LABEL = "label"; //$NON-NLS-1$
	var DETAIL_URL = "url"; //$NON-NLS-1$
	var DETAIL_DEFERRED = "deferred";  //$NON-NLS-1$
	var UPDATED = "update"; //$NON-NLS-1$
	var DIFFERENCE = "difference"; //$NON-NLS-1$
	var ADDED = "add"; //$NON-NLS-1$
	var REMOVED = "remove"; //$NON-NLS-1$
	var CUSTOM_CHANGE = "custom-change"; //$NON-NLS-1$
	var SCOPE = "scope"; //$NON-NLS-1$
	var OLD_VALUE = "old-value"; //$NON-NLS-1$
	var NEW_VALUE = "new-value"; //$NON-NLS-1$
	var FORMAT = "format"; //$NON-NLS-1$
	var BOOLEAN_FORMAT = "boolean"; //$NON-NLS-1$
	var DATE_FORMAT = "date"; //$NON-NLS-1$
	var DATE_ONLY_FORMAT = "date-only"; //$NON-NLS-1$
	var CUSTOM_TITLE = "title"; //$NON-NLS-1$
	var CUSTOM_LIST_ITEM = "list-item"; //$NON-NLS-1$
	var CUSTOM_LIST_ITEM_TEXT = "text"; //$NON-NLS-1$
	var CUSTOM_LIST_ITEM_DETAILS = "details"; //$NON-NLS-1$
	var HISTORY_ERROR = "error"; //$NON-NLS-1$
	var HISTORY_ERROR_MESSAGE = "message"; //$NON-NLS-1$
	
	//determines the number of pages to keep in cache (includes current page)
	var CACHE_SIZE = 11;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.HistoryTable", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/HistoryTable.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._deferredDetails = [];
			this._pageCache = {};
		},
		
		postCreate: function() {
			this.inherited(arguments);
			this._topPager = new com.ibm.team.process.web.ui.internal.admin.projects._DatePager({
				table: this
			}).placeAt(this._topPagerArea);
			this._bottomPager = new com.ibm.team.process.web.ui.internal.admin.projects._DatePager({
				table: this
			}).placeAt(this._bottomPagerArea);
			this.refresh();
		},
		
		refresh: function() {
			if (this.editor.isTheActiveTab(ProcessConstants.TabEnum.HISTORY)) {
				this._clearState();
				this._showPage(0);
			} else {
				// wait to refresh until this tab is opened again
				this._refreshPending = true;
			}
		},
		
		activateTab: function() {
			if (this._refreshPending) {
				// editor refreshed since the last time this tab was open
				delete this._refreshPending;
				this.refresh();
			}
		},
		
		_showPage: function(pageNum) {
			var oldPageNum = this._pageNum;
			this._pageNum = pageNum;
			
			//delete cached pages
			if (pageNum - CACHE_SIZE > -1 && this._pageCache[pageNum - CACHE_SIZE] !== undefined) {
				delete this._pageCache[pageNum - CACHE_SIZE];
			}
			if (this._pageCache[pageNum + CACHE_SIZE] !== undefined) {
				delete this._pageCache[pageNum + CACHE_SIZE];
			}
			
			if (!this._pageCache[pageNum]) {
				// ask the server for this page
				this.editor.startLoading(this.messages.HistoryTable_3);
				var url = null;
				if (pageNum === 0) {
					url = this._getUrl();
				} else if (pageNum === oldPageNum - 1) {
					url = this._previousPage;
				} else if (pageNum === oldPageNum + 1) {
					url = this._nextPage;
				}
				this._xhrGet(url, dojo.hitch(this, this._populateTable));
			} else {
				// just show the cached result
				var cache = this._pageCache[pageNum];
				this._previousPage = cache.previousPage;
				this._nextPage = cache.nextPage;
				this._updatePagers(cache.startDate, cache.endDate);
				dojox.xml.parser.removeChildren(this._tbody);
				dojo.forEach(cache.rows, dojo.hitch(this, function(row) {
					this._tbody.appendChild(row);
				}));
				this._showWarning(cache.error);
			}
		},
		
		/**
		 * Constructs the appropriate URL to initiate a new history query.
		 */
		_getUrl: function() {
			var url = net.jazz.ajax.contextRoot() + PROJECT_AREAS_SERVICE + this.editor.projectAreaItemId;
			if (this.editor.context === ProcessConstants.EditorContext.TEAM) {
				url += TEAM_AREAS_SERVICE + this.editor.itemId;
			}
			url += HISTORY_SERVICE + PAGE_SIZE_PARAMETER + PAGE_SIZE;
			url += START_TIME_PARAMETER + this.editor.accessTime;
			if (window.location.hash.indexOf(EXPLAIN_ALL_PARAMETER) >= 0) {
				url += "&" + EXPLAIN_ALL_PARAMETER; //$NON-NLS-1$
			} else if (window.location.hash.indexOf(EXPLAIN_EMPTY_PARAMETER) >= 0) {
				url += "&" + EXPLAIN_EMPTY_PARAMETER; //$NON-NLS-1$
			}
			return url;
		},
		
		/**
		 * Fills in the table according to the given response from the server.
		 * 
		 * @param response {XMLDocument} response from the /history service 
		 */
		_populateTable: function(response) {
			UIHelper.runSafe(dojo.hitch(this, function() {
				dojox.xml.parser.removeChildren(this._tbody);
				var history = UIHelper.getNode(response.childNodes, HISTORY, ProcessConstants.NAMESPACE_JP);
				// get info needed for pager
				this._previousPage = UIHelper.getAttributeNodeValue(history, PREVIOUS_PAGE, ProcessConstants.NAMESPACE_JP);
				this._nextPage = UIHelper.getAttributeNodeValue(history, NEXT_PAGE, ProcessConstants.NAMESPACE_JP);
				var events = UIHelper.getNodes(history.childNodes, HISTORY_ENTRY, ProcessConstants.NAMESPACE_JP);
				var startDate = this._format(UIHelper.getAttributeNodeValue(events[0], DATE, ProcessConstants.NAMESPACE_JP), DATE_ONLY_FORMAT);
				var endDate = this._format(UIHelper.getAttributeNodeValue(events[events.length-1], DATE, ProcessConstants.NAMESPACE_JP), DATE_ONLY_FORMAT);
				this._updatePagers(startDate, endDate);
				// initialize a cache for this page
				var cache = this._pageCache[this._pageNum] = {};
				cache.previousPage = this._previousPage;
				cache.nextPage = this._nextPage;
				cache.startDate = startDate;
				cache.endDate = endDate;
				cache.rows = [];
				// create a row for and cache each change event returned
				dojo.forEach(events, dojo.hitch(this, function(event) {
					var user = UIHelper.getAttributeNodeValue(event, USERNAME, ProcessConstants.NAMESPACE_JP);
					var date = this._format(UIHelper.getAttributeNodeValue(event, DATE, ProcessConstants.NAMESPACE_JP), DATE_FORMAT);
					var details = UIHelper.getNodes(event.childNodes, HISTORY_DETAIL, ProcessConstants.NAMESPACE_JP);
					var row = this._addRow(user, date, details);
					cache.rows.push(row);
				}));
				// check for an error message
				var error = UIHelper.getNode(history.childNodes, HISTORY_ERROR, ProcessConstants.NAMESPACE_JP);
				this._showWarning(error);
				cache.error = error;
			}));
			this.editor.stopLoading();
		},
		
		_updatePagers: function(startDate, endDate) {
			this._topPager.update(startDate, endDate);
			this._bottomPager.update(startDate, endDate);
			dojo.style(this._bottomPagerArea, "display", this._pageNum === 0 && !this._nextPage ? "none" : "block"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		},
		
		_addRow: function(user, date, details) {
			var row = dojo.create("tr"); //$NON-NLS-1$
			var td = dojo.create("td", { //$NON-NLS-1$
				"class": "historyRow" //$NON-NLS-1$ //$NON-NLS-2$
			}, row); 
			dojo.create("div", { //$NON-NLS-1$
				innerHTML: dojo.string.substitute(this.messages.HistoryTable_5, [user, date]) + "<br><br>" //$NON-NLS-1$
			}, td);
			td.appendChild(this._getDetails(details));
			this._tbody.appendChild(row);
			return row;
		},
		
		/**
		 * Returns a node displaying what fields of this area were changed and
		 * how each was changed.
		 */
		_getDetails: function(details) {
			if (!details || details.length === 0) {
				return dojo.create("div", { //$NON-NLS-1$
					innerHTML: this.messages.HistoryTable_4,
					"class": "noDetails" //$NON-NLS-1$ //$NON-NLS-2$
				});
			}
			var table = dojo.create("table", { //$NON-NLS-1$
				"class": "detailTable" //$NON-NLS-1$ //$NON-NLS-2$
			});
			var tbody = dojo.create("tbody", {}, table); //$NON-NLS-1$
			// make a first pass to combine any details of the same type
			var detailCache = {};
			dojo.forEach(details, dojo.hitch(this, function(detail) {
				var changeType = UIHelper.getAttributeNodeValue(detail, DETAIL_LABEL, ProcessConstants.NAMESPACE_JP);
				if (!detailCache[changeType]) {
					detailCache[changeType] = detail;
				} else {
					// already processed a detail of this type, combine the
					// child nodes and/or detailUrl of this one with the first
					while (detail.childNodes.length > 0) {
						detailCache[changeType].appendChild(detail.childNodes[0]);
					}
					if (UIHelper.getAttributeNodeValue(detail, DETAIL_DEFERRED, ProcessConstants.NAMESPACE_JP) === "true") { //$NON-NLS-1$
						var detailUrl = UIHelper.getAttributeNodeValue(detail, DETAIL_URL, ProcessConstants.NAMESPACE_JP);
						if (detailUrl !== null) {
							var previous = UIHelper.getAttributeNodeValue(detailCache[changeType], DETAIL_URL, ProcessConstants.NAMESPACE_JP);
							var concatenated = (previous ? previous + "$#^#$" : "") + detailUrl; //$NON-NLS-1$ //$NON-NLS-2$
							UIHelper.setAttributeNodeValue(detailCache[changeType], DETAIL_URL, ProcessConstants.NAMESPACE_JP, concatenated);
						}
					}
				}
			}));
			// now generate a row with all details for each type of change
			for (var changeType in detailCache) {
				var detail = detailCache[changeType];
				var tr = dojo.create("tr", {}, tbody); //$NON-NLS-1$
				dojo.create("td", { //$NON-NLS-1$
					innerHTML: changeType,
					"class": "summaryColumn" //$NON-NLS-1$ //$NON-NLS-2$
				}, tr);
				var detailArea = dojo.create("td", {}, tr); //$NON-NLS-1$
				var detailUrl = UIHelper.getAttributeNodeValue(detail, DETAIL_URL, ProcessConstants.NAMESPACE_JP);
				var detailUrls = detailUrl ? detailUrl.split("$#^#$") : []; //$NON-NLS-1$
				if (detailUrls.length > 0 && UIHelper.getAttributeNodeValue(detail, DETAIL_DEFERRED, ProcessConstants.NAMESPACE_JP) === "true") { //$NON-NLS-1$
					var deferred = new com.ibm.team.process.web.ui.internal.admin.projects._DeferredDetail({
						callback: this._getDetailUrlsCallback(detailUrls, detail)
					}).placeAt(detailArea);
					this._deferredDetails.push(deferred);
				} else {
					detailArea.appendChild(this._getDetail(detail));
				}
			}
			return table;
		},
		
		/**
		 * Returns a function that will sequentially contact the given urls,
		 * combine their results into the given detail node, and then call its
		 * callback to display all results when all calls are complete.
		 */
		_getDetailUrlsCallback: function(detailUrls, detail) {
			return dojo.hitch(this, function(callback) {
				this._xhrGet(detailUrls.shift(), dojo.hitch(this, function(response) {
					var responseDetail = UIHelper.getNode(response.childNodes, HISTORY_DETAIL, ProcessConstants.NAMESPACE_JP);
					while (responseDetail.childNodes.length > 0) {
						detail.appendChild(responseDetail.childNodes[0]);
					}
					if (detailUrls.length > 0) {
						this._getDetailUrlsCallback(detailUrls, detail)(callback);
					} else {
						var detailNode = this._getDetail(detail);
						callback(detailNode, detailNode.className === "noDetails"); //$NON-NLS-1$
					}
				}));
			});
		},
		
		/**
		 * Constructs an HTML element to convey the information contained by the
		 * given <change-detail> node.
		 */
		_getDetail: function(detail) {
			if (detail && detail.childNodes && detail.childNodes.length > 0) {
				var div = dojo.create("div"); //$NON-NLS-1$
				var scopes = {};
				// parse <update> nodes
				var updated = this._getScopedAndUnscopedNodes(detail.childNodes, UPDATED);
				dojo.forEach(updated.unscoped, dojo.hitch(this, function(updated) {
					this._getUpdatedDetail(updated, dojo.create("div", {}, div)); //$NON-NLS-1$
				}));
				this._getScopedUpdatedDetails(updated.scoped, scopes);
				// parse <difference> nodes
				var differences = this._getScopedAndUnscopedNodes(detail.childNodes, DIFFERENCE);
				dojo.forEach(differences.unscoped, dojo.hitch(this, function(difference) {
					this._getDiffDetail(difference, dojo.create("div", {}, div)); //$NON-NLS-1$
				}));
				this._getScopedDiffDetails(differences.scoped, scopes);
				// parse <add> and <remove> nodes
				var added = this._getScopedAndUnscopedNodes(detail.childNodes, ADDED);
				var removed = this._getScopedAndUnscopedNodes(detail.childNodes, REMOVED);
				if (added.unscoped.length > 0 || removed.unscoped.length > 0) {
					this._getAddedRemovedDetail(added.unscoped, removed.unscoped, dojo.create("div", {}, div)); //$NON-NLS-1$
				}
				this._getScopedAddedRemovedDetails(added.scoped, removed.scoped, scopes);
				// parse <custom-change> nodes
				var customChanges = this._getScopedAndUnscopedNodes(detail.childNodes, CUSTOM_CHANGE);
				dojo.forEach(customChanges.unscoped, dojo.hitch(this, function(customChange) {
					this._getCustomChangeDetail(customChange, dojo.create("div", {}, div)); //$NON-NLS-1$
				}));
				this._getScopedCustomChangeDetails(customChanges.scoped, scopes);
				// show scoped changes now that they've been grouped together
				for (var scope in scopes) {
					dojo.create("div", { //$NON-NLS-1$
						"class": "scope", //$NON-NLS-1$ //$NON-NLS-2$
						innerHTML: this._escape(scope)
					}, div);
					div.appendChild(scopes[scope]);
				}
				return div;
			} else {
				return dojo.create("div", { //$NON-NLS-1$
					"class": "noDetails", //$NON-NLS-1$ //$NON-NLS-2$
					innerHTML: this.messages.HistoryTable_13
				});
			}
		},
		
		_getScopedAndUnscopedNodes: function(nodes, name) {
			var result = {
				scoped: [],
				unscoped: []
			};
			var matchingNodes = UIHelper.getNodes(nodes, name, ProcessConstants.NAMESPACE_JP) || [];
			dojo.forEach(matchingNodes, function(node) {
				var scoped = UIHelper.getAttributeNodeValue(node, SCOPE, ProcessConstants.NAMESPACE_JP) !== null; 
				result[scoped ? "scoped" : "unscoped"].push(node); //$NON-NLS-1$ //$NON-NLS-2$
			});
			return result;
		},
		
		//======================================================================
		// different detail formats
		
		_getUpdatedDetail: function(updated, parentNode) {
			var format = UIHelper.getAttributeNodeValue(updated, FORMAT, ProcessConstants.NAMESPACE_JP);
			var oldValue = UIHelper.getNode(updated.childNodes, OLD_VALUE, ProcessConstants.NAMESPACE_JP);
			if (oldValue) {
				oldValue = this._format(this._getTextContent(oldValue), format);
			}
			if (oldValue === null || oldValue === undefined) {
				oldValue = "&lt;" + this.messages.HistoryTable_7 + "&gt;"; //$NON-NLS-1$ //$NON-NLS-2$
			}
			var newValue = UIHelper.getNode(updated.childNodes, NEW_VALUE, ProcessConstants.NAMESPACE_JP);
			if (newValue) {
				newValue = this._format(this._getTextContent(newValue), format);
			}
			if (newValue === null || newValue === undefined) {
				newValue = "&lt;" + this.messages.HistoryTable_7 + "&gt;"; //$NON-NLS-1$ //$NON-NLS-2$
			}
			parentNode.innerHTML += oldValue + "&nbsp;&nbsp;&rarr;&nbsp;&nbsp;" + newValue + "<br>"; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_getDiffDetail: function(difference, parentNode, format) {
			var diff = this._format(this._getTextContent(difference, true));
			// restore <del> and <ins> elements
			diff = diff.replace(/&lt;del&gt;/g, "<del>").replace(/&lt;\/del&gt;/g, "</del>").replace(/&lt;ins&gt;/g, "<ins>").replace(/&lt;\/ins&gt;/g, "</ins>"); //$NON-NLS-1$ //$NON-NLS-2$  //$NON-NLS-3$ //$NON-NLS-4$
			dojo.create("div", { innerHTML: diff }, parentNode); //$NON-NLS-1$
		},
		
		_getAddedRemovedDetail: function(addeds, removeds, parentNode) {
			var text = ""; //$NON-NLS-1$
			var populateList = dojo.hitch(this, function(list, title) {
				if (list.length > 0) {
					text += title + "<br>"; //$NON-NLS-1$
					dojo.forEach(list, dojo.hitch(this, function(item, i) {
						var format = UIHelper.getAttributeNodeValue(item, FORMAT, ProcessConstants.NAMESPACE_JP);
						text += "&nbsp;&nbsp;&nbsp;&nbsp;" + this._format(this._getTextContent(item), format) + "<br>"; //$NON-NLS-1$ //$NON-NLS-2$
					}));
				}
			});
			populateList(addeds, this.messages.HistoryTable_8);
			populateList(removeds, this.messages.HistoryTable_9);
			parentNode.innerHTML += text;
		},
		
		_getCustomChangeDetail: function(customChange, parentNode) {
			var label = UIHelper.getAttributeNodeValue(customChange, CUSTOM_TITLE, ProcessConstants.NAMESPACE_JP);
			if (label) {
				parentNode.appendChild(document.createTextNode(dojo.string.substitute(this.messages.HistoryTable_16, [label])));
			}
			var div = dojo.create("div", { "class": label ? "indented" : "" }, parentNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			var oldValue = UIHelper.getNode(customChange.childNodes, OLD_VALUE, ProcessConstants.NAMESPACE_JP);
			var newValue = UIHelper.getNode(customChange.childNodes, NEW_VALUE, ProcessConstants.NAMESPACE_JP);
			if (oldValue || newValue) {
				this._getUpdatedDetail(customChange, div);
			}
			var difference = UIHelper.getNode(customChange.childNodes, DIFFERENCE, ProcessConstants.NAMESPACE_JP);
			if (difference) {
				this._getDiffDetail(difference, div);
			}
			var listItems = UIHelper.getNodes(customChange.childNodes, CUSTOM_LIST_ITEM, ProcessConstants.NAMESPACE_JP) || [];
			dojo.forEach(listItems, dojo.hitch(this, function(listItem) {
				this._getCustomListItemDetail(listItem, div);
			}));
		},
		
		_getCustomListItemDetail: function(listItem, parentNode) {
			var text = UIHelper.getNode(listItem.childNodes, CUSTOM_LIST_ITEM_TEXT, ProcessConstants.NAMESPACE_JP);
			if (text) {
				var format = UIHelper.getAttributeNodeValue(listItem, FORMAT, ProcessConstants.NAMESPACE_JP);
				this._getDiffDetail(text, parentNode, format);
			}
			var details = UIHelper.getNode(listItem.childNodes, CUSTOM_LIST_ITEM_DETAILS, ProcessConstants.NAMESPACE_JP);
			if (details) {
				var div = dojo.create("div"); //$NON-NLS-1$
				this._getDiffDetail(details, div);
				if (text) {
					dojo.toggleClass(parentNode.lastChild, "inline"); //$NON-NLS-1$
				}
				var linkHolder = dojo.create("div", { "class": "inline indented" }, parentNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				dojo.create("br", {}, parentNode); //$NON-NLS-1$
				var detailNode = dojo.create("div", { "class": "indented" }, parentNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				var deferred = new com.ibm.team.process.web.ui.internal.admin.projects._DeferredDetail({
					detailNode: detailNode,
					callback: function(callback) { callback(div); }
				}).placeAt(linkHolder);
				this._deferredDetails.push(deferred);
			}
		},
		
		_getScopedUpdatedDetails: function(updateds, scopes) {
			dojo.forEach(updateds, dojo.hitch(this, function(node) {
				var scope = UIHelper.getAttributeNodeValue(node, SCOPE, ProcessConstants.NAMESPACE_JP);
				this._initializeScope(scopes, scope);
				this._getUpdatedDetail(node, scopes[scope]);
			}));
		},
		
		_getScopedDiffDetails: function(differences, scopes) {
			dojo.forEach(differences, dojo.hitch(this, function(node) {
				var scope = UIHelper.getAttributeNodeValue(node, SCOPE, ProcessConstants.NAMESPACE_JP);
				this._initializeScope(scopes, scope);
				this._getDiffDetail(node, scopes[scope]);
			}));
		},
		
		_getScopedAddedRemovedDetails: function(addeds, removeds, scopes) {
			var args = {};
			var groupArgsByScope = function(list, variable) {
				dojo.forEach(list, function(node) {
					var scope = UIHelper.getAttributeNodeValue(node, SCOPE, ProcessConstants.NAMESPACE_JP);
					if (!args[scope]) {
						args[scope] = {
							added: [],
							removed: []
						};
					}
					args[scope][variable].push(node);
				});
			};
			groupArgsByScope(addeds, "added"); //$NON-NLS-1$
			groupArgsByScope(removeds, "removed"); //$NON-NLS-1$
			for (var scope in args) {
				this._initializeScope(scopes, scope);
				this._getAddedRemovedDetail(args[scope].added, args[scope].removed, scopes[scope]);
			}
		},
		
		_getScopedCustomChangeDetails: function(customChanges, scopes) {
			dojo.forEach(customChanges, dojo.hitch(this, function(node) {
				var scope = UIHelper.getAttributeNodeValue(node, SCOPE, ProcessConstants.NAMESPACE_JP);
				this._initializeScope(scopes, scope);
				this._getCustomChangeDetail(node, scopes[scope]);
			}));
		},
		
		_initializeScope: function(scopesObj, scope) {
			if (scopesObj[scope] === undefined) {
				var div = dojo.create("div", { "class": "indented" }); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				scopesObj[scope] = div;
			}
		},
		
		//======================================================================
		
		_format: function(text, format) {
			if (text !== null && text !== undefined) {
				if (format === BOOLEAN_FORMAT) {
					return text === "true" ? this.messages.HistoryTable_11 : this.messages.HistoryTable_12; //$NON-NLS-1$
				} else if (format === DATE_FORMAT || format === DATE_ONLY_FORMAT) {
					var dateObj = new jazz.app.i18n.Date(dojo.date.stamp.fromISOString(text));
					var date = jazz.app.i18n.date.locale.format(dateObj, { selector: "date", formatLength: "medium" }); //$NON-NLS-1$ //$NON-NLS-2$ 
					if (format === DATE_FORMAT) {
						date += " " + jazz.app.i18n.date.locale.format(dateObj, { selector: "time", formatLength: "short" }); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}
					return date;
				}
			}
			return text;
		},
		
		_getTextContent: function(node) {
			var textContent = ""; //$NON-NLS-1$
			if (node.textContent !== undefined && node.textContent !== null && dojo.string.trim(node.textContent).length > 0) {
				textContent = node.textContent;
			} else {
				// IE 8 doesn't define textContent
				dojo.forEach(node.childNodes, function(child) {
					// the text may be broken up into chunks
					textContent += child.nodeValue;
				});
			}
			if (dojo.string.trim(textContent).length === 0) {
				return null;
			} else {
				return this._escape(textContent);
			}
		},
		
		/**
		 * Custom escaping function thats does additional encoding on line
		 * breaks and tabs so that they will display properly.
		 * 
		 * @return {String} escaped html
		 */
		_escape: function(html) {
			// defect 255399: IE8 removes some line breaks when escaping,
			// instead of encoding them, so we have to do it ourselves
			var escapedHtml = html.replace(/\r\n/g, "*%*br*%*").replace(/\n/g, "*%*br*%*"); //$NON-NLS-1$ //$NON-NLS-2$
			escapedHtml = jazz.util.html.escape(escapedHtml);
			escapedHtml = escapedHtml.replace(/\*%\*br\*%\*/g, "<br>"); //$NON-NLS-1$
			// additional whitespace encoding
			escapedHtml = escapedHtml.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;"); //$NON-NLS-1$
			return escapedHtml;
		},
		
		/**
		 * Performs a GET on the given URL, expecting an XML response, and calls
		 * the given callback on success.
		 */
		_xhrGet: function(url, callback) {
			jazz.client.xhrGet({
				url: url,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: callback,
				error: dojo.hitch(this, this._handleError)
			});
		},
		
		_showWarning: function(error) {
			if (error) {
				this.editor._displayLongWarningMessage({
					headerMessage: UIHelper.getAttributeNodeValue(error, HISTORY_ERROR_MESSAGE, ProcessConstants.NAMESPACE_JP),
					responseText: this._getTextContent(error)
				}, this);
			} else if (this.editor._getMessagePage() === this) {
				this.editor._clearMessage();
			}
		},
		
		_handleError: function(error) {
			this.editor.stopLoading();
			this.editor.handleServerResponseError(error, this.messages.HistoryTable_10);
		},
		
		_clearState: function() {
			dojo.forEach(this._deferredDetails, function(detail) {
				if (detail) {
					detail.destroyRecursive();
				}
			});
			this._deferredDetails = [];
			this._pageCache = {};
		},
		
		uninitialize: function() {
			this._clearState();
			if (this._topPager) {
				this._topPager.destroyRecursive();
			}
			if (this._bottomPager) {
				this._bottomPager.destroyRecursive();
			}
			this.inherited(arguments);
		}
		
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects._DatePager", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		
		templateString: "<div class=\"pager\"><span dojoattachpoint=\"_previous\" dojotype=\"jazz.ui.toolbar.Button\" label=\"${messages.HistoryTable_0}\" enabled=\"false\" dojoattachevent=\"onClick: _onPreviousClick\"></span>" + //$NON-NLS-1$
						"<span class=\"vertical-bar\"></span><span dojoattachpoint=\"_pgRange\"></span><span class=\"vertical-bar\"></span>" + //$NON-NLS-1$
						"<span dojoattachpoint=\"_next\" dojotype=\"jazz.ui.toolbar.Button\" label=\"${messages.HistoryTable_1}\" enabled=\"false\" dojoattachevent=\"onClick: _onNextClick\"></span></div>", //$NON-NLS-1$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		update: function(startDate, endDate) {
			this._previous.set("enabled", this.table._previousPage ? true : false); //$NON-NLS-1$
			this._next.set("enabled", this.table._nextPage ? true : false); //$NON-NLS-1$
			this._pgRange.innerHTML = dojo.string.substitute(this.messages.HistoryTable_2, [startDate, endDate]);
		},
						
		_onPreviousClick: function() {
			this.table._showPage(this.table._pageNum - 1);
		},
		
		_onNextClick: function() {
			this.table._showPage(this.table._pageNum + 1);
		}
		
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects._DeferredDetail", dijit._Widget, { //$NON-NLS-1$
		
		/**
		 * @param callback {Function} function to call to get the details the
		 *            first time the node is expanded. This function will be
		 *            passed a single callback function that it should pass an
		 *            HTMLElement to
		 * @param detailNode {HTMLElement} [optional] the element to add the
		 *            details to. If not specified, one will be created
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._getDetails = args.callback;
			this._detailArea = args.detailNode;
		},
		
		postCreate: function() {
			this.inherited(arguments);
			this.domNode = dojo.create("div"); //$NON-NLS-1$
			this._topLink = dojo.create("a", { //$NON-NLS-1$
				"class": "showDetailsLink j-small j-action-secondary", //$NON-NLS-1$ //$NON-NLS-2$
				innerHTML: this.messages.HistoryTable_14,
				onclick: dojo.hitch(this, this._onClick)
			}, this.domNode);
		},
		
		_onClick: function() {
			if (!this._inProgress) {
				this._inProgress = true;
				if (!this._expanded) {
					if (!this._details) {
						if (!this._detailArea) {
							this._detailArea = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
						}
						// initial click, call the callback
						this._detailArea.innerHTML = "<div class=\"noDetails\">" + this.messages.HistoryTable_6 + "</div>"; //$NON-NLS-1$ //$NON-NLS-2$
						this._getDetails(dojo.hitch(this, this._gotDetails));
					} else {
						this._setExpanded(true);
					}
				} else {
					this._setExpanded(false);
				}
			}
		},
		
		_gotDetails: function(detailNode, collapse) {
			this._details = detailNode;
			if (collapse) {
				// no need for show/hide link
				dojo.place(this._details, this.domNode, "only"); //$NON-NLS-1$
			} else {
				this._setExpanded(true);
			}
		},
		
		_setExpanded: function(expanded) {
			this._expanded = expanded;
			dojo.toggleClass(this._topLink, "hideDetailsLink"); //$NON-NLS-1$
			dojo.toggleClass(this._topLink, "showDetailsLink"); //$NON-NLS-1$
			this._topLink.innerHTML = expanded ? this.messages.HistoryTable_15 : this.messages.HistoryTable_14;
			if (expanded) {
				// fix the height to 0 or the height of the loading message,
				// then the animation will grow from this height to full
				this._detailArea.style.height = this._detailArea.offsetHeight + "px"; //$NON-NLS-1$
				dojo.place(this._details, this._detailArea, "only"); //$NON-NLS-1$
				dojo.fx.wipeIn({
					node: this._detailArea,
					duration: 200,
					onEnd: dojo.hitch(this, function() {
						if (this._detailArea.clientHeight > 400) {
							// put a hide link at the bottom as well as the top
							// if the content is long enough 
							this._bottomLink = dojo.create("a", { //$NON-NLS-1$
								"class": "hideDetailsLink j-small j-action-secondary", //$NON-NLS-1$ //$NON-NLS-2$
								innerHTML: this.messages.HistoryTable_15,
								onclick: dojo.hitch(this, function() {
									this._topLink.click();
								})
							}, this._detailArea, "after"); //$NON-NLS-1$
						}
						delete this._inProgress;
					})
				}).play();
			} else {
				if (this._bottomLink) {
					this._bottomLink.parentElement.removeChild(this._bottomLink);
				}
				dojo.fx.wipeOut({
					node: this._detailArea,
					duration: 200,
					onEnd: dojo.hitch(this, function() {
						this._detailArea.removeChild(this._details);
						delete this._inProgress;
					})
				}).play();
			}
		}
	});
})();
