/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.InvitationCustomizationDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
(function(){

	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.InvitationCustomizationDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/InvitationCustomizationDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.descriptionNode = document.createTextNode(args.description);
			this.emptySubjectNode = document.createTextNode(this.messages.InvitationCustomizationDialog_1);
			this.emptyBodyNode = document.createTextNode(this.messages.InvitationCustomizationDialog_2);
			this.isCopySelf = args.copySelf;
			this.checkingMailerServiceStatusNode = document.createTextNode(this.messages.InvitationCustomizationDialog_3);
			this.mailerServiceDisabledNode = document.createTextNode(this.messages.InvitationCustomizationDialog_4);
		},
		
		postCreate: function(){
			this.initialize(this);
		},
		
		startup: function(){
			com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.resizeI18NButtons(this.dialogFooter, [this.finishButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		verifySubjectAndBody: function(e){
			if (!this.emailSubject.value || dojo.string.trim(this.emailSubject.value).length === 0) {
				this.setDescription(this.emptySubjectNode, true);
			} else {
				if (!this.emailBody.value || dojo.string.trim(this.emailBody.value).length === 0) {
					this.setDescription(this.emptyBodyNode, true);
				} else {
					this.setDescription(this.descriptionNode, false);
				}
			}
		},
		
		setDescription: function(node, isError){
			dojox.xml.parser.removeChildren(this.invitationDescription);
			this.invitationDescription.appendChild(node);
			if (isError) {
				dojo.removeClass(this._errorIcon, "hidden"); //$NON-NLS-1$
			} else {
				dojo.addClass(this._errorIcon, "hidden"); //$NON-NLS-1$
			}
			this.finishButton.disabled = isError;
		},
		
		initialize: function(args){
			this.setDescription(this.descriptionNode, false);
			if (args.invalidEmailSubjectOrBody) {
				this.finishButton.disabled = true;
			}
			// since the team invitation configuration data are "escaped" content,
			// we need to unescape the XML spcial characters in email subject and mail body
			this.emailSubject.value = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.unescapeXML(args.subject);
			this.emailBody.value = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.unescapeXML(args.message);
			this.copySelf.checked = args.isCopySelf;
			
			if (this.singleUserObj && args.includeEclipseConnection) {
				this.emailBody.rows = 6;
				this.invitationPropertiesDiv.style.display = "inline"; //$NON-NLS-1$
				this.invitationProperties.value = args.properties;
			} else {
				this.emailBody.rows = 12;
				this.invitationPropertiesDiv.style.display = "none"; //$NON-NLS-1$
			}
		},
		
		disableEmailCustomization: function(node, isError) {
			this.setDescription(node, isError);
			this.emailSubject.disabled = true;
			this.emailBody.disabled = true;
			this.copySelf.disabled = true;
			dojo.addClass(this._copySelfLabel, "disabled"); //$NON-NLS-1$			
			this.finishButton.disabled = true;
		},
		
		enableEmailCustomization: function() {
			this.verifySubjectAndBody();
			this.emailSubject.disabled = false;
			this.emailBody.disabled = false;
			this.copySelf.disabled = false;
			dojo.removeClass(this._copySelfLabel, "disabled"); //$NON-NLS-1$	
			this.connect(this.emailSubject, "onkeyup", "verifySubjectAndBody"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.emailSubject, "onchange", "verifySubjectAndBody"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.emailBody, "onkeyup", "verifySubjectAndBody"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.emailBody, "onchange", "verifySubjectAndBody"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		
		getEmailSubject: function(){
			return this.emailSubject.value;
		},
		
		getEmailBody: function(){
			return this.emailBody.value;
		},
		
		getCopySelf: function(){
			return this.copySelf.checked;
		}
	});
	
})();
