/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.IterationDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.form.FilteringSelect"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypesSelect"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DateTimeTextBox"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;

(function(){
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.IterationDialog", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/IterationDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this.iterationTypesSelect = new com.ibm.team.process.web.ui.internal.admin.projects.IterationTypesSelect({
				timelineStore: this.timelineStore,
				validateCallback: dojo.hitch(this, this._validateData)
			});
			this.iterationTypesCell.appendChild(this.iterationTypesSelect.domNode);
			
			this.inputFields = [this.identifier, this.displayName, this.startDate, this.endDate];
			this.identifier.set('intermediateChanges', true); //$NON-NLS-1$
			this.identifier.onChange = dojo.hitch(this, function(newValue) {
				this._validateData();
			});
			this.startDate.set('intermediateChanges', true); //$NON-NLS-1$
			this.startDate.onChange = dojo.hitch(this, function(newValue) {
				this._validateData();
			});
			this.endDate.set('intermediateChanges', true); //$NON-NLS-1$
			this.endDate.onChange = dojo.hitch(this, function(newValue) {
				this._validateData();
			});
			
			this._populateDialog();
		},
		
		startup: function(){
			com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.resizeI18NButtons(this.dialogFooter, [this.finishButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		_validateData: function(){
			if (this._isValidIdentifier() && this._isValidDateRange() && this.iterationTypesSelect.isValid()) {
				this.finishButton.disabled = false;
			} else {
				this.finishButton.disabled = true;
			}
			
		},
		
		_isValidDateRange: function(){
			var startDate = this.startDate.get('value'); //$NON-NLS-1$
			var endDate = this.endDate.get('value'); //$NON-NLS-1$
			if (startDate !== null && !this.startDate.isValid()) {
				return false; // Start date must contain a valid date
			}
			if (endDate === null) {
				return true; // Always ok if there is no end date
			}
			if (startDate === null && endDate !== null) {
				return false; // Can't have end date without start date
			}
			
			return endDate >= startDate; // End date must be greater than or equal to the start date
		},
		
		_isValidIdentifier: function(){
			var identifier = this.identifier.get('value'); //$NON-NLS-1$
			return identifier && dojo.string.trim(identifier).length > 0;
		},
		
		_populateDialog: function() {
			if (!this._iterationHasIterationType()) {
				this.iterationTypesSelect.set("displayedValue", this.messages.IterationDialog_IterationTypeNone); //$NON-NLS-1$
			} else {
				this.timelineStore.fetchItemByIdentity({
					identity: this.iterationTypeItemId,
					onItem: dojo.hitch(this, function(item, request) {
						var iterationName = this.timelineStore.getValue(item, "name"); //$NON-NLS-1$
						this.iterationTypesSelect.originalIterationTypeName = iterationName;
						this.iterationTypesSelect.set("displayedValue", iterationName); //$NON-NLS-1$
					})
				});
				
			}
			this.scheduledReleaseCheckbox.checked = this.hasDeliverable;
			for (var i = 0; i < this.inputFields.length; i++) {
				this.inputFields[i].set('value', this.initialValues[i]); //$NON-NLS-1$
			}
		},
		
		_iterationHasIterationType: function() {
			if (this.iterationTypeItemId) {
				var iterationType = null;
				this.timelineStore.fetchItemByIdentity({
					identity: this.iterationTypeItemId,
					onItem: function(item, request){
						iterationType = item;
					}
				});
				return iterationType === null ? false : true;
			}
			return false;
		},
		
		changesPending: function(){
			for (var i = 0; i < this.inputFields.length; i++) {
				if (this._compare(this.inputFields[i].get("value"), this.initialValues[i]) !== 0) { //$NON-NLS-1$
					return true;
				}
			}
			
			if (this.hasDeliverable !== this.scheduledReleaseCheckbox.checked) {
				return true;
			}
			
			if (this.iterationTypeItemId !== this.getSelectedIterationTypeId()) {
				return true;
			}
			return false;
		},
		
		getSelectedIterationTypeId: function() {
			if (!this.iterationTypesSelect.item && this.iterationTypesSelect.isValid()) {
				// if this is the case it means that this iteration is using an
				// iteration type from the process provider that is eclipsed by
				// a local type using the same id - since they couldn't have
				// selected such an iteration type, they must not have changed
				// this field, so just return the original value
				return this.iterationTypeItemId;
			}
			var store = this.iterationTypesSelect.store;
			return store.getIdentity(this.iterationTypesSelect.item);
		},
		
		_compare: function(newValue, oldValue){
			if (newValue > oldValue) {
				return 1;
			} else if (newValue < oldValue) {
				return -1;
			} else {
				return 0;
			}
		},
		
		close: function() {
			// Workitem 110877..on IE 7 the change handlers are being called
			// after the controls have been destroyed.  Make them do nothing.
			var emptyChangeHandler = function(newValue){
				// Intentionally do nothing
			};
			this.identifier.onChange = this.startDate.onChange = this.endDate.onChange = emptyChangeHandler;
			// Workitem 181522..doesn't always close on IE 9
			this.iterationTypesSelect.closeDropDown();
		}
		
	});
	
})();
