/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorDialog"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/OperationBehaviorDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/**
		 * Creates and opens a dialog that allows the user to select any of the
		 * possible preconditions/follow-up actions for the given operation.
		 * 
		 * @param item {dojo.data.Item} operation extension to let the user
		 *            choose a behavior from
		 * @param extensionStore {OperationBehaviorExtensionStore} store that
		 *            item comes from
		 * @param onOk {Function} ok callback, passed the item selected
		 * @param onCancel {Function} cancel callback
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	
		},
		
		postCreate: function() {
			this._tree = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTree({
				item: this.item,
				extensionStore: this.extensionStore,
				emptyText: this.messages.OperationBehaviorDialog_9,
				sorted: true
			}).placeAt(this.operationBehaviorTree);
			
			var hasPreconditions = this.extensionStore.getPreconditions(this.item).length > 0;
			var hasFollowups = this.extensionStore.getFollowupActions(this.item).length > 0;
			this.dialogHeading.innerHTML = this._getHeading(hasPreconditions, hasFollowups);
			this.description.innerHTML = this._getDescriptionMessage(hasPreconditions, hasFollowups);
			
			this.connect(this._tree, "behaviorSelected", this._selectionChanged); //$NON-NLS-1$
			this.connect(this._tree, "behaviorDblClicked", this._selectionDblClicked); //$NON-NLS-1$
			this.connect(this.okButton, "onclick", this._okCallback); //$NON-NLS-1$
			this.connect(this.cancelButton, "onclick", this._cancelCallback); //$NON-NLS-1$
			
			this.dialog = new jazz.ui.Dialog({
				primaryTitle: this._getTitle(hasPreconditions, hasFollowups),
				contentNode: this.domNode,
				width: "30em", //$NON-NLS-1$
				destroyContents: true,
				onOpen: dojo.hitch(this, this.startup)
			});
		},
		
		startup: function() {
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			this._tree.startup();
			this.inherited(arguments);
		},
		
		_getTitle: function(hasPreconditions, hasFollowups) {
			if (hasPreconditions && !hasFollowups) {
				return this.messages.OperationBehaviorDialog_1;
			} else if (!hasPreconditions && hasFollowups) {
				return this.messages.OperationBehaviorDialog_2;
			} else {
				return this.messages.OperationBehaviorDialog_0;
			}
		},
		
		_getHeading: function(hasPreconditions, hasFollowups) {
			if (hasPreconditions && !hasFollowups) {
				return this.messages.OperationBehaviorDialog_4;
			} else if (!hasPreconditions && hasFollowups) {
				return this.messages.OperationBehaviorDialog_5;
			} else {
				return this.messages.OperationBehaviorDialog_3;
			}
		},
		
		_getDescriptionMessage: function(hasPreconditions, hasFollowups) {
			if (hasPreconditions && !hasFollowups) {
				return this.messages.OperationBehaviorDialog_7;
			} else if (!hasPreconditions && hasFollowups) {
				return this.messages.OperationBehaviorDialog_8;
			} else {
				return this.messages.OperationBehaviorDialog_6;
			}
		},
		
		_okCallback: function() {
			this.dialog.close();
			if (this.onOk && dojo.isFunction(this.onOk)) {
				this.onOk(this._selected);
			}
		},
		
		_cancelCallback: function() {
			this.dialog.close();
			if (this.onCancel && dojo.isFunction(this.onCancel)) {
				this.onCancel();
			}
		},
		
		_selectionChanged: function(item) {
			this.okButton.disabled = false;
			var description = this.extensionStore.getValue(item, ProcessConstants.ExtensionDTO.DESCRIPTION);
			this.description.innerHTML = jazz.util.html.escape(description);
			this._selected = item;
		},
		
		_selectionDblClicked: function(item) {
			// will be processed after _selectionChanged
			this._okCallback();
		},
		
		uninitialize: function() {
			if (this._tree) {
				this._tree.destroyRecursive();
			}
			this.inherited(arguments);
		}
		
	});
})();
