/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditContextDialog"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectContextWidget"); //$NON-NLS-1$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	/**
	 * A simple dialog allowing the user to change the context for an existing
	 * operation behavior configuration.
	 * 
	 * See the constructor of OperationBehaviorEditRoleDialog for the required
	 * arguments to the constructor of this widget. In addition, the following
	 * fields in argument 'model' are required:
	 * 
	 * - extensionStore				A dojo.data.ItemFileReadStore which contains
	 * 								categories and operations for which
	 * 								operation behavior can be configured
	 * 	- editorContext				The editor context that the widget is being
	 * 								used in: a value in
	 * 								ProcessConstants.EditorContext
	 * 	- developmentLineItemId		A string that is used if editorContext is
	 * 								ProcessConstants.EditorContext.TEAM to offer
	 * 								child iterations of the team area's selected
	 * 								development line
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditContextDialog", //$NON-NLS-1$
		[com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditDialog], {
		
		createCustomContents: function() {
			this._operationBehaviorSelectContextWidget = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectContextWidget({
				model: this.model,
				behaviorConfigurationItem: this.behaviorConfigurationItem,
				selectionChangedCallback: dojo.hitch(this, this._selectionChanged)
			});
			this.widgetContainer.appendChild(this._operationBehaviorSelectContextWidget.domNode);
		},
		
		getSummaryTextString: function() {
			return this._operationBehaviorSelectContextWidget.getSummaryTextString();
		},
		
		hideSummaryTextNode: function() {
			this._operationBehaviorSelectContextWidget.hideSummaryTextNode();
		},
		
		getResultObject: function() {
			var obj = {};
			obj[ProcessConstants.BehaviorConfigurationDTO.CONTEXT] = this._selectedContext;
			obj[ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID] = this._selectedContextId;
			return obj;
		},
		
		_selectionChanged: function(selectedValues, doubleClicked) {
			if (selectedValues.context === undefined) {
				//"Always" was selected. Set the context based on whether the
				//operation is a project or team operation...
				var operationItem = this.model.extensionStore.fetchItemByIdentity(this.model.configurationStore.getValue(this.behaviorConfigurationItem, ProcessConstants.ExtensionDTO.ID));
				this._selectedContext = ProcessConstants.ExtensionContext.PROJECT === this.model.extensionStore.getValue(operationItem, "context") //$NON-NLS-1$
					? ProcessConstants.PermissionContext.PROJECT
					: ProcessConstants.PermissionContext.TEAM;
				//...and clear out the contextId
				this._selectedContextId = ""; //$NON-NLS-1$
			} else {
				//An iteration type or timeline/iteration was selected
				this._selectedContext = selectedValues.context;
				this._selectedContextId = selectedValues.contextId;
			}
			
			this.notifySelectionChanged(doubleClicked);
		},
		
		canFinish: function() {
			var currentContext = this.model.configurationStore.getValue(
				this.behaviorConfigurationItem,
				ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
			if (this._selectedContext === undefined) {
				if (currentContext !== ProcessConstants.PermissionContext.PROJECT &&
				    currentContext !== ProcessConstants.PermissionContext.TEAM) {
					return true;
				}
			} else {
				if (this._selectedContext !== ProcessConstants.PermissionContext.PROJECT
					&& this._selectedContext !== ProcessConstants.PermissionContext.TEAM
					&& this._selectedContextId === undefined) {
					//The user has not selected a specific iteration type
					//(perhaps because there are none) or timeline/iteration.
					//They cannot finish yet.
					return false;
				} else if (this._selectedContext !== currentContext) {
					return true;
				}
			}
			
			var currentContextId = this.model.configurationStore.getValue(
				this.behaviorConfigurationItem,
				ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
			if (this._selectedContextId === undefined) {
				return currentContextId !== ""; //$NON-NLS-1$
			} else {
				return this._selectedContextId !== currentContextId;
			}
		},
		
		uninitialize: function() {
			if (this._operationBehaviorSelectContextWidget) {
				this._operationBehaviorSelectContextWidget.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
})();
