/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorInlineEditor"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
	dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
	dojo.require("dijit.form.CheckBox"); //$NON-NLS-1$
	dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
	dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
	dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
	dojo.require("jazz.app.ua.TextHoverHelp"); //$NON-NLS-1$
	dojo.require("jazz.ui.StyledBox"); //$NON-NLS-1$
	dojo.require("jazz.util.html"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ErrorInCustomEditorWidget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTreeEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditContextDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditRoleDialog"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorInlineEditor", jazz.ui.StyledBox, { //$NON-NLS-1$
		
		trim: jazz.ui.StyledBox.TrimStyles.EDITOR,
		
		constructor: function(args) {
			this.iconPath = args.configurationStore.getCategoryIconUrl(args.item);
			this.primaryTitle = args.configurationStore.getValue(args.item, ProcessConstants.ExtensionDTO.NAME);
			
			this._content = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorInlineEditorContent({
				item: args.item,
				editor: args.editor,
				extensionStore: args.extensionStore,
				configurationStore: args.configurationStore,
				rolesStore: args.rolesStore,
				contextStore: args.contextStore,
				customEditorManager: args.customEditorManager
			});
			this.contentNode = this._content.domNode;
			
			this._customOnClose = args.onClose;
			delete args.onClose;
		},
		
		onClose: function() {
			this._customOnClose();
			// prevent default onClose behavior from StyledBox
			return false;
		},
		
		aboutToShowEditor: function() {
			if (this._content._currentCustomEditor) {
				this._content._currentCustomEditor.aboutToShowEditor();
			}
		},
		
		canSave: function() {
			return this._content._canSave();
		},
		
		validate: function() {
			return this._content._validate();
		}
		
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorInlineEditorContent", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/OperationBehaviorInlineEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._textHoverHelps = [];
			this._pageHoverHelps = [];
		},
		
		postCreate: function() {
			this._behaviorTree = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTreeEditor({
				item: this.item,
				configurationStore: this.configurationStore,
				extensionStore: this.extensionStore,
				persist: false
			}).placeAt(this.behaviorTree);
			this.connect(this._behaviorTree, "behaviorSelected", "_operationBehaviorSelected"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this.finalCheckBox.checked = this.configurationStore.getValue(this.item, ProcessConstants.BehaviorConfigurationDTO.FINAL_OP);
			this.connect(this.finalCheckBox, "onclick", "_onFinalClick"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this.connect(this.whoScopeButton, "onclick", "_changeWho"); //$NON-NLS-1$ //$NON-NLS-2$
			
			//Project operations will apply Always, and the context cannot be changed
			var extensionItem = this.extensionStore.fetchItemByIdentity(this.configurationStore.getValue(this.item, ProcessConstants.ExtensionDTO.ID));
			if (ProcessConstants.ExtensionContext.PROJECT === this.extensionStore.getValue(extensionItem, ProcessConstants.ExtensionDTO.CONTEXT)) {
				dojo.addClass(this.whenScopeButton, "hidden"); //$NON-NLS-1$
				
				var operationName = this.extensionStore.getValue(extensionItem, ProcessConstants.ExtensionDTO.NAME);
				this._textHoverHelps.push(new jazz.app.ua.TextHoverHelp({
					contentNode: this.whenScopeText,
					helpContentString: dojo.string.substitute(this.messages.OperationBehaviorInlineEditor_4, [operationName])
				}));
			} else {
				this.connect(this.whenScopeButton, "onclick", "_changeWhen"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			var extension = this.extensionStore.fetchItemByIdentity(this.configurationStore.getValue(this.item, ProcessConstants.ExtensionDTO.ID));
			var preconditionsPossible = this.configurationStore.getPreconditions(this.item).length > 0 || this.extensionStore.getPreconditions(extension).length > 0;
			var followupsPossible = this.configurationStore.getFollowupActions(this.item).length > 0 || this.extensionStore.getFollowupActions(extension).length > 0;
			if (preconditionsPossible && !followupsPossible) {
				this.moreDetails.innerHTML = this.messages.OperationBehaviorInlineEditor_5;
			} else if (!preconditionsPossible && followupsPossible) {
				this.moreDetails.innerHTML = this.messages.OperationBehaviorInlineEditor_6;
			} else {
				this.moreDetails.innerHTML = this.messages.OperationBehaviorInlineEditor_7;
			}
			
			this.operationBehaviorName.watch("displayedValue", dojo.hitch(this, function() { //$NON-NLS-1$
				this._behaviorNameChanged();
			}));
			
			this.operationBehaviorDescription.watch("displayedValue", dojo.hitch(this, function() { //$NON-NLS-1$
				this._behaviorDescriptionChanged();
			}));
			
			this.connect(this.configurationStore, "onUpdateConfiguration", "_onUpdateConfiguration"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.rolesStore, "onSet", "_onSetRole"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.contextStore, "onSet", "_onSetContext"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Overridden to handle elements which are going to require dynamic
		 * resizing, now that those elements will be in the document.
		 */
		startup: function() {
			this.inherited(arguments);
			
			this._behaviorTree.startup();
			this._truncate(this.whoScopeText, this.configurationStore.getRoleLabel(this.item));
			this._truncate(this.whenScopeText, this.configurationStore.getContextLabel(this.item));
		},
		
		_truncate: function(field, text) {
			UIHelper.truncate(field, text, function(text) {
				field.innerHTML = jazz.util.html.escape(text);
			}, function() {
				return field.offsetHeight;
			}, this);
		},
		
		/**
		 * Called when a precondition/follow-up is selected from the given list
		 */
		_operationBehaviorSelected: function(item) {
			this._selected = item;
			if (item === null) {
				// behavior was just deleted, clear details editor
				dojo.style(this.moreDetails, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(this.nameAndDescriptionEditor, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(this.customConfigurationMessage, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(this.customEditor, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				dojo.style(this.nameAndDescriptionEditor, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(this.moreDetails, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
				// populate fields
				var id = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
				var type = id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR)[2];
				this.behaviorLabel.innerHTML = type === ProcessConstants.PRECONDITIONS ? this.messages.OperationBehaviorInlineEditor_2 : this.messages.OperationBehaviorInlineEditor_3;
				dojo.place(this._getExtensionNameElement(id), this.extensionName, "only"); //$NON-NLS-1$
				// show context sensitive help widget if the attribute "uaKey" in the editor extension is provided
				var uaKey = this.customEditorManager.getUaKey(item);
				if (uaKey) {
					var helpWidget = new jazz.app.ua.PageHoverHelp({
						uaKey: uaKey
					});
					this._pageHoverHelps.push(helpWidget);
					dojo.place(helpWidget.domNode, this.helpWidget, "only"); //$NON-NLS-1$
				} else {
					dojox.xml.parser.removeChildren(this.helpWidget);
				}
				
				this._lastNameValue = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.NAME);
				this.operationBehaviorName.set("value", this._lastNameValue); //$NON-NLS-1$
				
				this._lastDescriptionValue = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.DESCRIPTION);
				this.operationBehaviorDescription.set("value", this._lastDescriptionValue); //$NON-NLS-1$
				
				// show custom editor, if necessary
				dojox.xml.parser.removeChildren(this.customEditor);
				var showCustomEditor = dojo.hitch(this, function(widgetId) {
					// helper function, called below
					try {
						this._currentCustomEditor = this.customEditorManager.createEditor(item, this.customEditor, widgetId);
						if (this._currentCustomEditor) {
							dojo.style(this.customEditor, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
							dojo.style(this.customConfigurationMessage, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
						} else {
							this.customConfigurationMessage.innerHTML = this.messages.OperationBehaviorInlineEditor_0;
							dojo.style(this.customConfigurationMessage, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
							dojo.style(this.customEditor, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
						}
					} catch (error) {
						this._handleErrorLoadingWidget(item, type, error);
					}
				});
				if (this.customEditorManager.needsNewEditor(item)) {
					// show loading message
					this.customConfigurationMessage.innerHTML = this.messages.OperationBehaviorInlineEditor_1;
					dojo.style(this.customConfigurationMessage, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
					dojo.style(this.customEditor, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
					// async load widget
					var widgetId = this.customEditorManager.getWidgetId(item);
					try {
						jazz.core.loader.load_async(widgetId, function() {
							showCustomEditor(widgetId);
						});
					} catch (error) {
						this._handleErrorLoadingWidget(item, type, error);
					}
				} else {
					// run sync if not new editor, to avoid unnecessary flicker
					showCustomEditor();
				}
			}
		},
		
		_handleErrorLoadingWidget: function(item, type, error) {
			dojo.empty(this.customEditor);
			
			if (this._errorInCustomEditorWidget) {
				this._errorInCustomEditorWidget.destroyRecursive();
			}
			
			var itemId = this.configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
			var operationBehaviorId = itemId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR).pop();
			this._errorInCustomEditorWidget = new com.ibm.team.process.web.ui.internal.admin.projects.ErrorInCustomEditorWidget({
				summaryText: type === ProcessConstants.PRECONDITIONS
					? this.messages.OperationBehaviorInlineEditor_9
					: this.messages.OperationBehaviorInlineEditor_10, 
				extensionId: operationBehaviorId,
				editorWidget: error.widgetId,
				editorError: error	
			});
			this.customEditor.appendChild(this._errorInCustomEditorWidget.domNode);
			
			dojo.style(this.customEditor, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(this.customConfigurationMessage, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Returns an element that can be inserted into the document to
		 * display the name of the extension with this id.
		 */
		_getExtensionNameElement: function(id) {
			var extension = this.extensionStore.fetchItemByIdentity(id);
			var name = dojo.create("span", { //$NON-NLS-1$
				innerHTML: this.extensionStore.getValue(extension, ProcessConstants.ExtensionDTO.NAME)
			});
			return name;
		},
		
		/**
		 * React to changes to the input for the behavior's name.
		 */
		_behaviorNameChanged: function() {
			var name = this.operationBehaviorName.get("value"); //$NON-NLS-1$
			if (name !== this._lastNameValue) {
				// determine if the editor can currently be saved
				if (dojo.string.trim(name).length === 0 && dojo.string.trim(this._lastNameValue).length > 0) {
					this._cantSaveCount = this._cantSaveCount || 0;
					this._cantSaveCount++;
				} else if (dojo.string.trim(name).length > 0 && dojo.string.trim(this._lastNameValue).length === 0) {
					this._cantSaveCount--;
				}
				// update the store
				this.configurationStore.setValue(this._selected, ProcessConstants.ExtensionDTO.NAME, name);
				this._lastNameValue = name;
			}
		},
		
		/**
		 * React to changes to the input for the behavior's description.
		 */
		_behaviorDescriptionChanged: function() {
			var description = this.operationBehaviorDescription.get("value"); //$NON-NLS-1$
			if (description !== this._lastDescriptionValue) {
				this.configurationStore.setValue(this._selected, ProcessConstants.ExtensionDTO.DESCRIPTION, description);
				this._lastDescriptionValue = description;
			}
		},
		
		_onFinalClick: function() {
			this.configurationStore.setValue(this.item, ProcessConstants.BehaviorConfigurationDTO.FINAL_OP, this.finalCheckBox.checked);
		},
		
		_changeWho: function() {
			var dialog = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditRoleDialog({
				model: {
					contextStore: this.contextStore,
					editorContext: this.editor.editor.context,
					configurationStore: this.configurationStore,
					extensionStore: this.extensionStore,
					rolesStore: this.rolesStore
				},
				behaviorConfigurationItem: this.item
			});
			dialog.startup();
		},
		
		_changeWhen: function() {
			var model = {
				contextStore: this.contextStore,
				editorContext: this.editor.editor.context,
				configurationStore: this.configurationStore,
				extensionStore: this.extensionStore,
				rolesStore: this.rolesStore
			};
			if (this.editor.editor.isTeamArea) {
				model.developmentLineItemId = this.editor.editor.getSelectedDevelopmentLineItemId();
			}
			var dialog = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorEditContextDialog({
				model: model,
				behaviorConfigurationItem: this.item
			});
			dialog.startup();
		},
		
		// =====================================================================
		// React to store events of relevance
		
		_onUpdateConfiguration: function(oldItem, newItem) {
			if (this.item === oldItem) {
				if (this.item !== newItem) {
					// this editor's item changed as part of this update
					this.item = newItem;
				}
				this._truncate(this.whoScopeText, this.configurationStore.getRoleLabel(this.item));
				this._truncate(this.whenScopeText, this.configurationStore.getContextLabel(this.item));
			}
		},
		
		_onSetRole: function(item, attribute, oldValue, newValue) {
			if (attribute === ProcessConstants.ProcessRoleDTO.LABEL) {
				this._truncate(this.whoScopeText, this.configurationStore.getRoleLabel(this.item));
			}
		},
		
		_onSetContext: function(item, attribute, oldValue, newValue) {
			if (attribute === "name") { //$NON-NLS-1$
				this._truncate(this.whenScopeText, this.configurationStore.getContextLabel(this.item));
			}
		},
		
		// =====================================================================
		
		/**
		 * Indicates whether the containing editor should be able to save or
		 * not. If any behavior's name (title) is empty it will not be able to.
		 */
		_canSave: function() {
			return !this._cantSaveCount; // false if count > 0
		},
		
		_validate: function() {
			if (this._cantSaveCount) {
				return this.messages.OperationBehaviorInlineEditor_8;
			}
			return null;
		},
		
		uninitialize: function() {
			for (var i = 0; i < this._textHoverHelps.length; i++) {
				if (this._textHoverHelps[i]) {
					this._textHoverHelps[i].destroyRecursive();
				}
			}
			this._textHoverHelps = [];
			
			for (var i = 0; i < this._pageHoverHelps.length; i++) {
				if (this._pageHoverHelps[i]) {
					this._pageHoverHelps[i].destroyRecursive();
				}
			}
			this._pageHoverHelps = [];

			if (this._behaviorTree) {
				this._behaviorTree.destroyRecursive();
				delete this._behaviorTree;
			}
			
			if (this._errorInCustomEditorWidget) {
				this._errorInCustomEditorWidget.destroyRecursive();
				delete this._errorInCustomEditorWidget;
			}
			
			this.inherited(arguments);
		}
		
	});
})();
