/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTree"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dijit.tree.TreeStoreModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTree", com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree, { //$NON-NLS-1$
		
		/**
		 * If true, rows are not highlighted during hover/selection and events
		 * are not listened to (model and store events) or triggered
		 * (behaviorSelected() and behaviorDblClicked() are never called). The
		 * tree is read-only and does not change (better performance).
		 */
		isStatic: false,
		
		/**
		 * Specify to provide a custom message to display when the tree would
		 * otherwise be empty.
		 */
		emptyText: null,
		
		/**
		 * Specify if behaviors should be sorted by name inside of each group
		 */
		sorted: false,
		
		/**
		 * Creates a tree that displays all preconditions/follow-ups of the
		 * given operation configuration or extension in a flat list. When both
		 * preconditions/follow-ups exist, sub-headings are included above each
		 * section.
		 * 
		 * @param item {dojo.data.Item} operation item to create a tree for,
		 * 			  required
		 * @param configurationStore {OperationBehaviorConfigurationStore}
		 * 			  optional, but extensionStore needed if not specified
		 * @param extensionStore {OperationBehaviorExtensionStore}
		 * 			  optional, but configurationStore needed if not specified
		 */
		constructor: function(args) {
			this.showRoot = false;
			this.model = new com.ibm.team.process.web.ui.internal.admin.projects._OperationBehaviorTreeModel({
				root: args.item,
				store: args.configurationStore || args.extensionStore,
				configurationStore: args.configurationStore,
				extensionStore: args.extensionStore,
				emptyText: args.emptyText,
				sorted: args.sorted
			});
			if (this.isStatic) {
				// static, don't listen to store events
				dojo.forEach(this.model.connects, dojo.disconnect);
			}
		},
		
		postCreate: function() {
			this.model.onChildrenChange = dojo.hitch(this, this._beforeOnChildrenChange);
			this.inherited(arguments);
			
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-OperationBehaviorTree"); //$NON-NLS-1$
			
			this.connect(this.model, "onChildrenChange", this._afterOnChildrenChange); //$NON-NLS-1$
			this.connect(this.model, "onChange", this._onChange); //$NON-NLS-1$
			
			this.connect(this.model.store, "onUpdateConfiguration", "_onUpdateConfiguration"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		connect: function(args) {
			var handle = this.inherited(arguments);
			if (this.isStatic) {
				// static, don't listen to model/store events
				this.disconnect(handle);
			}
			return handle;
		},
		
		_onUpdateConfiguration: function(oldItem, newItem) {
			if (this.item === oldItem && this.item !== newItem) {
				// update this tree's item reference
				this.item = newItem;
			}
		},
		
		onLoad: function() {
			this.inherited(arguments);
			this.model.getChildren(this.model.root, dojo.hitch(this, function(children) {
				// update now that all rows have been created
				this._updateNodes(children);
			}));
			this.connect(this.model, "onChildrenChange", function(parent, children) { //$NON-NLS-1$
				// update as rows are added/deleted/moved
				this._updateNodes(children);
			});
		},
		
		/**
		 * Gets the node for each item, and any other info necessary, so that
		 * updateNode() can be called on each node.
		 */
		_updateNodes: function(items) {
			var nodes = items.length > 0 ? [this.getNodesByItem(items[0])[0]] : [];
			for (var i = 0; i < items.length; i++) {
				if (i+1 < items.length) {
					nodes.push(this.getNodesByItem(items[i+1])[0]);
				}
				this.updateNode(nodes[i], i > 0 ? nodes[i-1] : null, nodes[i+1] || null);
			}
		},
		
		/**
		 * Called on each node in the tree each time a change occurs in the
		 * tree's structure, so that any necessary updates can be made.
		 * 
		 * @param node {dijit._TreeNode} tree node to update
		 * @param previous {dijit._TreeNode} previous sibling node of this node,
		 * 			  or null
		 * @param next {dijit._TreeNode} next sibling node of this node, or null
		 */
		updateNode: function(node, previous, next) {
			if (this.isStatic || dojo.isString(node.item)) {
				dojo.addClass(node.rowNode, "readOnlyRow"); //$NON-NLS-1$
			}
			if (!node._truncating && !dojo.isString(node.item)) {
				// run async to wait until all nodes have been added, and the
				// scrollbar has appeared if its going to
				setTimeout(dojo.hitch(this, function() {
					this._truncateNode(node);
				}));
				node._truncating = true;
			}
		},
		
		_onChange: function(item) {
			var node = this.getNodesByItem(item)[0];
			if (node) {
				this._truncateNode(node);
			}
		},
		
		/**
		 * Prevents a tree node from wrapping by truncating the label text.
		 */
		_truncateNode: function(node) {
			node.truncate = UIHelper.truncate(node.labelNode, node.labelNode.innerHTML, function(text) {
				node.labelNode.innerHTML = text;
			}, function() {
				return node.contentNode.offsetHeight;
			}, this);
		},
		
		getLabel: function(item) {
			var label = this.inherited(arguments);
			if (label === undefined || label === null || dojo.string.trim(label).length === 0) {
				// return the id if there is no name
				var id = this.model.store.getValue(item, ProcessConstants.ExtensionDTO.ID);
				var segments = id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR);
				label = segments[segments.length-1];
			}
			return label;
		},
		
		getLabelClass: function(item) {
			if (dojo.isString(item)) {
				return "subheadingLabel"; //$NON-NLS-1$
			}
		},
			
		getIconClass: function(item) {
			if (!dojo.isString(item)) {
				var id = this.model.store.getValue(item, ProcessConstants.ExtensionDTO.ID);
				var type = id.split(ProcessConstants.BEHAVIOR_PATH_SEPARATOR)[2];
				if (type === ProcessConstants.PRECONDITIONS) {
					return "behaviorIcon preconditionIcon"; //$NON-NLS-1$
				} else if (type === ProcessConstants.FOLLOWUP_ACTIONS) {
					return "behaviorIcon followupIcon"; //$NON-NLS-1$
				}
			}
			return null;
		},
		
		/**
		 * Overridden from dijit.Tree to account for the fact that the tree may
		 * contain some entries (string entries) that shouldn't react when the
		 * user clicks on them. For example, if the user clicks on a
		 * precondition, then clicks on the "Preconditions" node, we want the
		 * precondition to stay selected, and to not call behaviorSelected().
		 */
		focusNode: function(node) {
			if (!this.isStatic) {
				var lastFocusedIsItem = this.lastFocused && !dojo.isString(this.lastFocused.item);
				if (dojo.isString(node.item) && lastFocusedIsItem && !this._lastFocusedItem) {
					// about to focus on a dummy node, remember the last real
					// node selected, so we can re-select it
					this._lastFocusedItem = this.lastFocused;
				} else if (!dojo.isString(node.item) && this._lastFocusedItem) {
					// about to focus on a real node, and we previously
					// re-selected a node
					if (this._lastFocusedItem !== node) {
						// previously re-selected node was a different one than
						// the one just focused, un-select it
						this._lastFocusedItem.setSelected(false);
					}
					// no longer need to remember the last real node selected,
					// the current node is real
					delete this._lastFocusedItem;
				}
			}
			this.inherited(arguments);
		},
		
		/**
		 * Overridden along with focusNode() to only call behaviorSelected()
		 * when a node with an item is focused, and to keep the last node with
		 * an item "selected" when a string node is focused.
		 */
		focusChanged: function(node) {
			if (!this.isStatic) {
				if (this._lastFocusedItem) {
					// a dummy node was just focused, keep the last real node
					// selected (for display purposes, this doesn't alter which
					// node is currently focused and tab-indexable
					this._lastFocusedItem.setSelected(true);
				} else if (!dojo.isString(node.item)) {
					// a real node selected, alert listeners
					this.behaviorSelected(node.item);
				}
			}
		},
		
		/**
		 * Overriden to allow behaviors to be selected (in the sense of
		 * double-clicked) when the Enter key is pressed while they have
		 * focus. dijit.Tree will send an onClick event in that case.
		 */
		onClick: function(item, node, event) {
			if (!this.isStatic) {
				if ("keypress" === event.type && dojo.keys.ENTER === event.which) { //$NON-NLS-1$
					if (!this._lastFocusedItem && !dojo.isString(node.item)) {
						this.behaviorSelected(node.item);
						this.behaviorDblClicked(item);
					}
				}
			}
		},
		
		/**
		 * Overrides the model's onChildrenChange, rather than dojo.connects to,
		 * because we need to read "lastFocused" before onChildrenChange
		 * listeners change it. This method is meant to be overridden though (by
		 * the model itself, we do it at arms length here)
		 */
		_beforeOnChildrenChange: function(parent, newChildrenList) {
			this._focusedBeforeChildrenChange = null;
			if (this.lastFocused) {
				if (dojo.some(newChildrenList, dojo.hitch(this, function(item) {
					return this.lastFocused.item === item;
				}))) {
					// a node already had focus, and is still a child, we will
					// restore it in our onChildrenChange listener below
					this._focusedBeforeChildrenChange = this.lastFocused;
				} else if (!dojo.isString(this.lastFocused.item) && !this.isStatic) {
					// previously selected item is no longer part of this tree,
					// notify listeners that the selection has been cleared
					this.behaviorSelected(null);
				}
			}
		},
		
		/**
		 * dijit.Tree resets the currently focused node when the child items of
		 * the root are refreshed, this listener is connected (to run after
		 * dijit.Tree's listeners) to prevent that behavior
		 */
		_afterOnChildrenChange: function(parent, newChildrenList) {
			if (this._focusedBeforeChildrenChange && this._focusedBeforeChildrenChange.item !== this.lastFocused.item) {
				// dijit.Tree focused a new item, unfocus it
				this.lastFocused.setFocusable(false);
				// and restore the reference to the previously focused item
				this.lastFocused = this._focusedBeforeChildrenChange;
			}
		},
		
		onDblClick: function(item, node, event){
			if (!dojo.isString(item) && !this.isStatic) {
				this.behaviorDblClicked(item);
			}
		},
		
		behaviorSelected: function(item) {
			// implementors can connect to this function to be notified when an
			// item is selected in the tree (and not in read-only mode)
		},
		
		behaviorDblClicked: function(item) {
			// implementors can connect to this function to be notified when an
			// item in the tree is double-clicked (and not in read-only mode)
		},
		
		uninitialize: function() {
			this.model.destroy();
			this.inherited(arguments);
		}
		
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects._OperationBehaviorTreeModel", dijit.tree.TreeStoreModel, { //$NON-NLS-1$
		
		labelAttr: ProcessConstants.ExtensionDTO.NAME,
		childrenAttrs: [ProcessConstants.ExtensionDTO.ITEMS],
		sorted: false,
		
		/**
		 * @param root {dojo.data.Item} operation item to model behavior for,
		 * 			  required
		 * @param store {dojo.data.Store} underlying store, one of
		 * 			  configurationStore or extensionStore
		 * @param configurationStore {OperationBehaviorConfigurationStore}
		 * 			  optional, but extensionStore needed if not specified
		 * @param extensionStore {OperationBehaviorExtensionStore}
		 * 			  optional, but configurationStore needed if not specified
		 */
		constructor: function(args) {
			// store reference to root item, to use to determine children, etc.
			this._item = this.root;
			// we don't actually use the given root as the root of the tree,
			// since this item can be replaced with another, and dijit.Tree does
			// not support having its model's root replaced
			this.root = "#$#root#$#"; //$NON-NLS-1$
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connects.push(dojo.connect(this.store, "onUpdateConfiguration", this, "_onUpdateConfiguration")); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		// =====================================================================
		// Events from data store; only react to those affecting this model's
		// item's hierarchy
		
		_onUpdateConfiguration: function(oldItem, newItem) {
			if (this._item === oldItem && this._item !== newItem) {
				// update this model's item reference
				this._item = newItem;
				this.getChildren(this.root, dojo.hitch(this, function(children) {
					this.onChildrenChange(this.root, children);
				}));
			}
		},
		
		onNewItem: function(item, parentInfo) {
			if (this._inHierarchy(item)) {
				this.getChildren(this.root, dojo.hitch(this, function(children) {
					this.onChildrenChange(this.root, children);
				}));
			}
		},
		
		onDeleteItem: function(item) {
			// do nothing, when this item is removed from its parent's children
			// we will update the tree. We can't determine if this item was in
			// the hierarchy anyways, because the store no longer recognizes it
		},
		
		onSetItem: function(item, attribute, oldValue, newValue) {
			if (this._inHierarchy(item)) {
				if (attribute === ProcessConstants.BehaviorConfigurationDTO.CONFIGURED || attribute === this.childrenAttrs[0]) {
					this.getChildren(this.root, dojo.hitch(this, function(children) {
						this.onChildrenChange(this.root, children);
					}));
				} else {
					this.onChange(item);
				}
			}
		},
		
		// =====================================================================
		// dijit.TreeStoreModel methods; account for the fact that the root is
		// just a placeholder, and that some child items may just be strings
		
		getChildren: function(parentItem, callback, onError) {
			if (parentItem === this.root) {
				callback(this._getRootChildren());
			} else if (dojo.isString(parentItem)) {
				callback([]);
			} else {
				this.inherited(arguments);
			}
		},
		
		mayHaveChildren: function(item) {
			if (item === this.root) {
				return true;
			} else if (dojo.isString(item)) {
				return false;
			}
			return this.inherited(arguments);
		},
		
		getIdentity: function(item) {
			if (dojo.isString(item)) {
				return item;
			}
			return this.inherited(arguments);
		},
		
		getLabel: function(item) {
			if (dojo.isString(item)) {
				return item;
			}
			return this.inherited(arguments);
		},
		
		// =====================================================================
		
		_inHierarchy: function(item) {
			if (item === this._item) {
				return true;
			} else if (item === null) {
				return false;
			} else {
				return this._inHierarchy(this.store.getParent(item));
			}
		},
		
		_getRootChildren: function() {
			var preconditions = this.store.getPreconditions(this._item);
			var followups = this.store.getFollowupActions(this._item);
			
			if (this.sorted) {
				//alphabetical sort
				preconditions.sort(function(a,b){return ((a.name == undefined) ? a.id : a.name) > ((b.name == undefined) ? b.id : b.name)});
				followups.sort(function(a,b){return ((a.name == undefined) ? a.id : a.name) > ((b.name == undefined) ? b.id : b.name)});
			}
			
			if (preconditions.length > 0 && followups.length > 0) {
				// add in the sub-heading items (strings)
				preconditions.splice(0, 0,  this.messages.OperationBehaviorTree_0);
				followups.splice(0, 0,  this.messages.OperationBehaviorTree_1);
			}
			var children = preconditions.concat(followups);
			if (children.length === 0) {
				// show some text if there are no children
				if (this.emptyText !== undefined) {
					children.push(this.emptyText);
				} else {
					var preconditionsPossible = preconditions.length > 0;
					var followupsPossible = followups.length > 0;
					if (this.store === this.configurationStore && this.extensionStore) {
						// check the extension store for behaviors too
						var extension = this.extensionStore.fetchItemByIdentity(this.store.getValue(this._item, ProcessConstants.ExtensionDTO.ID));
						preconditionsPossible = preconditionsPossible || this.extensionStore.getPreconditions(extension).length > 0;
						followupsPossible = followupsPossible || this.extensionStore.getFollowupActions(extension).length > 0;
					}
					if (preconditionsPossible && !followupsPossible) {
						children.push(this.messages.OperationBehaviorTree_3);
					} else if (!preconditionsPossible && followupsPossible) {
						children.push(this.messages.OperationBehaviorTree_4);
					} else {
						children.push(this.messages.OperationBehaviorTree_2);
					}
				}
			}
			return children;
		}
		
	});
})();
