/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataClient"); //$NON-NLS-1$
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");//$NON-NLS-1$
    var OsgiBasedConfigurationDataClient = com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataModel", null, { //$NON-NLS-1$

		// A list of configuration data elements.
		// Access the element by id, for example: configurationDataElements[configurationDataId]
		// Each element has the following data structure:
		//  {
		//   configurationDataId: the id of this configuration data
		//   isFinal: if this configuration data is final or not
		//   isInherited: if this configuration data is inherited from other process provider or not
		//   isFinalStatusChanged: if the final status of this configuration data is changed or not, 
		//                         sometimes, only the final status is changed while content is NOT changed.  
		//   isDeleted: if this configuration data has been ed or not
		//   savedContent: the original configuration data content retrieved from repository,
		//   unsavedContent: the latest modified configuration data content collected from the editor
		//   }
		
		constructor: function(args) {
			this._configurationDataElements = [];
		},
		
		/**
		 * Get an array of configuration data element for the given project area and configuration data ids
		 * 
		 * @param {String} projectAreaUUID
		 * @param {Object} configurationDataIds
		 * @param {Object} callbackContext
		 * @param {String} successCallbackName
		 * @param {String} errorCallbackName
		 */
		getConfigurationDataElements: function(projectAreaUUID, configurationDataIds, callbackContext, successCallbackName, errorCallbackName) {
			var returnValue = [];
			var newIds = [];
			for (var i = 0; i < configurationDataIds.length; i++) {
				var cachedElement = this._getConfigurationDataElement(configurationDataIds[i]);
				if (cachedElement) {
					returnValue.push(cachedElement);
				} else {
					// collect all the configuration data that are NOT stored in the cache
					newIds.push(configurationDataIds[i]);
				}
			}
			if (newIds.length > 0) {
				// if there are some configuration data are not stored in the cache
				// we need to retrieve these data from server and then store in the cache
				var getConfigurationDataObject = {
					success: dojo.hitch(this, function(response){
						if (response) {
							var elements = dojo.fromJson(response);
							for (var j = 0; j < elements.length; j++) {
								var newCachedElement = {};
								newCachedElement.configurationDataId = elements[j].configurationDataId;
								newCachedElement.savedContent = elements[j].configurationDataContent;
								newCachedElement.isFinal = elements[j].isFinal;
								newCachedElement.isInherited = elements[j].isInherited;
								newCachedElement.isFinalStatusChanged = false;
								newCachedElement.isDeleted = false;
								// add to cache
								this._configurationDataElements.push(newCachedElement);
								// add to returnValue
								returnValue.push(newCachedElement);
							}
							callbackContext[successCallbackName].call(callbackContext, returnValue);
						}
					}),
					failure: dojo.hitch(this, function(error){
						// call the errorCallback
						if (errorCallbackName) {
							callbackContext[errorCallbackName].call(callbackContext, error);
						}
					})
				};
				
				// retrieve these configuration data from server
				var srh = new ServiceResponseHandler(getConfigurationDataObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
				var args = {};
				args.configurationDataIds = newIds;
				args.projectAreaUUID = projectAreaUUID;
				OsgiBasedConfigurationDataClient.getConfigurationData(args, srh);
			} else {
				callbackContext[successCallbackName].call(callbackContext, returnValue);
			}
		},
		
		/**
		 * Get an array of only the content of the given configuration data ids
		 * (excludes additional info provided by #getConfigurationDataElements).
		 * Each element is a JSON object, the format is:
		 * 	{ "configurationDataId":<configurationDataId>, 
		 *    "configurationDataContent": <changedConfigurationDataContent>
		 *  }
		 * 
		 * @param {String} projectAreaUUID
		 * @param {Object} configurationDataIds
		 * @param {Object} callbackContext
		 * @param {String} successCallbackName
		 * @param {String} errorCallbackName
		 */
		getConfigurationDataContent: function(projectAreaUUID, configurationDataIds, callbackContext, successCallbackName, errorCallbackName) {
			var getConfigurationDataContentObject = {
				success: dojo.hitch(this, function(configDataElementList){
					var configurationDataContentList = [];
					// We just only need to pass the content of configuration
					// data to editor because there is no need to let the editor
					// knows "isFinal" or "isInherited" or other attributes.
					// We also need store the editor's status, such as
					// isFinalChecked, isDataInherited
					for (var i = 0; i < configDataElementList.length; i++) {
						var content = {};
						content.configurationDataId = configDataElementList[i].configurationDataId;
						if (configDataElementList[i].isDeleted) {
							// if this element has been deleted,
							// we just pass the null object
							content.configurationDataContent = null;				
						} else {
							content.configurationDataContent = configDataElementList[i].savedContent;
							if (configDataElementList[i].unsavedContent) {
								content.configurationDataContent = configDataElementList[i].unsavedContent;
							}
						}
						configurationDataContentList.push(content);
					}
					callbackContext[successCallbackName].call(callbackContext, configurationDataContentList);
				}),
				failure: dojo.hitch(this, function(error){
					// call the errorCallback
					if (errorCallbackName) {
						callbackContext[errorCallbackName].call(callbackContext, error);
					}
				})
			};
			this.getConfigurationDataElements(projectAreaUUID, configurationDataIds, getConfigurationDataContentObject, "success", "failure"); //$NON-NLS-1$  //$NON-NLS-2$ 
		},
		
		/**
		 * Get an array of configuration data element from the cached model for the given configurationDataIds
		 * 
		 * @param {Object} configurationDataIds
		 */
		getConfigurationDataElementsDirectlyFromCache: function(configurationDataIds) {
			// Retrieve the elements from cache directly
			// So, make sure all your interested elements have been cached before you invoke this method.
			var returnValue = [];
			for (var i = 0; i < configurationDataIds.length; i++) {
				var element = this._getConfigurationDataElement(configurationDataIds[i]);
				if (element) {
					returnValue.push(element);
				}
			}
			return returnValue;
		}, 
		
		/**
		 * Get configuration data element for the given configuration data id from the cached model
		 * 
		 * @param {Object} id
		 */
		_getConfigurationDataElement: function(id) {
			for (var i = 0; i < this._configurationDataElements.length; i++) {
				if (this._configurationDataElements[i].configurationDataId === id) {
					return this._configurationDataElements[i];
				}
			}
			return null;
		},
		
		/**
		 * Update the configuration data in the model with the latest modified content
		 * @param {Object} unSavedConfigurationDataList an array of configuration data that has changed configuration data content
		 *    Each element is an JSON object, the format is:
		 *    { "configurationDataId":<configurationDataId>, 
		 *  	"configurationDataContent": <changedConfigurationDataContent>
		 *    }
		 */
		updateUnsavedConfigurationData: function(unSavedConfigurationDataList) {
			for (var i = 0; i < unSavedConfigurationDataList.length; i++) {
				var unSavedConfigurationDataJsonObj = unSavedConfigurationDataList[i];
				var id = unSavedConfigurationDataJsonObj.configurationDataId;
				var element = this._getConfigurationDataElement(id);
				if (element) {
					element.unsavedContent = unSavedConfigurationDataJsonObj.configurationDataContent;
				}
			}
		},
		
		/**
		 * Get all the configuration data from the model that has latest modified configuration data
		 */
		getUnsavedConfigurationDataList: function() {
			var unsavedConfigurationData = [];
			for (var i = 0; i < this._configurationDataElements.length; i++) {
				var element = this._configurationDataElements[i];
				if (element.isDeleted) {
					if (element.isInherited) {
						// if user just delete the customization of inherited configuration data
						// we don't need to do anything
						continue;
					}
					// if this element has been deleted, other attributes will be ignored.
					var deletedElement = {};
					deletedElement.configurationDataId = element.configurationDataId;
					deletedElement.isDeleted = true;
					deletedElement.isInherited = element.isInherited;
					unsavedConfigurationData.push(dojo.toJson(deletedElement));
				} else if (element.unsavedContent || element.isFinalStatusChanged) {
					// Sometimes, the content is not changed, but the final status has been changed.
					// In this case, we also need to store the final status to process specification.
					var unsavedElement = {};
					unsavedElement.configurationDataId = element.configurationDataId;
					unsavedElement.isDeleted = false;
					unsavedElement.isFinal = element.isFinal;
					unsavedElement.isInherited = element.isInherited;
					unsavedElement.isDeleted = element.isDeleted;
					unsavedElement.configurationDataContent = element.unsavedContent;
					if (!unsavedElement.configurationDataContent) {
						unsavedElement.configurationDataContent = "";//$NON-NLS-1$ 
					}
					unsavedConfigurationData.push(dojo.toJson(unsavedElement));
				}
			}
			return unsavedConfigurationData;
		}, 
		
		/**
		 * Restore the given configuration data elements in the model to the original status.
		 * 
		 * @param {Object} configurationDataIds the array of id of the configuration data elements that need to be restored.
		 */
		restoreConfigurationDataElements: function(configurationDataIds) {
			for (var i = 0; i < configurationDataIds.length; i++) {
				var cachedElement = this._getConfigurationDataElement(configurationDataIds[i]);
				if (cachedElement) {
					if (cachedElement.isFinalStatusChanged) {
						// roll back the final status
						cachedElement.isFinal = !cachedElement.isFinal;
					}
					cachedElement.isFinalStatusChanged = false;
					cachedElement.isDeleted = false;
					cachedElement.unsavedContent = null;
				} 
			}
		},
		
		/**
		 * Empty the cached model
		 */
		clear: function() {
			this._configurationDataElements = [];
		}
	});
	
})();
