/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByActionEditor"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsActionsTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForActionTable"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy"); //$NON-NLS-1$)
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore"); //$NON-NLS-1$)
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var PermittedType = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType;
	
	/**
	 * Sub-editor in the Permissions tab that is displayed when the option to
	 * show permissions by operation is selected. Presents a tree of operations
	 * to pick from. When one is clicked, its details and a table of roles
	 * are presented where the permissions for each role can be edited.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByActionEditor", [dijit._Widget], { //$NON-NLS-1$
		
		/**
		 * @param {Object} An object with the following fields:
		 * 
		 * processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
		 * 						the Permissions tab
		 * processAreaStore		Store of artifacts related to project or team,
		 * 						used for time scopes such as timelines and
		 * 						iteration types
		 * workingCopy          Working copy of the permissions
		 * roleSelectionChangedCallback Function that accepts a role item that
		 * 								will be invoked when the current
		 * 								selected role for the permissions
		 * 								editor changes. For this sub-editor,
		 * 								the argument will always be 'null'.
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * This widget follows the structure required for sub-editors of the
		 * Permissions editor: two fields "firstColumn" and "secondColumn"
		 * that are widgets. Each has a field "initialWidth" which is used
		 * for their initial layout. Creation of the widgets is deferred 
		 * until the permission extensions have been loaded
		 * 
		 * See function _createSubEditor in PermissionsEditor for how these
		 * fields are incorporated.
		 */
		postCreate: function() {
			this.firstColumn = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByActionActionsColumn({
				messages: this.messages,
				processAreaEditor: this.processAreaEditor,
				processAreaStore: this.processAreaStore,
				workingCopy: this.workingCopy,
				editorState: this.editorState,
				initialWidth: "45%" //$NON-NLS-1$
			});
			
			this.secondColumn = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByActionPermissionsColumn({
				messages: this.messages,
				processAreaEditor: this.processAreaEditor,
				workingCopy: this.workingCopy,
				processAreaStore: this.processAreaStore,
				initialWidth: "54%" //$NON-NLS-1$
			});
			this.workingCopy.loadPermissionExtensions(dojo.hitch(this, this._extensionsLoaded), dojo.hitch(this, this._extensionsLoadFailed));
			
			this.roleSelectionChangedCallback(null);	
		},
		
		/**
		 * Called by Permissions editor when it is being cleared for a refresh
		 * or state. Updates a memento with any state that should be persisted
		 * after the editor is re-loaded, such as the selected action.
		 */
		saveState: function(editorState) {
			this.firstColumn.saveState(editorState);
		},
		
		startLoading: function() {
			//If necessary, disable controls that would result in calls to
			//the working copy when it is already loading data
		},
		
		stopLoading: function() {
			//Enable controls that were disabled in startLoading
		},
			
		_extensionsLoaded: function(extensionStore) {
			this._extensionStore = extensionStore;
			this.secondColumn.setExtensionStore(extensionStore);
			this.firstColumn.setFocusChangedCallout(dojo.hitch(this.secondColumn, this.secondColumn.actionSelected));
			this.firstColumn.setExtensionStore(extensionStore);
		},
		
		_extensionsLoadFailed:function(error) {
			this.processAreaEditor.handleServerResponseError(error, this.messages.PermissionsByActionEditor_3);
		},
		
		/**
		 * Will be called by Permissions editor when a time period, including
		 * "Always", is selected to edit permissions for. The passed item must
		 * be an item in the store processAreaStore that was passed to this
		 * widget's constructor. 
		 */
		timePeriodSelected: function(scopeItem) {
			this.firstColumn.timePeriodSelected(scopeItem);
			this.secondColumn.timePeriodSelected(scopeItem);
		},
		
		/**
		 * Will be called by Permissions editor when the configured permissions
		 * per role and time period have been loaded, i.e. functions in the
		 * working copy like getConfiguredRoles() are ready to be called.
		 */
		configuredScopesLoaded: function() {
		},
		
		/**
		 * Called by enclosing tab PermissionsEditor when either:
		 * 
		 * 1. The Permissions editor is navigated to and this sub-editor
		 *    is the active one
		 * 2. This sub-editor is made the active one in the Permissions editor
		 * 
		 * The sub-editor will react to changes made outside of itself or
		 * of the Permissions editor, (eg. roles added, removed, ...), in ways
		 * that would not be appropriate immediately when the changes were
		 * made, (eg. because loading from the server could be required).
		 */
		editorShown: function() {
			this.secondColumn.editorShown();
			
			this.roleSelectionChangedCallback(null);
		}
		
		//There is no need to override uninitialize to destroy the columns.
		//They are added to containers that destroy them on uninitialization
	});
	
	/**
	 * @param {Object} An object with the following fields:
	 * 
	 * processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
	 * 						the Permissions tab
	 * messages             Translatable messages
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByActionActionsColumn", [dijit._Widget], { //$NON-NLS-1$
		constructor: function(args) {
			this._selectedTimePeriod = args.processAreaStore.alwaysRootItem;
		},
		
		postCreate: function() {
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsByActionActionsColumn"); //$NON-NLS-1$
		},
		
		/**
		 * Sets a callout to be used when the focus changed
		 * @param focusChanged(item) a function that is passed an item in
		 * extensions store
		 */
		setFocusChangedCallout: function(callout)
        {
			this.focusChangedCallout = callout;
        },	
        
		/**
		 * Sets the store which will be used to display the tree of actions
		 * @param extensionStore    A store containing the permission extensions
		 */
        setExtensionStore: function(extensionStore) {
        	this._actionsTree = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsActionsTree({
        		messages: this.messages,
        		store: extensionStore,
        		workingCopy: this.workingCopy,
        		headingText: this.messages.PermissionsByActionEditor_1,
        		focusChangedCallout: this.focusChangedCallout,
        		processAreaStore: this.processAreaStore,
        		initialTimePeriod: this._selectedTimePeriod,
        		editorContext: this.processAreaEditor.context,
        		editorState: this.editorState
        	});
        	this.domNode.appendChild(this._actionsTree.domNode);
        },
		
		resized: function() {
			if (this._actionsTree && this._actionsTree.resized) {
				this._actionsTree && this._actionsTree.resized();
			}
		},
		
		timePeriodSelected: function(scopeItem) {
			if (scopeItem !== null) {
				this._selectedTimePeriod = scopeItem;
				if (this._actionsTree) {
					this._actionsTree.timePeriodSelected(this._selectedTimePeriod);
				}
			}
		},
		
		saveState: function(editorState) {
			this._actionsTree.saveState(editorState);
		},
		
        uninitialize: function() {
        	if (this._actionsTree) {
			    this._actionsTree.destroyRecursive();
			    delete this._actionsTree;
        	}
		}
	});
	
	/**
	 * @param {Object} An object with the following fields:
	 * 
	 * processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
	 * 						the Permissions tab
	 * messages             Translatable messages
	 * workingCopy          A PermissionsWorkingCopy object which contains the permissions state
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByActionPermissionsColumn", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
			templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/PermissionsByActionPermissionsColumn.html"), //$NON-NLS-1$ //$NON-NLS-2$	
		
	   /**
        * @param {Object} An object with the following fields:
		* 
		* processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
		* 						the Permissions tab
		* processAreaStore		Store of artifacts related to project or team,
		* 						used for time scopes such as timelines and
	    * 						iteration types
		* workingCopy          Working copy of the permissions
	    */
		constructor: function(args) {
			this._selectedTimePeriod = args.processAreaStore.alwaysRootItem;
	    },
			
		postCreate: function() {
			this.connect(this.processAreaEditor.getRolesStore(), "onNew", this._rolesChanged); //$NON-NLS-1$
			this.connect(this.processAreaEditor.getRolesStore(), "aboutToDelete", this._rolesChanged); //$NON-NLS-1$
			this.connect(this.processAreaEditor.getRolesStore(), "switchOrder", this._rolesChanged); //$NON-NLS-1$
			this.connect(this.processAreaEditor.getRolesStore(), "onSet", this._rolesChanged); //$NON-NLS-1$
		},
		
		resized: function() {
			setTimeout(dojo.hitch(this, function() {
				if (this.domNode) {
					this.heightChanged();
				}
			}), 0);
		},
		
		/**
		 * Sets the store of extensions
		 * @param extensionStore
		 */
		setExtensionStore: function(extensionStore) {
			this._extensionStore = extensionStore;
			this._createPermissionsTableToolbar();
			this._createPermissionsTable();
		},
		
		/**
		 * Method that is called when focus changed in the actions tree
		 * @param item an item in the extensions store
		 */
		actionSelected: function(item) {
			this._selectedActionItem = item;
			
			if (item) {
				this._selectedActionId = this._extensionStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
			} else {
				this._selectedActionId = null;
			}
		    		   
			this._updatePermissionsTableAndDescriptionSection();
		},
		
		/**
		 * Will be called when a time period is selected
		 */
		timePeriodSelected: function(scopeItem) {
			this._selectedTimePeriod = scopeItem;
			if (this._selectedTimePeriod !== null) {
				this._updatePermissionsTableAndDescriptionSection();
			}
		},
		
		/**
		 * See editorShown in PermissionsByActionEditor above.
		 */
		editorShown: function() {
			if (this._tableUpdateRequired) {
				this._permissionsTable.destroyRecursive();
				dojo.empty(this.permissionsByActionPermissionsTable);
				this._createPermissionsTable();				
				this._updatePermissionsTableAndDescriptionSection();
				
				delete this._tableUpdateRequired;
			}
		},
		
		_rolesChanged: function() {
			this._tableUpdateRequired = true;
		},
		
		_getActionId: function(row) {
			return this._selectedActionId;
		},
		
		_getContextItem: function(row) {
			return this._selectedTimePeriod;
		},
		
		/*
		 * Show the name and description of the selected action/operation/category and
		 * hides and reveals the appropriate sections depending on whether a category was selected
         * Update the permissions table to match the selected action and time period
		 */
		_updatePermissionsTableAndDescriptionSection : function() {
			if (this._selectedActionItem) {
			    var name = this._extensionStore.getValue(this._selectedActionItem, ProcessConstants.ExtensionDTO.NAME);
			    var description = this._extensionStore.getValue(this._selectedActionItem, ProcessConstants.ExtensionDTO.DESCRIPTION);
	            var isCategory  = this.workingCopy.isCategory(this._selectedActionId);
				var isOperation  = !isCategory && this.workingCopy.isOperation(this._selectedActionId); 
				dojo.removeClass(this.permissionsByActionNameAndDescription, "hidden"); //$NON-NLS-1$
				dojo.addClass(this.permissionsByActionNothingSelected, "hidden"); //$NON-NLS-1$
				dojo.empty(this.permissionsByActionKind);
				dojo.toggleClass(this.permissionsByActionCategorySelected, "hidden", !isCategory); //$NON-NLS-1$
				dojo.toggleClass(this.permissionsByActionPermissionsTableHeading, "hidden", isCategory); //$NON-NLS-1$
				dojo.toggleClass(this.permissionsByActionPermissionsTable, "hidden", isCategory); //$NON-NLS-1$
				// Display whether this is an action, operation or category
				if (isCategory) {
					this.permissionsByActionKind.appendChild(document.createTextNode(this.messages.PermissionsByActionEditor_6)); 
				} else {
					var label = isOperation ? this.messages.PermissionsByActionEditor_5 : this.messages.PermissionsByActionEditor_4;  
					this.permissionsByActionKind.appendChild(document.createTextNode(label));  
				}
				// Display the action name
				dojo.empty(this.permissionsByActionName);
				this.permissionsByActionName.appendChild(document.createTextNode(name)); 
				// Update the description container visibility and the description text
				dojo.empty(this.permissionsByActionDescription);
				var hasDescription = description !== null && description.length > 0;
				dojo.toggleClass(this.permissionsByActionDescriptionContainer, "hidden", !hasDescription); //$NON-NLS-1$
				if (hasDescription) {
					this.permissionsByActionDescription.appendChild(document.createTextNode(description)); 
				}
				// Display the timeframe if set
				dojo.empty(this.permissionsByActionTimeframe);
				var isTimeframeAlways = this._selectedTimePeriod === this.processAreaStore.alwaysRootItem;
				dojo.toggleClass(this.permissionsByActionTimeframeContainer, "hidden", isTimeframeAlways); //$NON-NLS-1$
				if (!isTimeframeAlways && this._selectedTimePeriod) {
					var selectedContext = this.processAreaStore.getValue(this._selectedTimePeriod, "enabledContext"); //$NON-NLS-1$
					var scopeLabel = this.processAreaStore.getValue(this._selectedTimePeriod, "name"); //$NON-NLS-1$
					if (ProcessConstants.PermissionContext.ITERATION_TYPE === 
						selectedContext) {
						var scopeText = dojo.string.substitute(this.messages.PermissionsByActionEditor_7, [scopeLabel]);
						this.permissionsByActionTimeframe.appendChild(document.createTextNode(scopeText));
					} else {
						this.permissionsByActionTimeframe.appendChild(document.createTextNode(scopeLabel)); 
					}
				}
				if (!isCategory && this._selectedTimePeriod) {
				    this._permissionsTable.reloadPermissions(dojo.hitch(this, function() {
			            if (this._permissionsTable && this._permissionsTable.resizeHeader && !this._permissionsTable.headerResized) {
						    this._permissionsTable.resizeHeader(true);
						    this._permissionsTable.headerResized = true;
					    }
				    	//resized() must be called after permissions are
						//loaded, because changes to the rows to show permissions
						//may have impacted their height
						this.resized();
						// Sort the table if it was sorted by permission
						if (this._permissionsTable.sortBy === 1) {
						    this._permissionsTable.handleSort();
				        }
						this._determineGrantAndRevokeAllEnablement();
					}));
			    }
			} else {
				dojo.addClass(this.permissionsByActionNameAndDescription, "hidden"); //$NON-NLS-1$
				dojo.addClass(this.permissionsByActionPermissionsTableHeading, "hidden"); //$NON-NLS-1$
				dojo.addClass(this.permissionsByActionPermissionsTable, "hidden"); //$NON-NLS-1$
				dojo.addClass(this.permissionsByActionCategorySelected, "hidden"); //$NON-NLS-1$
				dojo.removeClass(this.permissionsByActionNothingSelected, "hidden"); //$NON-NLS-1$
			}
			this.resized();
		},
		
		_determineGrantAndRevokeAllEnablement: function() {
			var actionId = this._getActionId();
			var rolesStore = this.processAreaEditor.getRolesStore();
			var roles = rolesStore.getAllRoleItems(); 			
			// Check for any roles with this permission inherited			
		
			var allRolesInherited = true;
			for (var roleIndex = 0; roleIndex < roles.length && allRolesInherited; roleIndex++) {
				var roleId = rolesStore.getValue(roles[roleIndex], ProcessConstants.ProcessRoleDTO.ID);
				var permissionsStore = this.workingCopy.getPermissionsStore(roleId, this._selectedTimePeriod);
				var permissionForRole = permissionsStore.getPermission(
					permissionsStore.getPermissionItem(actionId),
					this._getContextItem()
				);
				allRolesInherited = (permissionForRole === PermittedType.INHERITED);				
			}
			this._enableGrantAndRevokeAll(!allRolesInherited);
		},
		
		_enableGrantAndRevokeAll: function(enabled) {
			this._grantAllButton.set("enabled", enabled); //$NON-NLS-1$
			this._revokeAllButton.set("enabled", enabled); //$NON-NLS-1$
		},
		
		_createPermissionsTable: function() {						
			this._permissionsTable = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForActionTable({
				workingCopy: this.workingCopy,
				heightChanged: dojo.hitch(this, this.heightChanged),
				permissionChangedInTable: dojo.hitch(this, this._permissionChangedInTable),
				processAreaEditor: this.processAreaEditor,
				processAreaStore: this.processAreaStore,
				getActionId: dojo.hitch(this, this._getActionId),
				getContextItem: dojo.hitch(this, this._getContextItem)
			});

			this.permissionsByActionPermissionsTable.appendChild(this._permissionsTable.domNode);
			this.heightChanged();
		},
		
		_createPermissionsTableToolbar: function() {
			this._permissionsTableToolbar = new jazz.ui.Toolbar();
			this.permissionsByActionPermissionsTableHeading.appendChild(this._permissionsTableToolbar.domNode);
			
			this._grantAllButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsForActionTable_2,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/grant-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, this._grantForAllRoles)
			});
			this._permissionsTableToolbar.addChild(this._grantAllButton);
			
			this._revokeAllButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsForActionTable_3,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/revoke-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, this._revokeForAllRoles)
			});
			this._permissionsTableToolbar.addChild(this._revokeAllButton);
		},
		
		_grantForAllRoles: function() {
			this._changePermissionForAllRoles(PermittedType.PERMITTED);
		},
		
		_revokeForAllRoles: function() {
			this._changePermissionForAllRoles(PermittedType.NOT_PERMITTED);
		},
		
		_changePermissionForAllRoles: function(permission) {
			var actionId = this._getActionId();
			var rolesStore = this.processAreaEditor.getRolesStore();
			var roles = rolesStore.getAllRoleItems(); 		
			this.workingCopy.startBatchUpdate();
						
			var name = this._extensionStore.getValue(this._selectedActionItem, ProcessConstants.ExtensionDTO.NAME);
			dojo.forEach(roles, dojo.hitch(this, function(role) {
				var roleId = rolesStore.getValue(role, ProcessConstants.ProcessRoleDTO.ID);
				var permissionsStore = this.workingCopy.getPermissionsStore(roleId, this._selectedTimePeriod);
				var permissionItem = permissionsStore.getPermissionItem(actionId);
				var currentPermission = permissionsStore.getPermission(
					permissionItem,
					this._getContextItem()
				);
				// Permissions which are inherited are left unchanged
				if (currentPermission !== PermittedType.INHERITED) {				
				    permissionsStore.setPermission(
						permissionItem,
						actionId,
						this._getContextItem(),
						permission
					);
			    }

			}));
			
			this.workingCopy.endBatchUpdate();
		},
		
		/*
		 * Called when one of the permissions for the selected operation and context is changed
		 */
		_permissionChangedInTable: function(permission) {
			if (permission === PermittedType.INHERITED) {
				this._determineGrantAndRevokeAllEnablement();
			} else {
				this._enableGrantAndRevokeAll(true);
			}
		},
		
		/**
		 * The container of this sub-editor listens to invocations of this
		 * function via dojo.connect. heightChanged() should be called whenever
		 * this widget requires more or less vertical space to display all
		 * contents without scrolling.
		 */
		heightChanged: function() {
		},
		
        uninitialize: function() {
        	if (this._permissionsTableToolbar) {
			    this._permissionsTableToolbar.destroyRecursive();
			    delete this._permissionsTableToolbar;
        	}
        	
        	if (this._permissionsTable) {
			    this._permissionsTable.destroyRecursive();
			    delete this._permissionsTable;
        	}
		}
	});
})();
