/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessProviderDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.SearchBox"); //$NON-NLS-1$
dojo.require("jazz.ui.ListBox"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
//STATUS MESSAGE TYPES allowed to be used in getProcessProviderContainerInfo() for
// ProcessProviderContainerInfoDTO (Keep in sync with ProcessWebUIService.java).
var PROVIDER_CONTAINER_OK                        = 0;
var PROVIDER_CONTAINER_ERROR_INCOMPATIBLE_TYPE   = 1;
var PROVIDER_CONTAINER_ERROR_INCOMPATIBLE_VERSION= 2;
var PROVIDER_CONTAINER_ERROR_CONNECTION_PROBLEM  = 4;

var ICON_OK = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/success_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var ICON_WARNING = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/warning.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var ICON_ERROR = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/error_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessProviderDialog", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ProcessProviderDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages");//$NON-NLS-1$ //$NON-NLS-2$	
			this._owningApplicationKey = args.owningApplicationKey;
			this._projectAreaItemId = args.projectAreaItemId;
			this._isEmbedded = args.isEmbedded;
		},
		
		postCreate: function(){
			this._processProviderContainerSelector.iconMap[PROVIDER_CONTAINER_OK] = ICON_OK;
			this._processProviderContainerSelector.iconMap[PROVIDER_CONTAINER_ERROR_INCOMPATIBLE_VERSION] = ICON_ERROR;
			this._processProviderContainerSelector.iconMap[PROVIDER_CONTAINER_ERROR_CONNECTION_PROBLEM] = ICON_WARNING;
			this._processProviderContainerSelector.onClick = dojo.hitch(this, this._handleProcessProviderContainerSelection);
			this._processSelector.onClick = dojo.hitch(this, this._handleProcessProviderSelection);
			
			this._createSearchBox();
			this._getProcessProviderContainers();
		},
		
		startup: function(){
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			// not ideal, but need to do this to make the search box focusable
			this._searchBox._setScope({
				scope: this._searchBox.scopes[0],
				focusText: true
			});
			
			this.inherited(arguments);
		},

		/**
		 * Gets the list of all applications registered as friends with this
		 * application. Always filters out internal friends and this application's
		 * configured JTS.
		 */
		_getProcessProviderContainers: function(){
			this._showMessage(this.messages.ProcessProviderDialog_FetchingProcessProviderContainers);
			var args = {};

			var callback = "_populateProcessProviderContainersInitial"; //$NON-NLS-1$
			var srh = new ServiceResponseHandler(this, callback, "_handleFetchError"); //$NON-NLS-1$
            ProcessClient.getProcessProviderContainers(args, srh);
		},
		
		/**
		 * Gets the list of applications registered as friends with this
		 * application, whose title or rootservices location's hostname contain
		 * the given filter. Returns all friends if an undefined/empty filter is
		 * given. Always filters out internal friends and this application's
		 * configured JTS.
		 * 
		 * @param filter {String} the string to query with
		 */
		_getFilteredProcessProviderContainers: function(filter){
			var args = {};
			delete this._newFilteredProviderContainers;
			if (filter && filter !== "") { //$NON-NLS-1$
				args.filter = filter;
			}
			var callback = "_filterProcessProviderContainers"; //$NON-NLS-1$ 
			var srh = new ServiceResponseHandler(this, callback, "_handleFetchError"); //$NON-NLS-1$
            ProcessClient.getProcessProviderContainers(args, srh);
		},
		
		/**
		 * Populates the application list with retrieved provider containers.
		 * 
		 * @param containers
		 *            {ProcessProviderContainerDTO[]}
		 */
		_populateProcessProviderContainersInitial: function(containers) {
			if (containers) {
				this._newProviderContainers = dojo.map(containers, function(container) {
					container.displayName = dojo.string.substitute(this.messages.ProcessProviderDialog_ProviderContainerName, [container.name, container.hostName]);
					return container;
				}, this);

				this._newProviderContainers.currentIndex = 0;

				this._newProviderContainers.sort(function(a, b) {
					if (a.local) {
						return -1;
					}
					if (b.local) {
						return 1;
					}
					return a.displayName.localeCompare(b.displayName);
				});
				this._retrieveProviderContainers();
			} 
		},
		
		/**
		 * Updates populated by _populateProcessProviderContainersInitial()
		 *  application list with retrieved filtered provider containers.
		 * 
		 * @param containers
		 *            {ProcessProviderContainerDTO[]}
		 */
		_filterProcessProviderContainers: function(containers) {
			if (containers) {
				this._newFilteredProviderContainers = dojo.map(containers, function(container) {
					container.displayName = dojo.string.substitute(this.messages.ProcessProviderDialog_ProviderContainerName, [container.name, container.hostName]);
					return container;
				}, this);
			} 

			this._filterDisplayedProviderContainers();
		},

		_isNewFilteredProviderContainer: function(containerId) {
			if (this._newFilteredProviderContainers) {
				for ( var i = 0; i < this._newFilteredProviderContainers.length; i++) {
					if (this._newFilteredProviderContainers[i].displayName === containerId) {
						return true;
					}
				}
			}
			return false;
		},

		_filterDisplayedProviderContainers: function() {
			if (this._processProviderContainerSelector) {

				var allEntries = this._processProviderContainerSelector.getAllEntries();
				var selected = this._processProviderContainerSelector.getSelection();
				var selectedId = (selected.length > 0) ? selected[0].id : ""; //$NON-NLS-1$
				var visibleEntry = false;
				var keepSelection = false;

				for ( var i = 0; i < allEntries.length; i++) {
					var entry = allEntries[i];
					if (this._isNewFilteredProviderContainer(entry.id)) {
						visibleEntry = true;
						this._processProviderContainerSelector.showEntry(entry, true);
						if (selectedId === entry.id) {
							keepSelection = true;
						}
					} else {
						this._processProviderContainerSelector.showEntry(entry, false);
					}
				}
				this._processProviderContainerSelector.setSelection(selected, keepSelection);

				if (visibleEntry) {
					if(keepSelection){
						this._showMessage(this.messages.ProcessProviderDialog_Prompt);
					} else {
						this._showMessage(this.messages.ProcessProviderDialog_0);
						this._clearProcessSelector();					
					} 
				} else {
					this._showMessage(this.messages.ProcessProviderDialog_NoApplicationsFound);
					this._clearProcessSelector();					
				}
			}
		},
		
		_clearProcessSelector: function() {
			this._processSelector.removeAllEntries();
			this.okButton.disabled = true;
		},
		
		_retrieveProviderContainers: function() {
			if (this._newProviderContainers) {
				if (this._newProviderContainers.currentIndex < this._newProviderContainers.length) {
					var providerContainer = this._newProviderContainers[this._newProviderContainers.currentIndex];

					if (providerContainer && !providerContainer.local) {
						// Report progress
						var infoStr = dojo.string.substitute(this.messages.ProcessProviderDialog_LoadingPercent, [Math.round(this._newProviderContainers.currentIndex * 100 / this._newProviderContainers.length)]);
						this._showMessage(infoStr);

						// need to get the providers service and compatibility
						// of this application
						var handlerObj = {
							_handleGetInfo: dojo.hitch(this, function(info) {
								providerContainer.statusCode = info.statusCode;
								if (info.providersServiceUrl) {
									providerContainer.providersServiceUrl = info.providersServiceUrl;
								}
								if (info.statusMessage) {
									providerContainer.statusMessage = info.statusMessage;
								}
								this._processCurrentOptionAndContinue();
							}),
							_handleError: dojo.hitch(this, function(error) {
								providerContainer.statusMessage = this._handleFetchError(error);
								providerContainer.statusCode = PROVIDER_CONTAINER_ERROR_CONNECTION_PROBLEM;
								this._processCurrentOptionAndContinue();
							})
						};
						var srh = new ServiceResponseHandler(handlerObj, "_handleGetInfo", "_handleError"); //$NON-NLS-1$ //$NON-NLS-2$
						ProcessClient.getProcessProviderContainerInfo({
							'owningApplicationKey': this._owningApplicationKey, //$NON-NLS-1$
							'providerRootServicesUrl': providerContainer.rootServicesUrl //$NON-NLS-1$
						}, srh);

					} else {
						providerContainer.statusCode = PROVIDER_CONTAINER_OK;
						this._processCurrentOptionAndContinue();
					}

				} else {
					delete this._newProviderContainers;

					if(this._newFilteredProviderContainers) {
						this._filterDisplayedProviderContainers();
					} else {
						var selected = this._processProviderContainerSelector.getSelection();
						if (selected.length == 0) {
							// start with "Local" selected
							var entryLocal = this._processProviderContainerSelector.getEntriesByIndices([0]);
							this._processProviderContainerSelector.setSelection(entryLocal, true);
						}
						this._handleProcessProviderContainerSelection();
					}
				}
			}
		},

		_processCurrentOptionAndContinue: function(option) {
			if (this._processProviderContainerSelector && this._newProviderContainers) {
				var providerContainer = this._newProviderContainers[this._newProviderContainers.currentIndex++];

				// ADD PROVIDER CONTAINER INFO TO THE SELECTOR
				// (Do not add providers with incompatible type)
				if (providerContainer.statusCode != PROVIDER_CONTAINER_ERROR_INCOMPATIBLE_TYPE) {
					// Do not show remote providers if this page is running on
					// the JTS - Task 204695
					if (providerContainer.local || !this._isEmbedded) {
						// This check is to handle cancel during loading.
						this._processProviderContainerSelector.addEntry({
							id: providerContainer.displayName,
							label: providerContainer.displayName,
							value: providerContainer,
							type: providerContainer.statusCode
						});
					}
				}

				this._retrieveProviderContainers();
			}
		},

		/**
		 * Populates the project area list based on the currently selected
		 * application. If a remote application is selected, uses the
		 * rootservices URL for that application to get its process providers
		 * service URL, and then makes a GET to that URL to get any process
		 * providing projects it has. Asks ProcessWebUIService to query the
		 * database for process providers in the local case.
		 */
		_handleProcessProviderContainerSelection : function() {
			var selected = this._processProviderContainerSelector.getSelection();
			if (selected.length > 0) { 
				this._selectedProcessProviderContainer = selected[0].value;
				this._showMessage(this.messages.ProcessProviderDialog_Loading);
			} else {
				this._selectedProcessProviderContainer = null;
				this._showMessage(this.messages.ProcessProviderDialog_0);
			}
			
			this._clearProcessSelector();
			
			if (this._selectedProcessProviderContainer && this._selectedProcessProviderContainer.local) {
				// local case
				var srh = new ServiceResponseHandler(this, "_populateProcessProvidersFromLocalContainer", "_handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getAllProcessProviders({
            		owningApplicationKey: this._owningApplicationKey
            	}, srh);
			} else if (this._selectedProcessProviderContainer) {
				// remote case
				if (this._selectedProcessProviderContainer.statusCode != PROVIDER_CONTAINER_OK) {
					// already tried to contact this application
					this._reportApplicationAccessError(this._selectedProcessProviderContainer);
				} else {
					if (this._selectedProcessProviderContainer.providersServiceUrl) {
						// already have providers service url, GET it
						jazz.client.xhrGet({
							url: this._selectedProcessProviderContainer.providersServiceUrl,
							headers: {
								"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
							},
							handleAs: "xml", //$NON-NLS-1$
							load: dojo.hitch(this, this._populateProcessProvidersFromRemoteContainer),
							error: dojo.hitch(this, this._handleXhrError)
						});
					}
				}
			}
		},
		
		_populateProcessProvidersFromLocalContainer: function(processes){
			if (processes) {
				this._populateProjectAreasFromLocalContainer(processes);
				this._showMessage(this.messages.ProcessProviderDialog_Prompt);
			} else {
				this._handleNoProcessProviders();
			}
		},

		_populateProjectAreasFromLocalContainer: function(processes) {
			if (this._processSelector) {
				// If cancel is pressed during load the this._processSelector
				// may be undefined
				var processesToShow = new Array();
				// first get rid of the project area that this provider will
				// provide for (if it is in the collection)
				dojo.forEach(processes, function(process) {
					if (process.itemId !== this._projectAreaItemId) {
						processesToShow.push(process);
					}
				}, this);

				dojo.forEach(processes, function(process) {
					this._processSelector.addEntry({
						id: process.name,
						label: process.name,
						value: process
					});
				}, this);
			}
		},
		
		_populateProcessProvidersFromRemoteContainer: function(processesXML){
			var remoteProcesses = this._parseRemoteProcesses(processesXML);
			if (remoteProcesses && remoteProcesses.length !== 0) {
				this._populateProjectAreasOptionsFromRemoteContainer(remoteProcesses);
				this._showMessage(this.messages.ProcessProviderDialog_Prompt);
			} else {
				this._handleNoProcessProviders();
			}
		},
		
		_parseRemoteProcesses: function(processesXML) {
			var remoteProcesses = [];
			
			var children = processesXML.childNodes;
			var projectAreasNode = UIHelper.getNode(children, "project-areas", ProcessConstants.NAMESPACE_JP06); //$NON-NLS-1$
			if (projectAreasNode) {
				var projectAreaNodes = UIHelper.getNodes(projectAreasNode.childNodes, "project-area", ProcessConstants.NAMESPACE_JP06); //$NON-NLS-1$
				dojo.forEach(projectAreaNodes, dojo.hitch(this, function(projectAreaNode){
					var urlNode = UIHelper.getNode(projectAreaNode.childNodes, "url", ProcessConstants.NAMESPACE_JP06); //$NON-NLS-1$
					var url = urlNode.firstChild.data;
					
					var title = UIHelper.getAttributeNodeValue(projectAreaNode, "name", ProcessConstants.NAMESPACE_JP06); //$NON-NLS-1$
					
					remoteProcesses.push({
						url: url,
						name: title
					});
				}));
				
				return remoteProcesses;
			}
		},
		
		_populateProjectAreasOptionsFromRemoteContainer: function(remoteProcesses) {

			remoteProcesses.sort(function(a, b) {
				return a.name.localeCompare(b.name);
			});

			dojo.forEach(remoteProcesses, function(process) {
				this._processSelector.addEntry({
					id: process.name,
					label: process.name,
					value: process
				});
			}, this);
		},
		
        /**
         * Sets the selected process information and enables the OK button
         */
		_handleProcessProviderSelection: function(){
			var selected = this._processSelector.getSelection();
			this.okButton.disabled = selected.length==0;
			if (selected.length > 0) {
				this.selectedProcess = selected[0].value;
				if (!this._selectedProcessProviderContainer.local) {
					this.selectedProcess.serverTitle = this._selectedProcessProviderContainer.name;
				}
			}
		},
		
		_handleNoProcessProviders: function(){
			this._showMessage(this.messages.ProcessProviderDialogHTML_NoProcesses);
		},
		
		/**
		 * Handles errors coming from the XHR
		 * 
		 * @param error
		 * @param response
		 * @returns
		 */
		_handleXhrError: function(error, response) {
			if (error.dojoType && error.dojoType === "popupBlocked" && error.message) {  //$NON-NLS-1$
				this._reportErrorFetchingRemote(error.message);
				return error.message;
			} else if (response.xhr.statusText && response.xhr.statusText !== "undefined") { //$NON-NLS-1$
				this._reportErrorFetchingRemote(response.xhr.statusText);
				return response.xhr.statusText;
			} else {
				return this._handleFetchError(error);
			}
		},
		
		_handleFetchError: function(error){
			 if (error.message && error.message !== "undefined") { //$NON-NLS-1$
				this._reportErrorFetchingRemote(error.message);
				return error.message;
			} else if (error.responseText && error.responseText !== "undefined") { //$NON-NLS-1$
				this._reportErrorFetchingRemote(error.responseText);
				return error.responseText;
			} else {
				this._reportErrorFetchingRemote(this.messages.ProcessProviderDialog_NoErrorDetails);
				return this.messages.ProcessProviderDialog_NoErrorDetails;
			}			
		},

		_reportApplicationAccessError: function(provider) {
			switch (provider.statusCode) {
				case PROVIDER_CONTAINER_ERROR_INCOMPATIBLE_VERSION:
					this._reportStatus(ICON_ERROR, provider.statusMessage);
					break;
				case PROVIDER_CONTAINER_ERROR_CONNECTION_PROBLEM:
					this._reportStatus(ICON_WARNING, provider.statusMessage);
					break;
			}
		},

		_showMessage: function(message) {
			this.dialogHeading.innerHTML = message;
		},

		_reportStatus: function(statusIcon, statusMsg) {
			var imgHtml = "<img src=\"" + statusIcon + "\"\/>"; //$NON-NLS-1$ //$NON-NLS-2$
			this._showMessage( imgHtml + " " + statusMsg); //$NON-NLS-1$
		},

		_reportErrorFetchingRemote: function(errorMessage) {
			this._reportStatus(ICON_WARNING, errorMessage);
		},

		/**
		 * Adds the application search box to the dialog. A search will be
		 * triggered after typing or clicking the search button.
		 */
		_createSearchBox: function(){
			this._searchBox = new jazz.ui.SearchBox({
				allowEmptySearches: true,
				scopes: [{
					id: "", //$NON-NLS-1$
					label: this.messages.ProcessProviderDialog_1,
					icon: "" //$NON-NLS-1$
				}],
				onUpdate: dojo.hitch(this, function(filter){
					this._getFilteredProcessProviderContainers(filter);
				}),
				onExecute: dojo.hitch(this, function(filter){
					this._getFilteredProcessProviderContainers(filter);
				})
			});
			this.processProviderContainerHeading.appendChild(this._searchBox.domNode);
		}
	});
})();
