/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.SelectVariableDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
(function() {
	
	/**
	 * Constructs a new Select Variable dialog that can be used to select a predefined 
	 * variable. These predefined variables can be used in team invitation email template
	 * and are substituted for actual system values when the invitation is sent. 
	 * 
	 * @com.ibm.team.process.web.ui.internal.admin.projects.SelectVariableDialog
	 * @constructor
	 * @extends dijit._Widget
	 * @extends dijit._Templated
	 */ 
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.SelectVariableDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/SelectVariableDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$

		/* 
		 * Constructs the dialog to select a variable.
		 * 
		 * Expects an argument of the format:
		 *  args = {
		 * 		predefinedVariables: an array of the predefined variables that can be selected (optional, Object),
		 *  	onOK: the function callback when the OK button is clicked (required), the selected variable will be passed as an argument
		 *  }
		 *  
		 *  The format of a variable:
		 *  variable = {
		 *  	variableKey: the key of this variable,
		 *  	variableLabel: the label of this variable,
		 *  	variableDescription: the description of this variable
		 *  }
		 *  
		 */
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			if (!dojo.isFunction(args.onOK)) {
				throw new Error("Missing function for 'onOK' parameter."); //$NON-NLS-1$
			}			

			this._predefinedVariables = args.predefinedVariables;
			//sort the variables by label
			if (this._predefinedVariables && this._predefinedVariables.length > 0) {
				this._predefinedVariables.sort(function(a, b){
					return a.variableLabel.localeCompare(b.variableLabel);
				});				
			}
		},
		
		postCreate: function(){
			// create jazz.ui.Dialog
			this.dialog = new jazz.ui.Dialog({
				primaryTitle: this.messages.SelectVariableDialog_1,
				contentNode: this.domNode,
				width: "30em", //$NON-NLS-1$
				destroyContents: true,
				onClose: dojo.hitch(this, function(){
				})
			});
			
			// create variable options list
			if (this._predefinedVariables && this._predefinedVariables.length > 0) {
				for(var i = 0; i < this._predefinedVariables.length; i++) {
					var variable = this._predefinedVariables[i];
					var option = new Option(variable.variableLabel, variable.variableKey);
					option.description = variable.variableDescription;
					if (dojo.isIE) {
						this._variableSelectionList.options[i] = option;
					} else {
						this._variableSelectionList.appendChild(option);
					}
				}
				
				this.connect(this._variableSelectionList, "onchange", "_onSelectionChanged"); //$NON-NLS-1$ //$NON-NLS-2$
				this.connect(this._variableSelectionList, "ondblclick", "_onOK"); //$NON-NLS-1$ //$NON-NLS-2$
			}

			this.connect(this._cancelButton, "onclick", "_onCancel"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._okButton, "onclick", "_onOK"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		startup: function(){
			com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.resizeI18NButtons(this.dialogFooter, [this._okButton, this._cancelButton]);
			this.inherited(arguments);
		},
		
		/**
		 * Click handler for Cancel button.
		 * 
		 * @param {Object} e
		 */
		_onCancel: function(e) {
			dojo.stopEvent(e);
			this.dialog.close();
		},
		
		/**
		 * Click handler for OK button.
		 * 
		 * @param {Object} e
		 */
		_onOK: function(e) {
			dojo.stopEvent(e);
			this.dialog.close();
			if (this.onOK) {
				this.onOK(this._getSelectedVariable());
			}
		},
		
		/**
		 * Get the currently selected variable in the list.
		 */
		_getSelectedVariable: function() {
			if (this._variableSelectionList.selectedIndex >= 0) {
				var selectedOption = this._variableSelectionList.options[this._variableSelectionList.selectedIndex];
				var selectedVariable = {};
				selectedVariable.variableKey = selectedOption.value;
				selectedVariable.variableLabel = selectedOption.label;
				selectedVariable.description = selectedOption.description;
				return selectedVariable;
			}
		},

		/**
		 * Selection change handler of variables list.
		 * 
		 * @param {Object} e
		 */
		_onSelectionChanged: function(e) {
			dojo.stopEvent(e);
			if (this._variableSelectionList.selectedIndex >= 0) {
				this._okButton.disabled = false;
				var selected = this._variableSelectionList.options[this._variableSelectionList.selectedIndex];
				this._variableDescriptionTextArea.value = selected.description;
			}
		}
		
	});
})();
