/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TeamInvitationEditor"); //$NON-NLS-1$

	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.SelectVariableDialog");//$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.AdminMenu");//$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var AbstractConfigurationDataEditor = com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var SelectVariableDialog = com.ibm.team.process.web.ui.internal.admin.projects.SelectVariableDialog;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TeamInvitationEditor", AbstractConfigurationDataEditor, { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/TeamInvitationEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._configurationDataId = "com.ibm.team.process.service.configurationData.teamInvitation"; //$NON-NLS-1$
		},
		
		postCreate: function(){
			// initialize the editor
			if (this._configurationDataContentList[0] && this._configurationDataContentList[0].configurationDataContent) {
				// initialize the editor with the configuration data
				var content = dojo.fromJson(this._configurationDataContentList[0].configurationDataContent);
				if (content) {
					this._emailSubjectText.value = UIHelper.unescapeXML(content.emailSubject);
					this._emailBodyTextArea.value = UIHelper.unescapeXML(content.emailBody);
					this._copySelfCheckbox.checked = content.copySelf;
				}
			} else {
				// if there is no team invitation configuration data found
				// try to get team invitation from generator
				var args = {
					processAreaItemId: this._processAreaEditor.itemId, 
					isTeamArea: false
				};
				var srh = new ServiceResponseHandler(this, "_handleGetTeamInvitationFromGenerator", "_handleGetTeamInvitationFromGeneratorError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getTeamInvitationFromGenerator(args, srh);	
			}

			this._originalEmailSubject = this._emailSubjectText.value;
			this._originalEmailBody = this._emailBodyTextArea.value;
			this._originalCopySelf = this._copySelfCheckbox.checked;
			
			// load predefined variables from server
			this._loadVariables();

			// connect "onchange" and "onkeyup" handler
			this.connect(this._emailSubjectText, "onchange", "_commonOnchangeHandler"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._emailSubjectText, "onkeyup", "_commonOnchangeHandler"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._emailBodyTextArea, "onchange", "_commonOnchangeHandler"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._emailBodyTextArea, "onkeyup", "_commonOnchangeHandler"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._copySelfCheckbox, "onchange", "_commonOnchangeHandler"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * Action handler for inserting variable to email subject.
		 */
		_insertToSubjectHandler: function() {
			this._showInsertVariableDialog(this._emailSubjectText);
		},
		
		/**
		 * Action handler for inserting variable to email body.
		 */
		_insertToBodyHandler: function() {
			this._showInsertVariableDialog(this._emailBodyTextArea);
		},
		
		/**
		 * Popup the varialbe selection dialog for the given textarea.
		 * 
		 * @param {Object} textArea
		 */
		_showInsertVariableDialog: function(textArea) {
			var parms = {};
			parms.predefinedVariables = this._predefinedVariables;
			parms.onOK = dojo.hitch(this, function(selectedVariable) {
				if (textArea) {
					var	insertValue = "${" + selectedVariable.variableKey + "}"; //$NON-NLS-1$ //$NON-NLS-2$
					if (dojo.isIE) {
						if (document.selection) {
							textArea.focus();
							var selection = document.selection.createRange();
							selection.text = insertValue;
						} else {
							textArea.value += insertValue;
						}
					} else {
						if (textArea.selectionStart || textArea.selectionStart === 0) {
							var startPos = textArea.selectionStart;
							var endPos = textArea.selectionEnd;
							textArea.value = textArea.value.substring(0, startPos) + insertValue + textArea.value.substring(endPos, textArea.value.length);
							textArea.selectionStart = startPos + insertValue.length;
							textArea.selectionEnd = textArea.selectionStart;
							this._processAreaEditor.setDirty(this._processAreaEditor.changesPending());			
							textArea.focus();
						} else {
							textArea.value += insertValue;
						}
					}
				}
			});
			var dialog = new SelectVariableDialog(parms);
			dialog.startup();

		},

		/**
		 * Common onchange event handler.
		 */
		_commonOnchangeHandler: function() {
			this._processAreaEditor.setDirty(this._processAreaEditor.changesPending());
		},
		
		/**
		 * Show insert variable action buttons.
		 */
		_showInsertVariableActionButton: function() {
			// show insert action buttons
			dojo.removeClass(this._insertToSubjectButton.domNode, "hidden"); //$NON-NLS-1$
			dojo.removeClass(this._insertToBodyButton.domNode, "hidden"); //$NON-NLS-1$
		},
		
		/**
		 * Load predefined variables from the server.
		 */
		_loadVariables: function() {
			var srh = new ServiceResponseHandler(this, "_handleGetVariables", "_handleGetVariablesError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getTeamInvitationPredefinedVariables(srh);	
		},
		
		/**
		 * Success handler for loading variables.
		 * 
		 * @param {Object} data
		 */
		_handleGetVariables: function(predefinedVariables) {
			this._predefinedVariables = predefinedVariables;
			// show insert variable action buttons
			this._showInsertVariableActionButton();
		},
		
		/**
		 * Error handler for loading variables.
		 * 
		 * @param {Object} error
		 */
		_handleGetVariablesError: function(error) {
			this.setErrorStatus(error);
			this._error = error;
		},

		/**
		 * Success handler for loading team invitation from generator.
		 * 
		 * @param {Object} invitation
		 */
		_handleGetTeamInvitationFromGenerator: function(invitation) {
			invitation = dojo.fromJson(invitation);
			this._emailSubjectText.value = invitation.emailSubject;
			this._emailBodyTextArea.value = invitation.emailBody;
			this._copySelfCheckbox.checked = invitation.copySelf;
			this._originalEmailSubject = this._emailSubjectText.value;
			this._originalEmailBody = this._emailBodyTextArea.value;
			this._originalCopySelf = this._copySelfCheckbox.checked;
		},
		
		/**
		 * Error handler for loading team invitation from generator.
		 * 
		 * @param {Object} error
		 */
		_handleGetTeamInvitationFromGeneratorError: function(error) {
			this.setErrorStatus(error);
			this._error = error;
		},
		
		/**
		 * Returns true if the given string is undefined or null or an empty string.
		 * 
		 * @param {String} value
		 */
		_isEmptyString: function(value) {
			if (value && dojo.string.trim(value).length > 0) {
				return false;
			} else {
				return true;
			}
		},

		/**
		 * See com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor#changesPending.
		 */
		changesPending: function() {
			if (this._originalEmailSubject !== this._emailSubjectText.value
					|| this._originalEmailBody !== this._emailBodyTextArea.value
					|| this._originalCopySelf !== this._copySelfCheckbox.checked) {
				return true;
			}
			
			return false;
		},
		
		/**
		 * See com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor#changesPending.
		 */
		getUnsavedConfigurationDataList: function() {
			var unSavedContentList = [];
			var unsavedContent = {};
			var invitationObj = {};
			invitationObj.emailSubject = UIHelper.escapeXML(this._emailSubjectText.value);
			invitationObj.emailBody = UIHelper.escapeXML(this._emailBodyTextArea.value);
			invitationObj.copySelf = this._copySelfCheckbox.checked;
			unsavedContent.configurationDataContent = invitationObj;
			unsavedContent.configurationDataId = this._configurationDataId;
			unSavedContentList.push(unsavedContent);
			return unSavedContentList;
		},
		
		/**
		 * See com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor#validate.
		 */
		validate: function() {
			if (this._error) {
				// this means there are some errors happening when loading this team invitation editor
				return this._error;
			} else {
				var err;
				if (this._isEmptyString(this._emailSubjectText.value)) {
					err = this.messages.TeamInvitationEditor_0;
				} else if (this._isEmptyString(this._emailBodyTextArea.value)) {
					err = this.messages.TeamInvitationEditor_1;
				}
				if (err) {
					this.setErrorStatus(err);
					return err;
				}
			}
		},
		
		/**
		 * See com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor#canSave.
		 */
		canSave: function() {
			return !this.validate();
		},
		
		/**
		 * See com.ibm.team.process.web.ui.configdata.AbstractConfigurationDataEditor#uninitialize.
		 */
		uninitialize: function() {
			this.inherited(arguments);
		}

	});
})();
