/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore", com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore, { //$NON-NLS-1$
		/*
		 * Used to ensure that new iterations, iteration types, and timelines all have unique identifiers
		 */
		counter: 0,
	
		/*
		 * Constructs the Timeline store
		 * @param {Object} args  No special args at this level see base class for any required args
		 */
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
				
		addNewIterationType: function(iterationType){
			var self = this;
			var newChildItem;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.ITERATION_TYPES,
				onItem: function(item, request){
					newChildItem = self.newItem({
						name: iterationType.name,
						id: ProcessConstants.EditorMode.NEW + self.counter++,
						stateId: ProcessConstants.EditorMode.NEW,
						parent: self.getIdentity(item),
						context: ProcessConstants.ExtensionContext.TEAM,
						contextId: iterationType.contextId,
						enabledContext: ProcessConstants.PermissionContext.ITERATION_TYPE,
						summary: iterationType.summary,
						archived: false,
						defined: true
					}, {
						parent: item,
						attribute: "items" //$NON-NLS-1$
					});
					
				}
			});
			return newChildItem;
			
		},
		
		updateIterationType: function(iterationType, iterationTypeData){
			this.setValue(iterationType, "contextId", iterationTypeData.contextId); //$NON-NLS-1$
			this.setValue(iterationType, "name", iterationTypeData.name); //$NON-NLS-1$
			this.setValue(iterationType, "summary", iterationTypeData.summary); //$NON-NLS-1$
		},
		
		addNewTimeline: function(timeline){
			var self = this;
			// Can be only one project time line so unmark any others in the store
			if (timeline.projectTimeline === true) {
				this.clearProjectTimeline();
			}
			
			// Add the new timeline
			var newChildItem;
			this.fetchItemByIdentity({
				identity: ProcessConstants.PermissionsScopeContext.DEV_LINES,
				onItem: function(item, request){
					newChildItem = self.newItem({
						name: timeline.name,
						id: ProcessConstants.EditorMode.NEW + self.counter++,
						stateId: ProcessConstants.EditorMode.NEW,
						parent: self.getIdentity(item),
						context: ProcessConstants.ExtensionContext.TEAM,
						contextId: timeline.id,
						enabledContext: ProcessConstants.PermissionContext.DEV_LINE,
						projectTimeline: timeline.projectTimeline,
						startDate: timeline.startDate,
						currentIterationItemId: "", //$NON-NLS-1$
						endDate: timeline.endDate,
						archived: false
					}, {
						parent: item,
						attribute: "items" //$NON-NLS-1$
					});
					
				}
			});
			return newChildItem;
			
		},
		
		updateTimeline: function(timeline, timelineData){
			// Can be only one project time line so unmark any others in the store
			if (timelineData.projectTimeline === true) {
				this.clearProjectTimeline();
			}
			var contextId = this.getValue(timeline, "contextId"); //$NON-NLS-1$
			this.setValue(timeline, "contextId", timelineData.contextId); //$NON-NLS-1$
			if (contextId !== timelineData.contextId) {
				// Context id changed...recompute for the children
				this._recomputeContextIdForChildren(timelineData.contextId, this.getValues(timeline, ProcessConstants.ExtensionDTO.ITEMS));
			}
			this.setValue(timeline, "name", timelineData.name); //$NON-NLS-1$
			this.setValue(timeline, "startDate", timelineData.startDate); //$NON-NLS-1$
			this.setValue(timeline, "endDate", timelineData.endDate); //$NON-NLS-1$
			this.setValue(timeline, "projectTimeline", timelineData.projectTimeline);//$NON-NLS-1$
		},
		
		_recomputeContextIdForChildren: function(parentContextId, children){
			if (children === undefined) {
				return;
			}
			dojo.forEach(children, function(child){
				var iterationContextId = this._getIterationId(this.getValue(child, "contextId")); //$NON-NLS-1$
				iterationContextId = parentContextId + ProcessConstants.PERMISSION_PATH_SEPARATOR + iterationContextId;
				this.setValue(child, "contextId", iterationContextId); //$NON-NLS-1$
				this._recomputeContextIdForChildren(iterationContextId, this.getValues(child, ProcessConstants.ExtensionDTO.ITEMS));
			}, this);
		},
		
		_getIterationId: function(contextId){
			var segments = contextId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			return segments[segments.length - 1];
		},
		
		setProjectTimeline: function(timeline){
			// Can be only one project time line so unmark any others in the store			
			this.clearProjectTimeline();
			this.setValue(timeline, "projectTimeline", true);//$NON-NLS-1$
		},
		
		clearProjectTimeline: function(){
			this.fetch({
				query: {
					projectTimeline: true
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this, function(items){
					for (var i = 0; i < items.length; i++) {
						this.setValue(items[i], "projectTimeline", false); //$NON-NLS-1$
					}
				})
			});
		},
		
		addNewIteration: function(iteration){
			var self = this;
			var newChildItem;
			
			this.fetchItemByIdentity({
				identity: iteration.parentId,
				onItem: function(item, request){
					newChildItem = self.newItem({
						name: iteration.name,
						id: ProcessConstants.EditorMode.NEW + self.counter++,
						stateId: ProcessConstants.EditorMode.NEW,
						parent: self.getIdentity(item),
						context: ProcessConstants.ExtensionContext.TEAM,
						contextId: iteration.id,
						enabledContext: ProcessConstants.PermissionContext.ITERATION,
						startDate: iteration.startDate,
						endDate: iteration.endDate,
						completed: iteration.completed,
						current: iteration.current,
						leaf: iteration.leaf,
						hasDeliverable: iteration.hasDeliverable,
						iterationTypeItemId: iteration.iterationTypeItemId,
						archived: false
					
					}, {
						parent: item,
						attribute: "items" //$NON-NLS-1$
					});
					
				}
			});
			return newChildItem;
			
		},
		
		updateIteration: function(iteration, iterationData){
			var contextId = this.getValue(iteration, "contextId"); //$NON-NLS-1$
			if (contextId !== iterationData.contextId) {
				var parent = this.getParent(iteration);
				var parentContextId = this.getValue(parent, "contextId"); //$NON-NLS-1$
				contextId = parentContextId + ProcessConstants.PERMISSION_PATH_SEPARATOR + iterationData.contextId;
				this.setValue(iteration, "contextId", contextId); //$NON-NLS-1$
				this.setValue(iteration, "parent", this.getIdentity(parent)); //$NON-NLS-1$
				this._recomputeContextIdForChildren(contextId, this.getValues(iteration, ProcessConstants.ExtensionDTO.ITEMS));
			}
			this.setValue(iteration, "name", iterationData.name); //$NON-NLS-1$
			this.setValue(iteration, "startDate", iterationData.startDate); //$NON-NLS-1$
			this.setValue(iteration, "endDate", iterationData.endDate); //$NON-NLS-1$
			this.setValue(iteration, "hasDeliverable", iterationData.hasDeliverable);//$NON-NLS-1$
			this.setValue(iteration, "iterationTypeItemId", iterationData.iterationTypeItemId);//$NON-NLS-1$
		},
		
		moveIteration: function(iteration, newParent, oldParent){
			var contextId = this._getIterationId(this.getValue(iteration, "contextId")); //$NON-NLS-1$
			var parentContextId = this.getValue(newParent, "contextId"); //$NON-NLS-1$
			contextId = parentContextId + ProcessConstants.PERMISSION_PATH_SEPARATOR + contextId;
			this.setValue(iteration, "contextId", contextId); //$NON-NLS-1$
			this.setValue(iteration, "parent", this.getIdentity(newParent)); //$NON-NLS-1$
			this._recomputeContextIdForChildren(contextId, this.getValues(iteration, ProcessConstants.ExtensionDTO.ITEMS));
			
			var self = this;
			var _getTimeline = function(parent) {
				if (self.isTimeline(parent)) {
					return parent;
				} else {
					while(parent && !self.isTimeline(parent)) {
						parent = self.getParent(parent);
					}
					return parent;				
				}
			}
			var newTimeline = _getTimeline(newParent);
			var oldTimeline = _getTimeline(oldParent)
			
			var oldCurrentIteration = this._getCurrentIterationForTimeline(oldTimeline);
			this._clearIterationState(oldTimeline);
			if (oldCurrentIteration) {
				this.setCurrentIteration(oldCurrentIteration);
			} else {
				this.clearCurrentIteration(oldTimeline);
			}
			if (newTimeline !== oldTimeline) {
				// Remove any current or completed iteration indications
				// for the moved iteration in the new timeline
				this._clearIterationState(iteration);
				var newCurrentIteration = this._getCurrentIterationForTimeline(newTimeline);
				if (newCurrentIteration) {
					this.setCurrentIteration(newCurrentIteration);
				}
			}			
		},
		
		_clearIterationState: function(iterationContainer) {
			if (this.isIteration(iterationContainer)) {
				var contextId = this.getValue(iterationContainer, "contextId"); //$NON-NLS-1$
				var originalContextId = this.getValue(iterationContainer, "originalContextId"); //$NON-NLS-1$
				if (contextId !== originalContextId) {
					this.setValue(iterationContainer, "current", false);	//$NON-NLS-1$
					this.setValue(iterationContainer, "leaf", false);		//$NON-NLS-1$
					this.setValue(iterationContainer, "completed", false);	//$NON-NLS-1$
				}
			}
			var children = this.getValues(iterationContainer, ProcessConstants.ExtensionDTO.ITEMS)
			for (var i=0; i < children.length; i++) {
				this._clearIterationState(children[i]);
			}
		},
		
		setCurrentIteration: function(iteration){
			// Can be only one current iteration in the development line
			var timeline = this.getTimelineForIteration(iteration);			
			this.clearCurrentIteration(timeline);
			this.setValue(timeline, "currentIterationItemId", this.getIdentity(iteration)); //$NON-NLS-1$
			
			// Set the new iteration and it's parent iterations as the current iteration
			this.setValue(iteration, "current", true);//$NON-NLS-1$
			this.setValue(iteration, "leaf", true);//$NON-NLS-1$
			var parent = this.getParent(iteration);
			while (parent && this.isIteration(parent)) {
				this.setValue(parent, "current", true);//$NON-NLS-1$
				parent = this.getParent(parent);
			}
			
			parent = this.getParent(iteration);
			// Mark the iterations before the current iteration as complete
			this._markCompletedIterations(this.getValues(parent, ProcessConstants.ExtensionDTO.ITEMS));
			
			// Mark all children of the current iteration as not complete
			this._markNonCompletedIterations(this.getValues(iteration, ProcessConstants.ExtensionDTO.ITEMS));
			
			
			// Mark all children that follow the current iteration in the timeline as not complete
			var iterations = this.getValues(parent, ProcessConstants.ExtensionDTO.ITEMS);
			if (iterations) {
				for (var i = 0; i < iterations.length; i++) {
					iteration = iterations[i];
					if (this.getValue(iteration, "current") === true) { //$NON-NLS-1$
						break;
					}
				}
				var nonCompleteIterations = iterations.slice(i);
				this._markNonCompletedIterations(nonCompleteIterations);
			}
		},
		
		_markCompletedIterations: function(iterations){
			if (iterations === undefined) {
				return;
			}
			for (var i = 0; i < iterations.length; i++) {
				var iteration = iterations[i];
				if (this.getValue(iteration, "current") === true) { //$NON-NLS-1$
					// Only iterations before the current are marked as completed
					break;
				} else if (this.getValue(iteration, "completed") !== true) { //$NON-NLS-1$
					this.setValue(iteration, "completed", true); //$NON-NLS-1$
					this._markCompletedIterations(this.getValues(iteration, ProcessConstants.ExtensionDTO.ITEMS));
				}
			}
			
		},
		
		_markNonCompletedIterations: function(iterations){
			if (iterations === undefined) {
				return;
			}
			for (var i = 0; i < iterations.length; i++) {
				var iteration = iterations[i];
				if (this.getValue(iteration, "completed") === true) { //$NON-NLS-1$
					this.setValue(iteration, "completed", false); //$NON-NLS-1$
					this._markNonCompletedIterations(this.getValues(iteration, ProcessConstants.ExtensionDTO.ITEMS));
				}
			}
			
		},
		
		/**
		 * Clears (unsets) the current iteration on the given timeline.
		 * 
		 * @param timeline the timeline whose current iteration to clear
		 */
		clearCurrentIteration: function(timeline){
			var currentIteration = this._getCurrentIterationForTimeline(timeline);
			this.setValue(timeline, "currentIterationItemId", ""); //$NON-NLS-1$ //$NON-NLS-2$
			
			if (currentIteration) {
				this.setValue(currentIteration, "current", false);//$NON-NLS-1$
				this.setValue(currentIteration, "leaf", false);//$NON-NLS-1$
				var parent = this.getParent(currentIteration);
				while (parent && this.isIteration(parent)) {
					this.setValue(parent, "current", false);//$NON-NLS-1$
					parent = this.getParent(parent);
				}
			}
		},
		
		_getCurrentIterationForTimeline: function(timeline) {
			var currentIteration;
			this.fetch({
				query: {
					current: true,
					leaf: true
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this, function(items){
					for (var i = 0; i < items.length; i++) {
						if (timeline === this.getTimelineForIteration(items[i])) {
							currentIteration = items[i];
						}
					}
				})
			});
			return currentIteration;
		}
		
	});
	
})();
