/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportView");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojox.xml.parser");
    
    dojo.require("com.ibm.team.repository.web.client.internal.licensing");
    
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigHelper");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportContributor");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenReportContributor");
    
    dojo.require("jazz.ui.MessageBox");
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    dojo.require("dojo.i18n");
    dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.team.repository.web.client.internal.ExternalLicenseClient");
    
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");
    
    

    var licensing = com.ibm.team.repository.web.client.internal.licensing;

    var MessageBox = jazz.ui.MessageBox;
    var LicenseTokenReportContributor = com.ibm.team.repository.web.ui.internal.admin.server.LicenseTokenReportContributor;
    var LICENSE_SERVICE_IMPL = "com.ibm.team.repository.service.internal.license.FloatingLicenseService";
    var TOKEN_LICENSE_SERVICE = "com.ibm.team.repository.service.internal.license.token.LicenseTokenService";
    var CONFIGURED_TOKEN_PROVIDER_PROP = "tokenProviderService";
    var ConfigHelper = com.ibm.team.repository.web.ui.internal.admin.server.ConfigHelper;
	var ExternalLicenseClient = com.ibm.team.repository.web.client.internal.ExternalLicenseClient;
	var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    function isNode(o){
        return o && o.nodeType && o.cloneNode;
    }
    
    function getWorkbench(){
        return net.jazz.ajax.ui.PlatformUI.getWorkbench();
    }
    
    function setText(e, text){
		if(!e || !text) return;
        dojox.xml.parser.removeChildren(e);
        
        if (!isNode(text)) {
            text = document.createTextNode(text);
        }
        
        e.appendChild(text);
    }
    
    
    var LICENSE_TOKEN_REPORT_CONTRIBUTOR = "token";
    var FLOATING_LICENSE_REPORT_CONTRIBUTOR = "floating";
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportView", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/FloatingLicenseReportView.html"),
        
        constructor: function(args){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");
            this.showForm = args.showForm;
        },
        
        postCreate: function(){
            com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportView.superclass.postCreate.apply(this);
            
            var self = this;
            
            var args = {
                implClass: TOKEN_LICENSE_SERVICE,
                handleServiceError: "_handleServiceError",
                parent: self
            
            };
            this.configHelper = new ConfigHelper(args);
            
            this.header = new MessageBox();
            this.headerArea.appendChild(this.header.domNode);
            
            var onLoad = function(licenseId, licenseInfo){
                self._handleTypeListLoad(licenseId, licenseInfo);
            }
            
            var onError = function(error){
                self._handleServiceError(error);
            }
            
            var floatHelper = {
                yes: function(status){
                    self._createForm();
                },
                no: function(s){
                    setText(self.description, self.messages.notEnabled);
                },
                error: function(err){
                    err.message = self.messages.failedFloatCheck + ": " + err.message;
                    self._handleServiceError(err);
                }
            }
            
            licensing.checkFloatingLicensesSupported(floatHelper.yes, floatHelper.no, floatHelper.error);
        },
        
        refresh: function(){
            this.header.hideMessage();
        },
        
        _handleTypeListLoad: function(licenseId, licenseInfo){
			if(!this.simpleViewArea) return;
            if (!licenseId) {
                // No floating licenses are installed in the server
                var descNode = document.createElement("span");
                var floatinglink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses'>";
                var installFloatingMessage = dojo.string.substitute(this.messages.installFloatingLink, [floatinglink, "</a>"]);
                descNode.innerHTML = installFloatingMessage;
                setText(this.description, descNode);
                dojo.toggleClass(this.simpleViewArea, "noDisplay", true);
            }
            else {
                dojo.toggleClass(this.simpleViewArea, "noDisplay", false);
            }
        },
        
		_initContributor : function(contributor){
			this.connect(contributor, "handleLicenseLoad", "_handleTypeListLoad");
			this.connect(contributor, "handleServiceError", "_handleServiceError")
			this.connect(contributor, "onFetchComplete", "_onFetchComplete");
			this.connect(contributor, "onChange", "_onContributorParmChange");
		},
		
        _createForm: function(){
            var self = this;
			if(!this.floatingLicenseContributorArea) return;
            
            var floatParms = {
                displayIn: this.floatingLicenseReportArea   
            };
            
            this.floatingReportContributor = new com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportContributor(floatParms);
			this._initContributor(this.floatingReportContributor);
			
            this.floatingLicenseContributorArea.appendChild(this.floatingReportContributor.domNode);
            
			var tokenParms = {
				displayIn: this.licenseTokenReportArea 
			};
						
            this.licenseTokenReportContributor = new LicenseTokenReportContributor(tokenParms);
			this._initContributor(this.licenseTokenReportContributor);
			
            this.licenseTokenContributorArea.appendChild(this.licenseTokenReportContributor.domNode);
                                                                        
            var tr = document.createElement("tr");            
            var floatingInput = this._createReportType(FLOATING_LICENSE_REPORT_CONTRIBUTOR, this.messages.floatingLicenses, tr);
            
            
            var licenseTokenInput = this._createReportType(LICENSE_TOKEN_REPORT_CONTRIBUTOR, this.messages.licenseTokens, tr);
            this.reportTypeSelection.appendChild(tr);
            
            
            
            this.reportTypes = [];
            // 
			// Bind the float/token contributor area , widget , input button , and report view so that
			// they can be toggled as they are selected.
			//
            var floatConfig = {
				reportArea: this.floatingLicenseReportArea,
                displayArea: this.floatingLicenseContributorArea,
                contributor: this.floatingReportContributor,
                input: floatingInput
            };
            
            this.reportTypes[FLOATING_LICENSE_REPORT_CONTRIBUTOR] = floatConfig;
           
            
            var tokenConfig = {
				reportArea: this.licenseTokenReportArea,
                displayArea: this.licenseTokenContributorArea,
                contributor: this.licenseTokenReportContributor,
                input: licenseTokenInput
            };
            
            this.reportTypes[LICENSE_TOKEN_REPORT_CONTRIBUTOR] = tokenConfig;
            
            this.requestButton = document.createElement("button");
            var label = document.createTextNode(this.messages.buttonLabel);
            this.requestButton.appendChild(label);
            this.connect(this.requestButton, "onclick", "_fetchAllReports");
            this.buttonArea.appendChild(this.requestButton);
            
            this._contributorSelected(floatConfig);
			
			var srh = new SRH(this, "_onTokenProvidersFetched", "_handleTokenProvidersFetchError");
			ExternalLicenseClient.getRegisteredProviders(srh);   
        },
        
		//
		// Enables the request report button depending on whether or not
		// the current contributor is valid.
		//
		_enableRequestButton : function(){
			if (this.currentReportContributor) {
				this.requestButton.disabled = !this.currentReportContributor.isValid();
			}
		},
		
		//
		// An event sent when a contributors parameter has changed.  
		//
		_onContributorParmChange : function(){				
		  	this._enableRequestButton();			
		},
		
        _createReportType: function(value, label, parent){
            var td = document.createElement("td");
            parent.appendChild(td);
            var input = document.createElement("input");
            input.setAttribute("name", "reportType");            
            input.setAttribute("type", "radio");
            input.setAttribute("config" , value);
            td.appendChild(input);
            
            this.connect(input, "onchange", function() {            	
            	this._contributorSelected(this.reportTypes[input.getAttribute("config")]);
            });
            
            var labelElement = document.createElement("label");
            labelElement.setAttribute("class", "reportType");
            labelElement.innerHTML = label;
            td.appendChild(labelElement);
            return input;
        },
        
        _contributorSelected: function(config){
            this._hideReportContributors();
            this.currentReportContributor = config.contributor;
            dojo.toggleClass(config.displayArea, "noDisplay", false);
			dojo.toggleClass(config.reportArea, "noDisplay", false);
            config.input.checked = true;
			this._enableRequestButton(this.currentReportContributor);
        },
        
        
        _hideReportContributors: function(){
            if(!this.domNode) return;
            dojo.forEach(dojo.query('div.reportContributorArea'), function(element){
                dojo.toggleClass(element, "noDisplay", true);
            });
			dojo.forEach(dojo.query('div.reportViewArea'), function(element){
                dojo.toggleClass(element, "noDisplay", true);
            });
        },
        
        // Receives notification from a report contributor when it is done running the reports
        _onFetchComplete: function(){
            if(!this.domNode) return;
            this.requestButton.disabled = false;
            getWorkbench().hideMessageArea();
        },

        _fetchAllReports: function(){
            if (this.currentReportContributor) {
                this.requestButton.disabled = true;
                getWorkbench().showMessage(this.messages.loadingReportInfo);
                this.currentReportContributor.fetchAllReports();
            }
        },
               
        _handleServiceError: function(errorObj){
          	if(!this.domNode) return;
          	this.header.showMessage({
				errorObject: errorObj
			});
        },
        
		_onTokenProvidersFetched : function(providers){
			if(!this.domNode) return;
			dojo.toggleClass(this.reportTypeSelectionSection , "noDisplay" , providers == null || providers.length == 0);
		},
		
		_handleTokenProvidersFetchError : function(){
			// If there was an error fetching, then the report type selection remains hidden. No reason
			// to display an error 
		},
			
		uninitialize: function () {
			this.header.destroyRecursive();
			
			if(this.licenseTokenReportContributor) {
				this.licenseTokenReportContributor.destroyRecursive();
			}
			if(this.floatingReportContributor) {
				this.floatingReportContributor.destroyRecursive();
			}

			this.inherited(arguments);
	    }                         
    });
    
})();

