/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.OAuthView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.TitlePane");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.Wizard");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("jazz.app.ua.TextHoverHelp");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.HoverPopup");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");

var messages;
var setupMessages;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamContributorSelectionDialog = com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog;
var MessageBox = jazz.ui.MessageBox;
var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var TextHoverHelp = jazz.app.ua.TextHoverHelp;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var HoverPopup = jazz.ui.HoverPopup;

var PENDING = "pending";

function clearInlineError(elem) { 
	if(elem) {
		dojo.removeClass(elem,"inlineError");
		elem.innerHTML = "";
	}
};

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.EditOAuthConsumerTable", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/EditOAuthConsumerTable.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
		this.setupMessages = setupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},

	postCreate: function() {
		if (this.showAssociatedUser && this.showAssociatedUser === true)
			dojo.toggleClass(this._preAuthorizedUserSection, "noDisplay", false);
		
		this.connect(this._consumerSecretInput,"onchange","secretValueChanged");
		this.connect(this._usePublicKeyLink, "onclick", "switchToPublicKey");
		this.connect(this._useSecretLink, "onclick", "_useSecretHandler");
		this.connect(this._pickConsumerKeyLink, "onclick", "_pickConsumerKeyHandler");
		this.connect(this._generateConsumerKeyLink, "onclick", "_generateConsumerKeyHandler");
		this.connect(this._registerConsumerTable,"onkeyup","handleKeyUp");
		
		this.connect(this._selectUserLink,"onclick", function (e) {
			dojo.stopEvent(e);
            var userPicker = new TeamContributorSelectionDialog({
                allowMultiple: false,
				destroyContents: true,
				hideAdminGuest: true
            });
			this.connect(userPicker,"onAdd","associateUser");
		});
		
		this.connect(this._clearConsumerName,"onclick", function (e) {
			dojo.stopEvent(e);
			this.clearSelectedUser();
		});
		
		if (this.allowKeyEditing == false) {
			dojo.toggleClass(this._pickConsumerKeyLink, "noDisplay", true);
		}
		
		var allInputs = [
			this._consumerKeyInput,
			this._consumerNameInput,
			this._consumerSecretInput,
			this._consumerSecretInputAgain,
			this._publicKeyInput,
			this._consumerTrustedInput
		]
		var self = this;
		dojo.forEach(allInputs, function(elem) {
			if(elem) {
				self.connect(elem,"onchange","validateChange");
			}
		});
	},
	
	_isRegistringWithPublicKey: function() {
		return dojo.hasClass(this._consumerPublicKeySection, "noDisplay") === false;
	},
	
	_getRequiredInputs: function() {
		var requiredInputs = [
			this._consumerNameInput
		]
		
		if (this._isRegistringWithPublicKey()) {
			requiredInputs.push(this._publicKeyInput);
		} else {
			requiredInputs.push(this._consumerSecretInput);
			requiredInputs.push(this._consumerSecretInputAgain);
		}
		
		if (this.isPickingKey()) {
			requiredInputs.push(this._consumerKeyInput);
		}
		
		return requiredInputs;
	},
	
	setPublicKey: function(publicKey) {
		this._publicKeyInput.value = publicKey;
	},
	
	_pickConsumerKeyHandler: function() {
		dojo.toggleClass(this._staticConsumerKeyDiv, "noDisplay", true);
		dojo.toggleClass(this._pickConsumerKeyDiv, "noDisplay", false);
	},
	
	_generateConsumerKeyHandler: function() {
		dojo.toggleClass(this._staticConsumerKeyDiv, "noDisplay", false);
		dojo.toggleClass(this._pickConsumerKeyDiv, "noDisplay", true);
	},
	
	switchToPublicKey: function() {
		dojo.toggleClass(this._consumerSecretSection, "noDisplay", true);
		dojo.toggleClass(this._consumerSecretRepeatSection, "noDisplay", true);
		dojo.toggleClass(this._consumerPublicKeySection, "noDisplay", false);
	},
	
	_useSecretHandler: function () {
		dojo.toggleClass(this._consumerSecretSection, "noDisplay", false);
		dojo.toggleClass(this._consumerSecretRepeatSection, "noDisplay", false);
		dojo.toggleClass(this._consumerPublicKeySection, "noDisplay", true);
	},
	
	validate: function() {
		// change this function with care.  It is also used directly by the EditConsumerStep below, so make sure "this" works for both contexts	
		
		//Let's first assume we are valid ("ready").
		var isReady = true;
		clearTimeout(this.onKeyUpTimout);
		
		//Make sure none of the required input fields is blank
		dojo.forEach(this._getRequiredInputs(), function(elem) {
			if(elem && elem.value ==  "") {
				isReady = false;
			} 
		});
					
		//Do the secrets match?
		if(isReady && this._consumerSecretInput.value != this._consumerSecretInputAgain.value) {
			isReady = false;
			
			this.messageBox.showMessage({ 
				type: STATUS_TYPES.ERROR,
				summary: this.setupMessages.correctFields
			});
			dojo.addClass(this.mismatchErr,"inlineError");
			this.mismatchErr.innerHTML = this.setupMessages.secretMismatch;
		} else {
			this.messageBox.hideMessage();
			clearInlineError(this.mismatchErr);
		}
		
		return isReady;
	},	
	
	validateChange: function () {
		var isValid = this.validate();
		
		if (this.validateCallback)
			this.validateCallback(isValid);
	},	
	
	clearTable: function () {
		this._generateConsumerKeyHandler();
		this._consumerKeyInput.value = "";
		
		this._useSecretHandler();
		this._consumerSecretInputAgain.value = "";
		this._consumerSecretInput.value = "";
		this._consumerNameInput.value ="";
		this._publicKeyInput.value = "";
		this._consumerTrustedInput.checked = false;
		clearInlineError(this.mismatchErr);
	}, 
	
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validateChange);
			var timeout = 1500;
			if (e.target._jazzRow  && e.target._jazzRow.hasError) {
				timeout = 500;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	secretValueChanged: function () {
		this.secretUpdated = true;
	}, 
		
	clearSelectedUser: function () {
		this.associatedUser = null;
		this._selectedConsumerName.innerHTML = "";
		dojo.toggleClass(this._selectUserLink, "noDisplay", false);
		dojo.toggleClass(this._clearConsumerName, "noDisplay", true);
		this.validateChange();
	}, 

	associateUser: function (newUsers) {
		if (newUsers != null && newUsers[0] != null) {
			this.associatedUser = newUsers[0];
			this._selectedConsumerName.innerHTML = "";
			this._selectedConsumerName.appendChild(document.createTextNode(newUsers[0].name));
			dojo.toggleClass(this._selectUserLink, "noDisplay", true);
			dojo.toggleClass(this._clearConsumerName, "noDisplay", false);
			this.validateChange();
		}
	}, 
	
	getAssociatedUserId: function() {
		return this.associatedUser ? this.associatedUser.userId : null;	
	},
	
	getSecretType: function() {
		if (this._isRegistringWithPublicKey()) {
			return "RSA_PUBLIC_KEY";
		} else {
			return "STRING";
		}
	},
	
	getSecret: function() {
		if (this._isRegistringWithPublicKey()) {
			// Since the public key section is visible, it should be used for the
			// consumer registration
			return this._publicKeyInput.value;
		} else {
			return this._consumerSecretInput.value;
		}
	},
	
	setStringSecret: function(secret) {
		this._consumerSecretInput.value = secret;
		this._consumerSecretInputAgain.value = secret;
		this.secretUpdated = false;
	},
	
	isSecretUpdated: function() {
		return this.secretUpdated; 
	},	
	
	getName: function() {
		return this._consumerNameInput.value;
	},
	
	setName: function(name) {
		this._consumerNameInput.value = name;
	},
	
	isTrustedChecked: function() {
		return this._consumerTrustedInput.checked;
	},
	
	setTrustedChecked: function(isTrusted) {
		this._consumerTrustedInput.checked = isTrusted;
	},
	
	isPickingKey: function() {
		return dojo.hasClass(this._pickConsumerKeyDiv, "noDisplay") === false;		
	},
	
	getKey: function() {
		if (this.isPickingKey()) {
			return this._consumerKeyInput.value;
		} else {
			return null;
		}
	},
	
	setStaticKey: function(staticKey) {
		this._consumerKeySpan.value = staticKey;
	}

});
var EditOAuthConsumerTable = com.ibm.team.repository.web.ui.internal.admin.server.EditOAuthConsumerTable;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.OAuthView", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/OAuthView.html"),
	_actionToolbars: [],

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
		this.setupMessages = setupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	
	
	postCreate: function() {
		this.header = new MessageBox();
		this.messageBox.appendChild(this.header.domNode);
		
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		
		this._textHelpHovers = new Array();
		this._disabledDeleteHovers = new Array();
		
		//Add *dynamic* descriptions
		var appContext = this.isJTS ? this.messages.pageDescription_jts : this.messages.pageDescription_app;
		this.pageDescription.innerHTML = dojo.string.substitute(this.messages.pageDescription, [appContext]);
		appContext = this.isJTS ? this.messages.authorizeProvisionalDescription_jts : this.messages.authorizeProvisionalDescription_app;
		this.provisionalDescription.innerHTML = dojo.string.substitute(this.messages.authorizeProvisionalDescription, [appContext]);
		appContext = this.isJTS ? this.messages.authorizedKeysDescription_jts : this.messages.authorizedKeysDescription_app;
		this.authorizedKeysDescription.innerHTML = dojo.string.substitute(this.messages.authorizedKeysDescription, [appContext]);

		var self = this;
		this.editConsumerTable = new EditOAuthConsumerTable({
			validateCallback: function(isValid) {
				self._validateHandler(isValid);
			},
			messageBox: this.header
		});
		this._consumerEditTableDiv.appendChild(this.editConsumerTable.domNode);
		
		//Add Hover Text
		this._textHelpHovers.push(new TextHoverHelp({
			helpContentString: this.messages.help_consumerSecret,
			contentNode: this.editConsumerTable.help_consumerSecret
		}));
		this._textHelpHovers.push(new TextHoverHelp({
			helpContentString: this.messages.help_trusted,
			contentNode: this.editConsumerTable.help_trusted
		}));
		
		//Append the context sensitive help
		this._pageHelpHover = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.oauthMgmt" });
		this.helpHover.appendChild(this._pageHelpHover.domNode);
		
		this.connect(this._registerConsumerButton, "onclick",  "_registerConsumerHandler");

		this.refresh();
	},
	
	refresh: function() {
		this._getConsumerList();
		this._getProvisionalRegistrations();
		this.header.hideMessage();
		this._registerConsumerButton.disabled = true;
		this.editConsumerTable.clearTable();
	},
	
	_validateHandler: function(isValid) {
		this.setRegisterButton(isValid);
	},
	
	_registerConsumerHandler: function () {
		this._registerConsumerButton.disabled = true;

		//Let's just validate one more time before we send out the data.
		var isValid = this.editConsumerTable.validate();
		if(!isValid) return;
		
		var args = {consumerSecret: this.editConsumerTable.getSecret(),
		             secretType: this.editConsumerTable.getSecretType(),
			   	     name: this.editConsumerTable.getName()}
		if (this.editConsumerTable.isPickingKey()) {
			args.consumerKey = this.editConsumerTable.getKey();
		}
		var srh = new ServiceResponseHandler(this, "_handleConsumerRegistrationSuccess", "_handleServiceError");
		OAuthAdminClient.registerNewConsumer(srh, args);
		return false;
	},
	
	_handleConsumerRegistrationSuccess: function (consumerKey) {
		this.header.showMessage({ 
			type: STATUS_TYPES.OK,
			closable: true,
			summary: dojo.string.substitute(this.messages.successMessage, [consumerKey])
		});

		// Do the save in two steps, because it requires 
		// less privledges to save a consumer than it 
		// does to make a consumer trusted
		if (this.editConsumerTable.isTrustedChecked()) {
			var srh = new ServiceResponseHandler(this, "_handleConsumerTrustSuccess", "_handleServiceError");
			OAuthAdminClient.setConsumerTrusted(srh, consumerKey, true);		
		} else {
			this.editConsumerTable.clearTable();
			this._getConsumerList();
		}
		
	},
	
	_handleConsumerTrustSuccess: function () {
		this.editConsumerTable.clearTable();
		this._getConsumerList();
	},
	
	_getConsumerList: function () {
		this._destroyActionToolbars();
		dojox.xml.parser.removeChildren(this.consumersTableBody);	
		dojo.toggleClass(this.noConsumer, "noDisplay", true);
		var srh = new ServiceResponseHandler(this, "displayConsumersList", "_handleServiceError");
		OAuthAdminClient.getConsumers(srh);
	},
	
	_deleteConsumer: function(consumerKey) {
		var verifyDelete = confirm(this.messages.confirmDelete);
        if (verifyDelete) {
			this._keyPendingDeletion = consumerKey;
			var srh = new ServiceResponseHandler(this, "removeConsumerEntry", "_handleServiceError");
			OAuthAdminClient.deleteConsumer(srh, consumerKey);
		}
	},
	
	removeConsumerEntry: function() {
		var elem = dojo.byId("row-" + this._keyPendingDeletion);
		this.consumersTableBody.removeChild(elem);
		
	},
	
	sortConsumers: function () {	
		return this.consumerList.sort(function(a, b){
			var aName = a.name ? a.name.toLowerCase() : "";
			var bName = b.name ? b.name.toLowerCase() : "";
			return aName > bName ? 1 : -1;
		});
	},
	
	displayConsumersList: function (consumers) {
		if(!this.domNode) return;
		if (consumers){
			this.consumerList = consumers;
			if(this.consumerList.length > 1) {
				this.sortConsumers();
			}
			var self = this;
			dojo.forEach(this.consumerList, function(consumer){
                self.addConsumerListRow(consumer);
            });
		}
		dojo.toggleClass(this.consumerTable, "noDisplay", !consumers);
		dojo.toggleClass(this.noConsumer, "noDisplay", consumers);
	},
	
	addConsumerListRow: function(consumer) {
		
		var row = dojo.create("tr", { id: "row-" + consumer.key, className: "hasAction" }, this.consumersTableBody);
		var nameCell = dojo.create("td", {}, row);
		nameCell.appendChild(document.createTextNode(consumer.name));
		//Prevent the display of malicious scripts
		var keyCell = dojo.create("td", {}, row);
		keyCell.appendChild(document.createTextNode(consumer.key));
		
		var userCell = dojo.create("td", {}, row);
		userCell.appendChild(document.createTextNode(consumer.userId || " "));
		dojo.create("td", { innerHTML: consumer.trusted ? this.messages.trustedValue : this.messages.untrustedValue }, row);

		var actions = dojo.create("td", {}, row);	
		var toolbar = new Toolbar();
		this._actionToolbars.push(toolbar);
		var deleteAction = {};
		deleteAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/delete_hov.gif");
		deleteAction.label = this.messages.removeConsumer;
		deleteAction.callbackData = {
			key: consumer.key
		};
		var callback = function(e){
			dojo.stopEvent(e);
			this._deleteConsumer(e.currentTarget.callbackData.key);
		};
		var deleteButton = toolbar.addButton(this, callback, deleteAction);
		
		//See if we are allowed to delete this
		if(consumer.reasonsToDenyDeletion && consumer.reasonsToDenyDeletion.length > 0) {
			toolbar.disableButton(deleteButton);
			this._disabledDeleteHovers.push(new HoverPopup({
				around:  deleteButton.domNode,
				html: consumer.reasonsToDenyDeletion[0],
				openDelay: 300,
				closeDelay: 300
			}));
		}
		
		var editAction = {};
		editAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit-ellipses.gif");
		editAction.label = this.messages.editConsumer;
		editAction.callbackData = {
			consumer: consumer
		};
		var callback = function(e){
			dojo.stopEvent(e);
			this._launchEditConsumer(e.currentTarget.callbackData.consumer);
		};
		toolbar.addButton(this, callback, editAction);
		actions.appendChild(toolbar.domNode);
	},

	_launchEditConsumer: function (consumer) {
		this._destroyConsumerWizard();
		
		var caller = this;
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.EditConsumerStep({
			stepTitle: this.messages.editConsumerDialogTitle,
			consumer: consumer, 
			caller: caller
		});

		var wizardArgs = {
			steps: [step1],
			primaryTitle: dojo.string.substitute(this.messages.editConsumerHeading, [consumer.key]),
			showStepSummary: false,
			processFinish: function(){
				this._steps[0].saveUpdates();
			},
			processCancel: function() {		
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(caller.messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}				
			}, 	
			saveSuccess: function () {
				caller._getConsumerList();
				caller.header.hideMessage();
				this.container.viewBox.close();
			}
		}
		this.wizard = new jazz.ui.Wizard(wizardArgs);
					
	},
	
	_getProvisionalRegistrations: function () {
		// clear table & no provisional msg
		dojox.xml.parser.removeChildren(this.provisionalTableBody);
		dojo.toggleClass(this.noProvisional, "noDisplay", true);
		
		var srh = new ServiceResponseHandler(this, "displayProvisionalRegistrations", "_handleServiceError");
		OAuthAdminClient.getProvisionalRegistrations(srh);
	},
	
	displayProvisionalRegistrations: function (provisional) {
		if (provisional){
			this.provisionalKeys = provisional;
			var self = this;
			dojo.forEach(this.provisionalKeys, function(key){
                self.addProvisionalRegistrationRow(key);
            });
		}
		dojo.toggleClass(this.provisionalTable, "noDisplay", !provisional);
		dojo.toggleClass(this.noProvisional, "noDisplay", provisional);
		dojo.toggleClass(this.provisionalSaveButton, "noDisplay", !provisional);
	},
	
	addProvisionalRegistrationRow: function(provisionalKey) {
		var row = document.createElement("tr");

		var nameCell = document.createElement("td");
		var name = document.createElement("input");
		name.value = provisionalKey.name;
		name.type = "text";
		name.setAttribute("id","name-" + provisionalKey.key);
		nameCell.appendChild(name);
		row.appendChild(nameCell);
		provisionalKey._nameCell = nameCell;
		
		var cell1 = document.createElement("td");
		var key = document.createTextNode(provisionalKey.key);
		cell1.appendChild(key);
		row.appendChild(cell1);
		
		var cell1B = document.createElement("td");
		var userId = document.createTextNode(provisionalKey.userId ? provisionalKey.userId : "");
		cell1B.appendChild(userId);
		row.appendChild(cell1B);
		
		var cell2 = document.createElement("td");
		var trusted = document.createTextNode(provisionalKey.trusted ? messages.trustedValue : messages.untrustedValue);
		cell2.appendChild(trusted);
		row.appendChild(cell2);
		
		var cell3 = document.createElement("td");
		var grantTrusted = document.createElement("input");
		grantTrusted.type = "checkbox";
		provisionalKey._grantTrusted = grantTrusted;
		cell3.appendChild(grantTrusted);
		row.appendChild(cell3);
		grantTrusted.checked = provisionalKey.trusted;
		provisionalKey._cell3 = cell3;
		
		var cell4 = document.createElement("td");
		var select = this.createAcceptSelect(provisionalKey);
		provisionalKey._acceptSelect = select;
		this.connect(provisionalKey._acceptSelect, "onchange", "handleProvisionalChange");
		cell4.appendChild(select);
		row.appendChild(cell4);
		provisionalKey._cell4 = cell4;

		this.provisionalTableBody.appendChild(row);
	},
	
	createAcceptSelect: function (provisionalKey) {
		var node = document.createElement("select");
		var option = document.createElement("option");
		option.text = this.messages.pendingLabel;
		option.value = PENDING;
		node.options[node.options.length] = option;
		option = document.createElement("option");
		option.text = this.messages.approveLabel;
		option.value = "true";
		node.options[node.options.length] = option;
		option = document.createElement("option");
		option.text = this.messages.rejectLabel;
		option.value = "false";
		node.options[node.options.length] = option;
		return node;
	},
	
	handleProvisionalChange: function () {
		var dirty = false;
		for (var i = 0; i < this.provisionalKeys.length; i++) {
			if (this.provisionalKeys[i]._acceptSelect && this.provisionalKeys[i]._acceptSelect.value != PENDING) {
				dirty = true;
				break;
			}
		}
		this.provisionalSaveButton.disabled = !dirty;
	},		
			
	saveProvisional: function () {
		var self = this;
		this.provisionalSaveButton.disabled = true;
		var openRequests = 0;
		dojo.forEach(this.provisionalKeys, function(provisionalKey){   
			if (provisionalKey._acceptSelect && provisionalKey._acceptSelect.value != PENDING) {
				provisionalKey.grant = provisionalKey._grantTrusted.checked
				var acceptProvisional = {
					key: provisionalKey,
					
					displayAcceptedRegistration: function(response){
						openRequests--;
					
						var trust = this.key.grant ? self.messages.trustedValue : self.messages.untrustedValue;
						this.key._cell3.replaceChild(document.createTextNode(trust), this.key._cell3.firstChild);
						this.key._grantTrusted = null;
						
						var approval = provisionalKey._acceptSelect.value === "true" ? self.messages.approved : self.messages.rejected;
						this.key._cell4.replaceChild(document.createTextNode(approval), this.key._cell4.firstChild);
						this.key._acceptSelect = null;	
						
						var name = provisionalKey.name;
						this.key._nameCell.replaceChild(document.createTextNode(name), this.key._nameCell.firstChild);
						if(openRequests == 0) {
							self._getConsumerList();
						}
						
					},
					handleAcceptError: function(err){
						self._handleServiceError(err);
					}
				}
				provisionalKey.name = dojo.byId("name-" + provisionalKey.key).value;
				openRequests++;
				var srh = new ServiceResponseHandler(acceptProvisional, "displayAcceptedRegistration", "handleAcceptError");
				OAuthAdminClient.acceptProvisionalRegistration(srh, provisionalKey.name, provisionalKey.key, provisionalKey._grantTrusted.checked, provisionalKey._acceptSelect.value);
			}		
		});
		
	},
	
	_handleServiceError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	setRegisterButton: function (valid) {
		if(!valid) {			
			this._registerConsumerButton.disabled = true;	
		} else {
			this._registerConsumerButton.disabled = false;
		}
	},
	
	_destroyActionToolbars: function(){
		for (var i = 0; i < this._actionToolbars.length; i++) {
			this._actionToolbars[i].destroyRecursive();
		}
		this._actionToolbars = [];
	},
	
	_destroyConsumerWizard: function() {
		if (this.wizard) {
			this.wizard.destroyRecursive();
			delete this.wizard;
		}
	},
	
	_destroyInfoHovers: function() {
		// Info decorator hover popups
		if (this._disabledDeleteHovers) {
			for (textID in this._disabledDeleteHovers) {
				if (this._disabledDeleteHovers[textID]) {
					this._disabledDeleteHovers[textID].destroy();
					delete this._disabledDeleteHovers[textID];
				}
			}
			delete this._disabledDeleteHovers;
		}
	},
	
	uninitialize: function(){
		this._destroyActionToolbars();
		
		// Destroy the message box
		this.header.destroyRecursive();
		
		this._pageHelpHover.destroyRecursive();
		this.editConsumerTable.destroyRecursive();
		
		this._destroyInfoHovers();

		// TextHoverHelp Nodes
		if (this._textHelpHovers) {
			for (textID in this._textHelpHovers) {
				if (this._textHelpHovers[textID]) {
					this._textHelpHovers[textID].destroy();
					delete this._textHelpHovers[textID];
				}
			}
			delete this._textHelpHovers;
		}
		
		this._destroyConsumerWizard();
		this.inherited(arguments);
	}

});

var WizardStep = jazz.ui.wizard.WizardStep;
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.EditConsumerStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/EditOAuthConsumer.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
		this.setupMessages = setupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	

	postCreate: function () {
		this.header =  new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		var self = this;
		this.editConsumerTable = new EditOAuthConsumerTable({
			validateCallback: function(isValid) {
				self.triggerButtonStateUpdate();
			},
			showAssociatedUser: true,
			messageBox: this.header,
			allowKeyEditing: false
		});
		this._consumerEditTableDiv.appendChild(this.editConsumerTable.domNode);
		
		this.editConsumerTable.setStaticKey(this.consumer.key);
		
		if (this.consumer.secretType == "RSA_PUBLIC_KEY") {
			this.editConsumerTable.setPublicKey(this.consumer.secret);
			this.editConsumerTable.switchToPublicKey();	
		} else {
			this.editConsumerTable.setStringSecret("**********");
		}
		
		this.editConsumerTable.setTrustedChecked(this.consumer.trusted);
		this.editConsumerTable.setName(this.consumer.name);
		if (this.consumer.userId) {
			this.editConsumerTable.associateUser([{name: this.consumer.userId, userId:this.consumer.userId }]);
		}
		
		this._textHelpHovers = new Array();
		
		//Add Hover Text
		this._textHelpHovers.push(new TextHoverHelp({
			helpContentString: this.messages.help_consumerSecret,
			contentNode: this.editConsumerTable.help_consumerSecret
		}));
		this._textHelpHovers.push(new TextHoverHelp({
			helpContentString: this.messages.help_trusted,
			contentNode: this.editConsumerTable.help_trusted
		}));
		this._textHelpHovers.push(new TextHoverHelp({
			helpContentString: this.messages.help_functionalId,
			contentNode: this.editConsumerTable.help_functionalId
		}));
	}, 
	
	validate: function () {
		return this.editConsumerTable.validate();
	},
	
	saveUpdates: function () {
		var isValid = this.validate();
		if(!isValid) return;
		
		var args = {
			consumerKey: this.consumer.key,
			trusted: this.editConsumerTable.isTrustedChecked()
		};
		if (this.consumer.name != this.editConsumerTable.getName()) {
			args.name = this.editConsumerTable.getName();
		}
		
		var secretChanged = false;
		if (this.consumer.secretType == this.editConsumerTable.getSecretType()) {
			if ("STRING" == this.editConsumerTable.getSecretType()) {
				if (this.editConsumerTable.isSecretUpdated()) {
					secretChanged = true;
				}
			} else {
				// The consumer had a public key and it's content changed
				if (this.consumer.secret != this.editConsumerTable.getSecret().content) {
					secretChanged = true;
				}
			}
		} else {
			// The secret type of the consumer changed
			secretChanged = true;
		}
		
		if (secretChanged) {
			args.consumerSecretType = this.editConsumerTable.getSecretType();
			args.consumerSecret = this.editConsumerTable.getSecret();
		}
		
		var newUser = this.editConsumerTable.getAssociatedUserId();
		if (this.consumer.userId != newUser) {
			args.userId = newUser ? newUser : "";
		}
			
		var srh = new ServiceResponseHandler(this, "handleConsumerUpdateSuccess", "_handleServiceError");
		OAuthAdminClient.updateConsumer(srh, args);
	},
		
	handleConsumerUpdateSuccess: function () {
		this.wizard.saveSuccess();		
	},
	
	_handleServiceError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	canFinish: function(){
		var isValid = this.validate();
		return isValid && this.hasUpdates();
	}, 
	
	hasUpdates: function () {
		if (this.consumer.name != this.editConsumerTable.getName()) {
			return true;
		}
		
		if (this.consumer.secretType == this.editConsumerTable.getSecretType()) {
			if ("STRING" == this.editConsumerTable.getSecretType()) {
				if (this.editConsumerTable.isSecretUpdated()) {
					return true;
				}
			} else {
				// The consumer had a public key and it's content changed
				if (this.consumer.secret != this.editConsumerTable.getSecret()) {
					return true;
				}
			}
		} else {
			// The secret type of the consumer changed
			return true;
		}
		
		var newUser = this.editConsumerTable.getAssociatedUserId();
		if (this.consumer.userId != newUser) {
			return true;
		}
		if (this.consumer.trusted != this.editConsumerTable.isTrustedChecked()) {
			return true;
		}
		return false;
	},
	
	uninitialize: function(){
		// Destroy the message box
		this.header.destroyRecursive();
		
		// TextHoverHelp Nodes
		if (this._textHelpHovers) {
			for (textID in this._textHelpHovers) {
				if (this._textHelpHovers[textID]) {
					this._textHelpHovers[textID].destroy();
					delete this._textHelpHovers[textID];
				}
			}
			delete this._textHelpHovers;
		}
		this.editConsumerTable.destroyRecursive();
		this.inherited(arguments);
	}
	
	

});

})();