/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.SystemAlerts");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.TimeTextBox");
dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.client.xhr");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "SystemAlertsMessages");

(function() {
var messages;
var MessageBox = jazz.ui.MessageBox;
var SimpleToolbar = jazz.ui.SimpleToolbar;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;
var SYSTEM_ALERTS_ENDPOINT = "serverMessages";
var LOCAL_STORAGE_KEY = "com-ibm-team-repository-web-SystemAlerts";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.SystemAlerts", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/SystemAlerts.html"),
	_actionToolbars: [],
	widgetsInTemplate: true,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "SystemAlertsMessages");
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		this.alertsCache = new Array();
		this._actionToolbars = new Array();
	},
	
	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);

		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		
		this.buildTable();
		this.reload();
	},
	
	reload: function () {
		if(!this.domNode) return;
		this.header.hideMessage();
		
		this.getAlerts();
	},
	
	getAlerts: function() {
		this.showNoAlertsRow(/* show loading msg */ true);
		var deferred = jazz.client.xhrGet(	{
	    	url: net.jazz.ajax.contextRoot() + "/" + SYSTEM_ALERTS_ENDPOINT,
			handleAs: "json",
			headers: this.headers
		});
		deferred.addCallback(dojo.hitch(this, function(alerts){
			if(alerts && alerts.length > 0) {
				this.alertsCache = alerts;
				this.handleSort();
				this.handleSort(); //Do this twice since the first call is where we compute the status sortKey values... (hackish)
			} else {
				this.showNoAlertsRow();
			}			
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject){
			this.header.showMessage({
				errorObject: errorObject,
                summary: this.messages.errorLoadingAlerts
            });
		}));
		return deferred;
	},
	
	buildTable: function() {
		var columns = [
			{
				label: this.messages.status,
				sortable: true
			}, {
				label: this.messages.description,
				width: "50%",
				sortable: true
			}, {
				label: this.messages.author,
				sortable: true
			}, {
				label: this.messages.startDate,
				sortable: true
			}, {
				label: this.messages.endDate,
				sortable: true
			}, {
				label: this.messages.actions,
				sortable: false,
				className: "actionsCol"
			}
		];

		//Check for saved sort preferences
		var sortPrefs = dojo.fromJson(localStorage[LOCAL_STORAGE_KEY]);
		if(sortPrefs) {
			var sortBy = sortPrefs.sortBy;
			var asc = sortPrefs.asc;
		}
		
		this.styledTable = new jazz.ui.StyledTable({
			columns: columns,
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
			sortBy: (sortBy != null) ? sortBy : 0,
			sortByAscending: (asc != null) ? asc : true,
			showHover: true,
			handleSort: dojo.hitch(this, "handleSort")
		});
		this.contentArea.appendChild(this.styledTable.domNode);
		dojo.addClass(this.styledTable.domNode, "fullWidth");
	},
	
	handleSort: function(args) {
		var index = this.styledTable.sortBy;
		
		//Save the users' sort preferences
		try {
			localStorage[LOCAL_STORAGE_KEY] = dojo.toJson({
				sortBy: index,
				asc: this.styledTable.sortByAscending
			});
		} catch (e) {} //not a critical op
		
		//Determine which column to sort by
		if (index === 0) {
			var alerts = this.sortStatus();
		} else if(index === 1) {
			var alerts = this.sortDescription();
		} else if(index === 2) {
			var alerts = this.sortAuthor();
		} else if(index === 3) {
			var alerts = this.sortStartDate();
		} else if(index === 4) {
			var alerts = this.sortEndDate();
		} else {
			var alerts = this.alertsCache;
		}
		
		this.styledTable.clearBody();
		dojo.forEach(alerts, function(alert) {
			this.createRow(alert);
		}, this);
	},
	
	_getStatus: function(start, end) {
		var value, className;
		var now = new Date();
		if(now >= end) {
			value = this.messages.status_expired;
			className = "expired";
			sortKey = 3;
		}
		else if(start > now) {
			value = this.messages.status_upcoming;
			className = "upcoming";
			sortKey = 2;
		}
		else if(start <= now && now < end) {
			value = this.messages.status_active;
			className = "active";
			sortKey = 1;
		}
		return status = {
			value: value,
			className: className,
			sortKey: sortKey
		};
	},
	
	createRow: function(alert) {
		if(!this.styledTable.tbody) return;
		
		var tr = dojo.create("tr", {}, this.styledTable.tbody);
		
		var startDate = dojo.date.stamp.fromISOString(alert.startDate);
		var endDate = dojo.date.stamp.fromISOString(alert.endDate);
		
		//Compute status
		var status = this._getStatus(startDate, endDate);
		dojo.create("td", { innerHTML: status.value, className: "status " + status.className }, tr);
		alert.status = status.sortKey; //Custom sort order: Active, Future, Expired.
		
		dojo.create("td", { innerHTML: alert.description }, tr);
		
		var userTd = dojo.create("td", {}, tr);
		if(alert.contributorId && alert.contributorUserId) {
			var userLink = "#action=com.ibm.team.repository.editUser&itemId=" + alert.contributorId;
			dojo.create("a", { innerHTML: alert.contributorUserId, href: userLink }, userTd);
		} else if(alert.contributorId) {
			dojo.create("span", { innerHTML: alert.contributorUserId }, userTd);
		}
		
		dojo.create("td", { innerHTML: this.getExecutionTime(startDate) }, tr);
		dojo.create("td", { innerHTML: this.getExecutionTime(endDate) }, tr);
		
		
		var simpleToolbar = new SimpleToolbar();
		this._actionToolbars.push(simpleToolbar);
	
		var editHandler = {
			wasClicked: dojo.hitch(this, function(e) {
				dojo.stopEvent(e);
				this._editAlert(alert);
			})
		};
		simpleToolbar.addButton(editHandler, "wasClicked", {
			label: this.messages.editAlert,
			href:"#",
			imgSrc: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit-ellipses.gif")
		});
		
		var deleteHandler = {
			wasClicked: dojo.hitch(this, function(e) {
				dojo.stopEvent(e);
				this._deleteAlert(alert.itemId, tr);
			})
		};
		simpleToolbar.addButton(deleteHandler, "wasClicked", {
			label: this.messages.deleteAlert,
			href:"#",
			imgSrc: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/delete_hov.gif")
		});
		var actionsTd = dojo.create("td", { className: "action" }, tr);
		actionsTd.appendChild(simpleToolbar.domNode);
	},
	
	showNoAlertsRow: function(isLoading) {
		if(isLoading) {
			var msg = this.messages.loading;
		} else {
			var url = "<a id=\"_addAlertLink\" class=\"editLink\">" + this.messages.addAlertLink + "</a>";
			var msg = dojo.string.substitute(this.messages.noAlerts, [url]);
		}
		dojo.empty(this.styledTable.tbody);
		var row = dojo.create("tr", {}, this.styledTable.tbody);
		dojo.create("td", { className: "noRowsYet", colSpan: 6, innerHTML: msg} , row);
		if (!isLoading) {
			this.connect(dojo.byId("_addAlertLink"), "onclick", "_addAlert");
		}
	},
	
	_addAlert: function() {
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.AddSystemAlertStep({
			stepTitle: this.messages.createAlertTitle
		});	
		var self = this;
		var wizardArgs = {
			steps: [step1],
			primaryTitle: this.messages.createAlertTitle,
			width: "600px",
			showStepSummary: false,
			processFinish: function() {
				self.reload();
				this.container.viewBox.close();
			},			
			processCancel: function() {		
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}		
			}
		}
		this.addWizard = new jazz.ui.Wizard(wizardArgs);
		
	},
	
	_editAlert: function(alert) {
		
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.EditSystemAlertStep({
			stepTitle: this.messages.editAlertTitle,
			alert: alert
		});	
		var self = this;
		var wizardArgs = {
			steps: [step1],
			primaryTitle: this.messages.editAlertTitle,
			width: "600px",
			showStepSummary: false,
			processFinish: function() {
				self.reload();
				this.container.viewBox.close();
			},			
			processCancel: function() {		
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}		
			}
		}
		this.editWizard = new jazz.ui.Wizard(wizardArgs);
		
	},
	
	_deleteAlert: function(itemId, row) {
		var verifyDelete = confirm(this.messages.confirmDelete);
        if (verifyDelete) {
			dojo.xhrDelete({
		    	url: net.jazz.ajax.contextRoot() + "/" + SYSTEM_ALERTS_ENDPOINT + "/" + itemId ,
				handleAs: "text",
				headers: this.headers,
				load: dojo.hitch(this, function(response){
					 this.styledTable.tbody.removeChild(row);
				}),
				error: function(error){
					console.error(error);
				}
			});
		}
	},
	
	getExecutionTime: function(datetime) {
		if(!datetime) {
			return this.messages.never;
		}
		return executionTime = jazz.app.i18n.date.locale.format(datetime);
	},
	
	sortStartDate: function () {	
		return this.alertsCache.sort(dojo.hitch(this,function(a, b){
			var result = a.startDate > b.startDate ? 1 : -1;
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	sortEndDate: function () {	
		return this.alertsCache.sort(dojo.hitch(this,function(a, b){
			var result = a.endDate > b.endDate ? 1 : -1;
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	sortAuthor: function () {	
		return this.alertsCache.sort(dojo.hitch(this,function(a, b){
			var result;
			if (!a || !a.contributorId) {
				result = 1;
			} else if (!b || !b.contributorId) {
				result = -1;
			} else {
				result = a.contributorId.localeCompare(b.contributorId);
			}
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	sortStatus: function () {	
		return this.alertsCache.sort(dojo.hitch(this,function(a, b){
			var result;
			if (!a || !a.status) {
				result = 1;
			} else if (!b || !b.status) {
				result = -1;
			} else {
				result = a.status > b.status ? 1 : -1;
			}
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	sortDescription: function () {	
		return this.alertsCache.sort(dojo.hitch(this,function(a, b){
			var result;
			if (!a || !a.description) {
				result = 1;
			} else if (!b || !b.description) {
				result = -1;
			} else {
				result = a.description.localeCompare(b.description);
			}
			if (!this.styledTable.sortByAscending) {
				result = result * (-1);
			}
			return result;
		}));
	},
	
	destroyActionToolbars: function(){
		for (var i = 0; i < this._actionToolbars.length; i++) {
			this._actionToolbars[i].destroyRecursive();
		}
		this._actionToolbars = [];
	},	
	
	uninitialize: function(){
		this.destroyActionToolbars();
		this.header.destroyRecursive();
		if (this.addWizard) {
			this.addWizard.destroyRecursive();
			delete this.addWizard;
		}
		if (this.editWizard) {
			this.editWizard.destroyRecursive();
			delete this.editWizard;
		}
		this.inherited(arguments);
	}
	
});

/*
 * Wizard Step for adding new System Alerts
 */

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.AddSystemAlertStep", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/AddSystemAlert.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "SystemAlertsMessages");
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		this._dijitWidgets = new Array();
	},	

	postCreate: function() {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		this.connect(this._addTableBody,"onkeyup","updateFinishButton");
		this.isCompleted = false;
		
		var now = new Date();
		this._endDate = this.createDateTextBox(this._endDateContainer);
		this._endTime = this.createTimeTextBox(this._endTimeContainer);
		this._startDate = this.createDateTextBox(this._startDateContainer, now);
		this._startTime = this.createTimeTextBox(this._startTimeContainer, now);
		
		//Needed for comparing in hasUpdates() due to formatting issues when comparing directly to this.alert.*
		this._orignalStartTime = this._startTime.value;
		this._orignalStartDate = this._startDate.value;
	},
	
	createDateTextBox: function(attachNode, date){
        var constraints = {};
        if (dojo.i18n.normalizeLocale() == "en-us") {
            constraints = {
                datePattern: "MMM d, yyyy"
            };
        }
        var dateWidget = new dijit.form.DateTextBox({
            isDate: true,
            required: true,							
            containerToggle: "fade",
            constraints: constraints,
			placeHolder: this.messages.date
        }, attachNode);
        
        if (date) {
        	dateWidget.set('value', date);
        }
        
        // Date widget dynamically replaced its class so need to set element style.
        dateWidget.domNode.style.margin = "0";
        dateWidget.domNode.style.fontFamily = "inherit";
		
		this.connect(dateWidget, "onChange", "updateElements");
		this._dijitWidgets.push(dateWidget);
		return dateWidget;
    },
	
	createTimeTextBox: function(attachNode, date){
        var constraints = {
			clickableIncrement:'T00:30', 
			visibleIncrement:'T01:00', 
			visibleRange:'T03:00'
		};
        var timeWidget = new dijit.form.TimeTextBox({
        	required: true,
            constraints: constraints,
			placeHolder: this.messages.time
        }, attachNode);
        
        if (date) {
        	timeWidget.set('value', date);
        }
        
        // Date widget dynamically replaced its class so need to set element style.
        timeWidget.domNode.style.margin = "0";
        timeWidget.domNode.style.fontFamily = "inherit";
		
		this.connect(timeWidget, "onChange", "updateElements");
		this._dijitWidgets.push(timeWidget);
		return timeWidget;
    },
	
	updateElements: function() {
		this.updateConstraints();
		this.updateFinishButton();
	},
	
	updateFinishButton: function() {
		this.isCompleted = true;
		if( this._description.value == "" ||
			! this._startDate.isValid() ||
			! this._endDate.isValid() ||
			! this._startTime.isValid() ||
			! this._endTime.isValid() )
		{
			this.isCompleted = false;
		}
		this.triggerButtonStateUpdate();
	},
	
	updateConstraints: function() {
		if ( this._startDate.isValid() ) 
		{
			var sDate = this._startDate.value;
			var eDate = this._endDate.value;
			
			this._endDate.constraints.min = sDate;
			
			if ( dojo.date.compare(sDate, eDate, "date") === 0 ) {
				var sTime = this._startTime.value;
				this._endTime.constraints.min = dojo.date.add(sTime, "minute", 1); 
			} else {
				this._endTime.constraints.min = "T00:00:00";
			}
			
			this._validateDateAndTime();
		}
	},
	
	_validateDateAndTime: function() {
		this._startDate.validate();
		this._startTime.validate();
		this._endDate.validate();
		this._endTime.validate();
	},
	
	canFinish: function() {
		return this.isCompleted;
	},
	
	_getFullDateTime: function() {
		var sDate = this._startDate.value;
		var sTime = this._startTime.value;
		sDate.setHours(sTime.getHours());
		sDate.setMinutes(sTime.getMinutes());
		
		var eDate = this._endDate.value;
		var eTime = this._endTime.value;
		eDate.setHours(eTime.getHours());
		eDate.setMinutes(eTime.getMinutes());
		
		return [ sDate, eDate ];
	},
	
	onFinish: function() {
		var dates = this._getFullDateTime();
		var postArgs = {
			"description": this._description.value,
			"startDate": dojo.date.stamp.toISOString(dates[0]),
			"endDate": dojo.date.stamp.toISOString(dates[1])
		};

		var headers = this.headers;
		headers[HttpConstants.ACCEPT] = HttpConstants.CT_TEXT_JSON;
		headers[HttpConstants.CONTENT_TYPE] = HttpConstants.CT_TEXT_JSON;
		
		var deferred = jazz.client.rawXhrPost({
	    	url: net.jazz.ajax.contextRoot() + "/" + SYSTEM_ALERTS_ENDPOINT,
			handleAs: "json",
			headers: headers,
			postData: dojo.toJson(postArgs)
		});
		
		deferred.addCallback(dojo.hitch(this, function(response){
			this.wizard.processFinish();			
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject){
			this.header.showMessage({
				errorObject: errorObject,
                summary: this.messages.errorSavingNewAlert
            });
		}));
	},
	
	hasUpdates: function() {
		if(this._description.value != "") return true;
		if( ! this._startDate.value || ! this._startTime.value) return true;
		if(this._orignalStartTime !== this._startTime.value) return true;
		if(this._orignalStartDate !== this._startDate.value) return true;
		if( ! this._startTime.value) return true;
		if( ! this._endTime.value) return true;
		return false;
	},
	
	_destroyDijitWidgets: function(){
		for (var i = 0; i < this._dijitWidgets.length; i++) {
			this._dijitWidgets[i].destroyRecursive();
		}
		this._dijitWidgets = [];
	},
	
	uninitialize: function(){
		// Destroy the message box
		this.header.destroyRecursive();
		this._destroyDijitWidgets();
		this.inherited(arguments);
	}
});


/*
 * Wizard Step for editing System Alerts
 */

var AddSystemAlertStep = com.ibm.team.repository.web.ui.internal.admin.server.AddSystemAlertStep;
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.EditSystemAlertStep", [AddSystemAlertStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/AddSystemAlert.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "SystemAlertsMessages");
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
	},	

	postCreate: function() {
		this.inherited(arguments);
		
		this._description.value = this.alert.description;
		this._startDate._setValueAttr(this.alert.startDate);
		this._endDate._setValueAttr(this.alert.endDate);
		this._startTime._setValueAttr(this.alert.startDate);
		this._endTime._setValueAttr(this.alert.endDate);
		
		//Needed for comparing in hasUpdates() due to formatting issues when comparing directly to this.alert.*
		this._orignalStartTime = this._startTime.value;
		this._orignalEndTime = this._endTime.value;
		this._orignalStartDate = this._startDate.value;
		this._orignalEndDate = this._endDate.value;
	},
	
	hasUpdates: function() {
		if(this._description.value !== this.alert.description) return true;

		if(this._orignalStartDate !== this._startDate.value) return true;
		if(this._orignalStartTime !== this._startTime.value) return true;
		if(this._orignalEndDate !== this._endDate.value) return true;
		if(this._orignalEndTime !== this._endTime.value) return true;
		return false;
	},
	
	onFinish: function() {	
		var dates = this._getFullDateTime();
		var postArgs = {
			"description": this._description.value,
			"startDate": dojo.date.stamp.toISOString(dates[0]),
			"endDate": dojo.date.stamp.toISOString(dates[1])
		};

		var headers = this.headers;
		headers[HttpConstants.ACCEPT] = HttpConstants.CT_TEXT_JSON;
		headers[HttpConstants.CONTENT_TYPE] = HttpConstants.CT_TEXT_JSON;
		
		var deferred = jazz.client.rawXhrPut({
	    	url: net.jazz.ajax.contextRoot() + "/" + SYSTEM_ALERTS_ENDPOINT + "/" + this.alert.itemId,
			handleAs: "json",
			headers: headers,
			postData: dojo.toJson(postArgs)
		});
		
		deferred.addCallback(dojo.hitch(this, function(response){
			this.wizard.processFinish();			
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject){
			this.header.showMessage({
				errorObject: errorObject,
                summary: this.messages.errorSavingEdits
            });
		}));
	}
	
});

})();