/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ToggleDevMode");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

(function() {
	
var CheckBox	 			= dijit.form.CheckBox;
var MessageBox				= jazz.ui.MessageBox;
var STATUS_TYPES 			= MessageBox.STATUS_TYPES;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient 		= com.ibm.team.repository.web.client.internal.ServerStatusClient;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ToggleDevMode", [dijit._Widget, dijit._Templated], {

	templateString: "<div><div dojoAttachPoint=\"headerArea\"></div><div dojoAttachPoint=\"_loadingMsg\" style=\"display:none\">Loading...</div><div dojoAttachPoint=\"toggleDiv\"></div></div>",

	postCreate: function() {
		this._showLoadingMsg();
		
		this.header = new MessageBox();
        this.headerArea.appendChild(this.header.domNode);
		
		var srh = new ServiceResponseHandler(this, "_initToggle", "_handleInitError");
		ServerStatusClient.getDevMode(srh);
	},

	_initToggle: function(data) {
		this._hideLoadingMsg();
		this.toggle = new CheckBox({
			checked: (data && data.id === 1),
			id: "toggleDevModeCheckbox"
		});
		this.toggle.onChange = dojo.hitch(this, "_handleToggleDevMode");
		
		var _toggleLabel = dojo.create("label", { htmlFor: this.toggle.id, innerHTML: " Development Mode Enabled"});

		this.toggleDiv.appendChild(this.toggle.domNode);
		this.toggleDiv.appendChild(_toggleLabel);
		
		if ( data.code !== 'OK' && data.message) {
			this._displayErrorMessage(data);
		}
	},
	
	_handleInitError: function(errorObj) {
		this._hideLoadingMsg();
		
		this._displayErrorMessage(errorObj);
	},
	
	_showLoadingMsg: function () {
		this._loadingMsg.style.display = "";
	},
	
	_hideLoadingMsg: function () {
		this._loadingMsg.style.display = "none";
	},
	
	_handleToggleDevMode: function(value) {
		this.toggle.set('disabled', true);
		this.headerArea.style.display = "none";
		var srh = new ServiceResponseHandler(this, "_displayResult", "_requestFailed");
		ServerStatusClient.postDevMode(srh, value);
	},
	
	_displayResult: function(data) {
		var newMode = (data.id === 1);
		
		if ( data.code !== 'OK' && data.message) {
			this._displayErrorMessage(data);
		}

		this.toggle.set('disabled', false);
	},
	
	_requestFailed: function(errorObj) {
		if (this.toggle) {
			this.toggle.set('disabled', false);
		}
		
		this._displayErrorMessage(errorObj);
	},
	
	_displayErrorMessage: function(errorObj){	
		this.headerArea.style.display = "";
        this.header.showMessage({
			type: errorObj.code ? errorObj.code : "ERROR",
        	errorObject: errorObj
		});
            
	},
	
	uninitialize: function(){
		if( this.toggle ){
			this.toggle.destroyRecursive();
		}
		
		if (this.header) {
			this.header.destroyRecursive();
		}
		this.inherited(arguments);
	}
});	

})();
