/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.TokenLicenseFileUploadStep");
    
    dojo.require("dojo.date");
    dojo.require("dojo.date.locale");
    dojo.require("dojo.date.stamp");
    dojo.require("dojo.number");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojox.xml.parser");
    
    dojo.require("net.jazz.ajax.contextRoot");
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    
    dojo.require("com.ibm.team.repository.web.client.internal.licensing");
    dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FileUploader");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
    dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
    dojo.require("com.ibm.team.repository.web.ui.internal.Util");
    dojo.require("dojo.i18n");
    
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseQuickConnectStep");
    dojo.require("jazz.ui.wizard.WizardStep");
    dojo.require("jazz.ui.MessageBox");
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseTokenViewMessages");
    
    //
    // Aliases
    //
    
    var FileUploader = com.ibm.team.repository.web.ui.internal.admin.server.FileUploader;
    var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
    var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
    var LIC_P = com.ibm.team.repository.web.client.internal.licensing;
    var formatCALTypeDescription = LIC_P.formatCALTypeDescription;
    var getLicenseInfo = LIC_P.getLicenseInfo;
    var getUsedLicenseInfo = LIC_P.getUsedLicenseInfo;
    var getExpirationInfo = LIC_P.getExpirationInfo;
    var sortCALs = LIC_P.sortCALs;
    var postLicenseInfo = LIC_P.postLicenseInfo;
    var removeChildren = dojox.xml.parser.removeChildren;
    var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var UPLOAD_URI = LIC_P.FILE_UPLOAD_URI;
    var FLOATING_LICENSE_SERVICE_NAME = "com.ibm.team.repository.service.internal.license.LicenseService";
    var FLOATING_SUPPORT_OPERATION_ID = "com.ibm.team.repository.support.floatingLicenses";
    var LicenseQuickConnectStep = com.ibm.team.repository.web.ui.internal.admin.server.LicenseQuickConnectStep;
    var MessageBox = jazz.ui.MessageBox;
    var WizardStep = jazz.ui.wizard.WizardStep;
    var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
    var STATUS_TYPES = MessageBox.STATUS_TYPES;
    var LICENSE_SERVICE_IMPL = "com.ibm.team.repository.service.internal.license.token.LicenseTokenService";
    var LICENSE_TOKEN_SERVICE_PROVIDERS_PROP = "tokenProviderService";
    var JAZZ_TOKEN_PROVIDER_ID = "com.ibm.team.repository.service.internal.license.token.JazzLicenseTokenProviderService";
    var JAZZ_TOKEN_PROVIDER_IMPL_CLASS = "com.ibm.team.repository.service.internal.license.token.JazzLicenseTokenProviderService";
    var JAZZ_TOKEN_SERVER_URI_PROP = "jazzTokenProvider.tokenServerURI";
    var UPLOAD_SERVICE_URL = "service/com.ibm.team.jfs.license.internal.token.ILicenseTokenUploadService";
    var callContributedCode = net.jazz.ajax.core.internal.util.callContributedCode;
    var messages;
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.TokenLicenseFileUploadStep", [WizardStep, dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/TokenLicenseFileUploadStep.html"),
        constructor: function(args){
            this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseTokenViewMessages");
            this.elementPreventCSRF = {
                name: "X-Jazz-CSRF-Prevent",
                value: dojo.cookie("JSESSIONID")
            };
            this.formElements = args.formElements || [];
            this.formElements.push(this.elementPreventCSRF);
            this.formLabel = this.messages.tokenWizardFileTitle;
            
        },
        
        postCreate: function(){
        
			this.header = new MessageBox();		
			this.stepHeaderArea.appendChild(this.header.domNode);
			this.configUtil = new ConfigUtility();
            this.uploadForm = this.createForm();
            this.uploadFormArea.appendChild(this.uploadForm.domNode);
            var srh = new SRH(this, "_handleLicenseTokenServiceProps", "_handleServiceFetchError");
            ServerStatusClient.getServiceConfiguration(srh, LICENSE_SERVICE_IMPL);
            //this._resetFileInput();
        
        },
        
        _handleServiceFetchError: function(errorObj){
			this.header.showMessage({
				errorObject: errorObj
			});     
        },
        
        _handleJazzProviderServiceProps: function(serviceInfo){
            // find the url of the token server, if not the same as this , add the quick config step
            for (var i = 0; i < serviceInfo.properties.length; i++) {
                if (serviceInfo.properties[i].name == JAZZ_TOKEN_SERVER_URI_PROP) {
                    this.jazzTokenServerURI = serviceInfo.properties[i].overrideValue;
                }
            }
            
            this.localURI = window.location.protocol + "//" + window.location.hostname + ":" + window.location.port + net.jazz.ajax.contextRoot();
            if (this.licenseTokenServicePropvider && this.licenseTokenServicePropvider == JAZZ_TOKEN_PROVIDER_ID) {
                this.providerSetToJazz = true;
                
                if (this.jazzTokenServerURI && this.jazzTokenServerURI == this.localURI) 
                    this.jazzServerURISetToLocal = true;
            }
            
            if (!this.jazzServerURISetToLocal || !this.providerSetToJazz) 
                dojo.toggleClass(this.configArea, "noDisplay", false);
            this.connect(this.checkboxTokenServer, "onclick", function(e){
                if (e.target.checked) {
                    if (this.jazzTokenServerURI != null && this.jazzTokenServerURI != "") {
                        this.header.showMessage({
                            type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
                            summary: this.messages.overwriteServerURIWarning
                        });
                    }
                }
                else {
                    this.header.hideMessage();
                }
                
            });
            
        },
        
        _handleLicenseTokenServiceProps: function(serviceInfo){
            for (var i = 0; i < serviceInfo.properties.length; i++) {
                if (serviceInfo.properties[i].name == LICENSE_TOKEN_SERVICE_PROVIDERS_PROP) {
                    this.licenseTokenServicePropvider = serviceInfo.properties[i].overrideValue;
                }
            }
            
            var srh = new SRH(this, "_handleJazzProviderServiceProps", "_handleServiceFetchError");
            ServerStatusClient.getServiceConfiguration(srh, JAZZ_TOKEN_PROVIDER_IMPL_CLASS);
            
            
        },
        
        
        createForm: function(typeCode, formName){
            var self = this;
            var uploadUrl = net.jazz.ajax.contextRoot() + "/" + UPLOAD_SERVICE_URL;
            var args = {
                formLabel: this.messages.uploadTokenLicenseKey,
                onBeginUpload: function(){
                    self.header.showMessage({
                        type: STATUS_TYPES.INFO,
                        summary: self.messages.uploadingFile
                    });
                },
                onUpload: function(o){
                    self.onUpload(o);
                },
                onUploadFailure: function(errObj){
                    self.onUploadFailure(errObj);
                },
                fileInputChanged: function(){
                    self.triggerButtonStateUpdate();
                },
                postURI: uploadUrl,
                sendOnInputChange: false
            }
            return new FileUploader(args);
        },
        
        onUpload: function(o){
            dojo.publish("/token/changed", [{
                event: "added"
            }]);
            // Check to see if we need to set the provider & provider server url
            if (this.checkboxTokenServer.checked) {
                this.configure();
            }
            else 
                this.finished();
            
        },
        
        configure: function(){
            var changes = [];
            if (!this.providerSetToJazz) {
                changes.push({
                    "name": LICENSE_TOKEN_SERVICE_PROVIDERS_PROP,
                    "serviceImplementationName": LICENSE_SERVICE_IMPL,
                    "value": JAZZ_TOKEN_PROVIDER_ID
                })
            }
            if (!this.jazzServerURISetToLocal) 
                changes.push({
                    "name": JAZZ_TOKEN_SERVER_URI_PROP,
                    "serviceImplementationName": JAZZ_TOKEN_PROVIDER_IMPL_CLASS,
                    "value": this.localURI
                });
            
            
            var changeList = {
                "changes": changes,
                "properties": []
            };
            this.configUtil.save(changeList, this, "finished", "onUploadFailure");
        },
        
        cancelRequested: function(){
            this.parentPointer.wizard.contentArgs.container.close();
        },
        
        performFinish: function(){
            this.header.hideMessage();
            this.uploadForm.send();
        },
        
        onUploadFailure: function(responseObj){
            this.header.showMessage({
                type: STATUS_TYPES.ERROR,
                summary: responseObj.errorMessage
            });
        },
        
        finished: function(){
        
        },
        
        canFlipToNextPage: function(){
            return this.canFinish();
        },
        
        
        canFinish: function(){
            return this.uploadForm.fileInput.value;
        },
		
		uninitialize: function(){
			this.header.destroyRecursive();
			this.inherited(arguments);
		}
        
        
    });
})();
