/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.catalog.AddWidgetHelper");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("jazz.ensemble.internal.catalog.ErrorUtility");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "AddWidgetHelperMessages");

(function() {

/*
 * Aliases 
 */
var ErrorUtility = jazz.ensemble.internal.catalog.ErrorUtility;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DiscoveryRegistry = com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ViewletRegistry = com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry;

/*
 * Constants 
 */
var GADGET_TYPE = "org.opensocial.gadget";
var IWIDGET_TYPE = "com.ibm.iwidget";
var IMAGE_SERVICE = "/service/com.ibm.team.dashboard.common.internal.service.IDashboardImageService";
var PATH_ARG = "?path=";
var URL_ARG = "?url=";

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.catalog.AddWidgetHelper",
	[dijit._Widget], {

	// Properties //----------------------------------------------------------->

	onAddWidget: null,

	// ------------------------------------------------------------------------>

	/*
 	 * @param Object args
 	 * 
 	 * @return AddWidgetHelper 
 	 */
	constructor: function(args) {
		this.msg = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "AddWidgetHelperMessages");
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

		this._remoteServerData = null;
		
	},	

	/*
 	 * @param Object data
 	 * @param String rootServices
 	 * @param Boolean local
 	 * 
 	 * @return dojo.Deferred
 	 */		
	addWidget: function(data, rootServices, local) {
		// TEST
		//console.debug(data);

		var deferred = new dojo.Deferred();
		
		if (local) {
			// Local
			this._callRESTAddWidget(deferred, data);
		} else {
			// Remote
			var deferred2 = this._getRemoteServerData(rootServices);
			deferred2.addCallback(dojo.hitch(this, function(serverData) {
				this._callRESTAddWidget(deferred, data);
				return serverData; 
			}));			
			deferred2.addErrback(dojo.hitch(this, function(errorObject) {
				deferred.errback(errorObject);
				return errorObject; 
			}));
		}
		
		return deferred;
	},

	/*
 	 * @param String rootServices
 	 * 
 	 * @return dojo.Deferred
 	 */		
	_getRemoteServerData: function(rootServices) {
		var deferred = new dojo.Deferred();
		
		if (this._remoteServerData) {
			deferred.callback(this._remoteServerData);
		} else {
			this._callRESTGetRemoteServerData(deferred, rootServices);
			deferred.addCallback(dojo.hitch(this, function(serverData) {
				this._remoteServerData = serverData;
				return serverData; 
			}));			
		}

		return deferred;
	},	

	/*
 	 * @param dojo.Deferred deferred
 	 * @param String rootServices
 	 * 
 	 * @return void
 	 */
	_callRESTGetRemoteServerData: function(deferred, rootServices) {
		
		var successCallback = dojo.hitch(this, function(serverData) {
			// TEST
			//console.debug(serverData);
			deferred.callback(serverData);
		});
		var errorCallback = dojo.hitch(this, function(errorObject) {
			ErrorUtility.logError(errorObject, this.id);
			ErrorUtility.logError(new Error(this.msg.errServerData), this.id);
			deferred.errback(errorObject);
		});
		
		DiscoveryRegistry.getServerData(
			rootServices, 
			false, 
			successCallback, 
			errorCallback);

	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Object data
 	 * 
 	 * @return void
 	 */
	_callRESTAddWidget: function(deferred, data) {
		
		if (data.type === GADGET_TYPE) {
			// Internal OpenSocial Gadgets (catalog service document)
			this._processInternalWidget(deferred, data);
		} else {
			// External OpenSocial Gadgets
			// External IBM iWidgets
			// Viewlets
			this._getWidgetDefinition(deferred, data);
		}
	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Object data
 	 * 
 	 * @return void
 	 */		
	_processInternalWidget: function(deferred, data) {

		// Data is being cached, making a copy ...
		var dataCopy = dojo.clone(data);

		// Use image service to resolve images through the proxy
		// Assumption: Absolute url to the icon
		if (dataCopy.icon) {
			dataCopy.icon = Util.getServiceRoot() + 
						IMAGE_SERVICE + 
						URL_ARG + 
						encodeURIComponent(dataCopy.icon);
		}


		if (this.onAddWidget) {
			this.onAddWidget(dataCopy);
		} else {
			this._processAddWidgetFailure(deferred, new Error(this.msg.errUndefinedHandler));
			// TODO: MP: Call deferred error callback?
			return;
		}
		
		deferred.callback(true);
		
	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Object data
 	 * 
 	 * @return void
 	 */
	_getWidgetDefinition: function(deferred, data) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(definition) {
				this._processWidgetDefinitions(deferred, definition, data);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				// BUG: Expected this to be called when no definitions were found ...
				this._processAddWidgetFailure(deferred, errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		ViewletRegistry.getDefinition(
			data.url || data.widgetUri, 
			handler);		
	},

	/*
 	 * @param dojo.Deferred deferred
 	 * @param Object definition
 	 * @param Object data
 	 * 
 	 * @return void
 	 */		
	_processWidgetDefinitions: function(deferred, definition, data) {

		if (!definition) {
			var errorMessage = this.msg.errWidgetDefinition;
			if (data.title) {
				errorMessage = errorMessage + ": " + data.title;
			}
			var errorObject = new Error(errorMessage);
			this._processAddWidgetFailure(deferred, errorObject);
			return;
		}

		if (!data.title) {
			if (definition.name) {
				data.title = definition.name;
			} else if (this._isExternalGadget(definition.id)) {
				// External OpenSocial Gadget
				data.title = this.msg.gadgetTitle;
			} else if (this._isExternalIWidget(definition.id)) {
				// External IBM iWidget
				data.title = this.msg.iWidgetTitle;
			} else {
				data.title = this.msg.viewletTitle;			
			}
		}
		
		if (!data.icon) {
			if (definition.icon.indexOf("http://") == 0
					|| definition.icon.indexOf("https://") == 0) {
				data.icon = definition.icon;
			} else {
				/* URL is relative - use the Image Service to fetch the image*/
				var serverRoot = this._remoteServerData ? this._remoteServerData.viewletServiceRoot : Util.getServiceRoot();
				data.icon = serverRoot + IMAGE_SERVICE + PATH_ARG + encodeURIComponent(definition.icon);				
			}
		}

		if (!data.type) {
			if (this._isExternalGadget(definition.id)) {
				// External OpenSocial Gadget
				data.type = GADGET_TYPE;
				data.widgetUri = definition.url;
			} else if (this._isExternalIWidget(definition.id)) {
				// External IBM iWidget
				data.type = IWIDGET_TYPE;
				data.widgetUri = definition.url;
			}
		}
		
		if (this.onAddWidget) {
			this.onAddWidget(data);
		} else {
			this._processAddWidgetFailure(deferred, new Error(this.msg.errUndefinedHandler));
			// TODO: MP: Call deferred error callback?
			return;
		}
		
		deferred.callback(true);
		
	},

	/*
 	 * @param String id
 	 * 
 	 * @return Boolean
 	 */		
	_isExternalGadget: function(id) {
		return (id.indexOf("g:") === 0);
	},	

	/*
 	 * @param String id
 	 * 
 	 * @return Boolean
 	 */		
	_isExternalIWidget: function(id) {
		return (id.indexOf("i:") === 0);
	},	
	
	/*
 	 * @param dojo.Deferred deferred
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */		
	_processAddWidgetFailure: function(deferred, errorObject) {
		ErrorUtility.logError(errorObject, this.id);
		deferred.errback(errorObject);
	},

	/*
 	 * @param Boolean local
 	 * 
 	 * @return String
 	 */
	_getRepositoryRoot: function(local) {
		
		if (local || !this._remoteServerData) {
			return null;
		}
		
		return this._remoteServerData.viewletServiceRoot;
	}
	
});

})();
