/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.AttachmentManager"); //$NON-NLS-1$

	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.Attachment"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper");//$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.CopyResourceLinkDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.editors.UploadAttachmentDialog"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.Toolbar") //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button") //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var UploadAttachmentDialog = com.ibm.team.process.authoring.web.common.ui.internal.editors.UploadAttachmentDialog;
	var CopyResourceLinkDialog = com.ibm.team.process.authoring.web.common.ui.internal.editors.CopyResourceLinkDialog;
	var Attachment = com.ibm.team.process.authoring.web.common.common.internal.Attachment;
	
	function getRow(evt) {
		var e = evt.target;
		while (e) {
			if (e.tagName === "TR") { //$NON-NLS-1$
				return e;
			}
			e = e.parentNode;
		}
		return null;
	}
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.AttachmentManager", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "ui/internal/editors/templates/AttachmentManager.html"), //$NON-NLS-1$ //$NON-NLS-2$
		practiceResource: null,
		viewOnly: false,
		count: 0,
		toolBarList: null,
		container: null,
		contentEditor: null,
		btnNum: 2,
				
		constructor: function(args) {
			this.toolBarList = [];
			this._toolbarButtons = [];
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this.inherited(arguments);
			if (this.viewOnly) {
				this._toPreviewMode();
			} else {
				this.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_VIEWMODE, this._viewMode); //$NON-NLS-1$
				this.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_EDITMODE, this._editMode); //$NON-NLS-1$
				if ( this.contentEditor && this.contentEditor.isViewMode() ) {
					this._toPreviewMode();
				}
			}
		},
		
		_toPreviewMode: function() {
			dojo.toggleClass(this.addAttachmentArea, ProcessAuthoringConstants.CSS_HIDE, true); //$NON-NLS-1$
			this.btnNum = 1;
		},
		
		refresh: function() {
			dojo.empty(this._attachmentTableBody);
			this.destroyToolBars();
			
			if (this.container.editPractice) {
				if (!this.practiceLoadedInWorkingCopyHandle) {
					this.practiceLoadedInWorkingCopyHandle = dojo.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_PRACTICE_LOADED_IN_WORKINGCOPY, this, this._getAttachmentsByPractice);
				}
			} else {
				this._getAttachmentsByPractice();
			}
		},
		
		_getAttachmentsByPractice: function() {
			var practice = this.container.workingCopy.getNodeById(this.practiceResource).practice;
			ProcessAuthoringClient.getAttachmentsByPractice({
				url: practice.attachmentsUrl,
				load: dojo.hitch(this, function(data) {
					var attachments = UIHelper.getAttachmentNodes(data);
					this.setCount(attachments.length);
					dojo.forEach(attachments, function(attachment) {
						var attachmentObj = new Attachment(attachment);
						this.addRow(attachmentObj);
					}, this);
				})
			});
		},
		
		_viewMode: function() {
			this._toPreviewMode();
		},
		
		_editMode: function() {
			dojo.toggleClass(this.addAttachmentArea, ProcessAuthoringConstants.CSS_HIDE, false); //$NON-NLS-1$
			this.btnNum = 2;
		},
		
		/**
		 * @param {com.ibm.team.process.authoring.web.common.common.internal.Attachment} attachmentObj
		 */
		getRowData: function(attachmentObj) {
			var row = [];
			var path = attachmentObj.path;
			var size = attachmentObj.size;
			var creator = attachmentObj.creatorUrl;
			var date = attachmentObj.creationDate;
			var service = attachmentObj.service;
			
			var pathNode = document.createTextNode(path);
			var sizeNode = document.createTextNode(size);
			var creatorNode = document.createElement("a"); //$NON-NLS-1$
			var creatorId = creator.substring(creator.lastIndexOf("/") + 1, creator.length); //$NON-NLS-1$
			creatorNode.appendChild(document.createTextNode(creatorId));
			creatorNode.href = creator;
			creatorNode.title = creator;
			var dateNode = document.createTextNode(date);
			var serviceNode = document.createTextNode(service);
			
			row.push(pathNode);
			row.push(sizeNode);
			row.push(creatorNode);
			row.push(dateNode);
			row.push(serviceNode);
			return row;
		},
		
		/**
		 * @param {com.ibm.team.process.authoring.web.common.common.internal.Attachment} attachmentObj
		 */
		addRow: function(attachmentObj) {
			var row = this.getRowData(attachmentObj);
			var tr = document.createElement("tr"); //$NON-NLS-1$
			this.connect(tr, "onmouseover", this._onMouseOver); //$NON-NLS-1$
			this.connect(tr, "onmouseout", this._onMouseOut); //$NON-NLS-1$
			this._attachmentTableBody.appendChild(tr);
			for (var i = 0; i < row.length; i++) {
				var td = tr.insertCell(i);
				dojo.addClass(td, "data-column"); //$NON-NLS-1$
				td.appendChild(row[i]);
			}
			// add action bar
			var actionTd = tr.insertCell(i);
			dojo.addClass(actionTd, "data-column"); //$NON-NLS-1$
			var actionDiv = document.createElement("div"); //$NON-NLS-1$
			var editToolBar = new jazz.ui.Toolbar();
			editToolBar.rowNode = tr;
			actionDiv.appendChild(editToolBar.domNode);
			var btn = new jazz.ui.toolbar.Button({
				imgSrc: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "graphics/icons/copyurl-button.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title: this.messages.PracticeTree_9,
				onClick: dojo.hitch(this, function(evt) {
						dojo.stopEvent(evt);
						new CopyResourceLinkDialog({
							resourceLink: attachmentObj.contentUrl
					});
				})
			});
			editToolBar.addChild(btn);
			editToolBar._copyUrlButton = btn;
			this._toolbarButtons.push(btn);
			btn.domNode.style.display = "none"; //$NON-NLS-1$

			if (!this.viewOnly) {
				btn = new jazz.ui.toolbar.Button({
					imgSrc: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "graphics/icons/delete-button.gif"), //$NON-NLS-1$ //$NON-NLS-2$
					title: this.messages.AttachmentManager_1,
					onClick: dojo.hitch(this, function() {
						if (confirm(this.messages.AttachmentManager_2)) {
							ProcessAuthoringClient.deleteAttachment({
								'url': attachmentObj.url, //$NON-NLS-1$
								'load': dojo.hitch(this, function() { //$NON-NLS-1$
									this._attachmentTableBody.removeChild(tr);
									this.setCount(this.count - 1);
								}),
								'error': dojo.hitch(this, function(error, ioArgs) { //$NON-NLS-1$
									if (error.responseText) {
										error.message = error.responseText;
									}
									var licenseWarning = UIHelper.getLicenseWarning(ioArgs);
									error.headerMessage = licenseWarning ? licenseWarning : this.messages.PracticeError_1;
									this.container.displayErrorMessage(error);
									
								})
							});
						}
					})
				});
				editToolBar.addChild(btn);
				editToolBar._deleteButton = btn;
				this._toolbarButtons.push(btn);
				btn.domNode.style.display = "none"; //$NON-NLS-1$
			}
			this.toolBarList.push(editToolBar);
			actionTd.appendChild(actionDiv);
		},
		
		_onAddFile: function(evt) {
			if (this.practiceResource) {
				var uploadAttachment = new UploadAttachmentDialog({
					practiceURL: this.practiceResource
				});
				this.connect(uploadAttachment, "onSuccess", "onSuccess"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				alert(this.messages.AttachmentManager_0);
			}
		},
		
		onSuccess: function(content, resource, lastModified) {
			ProcessAuthoringClient.getAttachment({
				'lastModified': lastModified, //$NON-NLS-1$
				'url': resource, //$NON-NLS-1$
				'load': dojo.hitch(this, function(data) { //$NON-NLS-1$
					var attachmentObj = new Attachment(data.documentElement);
					this.addRow(attachmentObj);
					this.setCount(this.count + 1);
					if (this.contentEditor) {
						this.container.resizeEditor(this.contentEditor);
					}
				})
			});
		},
		
		_onMouseOver: function(evt) {
			var row = getRow(evt);
			dojo.some(this.toolBarList, function(toolBar) {
				if (toolBar.rowNode == row) {
					toolBar._copyUrlButton.domNode.style.display = ""; //$NON-NLS-1$
                    if (this.btnNum > 1) {
                    	toolBar._deleteButton.domNode.style.display = ""; //$NON-NLS-1$
                    }
					return true;
				}
			}, this);
		},
		
		_onMouseOut: function(evt) {
			var row = getRow(evt);
			dojo.some(this.toolBarList, function(toolBar) {
				if (toolBar.rowNode == row) {
					toolBar._copyUrlButton.domNode.style.display = "none"; //$NON-NLS-1$
                    if (this.btnNum > 1) {
                    	toolBar._deleteButton.domNode.style.display = "none"; //$NON-NLS-1$
                    }
					return true;
				}
			}, this);
		},
		
		getCount: function() {
			return this.count;
		},
		
		setCount: function(count) {
			this.count = count;
		},
		
		_destroyToolbarButtons: function() {
			dojo.forEach(this._toolbarButtons, function(button) {
				if (button) {
					button.destroyRecursive();
				}
			});
		},
		
		destroyToolBars: function() {
			this._destroyToolbarButtons();
			dojo.forEach(this.toolBarList, function(toolBar) {
				if (toolBar) {
					toolBar.destroyRecursive();
				}
			});
		},
		
		uninitialize: function() {
			if (this.practiceLoadedInWorkingCopyHandle) {
				dojo.unsubscribe(this.practiceLoadedInWorkingCopyHandle);
				this.practiceLoadedInWorkingCopyHandle = null;
			}
			
			this.destroyToolBars();
			this.inherited(arguments);
			
		}
	});
	
})();
