/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.LanguageDialog"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("dojo.io.iframe"); //$NON-NLS-1$
	dojo.require("dojox.uuid"); //$NON-NLS-1$
	dojo.require("dojox.uuid.generateTimeBasedUuid"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$ 
	dojo.require("jazz.ui.Dialog");//$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.LanguageDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "ui/internal/editors/templates/LanguageDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
		controller: null,
		resource: null,
		title: null,
		description: null,
		workingCopy: null,
		viewOnly: false,
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this.itemId = this.resource.substring(this.resource.lastIndexOf("/") + 1, this.resource.length); //$NON-NLS-1$
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.contentNode,
				primaryTitle: this.messages.LanguageDialog_6,
				destroyContents: true
			});
			
			var languageStore = new dojo.data.ItemFileReadStore({
				data: this.controller.getLanguageJsonObj()
			});
			this.languageSelect = new dijit.form.ComboBox({
				store: languageStore,
				labelAttr: 'label', //$NON-NLS-1$
				searchAttr: "label" //$NON-NLS-1$
			});
			this.languageSelectDiv.appendChild(this.languageSelect.domNode);
			
			var languages = this.workingCopy.getLanguagesById(this.itemId);
			this.initializeLanguageSelect();
			
			this.connect(this.cancelButton, "onclick", function(e) { //$NON-NLS-1$
				dojo.stopEvent(e);
				
				if (window.stop) {
					window.stop();
				}
				this.dialog.close();
			});
			
			if (!this.viewOnly && !this.controller.viewMode) {
				this.createButton.disabled = true;
				this.connect(this.languageSelect, "onFocus", "_onLanguageInput"); //$NON-NLS-1$ //$NON-NLS-2$
				this.connect(this.languageSelect, "onKeyUp", "_onLanguageChange"); //$NON-NLS-1$ //$NON-NLS-2$
				this.connect(this.createButton, "onclick", "_onCreate"); //$NON-NLS-1$ //$NON-NLS-2$
				if (languages.length > 1) {
					this.header.innerHTML = this.messages.LanguageDialog_0;
				} else {
					this.header.innerHTML = this.messages.LanguageDialog_13;
				}
			} else {
				dojo.addClass(this.editWrapper, "hide"); //$NON-NLS-1$
				if (languages.length > 1) {
					this.header.innerHTML = this.messages.LanguageDialog_11;
				} else {
					this.header.innerHTML = this.messages.LanguageDialog_12;					
				}
			}
			this.connect(this.languageSelect, "onChange", "_onLanguageChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.okButton, "onclick", "_onOk"); //$NON-NLS-1$ //$NON-NLS-2$ 
			this.connect(this.languageSelector, "ondblclick", "_onOk"); //$NON-NLS-1$ //$NON-NLS-2$
			this.languageSelect.setValue(""); //$NON-NLS-1$
		},
		
		initializeLanguageSelect: function() {
			var languages = this.workingCopy.getLanguagesById(this.itemId);
			var defaultLanguage = this.workingCopy.getDefaultLanguageById(this.itemId);
			if (languages.length > 0) {
				this.okButton.disabled = false;
			}
			dojo.forEach(languages, function(lang) {
				var label;
				if(lang == defaultLanguage) {
					label = dojo.string.substitute(this.messages.LanguageDialog_9, [this.controller.getFullLanguage(lang), this.messages.LanguageDialog_10]);
				} else {
					label = this.controller.getFullLanguage(lang);
				}
				var option = new Option(label, lang);
				this.languageSelector.options.add(option);
			}, this);
			this.languageSelector.selectedIndex = 0;
		},
		
		_onLanguageChange: function(e) {
			var langInput = this.languageSelect.getValue();
			if (langInput && dojo.trim(langInput) != "" && this.checkLanguageLegality(langInput)) { //$NON-NLS-1$
				this.createButton.disabled = false;
			} else {
				this.createButton.disabled = true;
			}
		},
		
		checkLanguageLegality: function(langInput) {
			if (!this.workingCopy.languageList) {
				this.workingCopy.languageList = this.controller.getLanguageJsonObj().items;
			}
			if (this.workingCopy.languageList) {
				return dojo.some(this.workingCopy.languageList, function(language) {
					if (language.label === langInput) {
						return true;
					}
					return false;
				}, this);
			}
			return false;
		},
		
		_onLanguageInput: function(e) {
			if (e) {
				dojo.stopEvent(e);
			}
			dojo.style(this.languageSelect.domNode, "color", "black"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(this.languageSelect.domNode, "fontStyle", "normal"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_onOk: function(e) {
			if (e) {
				dojo.stopEvent(e);
			}
			this.onOk();
		},
		
		_onCreate: function(e) {
			if (e) {
				dojo.stopEvent(e);
			}
			this.onCreate();
		},
		
		onCreate: function() {
		
		},
		
		onOk: function() {
		},
		
		getLanguageCode: function(lang) {
			var langCodeArray = lang.match(/\[.*]$/g);
			var langCode = null;
			if (langCodeArray != null) {
				langCode = langCodeArray[langCodeArray.length - 1];
				langCode = langCode.substring(1, langCode.length - 1);
			}
			if (langCode == null) {
				langCode = lang;
			}
			return langCode;
		},
		
		uninitialize: function() {
			this.inherited(arguments);
		}
	});
})();
