/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
	
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTreeModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("dijit.tree.ForestStoreModel"); //$NON-NLS-1$
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTreeModel", [dijit.tree.ForestStoreModel], { //$NON-NLS-1$

		showArchived: false,
		
		constructor: function(/* Object */ keywordParameters){
			this.inherited(arguments);
			this.refreshModel();
		},
		
		// Calls getChildren() on all nodes in the tree
		refreshModel: function() {
			var items = [this.root];
			while (items.length > 0) {
				var item = items.shift();
				this.getChildren(item, dojo.hitch(this, function(newChildren) {
					this.onChildrenChange(item, newChildren);
					items = items.concat(newChildren);
				}));
			}
		},
		
		/**
		 * Overridden to hide archived nodes and show unarchived nodes
		 * immediately after they are changed.
		 */
		onSetItem: function(item, attribute, oldValue, newValue) {
			this.inherited(arguments);
			if (attribute === "isArchived") { //$NON-NLS-1$
				var parent = this.getParent(item);
				if (parent) {
					this.getChildren(parent, dojo.hitch(this, function(newChildren) {
						this.onChildrenChange(parent, newChildren);
					}));
				}
			}
		},
		
		/**
		 * Get the parent of the given child item.
		 * 
		 * @param {Object} item
		 */
		getParent: function(item){
			var parentId = this.store.getValues(item, "parentId"); //$NON-NLS-1$
			if (typeof parentId !== "string") { //$NON-NLS-1$
				parentId = parentId[0];
			} 
			if (this.root.id === parentId || UIHelper.getResourceId(this.root.id) === parentId) {
				return this.root;
			} else {
				var parent;
				this.store.fetchItemByIdentity({
					identity: parentId,
					onItem: function(item, request){
						parent = item;
					}
				});
				return parent;
			}
		},
		
		/**
		 * Check if the given item is archived or not.
		 * 
		 * @param {Object} item
		 */
		isItemArchived: function(item) {
			var isArchived;
			if (item === this.root) {
				isArchived = this.root.isArchived;
			} else {
				isArchived = this.store.getValue(item, "isArchived"); //$NON-NLS-1$
			}
			
			if (isArchived === true) {
				return true;
			}
			
			return false;
		}	
	});
	
})();