/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.frontingapp.ui.internal.editors.RestoreArchivedDescriptionDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$ 
	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("jazz.ui.ResourceLink");//$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
	dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
	dojo.require("dijit.ProgressBar"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.frontingapp.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ResourceLink = jazz.ui.ResourceLink;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var _WIDTH_IN_EM = 30;
	
	dojo.declare("com.ibm.team.process.authoring.web.frontingapp.ui.internal.editors.RestoreArchivedDescriptionDialog", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.frontingapp", "ui/internal/editors/templates/RestoreArchivedDescriptionDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.frontingapp.ui.internal.editors", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
			this._ownerUrl = args.ownerUrl;
			this._archivedDescriptionUrl = args.archivedDescriptionUrl;
			this._width = _WIDTH_IN_EM + "em"; //$NON-NLS-1$
			this._resourceLinks = [];
		},
		
		postCreate: function() {
			this.connect(this.cancelButton, "onclick", "_onCancel"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.closeButton, "onclick", "onClose"); //$NON-NLS-1$ //$NON-NLS-2$
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.domNode,
				primaryTitle: this.messages.RestoreArchivedDescriptionDialog_0,
				width: this._width,
				destroyContents: true
			});
			this._restore();
		},
		
		/**
		 * Startup this dialog.
		 */
		startup: function(){
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.cancelButton, this.closeButton]);
			this.inherited(arguments);
		},
		
		_reportError: function(errorMessage) {
			dojo.addClass(this.dialogHeading, "ErrorStatus"); //$NON-NLS-1$
			this.dialogHeading.innerHTML = errorMessage;
			this._showProgressBar(false);
		},
		
		/**
		 * Event handler of Restore button in the dialog.
		 * 
		 * @param {Object} event
		 */
		_restore: function() {
			this._showProgressBar(true);
			ProcessAuthoringClient.getProcessDescription({
				resourceURL: this._archivedDescriptionUrl,
				load: dojo.hitch(this, function(data, ioArgs) {
					var descriptionsNode = UIHelper.getNode(data.childNodes, ProcessAuthoringConstants.XMLELEMENT_PROCESSDESCRIPTION_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
					if (descriptionsNode) {
						var archivedNode = UIHelper.getNode(descriptionsNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_ARCHIVED, ProcessAuthoringConstants.NAMESPACE_JTP);
						if (archivedNode && UIHelper.isArchivedXmlElement(descriptionsNode)) {
							UIHelper.setNodeText(archivedNode, "false"); //$NON-NLS-1$
							var _ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
							ProcessAuthoringClient.updateProcessDescriptionWithPutData({
								resourceURL: this._archivedDescriptionUrl,
								putData: dojox.xml.parser.innerXML(data),
								ETag: _ETag,
								load: dojo.hitch(this, function(data, ioArgs) {
									this.onRestoreSuccess({
										resourceUrl: this._archivedDescriptionUrl,
										message: this.messages.RestoreArchivedDescriptionDialog_1
										});
								}),
								error: dojo.hitch(this, function(error, ioArgs) {
									var message = error.responseText ? error.responseText : error.message;
									this._reportError(message);
								})
							});
						} else {
							this.onRestoreSuccess({
								resourceUrl: this._archivedDescriptionUrl,
								message: this.messages.RestoreArchivedDescriptionDialog_2
								});
						}
					}
				}),
				error: dojo.hitch(this, function(error) {
					var message = error.responseText ? error.responseText : error.message;
					this._reportError(message);
				})
			});
			
		},
		
		/**
		 * Show or hide the progress bar.
		 * 
		 * @param {Object} show
		 */
		_showProgressBar: function(show) {
			if (show) {
				dojo.removeClass(this.progressBarDiv, "hidden"); //$NON-NLS-1$
				this.progressBar.update({
					indeterminate: true
				});
			} else {
				dojo.addClass(this.progressBarDiv, "hidden"); //$NON-NLS-1$
			}
		},
		
		/**
		 * Success handler for restoring a selected process description.
		 * 
		 * @param {Object} args
		 */
		onRestoreSuccess: function(args) {
			dojo.addClass(this.progressBarDiv, "hidden"); //$NON-NLS-1$
			dojo.addClass(this.cancelButton, "hidden"); //$NON-NLS-1$
			dojo.removeClass(this.closeButton, "hidden"); //$NON-NLS-1$
			this.dialogHeading.innerHTML = args.message;
		},
		
		/**
		 * Event handler of Cancel button in the dialog.
		 * 
		 * @param {Object} event
		 */
		_onCancel: function(event) {
			dojo.stopEvent(event);
			if (window.stop) {
				window.stop();
			}
			this.dialog.close();
		},
		
		/**
		 * Event handler of Close button in the dialog.
		 * 
		 * @param {Object} event
		 */
		onClose: function(event) {
			dojo.stopEvent(event);
			this.dialog.close();
		},
		
		uninitialize: function() {
			this.inherited(arguments);
		}
	});
})();
