/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("jazz.client.xhr");

(function() {

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var inFlight = 0;
var queue = [];

/*
 * Provides access to viewlet definitions, with caching.
 */
com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry = {
	
	_deferredsByUrl: new Object(),
	
	/*
	 * Retrieves the viewlet definition object from the specified origin.
	 */
	/* void */ getDefinition: function(url, srh, passive, tracker) {
		// check cache
		var deferred = this._deferredsByUrl[url];
		if (!deferred || !deferred.success) {
			deferred = _fetchDefinition(url, passive, tracker);
			this._deferredsByUrl[url] = deferred;
		}
		deferred.addCallback(function(definition) {
			deferred.success = true;
			srh.doCallback(definition);
	    	return definition;
		});
		deferred.addErrback(function(/* Error */ error) {
			srh.doErrorCallback(error);
	    	return error;
		});
	}
};

function _demarshallResponse(evaldObj) {

	if (!evaldObj) {
		return null;
	} else if (!evaldObj.hasOwnProperty("soapenv:Body")) {
		return null;
	}
	var soapBody = evaldObj["soapenv:Body"];
	if (!soapBody.response) {
		return null;
	}
	var response = soapBody.response;
	if (!response.returnValue) {
		return null;
	}
	var returnValue = response.returnValue;
	
	if (returnValue.values instanceof Array) {          
		// array of EMF objects 
		var values = returnValue.values;
		if (values.length > 0) {
			return values[0];
		}
		return null;
	} else if (returnValue.value) {    
		// single EMF object
		return returnValue.value;
	}
	
	return null;
}

function decrementInFlight(response) {
	inFlight--;
	if (inFlight > 0) {
		var deferred = new dojo.Deferred();
		queue.push(function(){
			deferred.callback(response);
		});
		return deferred;
	} else {
		for (var k = 0; k < queue.length; k++) 
			queue[k]();
		queue = [];
	}
}

function watchInFlight(deferred, url) {
	if (!com.ibm.team.dashboard.web.util.internal.Util.isLocalDefinition(url))
		return;
	inFlight++;
	deferred.addBoth(decrementInFlight);
}

function _fetchDefinition(url, passive, tracker) {
	
	if (_isExternalWidget(url)) {
		// External OpenSocial Gadgets
		// External IBM iWidgets		
		return _fetchExternalWidgetDefinition(url, passive, tracker);
	}
	// Viewlet
	return _fetchViewletDefinition(url, passive);

}

function _fetchViewletDefinition(url, passive) {
	
	var args = {
		url: url,
		headers: {
			"Accept": "text/json",
			"X-com-ibm-team-configuration-versions": "LATEST",
			"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
		},
		handleAs: "json"
	};

	// Skip OAuth for passive requests
	if (passive) {
		args.skipInterceptors = ["jazz.oauth"];
	}		
	
	var deferred = jazz.client.xhrGet(args);
	watchInFlight(deferred, url);
	deferred.addCallback(function(/* any */ response) {
		return _demarshallResponse(response);
	});
	
	return deferred;	

}

function _fetchExternalWidgetDefinition(url, passive, tracker) {

	var deferred = new dojo.Deferred();
	var serviceObj = {
		_deferred: deferred,
		success: function(/* object[] */ definitions) {
			var definition = null;
			if ((definitions instanceof Array) &&
					(definitions.length > 0)) {
				definition = definitions[0];
			}
			this._deferred.callback(definition);
		},
		failure: function(/* Error */ error) {
			if (!(error instanceof Error)) {
				var orig = error;
				error = new Error();
				dojo.mixin(error, orig);
			}
			this._deferred.errback(error);
		}
	};
	var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
	// TODO: Write a servlet for serving external widget definitions 
	// External OpenSocial Gadgets
	// External IBM iWidgets
	DashboardClient.getViewletDefinitions([url], null, passive, srh, tracker);
	return deferred;

}

function _isExternalWidget(/* string */ url) {
	if (url.indexOf("g:") === 0) {
		// External OpenSocial Gadgets
		return true;
	} else if (url.indexOf("i:") === 0) {
		// External IBM iWidgets			
		return true;
	}
	return false;
}

})();
