/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.JPIUtil");

/*
 * Provides JPI/URL conversion utilities.
 */
com.ibm.team.dashboard.viewlets.web.ui.internal.utils.JPIUtil = {
	
	/*
	 * Accepts an array of HTML snippets and converts all the applicable URIs in all
	 * of them (JPIs to URLs or vice versa). The func param is a function that converts
	 * an array of URIs. You can use ViewletSite#urlsToJPIs and ViewletSite.jpisToURLs.
	 * 
	 * Note: If there is a risk of malicious script in the HTML, you should sanitize
	 * it before calling this function.
	 */
	/* dojo.Deferred */ convertURIs: function(/* string[] */ uris, /* string[] */ htmlSnippets, /* function */ func) {
		if (func) {
			// build a combined DOM
			var tempDiv = dojo.create("div");
			var len = htmlSnippets.length;
			for (var i=0;i<len;++i) {
				dojo.create("div", {innerHTML: htmlSnippets[i]}, tempDiv);
			}
			// gather all the URIs
			var allURIs = [], nodes = [];
			_findURIs(tempDiv, allURIs, nodes);
			allURIs.push.apply(allURIs, uris);
			if (allURIs.length > 0) {
				// convert the uris
				var dfd = func(allURIs);
				dfd.addCallback(function(/* string[] */ newURIs) {
					// swap in the new node values
					var nodesLen = nodes.length;
					for (var i=0;i<nodesLen;++i) {
						nodes[i].nodeValue = newURIs[i];
					}
					// serialize and replace the html snippets
					var node = tempDiv.firstChild;
					var len = htmlSnippets.length;
					for (var i=0;i<len;++i) {
						htmlSnippets[i] = node.innerHTML;
						node = node.nextSibling;
					}
					// update the uris
					var len = uris.length;
					for (var i=0;i<len;++i) {
						uris[i] = newURIs[nodesLen+i];
					}
					return htmlSnippets;
				});
				return dfd;
			}
		}
		// nothing to do; done
		var dfd = new dojo.Deferred();
		dfd.callback(htmlSnippets);
		return dfd;
	}
};

// elem attrs containing URIs
var URI_ATTRS_BY_ELEM = {
	"A": {"HREF": true},
	"IMG": {"SRC": true}
};

// elems with text containing URIs
var URI_TEXT_ELEMS = {
	"A": true
};

function _findURIs(elem, uris, nodes) {
	// is it a text URI-bearing element?
	var nodeName = elem.nodeName;
	var node = elem.firstChild;
	var isTextURI = URI_TEXT_ELEMS[nodeName];
	if (isTextURI && node && node.nodeType === 3) { /* Text */
		uris.push(node.nodeValue);
		nodes.push(node);
	}
	// check attributes
	var attrSet = URI_ATTRS_BY_ELEM[nodeName];
	if (attrSet) {
		var attrs = elem.attributes;
		var len = attrs.length;
		for (var i=0;i<len;++i) {
			var attr = attrs[i];
			if (attrSet[attr.name.toUpperCase()]) {
				uris.push(attr.nodeValue);
				nodes.push(attr);
			}
		}
	}
	if (!isTextURI) {
		// recursively search child elements
		while (node) {
			node.nodeType === 1 && _findURIs(node, uris, nodes);
			node = node.nextSibling;
		}
	}
	return nodes;
}

})();

