/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.viewlets.TimelineViewlet"); //$NON-NLS-1$

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.contextRoot"); //$NON-NLS-1$
dojo.require("jazz.app.context"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.util.binding"); //$NON-NLS-1$
dojo.require("jazz.util.i18n"); //$NON-NLS-1$
dojo.require("jazz.app.i18n"); //$NON-NLS-1$

dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("dojo.date.stamp"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.string"); //$NON-NLS-1$

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.TeamServerClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.viewlets", "TimelineMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

var TeamsClient = com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var messages;

var DEFAULT_VISIBLE = 3;

dojo.declare("com.ibm.team.process.web.ui.internal.viewlets.TimelineViewlet", [com.ibm.team.dashboard.web.ui.Viewlet], { //$NON-NLS-1$

	templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/viewlets/templates/TimelineViewlet.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	projectArea: null,
	teamArea: null,
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.viewlets", "TimelineMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	init: function() {
		this._updateTitle();
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var updateTitle = newSettings.userTitle != oldSettings.userTitle;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			this.getSite().refresh();
		} else if (newSettings.preferences.items != oldSettings.preferences.items) {
			this.update();
		} else if (newSettings.preferences.Presentation != oldSettings.preferences.Presentation) {
			this.update();
		}
		if (updateTitle) {
			this._updateTitle();
		}
	},

	_updateTitle: function() {
		if (this.teamArea != null && this.teamArea.name != null) {
			this.getSite().setTitle(dojo.string.substitute(messages.titleScoped, [this.teamArea.name]));
		} else if (this.projectArea != null && this.projectArea.name != null) {
			this.getSite().setTitle(dojo.string.substitute(messages.titleScoped, [this.projectArea.name]));
		} else if (this.getScopeItem() != null && this.getScopeItem().name != null) {
			this.getSite().setTitle(dojo.string.substitute(messages.titleScoped, [this.getScopeItem().name]));
		} else {
			this.getSite().setTitle(messages.title);
		}
	},
	
	getTitleHyperlinkURL: function() {
		if (this.projectArea == null || this.projectArea.itemId == null) {
			return null;
		}
		
		return this._getTimelineURL(this.projectArea);
	},
	
	refresh: function() {
		this._clearContent();
		this._setProjectArea();
		this._updateTitle();
	},
	
	_setProjectArea: function() {
		var serviceObj = {
			self: this,
			success: function(result) {
				this.self.projectArea.itemId = result.projectItemId;
				this.self.projectArea.name = result.projectName;
				this.self._fetchTimelines(this.self.projectArea);
			},
			failure: function(errorObj) {
				// read permission
				if (errorObj && errorObj.status == 403) {
					var span = document.createElement("span"); //$NON-NLS-1$
					span.className = "errorText"; //$NON-NLS-1$
					var message = this.self._getOperationalErrorText(messages.permissionDenied,
							messages.permissionDenied_explanation,
							messages.permissionDenied_useraction);
					span.appendChild(document.createTextNode(message));
					this.self._clearContent();
					this.self._content.appendChild(span);
					this.self.getSite().setLoading(false);
					this.self.getSite().doneRefresh();
				}
				else {
					this.self.getSite().setLoading(false);
					this.self.getSite().doneRefresh();
					this.self.getSite().error(errorObj);
				}
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
		if (this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			this.projectArea = this.getScopeItem();
			this.teamArea = null;
			this._fetchTimelines(this.projectArea);
		} else if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA) {
			this.projectArea = {};
			this.teamArea = this.getScopeItem();
			TeamsClient.getTeamsForTeamArea(this.getScopeItem().itemId, this.getScopeSubteams(), srh, this.getServiceTracker());
		} else {
			this.getSite().setLoading(false);
			this.getSite().doneRefresh();
			this._createNotScopeMessage();
		}
	},
	
	_getTimelineURL: function(projectArea) {
		if (projectArea == null || projectArea.itemId == null) {
			return null;
		} else {
			// Ex: https://localhost:9443/jazz/admin#action=com.ibm.team.process.editProjectArea&itemId=_WOqPMDujEeCIDJ7vsnU5zQ&tabId=com.ibm.team.process.web.Timelines
			return this.getWebURL() 
				+ "/admin" //$NON-NLS-1$
				+ (djConfig.isDebug ? "?debug=true" : "") //$NON-NLS-1$ //$NON-NLS-2$
				+ "#action=com.ibm.team.process.editProjectArea&itemId=" //$NON-NLS-1$
				+ projectArea.itemId
				+ "&tabId=" //$NON-NLS-1$
				+ com.ibm.team.process.web.common.internal.ProcessConstants.TabEnum.TIMELINE;
		}
	},
	
	_getResourceURL: function(projectArea) {
		if (projectArea == null || projectArea.itemId == null) {
			return null;
		} else {
			// Ex: https://localhost:9443/jazz/process/project-areas/_WOqPMDujEeCIDJ7vsnU5zQ/timelines
			return this.getWebURL() 
				+ "/process/project-areas/" //$NON-NLS-1$
				+ projectArea.itemId
				+ "/timelines" //$NON-NLS-1$
				+ (djConfig.isDebug ? "?debug=true" : ""); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},
	
	_fetchTimelines: function(projectArea) {
		var url = this._getResourceURL(projectArea);
		if (url == null) {
			this.getSite().setLoading(false);
			this.getSite().doneRefresh();
			this._createNotScopeMessage();
			return;
		} else {
			var dfd = jazz.client.xhrGet({
				url: url,
				headers: {
					accept: "application/rdf+xml" //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				handle: dojo.hitch(this, function(response, ioArgs) {
					if (response instanceof Error) {
						this.getSite().setLoading(false);
						this.getSite().doneRefresh();
						this.getSite().error(response);
					} else {
						this._timelines = jazz.util.binding.bind(response,root).getTimelines();
						this._mainTimeline = this._getMainTimeline(this._timelines);
						if (this._mainTimeline == null) {
							this._handleUAMessage(messages.noTimelines);
						} else {
							this._fetchIterations(this._mainTimeline);
						}
					}
				})
			});
		}
	},
	
	_getMainTimeline: function(timelines) {
		var mainTimeline = this._getProjectTimeline(this._timelines);
		if (mainTimeline == null) {
			if (timelines == null || timelines.length == 0) {
				return null;
			} else {
				//Assume first timeline is main timeline
				mainTimeline = timelines[0];
			}
		}
		var url = mainTimeline.getUrl();
		var timeline = {
			itemId: url.substring(url.lastIndexOf("/") + 1), //$NON-NLS-1$
			label: mainTimeline.getLabel()	
		};
		return timeline;
	},
	
	_getProjectTimeline: function(timelines) {
		if (timelines == null || timelines.length == 0) {
			return null;
		} else {
			var projectTimeline = null;
			for (var i = 0; i < timelines.length; i++) {
				if (timelines[i].isProjectTimeline()) {
					projectTimeline = timelines[i];
					break;
				}
			}
			if (!projectTimeline) {
				return null;
			} else {
				return projectTimeline;
			}
		}
	},
	
	_fetchIterations: function(timeline) {
		if (timeline == null || timeline.itemId == null) {
			this.getSite().setLoading(false);
			this.getSite().doneRefresh();
			this._handleUAMessage(messages.noTimelines);
		} else {
			// TODO: We should be using the process rest APIs for this, but they don't expose "current" iterations, see 193554
			var sr = new ServiceRequest("com.ibm.team.process.internal.service.web.IProcessWebUIService", "getIterations", {uuid: timeline.itemId}); //$NON-NLS-1$ //$NON-NLS-2$
			var serviceObj = {
				success: dojo.hitch(this, function(iterations) {
					this._iterations = iterations;
					this._mainIterations = this._getMainIterations(this._iterations);
					this._currentIteration = this._findCurrentIteration(this._mainIterations);
					if (this._currentIteration == null) {
						this._handleUAMessage(messages.noCurrentIteration);
					} else {
						this._populate(this._mainTimeline, this._mainIterations, this._currentIteration, false);
					}
				}),
				failure: dojo.hitch(this, function(error) {
					this.getSite().setLoading(false);
					this.getSite().doneRefresh();
					this.getSite().error(errorObj);
				})
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
			TeamServerClient.invokeService(sr, srh);
		}
	},
	
	_findCurrentIteration: function(iterations) {
		if (!iterations) {
			return null
		};
		for (var i = 0; i < iterations.length; i++) {
			if (iterations[i].current) {
				return i;
			}
		}
		return null;
	},
	
	_getMainIterations: function(iterations) {
		var current = this._findCurrentIteration(iterations);
		if (current == null) {
			return null;
		} else {
			var mainIterations = iterations;
			while (!mainIterations[current].leaf && mainIterations[current].iterations) {
				mainIterations = mainIterations[current].iterations;
				var next = this._findCurrentIteration(mainIterations);
				if (next == null) {
					break;
				}
				current = next;
			}
			return mainIterations;
		}
	},
	
	_setSecondaryTitle: function(timeline){
		if (timeline != null && timeline.label != null) {
			this.getSite().setSecondaryTitle(timeline.label);
			this._subHeading.innerHTML = timeline.label;
		} else {
			this.getSite().setSecondaryTitle(""); //$NON-NLS-1$
			this._subHeading.innerHTML = ""; //$NON-NLS-1$
		}
	},
	
	_clearContent: function() {
		this._setSecondaryTitle(null);
		this._content.innerHTML = ""; //$NON-NLS-1$
		this._setShowAll(null);
	},
	
	_setShowAll: function(showAll) {
		if (showAll === null || showAll === undefined) {
			this._bottomArea.style.display = "none"; //$NON-NLS-1$
		} else {
			this._bottomArea.style.display = "inline"; //$NON-NLS-1$
			dojo.toggleClass(this._showAllLink, "show-less", !showAll); //$NON-NLS-1$
			this._showAllLabel.innerHTML = showAll ? messages.showMore : messages.showLess;
		}
	},
	
	_createNotScopeMessage: function() {
		this._clearContent();
		dojo.addClass(this._content, "ua-message"); //$NON-NLS-1$
		if (this.getSite().isDashboardEditing()) {
			var nodes = jazz.util.i18n.addMessage(this._content, messages.notScoped + messages.notScopedEdit);
			this.connect(nodes[0], "onclick", this._handleToggleSettings); //$NON-NLS-1$
		} else {
			this._content.innerHTML = messages.notScoped;
		}
	},
	
	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},
	
	_handleUAMessage: function(message) {
		this._clearContent();
		this._setSecondaryTitle(this._mainTimeline);
		dojo.addClass(this._content, "ua-message"); //$NON-NLS-1$
		var link = jazz.util.i18n.addMessage(this._content, message)[0];
		link.href = this._getTimelineURL(this.projectArea);
	},
	
	_createIteration: function(iteration) {
		if (iteration == null || iteration.label == null) {
			return null;
		} else {		
			var className = "iteration"; //$NON-NLS-1$
			if (iteration.current) { 
				className += " current"; //$NON-NLS-1$
			}
			if (iteration.completed) {
				className += " completed"; //$NON-NLS-1$
			}
				
			var iterationDiv = dojo.create("div", {className: className}); //$NON-NLS-1$
			var iterationIcon = dojo.create("div", {className: "icon"}, iterationDiv); //$NON-NLS-1$ //$NON-NLS-2$
			var iterationLabelWrapper = dojo.create("div", {className: "label-wrapper"}, iterationDiv); //$NON-NLS-1$ //$NON-NLS-2$
			var iterationLabel = dojo.create("span", {className: "label", innerHTML: iteration.label}, iterationLabelWrapper); //$NON-NLS-1$ //$NON-NLS-2$
			var iterationEndDate = dojo.create("div", {className: "end-date", innerHTML: this._getEndDateString(iteration)}, iterationLabelWrapper); //$NON-NLS-1$ //$NON-NLS-2$
			return iterationDiv;
		}
	},
	
	_getEndDateString: function(iteration) {
		if (!iteration || !iteration.endDate) {
			return ""; //$NON-NLS-1$
		}
		var date = jazz.app.i18n.date.locale.format(new jazz.app.i18n.Date(dojo.date.stamp.fromISOString(iteration.endDate)), {formatLength: "medium", selector: "date"}); //$NON-NLS-1$ //$NON-NLS-2$
		return dojo.string.substitute(messages.endDate, [date]);
	},
	
	_populate: function(timeline, iterations, mainIteration, showAll) {
		this._clearContent();
		this._setSecondaryTitle(timeline);
		this._setShowAll(!showAll);
		var l = iterations.length;
		if (showAll) {
			var start = 0;
		} else {
			var c = start = mainIteration;
			if (l <= DEFAULT_VISIBLE) {
				start = 0;
			} else if (c + DEFAULT_VISIBLE > l) {
				start = l - 3;
			}
		}
		for (var i = start; i < l && (showAll || i < start + DEFAULT_VISIBLE); i++) {
			this._content.appendChild(this._createIteration(iterations[i]));
		}
	},
	
	_toggleShowAll: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		var showingAll = dojo.hasClass(this._showAllLink, "show-less"); //$NON-NLS-1$
		this._populate(this._mainTimeline, this._mainIterations, this._currentIteration, !showingAll);
	}
});

var NAMESPACES = {
	jp: "http://jazz.net/xmlns/prod/jazz/process/1.0/" //$NON-NLS-1$
};
var Iteration = {
	namespaces: NAMESPACES
};
var Timeline = {
	namespaces: NAMESPACES,
	getLabel: ["string", "jp:label", "1"], //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	getUrl: ["string", "jp:url", "1"], //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	getIterationsUrl: ["string", "jp:iterations-url", "1"], //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	isProjectTimeline: ["boolean", "jp:project-timeline", "1"] //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
};
var root = {
	namespaces: NAMESPACES,
	getTimelines: [Timeline, "jp:timelines/jp:timeline", "*"] //$NON-NLS-1$ //$NON-NLS-2$
};

})();