/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.wizard.ApplicationStep");

dojo.require("com.ibm.team.dashboard.web.ui.internal.wizard.AbstractDashboardStep");

dojo.require("jazz.app.internal.applicationAbout");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "WizardMessages");

var AbstractDashboardStep = com.ibm.team.dashboard.web.ui.internal.wizard.AbstractDashboardStep;
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var StyledTable = jazz.ui.StyledTable;

var APPLICATION_URI = "http://jazz.net/xmlns/prod/jazz/discovery/1.0/Application";
var PERSONAL_DASHBOARD_TEMPLATE_URI = "http://jazz.net/xmlns/prod/jazz/dashboard/1.0/PersonalDashboardTemplate";
var DEFAULT_VALUE = "default";
var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.wizard.ApplicationStep",
		[AbstractDashboardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/wizard/templates/ApplicationStep.html"),
	widgetsInTemplate: true,
	
	_dataByURL: null,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "WizardMessages");
		this.stepTitle = messages.appTitle;
	},
	
	postCreate: function() {
		var styledTable = new StyledTable({
			columns: [
				{ label: "", width: "16px" },
				{ label: messages.appHeadingLabel, width: "170px" },
				{ label: messages.headingDesc }
			]
		});
		var table = styledTable.domNode;
		var tBody = this._tBody = table.tBodies[0];
		// default entry
		this._addTableRow(messages.appDefaultLabel, null, DEFAULT_VALUE, messages.appDefaultDesc, null, "defaultIcon j-mirror", true);
		// loading row
		var tr = this.messageRow = dojo.create("tr", {"class": "loadingMsg lastRow"}, tBody);
		dojo.create("td", null, tr);
		var td = dojo.create("td", {colspan: "2"}, tr);
		td.appendChild(document.createTextNode(messages.loading));
		this.tableContent.appendChild(table);
	},
	
	init: function() {
		this.inherited(arguments);
		var dfd = DiscoveredService.loadDiscoveryServices([APPLICATION_URI, PERSONAL_DASHBOARD_TEMPLATE_URI], DISCOVERY_RESULT);
		dfd.addBoth(dojo.hitch(this, this._clearMsg));
		dfd.addCallback(dojo.hitch(this, this._handleCallback));
		dfd.addErrback(dojo.hitch(this, this.showError, messages.appErrorDiscovery));
	},
	
	canFinish: function() {
		return !this.data.projectScoped && this.inherited(arguments);
	},
	
	canFlipToNextPage: function() {
		return this.data.projectScoped && this.inherited(arguments);
	},
	
	isDefaultSelected: function() {
		return !this.data.templateUrl;
	},
	
	onEntrySelect: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			var value = e.target.value;
			if (value !== DEFAULT_VALUE) {
				var data = this._dataByURL[value];
				this.setData("applicationName", data.title);
				this.setData("templateUrl", value);
				this.setData("projectScoped", data.projectScoped);
				this.setData("rootServicesUrl", data.rootServicesUrl);
			}
			else {
				this.setData("applicationName", null);
				this.setData("templateUrl", null);
				this.setData("projectScoped", false);
				this.setData("rootServicesUrl", null);
			}
		}
	},
	
	_addEntry: function(/* object */ entry, /* boolean */ lastEntry) {
		this._addTableRow(entry.title, entry.secondaryTitle, entry.url, entry.description, entry.icon, null, false, lastEntry);
	},
	
	_addTableRow: function(/* string */ title, /* string? */ secondaryTitle, /* string */ value, /* string */ description, /* string? */ icon, /* string? */ iconClass, /* boolean */ checked, /* boolean */ lastRow) {
		var tr = dojo.create("tr", lastRow ? {"class": "lastRow"} : null);
		var td = dojo.create("td", {"class": "radioCol"}, tr);
		var input = dojo.create("input", {type: "radio", name: "app", value: value, id: value}, td);
		if (checked) {
			input.checked = true;
		}
		this.connect(input, "onclick", this.onEntrySelect);
		td = dojo.create("td", {"class": "labelCol"}, tr);
		var label = dojo.create("label", {"for": value}, td);
		if (icon) {
			dojo.create("img", {src: icon, "class": "icon"}, label);
		}
		else if (iconClass) {
			dojo.create("span", {"class": "icon " + iconClass}, label);
		}
		var span = dojo.create("span", {"class": "labelText"}, label);
		span.appendChild(document.createTextNode(title));
		if (secondaryTitle) {
			var span2 = dojo.create("span", {"class": "secondaryTitle"}, span);
			span2.appendChild(document.createTextNode(" (" + secondaryTitle + ")"));
		}
		td = dojo.create("td", {"class": "descCol"}, tr);
		td.appendChild(document.createTextNode(description));
		this._tBody.appendChild(tr);
	},
	
	_clearMsg: function() {
		this.messageRow.style.display = "none";
	},
	
	_handleCallback: function(/* object */ result) {
		// create lookup maps for application info
		var aboutsByUrl = jazz.app.internal.applicationAbout.get();
		var applicationsByPublisher = {};
		var applications = result.applications;
		var len = applications.length;
		for (var i=0;i<len;++i) {
			var application = applications[i];
			applicationsByPublisher[application.publisher] = application;
		}
		
		// combine template contributions with their respective application and about info
		var dataByURL = this._dataByURL = {};
		var templateEntries = result.dashboardTemplates;
		var len = templateEntries.length;
		for (var i=0;i<len;++i) {
			var entry = templateEntries[i];
			var publisher = entry.publisher;
			var application = applicationsByPublisher[publisher];
			if (application) {
				var about = aboutsByUrl[application.aboutUrl];
				if (about) {
					var rootServicesUrl = application.rootServicesUrl;
					if (rootServicesUrl) {
						dataByURL[entry.service] = {
							title: about.title,
							secondaryTitle: about.instanceName,
							description: about.description,
							icon: about.icon,
							url: entry.service,
							projectScoped: entry.projectScoped,
							rootServicesUrl: rootServicesUrl
						};
					}
					else {
						// log and skip
						console.warn("Unable to locate rootServices URL from discovery jd:Application entry for " + publisher + " (skipping)");
					}
				}
				else {
					// log and skip
					console.warn("Missing application-about for " + publisher + " (skipping)");
				}
			}
			else {
				// log and skip
				console.warn("Missing jd:Application for " + publisher + " (skipping)");
			}
		}
		
		this._populateTable(dataByURL);
	},
	
	_populateTable: function(/* object */ entriesByURL) {
		// sort alphabetically
		var arr = [];
		for (var i in entriesByURL) {
			arr.push(entriesByURL[i]);
		}
		arr.sort(_applicationComparator);
		var len = arr.length;
		for (var i=0;i<len;++i) {
			this._addEntry(arr[i], (i == len-1));
		}
	}
});

var NAMESPACES = {
	dc: "http://purl.org/dc/terms/",
	jd:	"http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
	jdb: "http://jazz.net/xmlns/prod/jazz/dashboard/1.0/",
	oslc: "http://open-services.net/ns/core#",
	rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
};

var Application = {
	namespaces: NAMESPACES,
	aboutUrl: ["string", "oslc:publisher/@rdf:resource", "1"],
	publisher: ["string", "dc:publisher", "1"],
	rootServicesUrl: ["string", "jd:rootServices/@rdf:resource", "1"]
};

var PersonalDashboardTemplate = {
	namespaces: NAMESPACES,
	service: ["string", "jd:service/@rdf:resource", "1"],
	projectScoped: ["boolean", "jdb:projectScoped", "1"],
	publisher: ["string", "dc:publisher", "1"]
};

var DISCOVERY_RESULT = {
	namespaces: NAMESPACES,
	applications: [Application, "jd:Application", "*"],
	dashboardTemplates: [PersonalDashboardTemplate, "jdb:PersonalDashboardTemplate", "*"]
};

/*
 * Compares two application entry objects for alphabetical sorting.
 */
function _applicationComparator(/* object */ a, /* object */ b) {
	if (a.title === b.title) {
		return a.secondaryTitle > b.secondaryTitle ? 1 : -1;
	}
	return a.title > b.title ? 1 : -1;
}

})();
