/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.wizard.CreateDashboardWizard");

dojo.require("com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.wizard.ApplicationStep");
dojo.require("com.ibm.team.dashboard.web.ui.internal.wizard.ProjectStep");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.contextRoot");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Wizard");
dojo.require("jazz.util.i18n");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "WizardMessages");

var contextRoot = net.jazz.ajax.contextRoot;
var ApplicationStep = com.ibm.team.dashboard.web.ui.internal.wizard.ApplicationStep;
var MessageBox = jazz.ui.MessageBox;
var PersonalDashboardClient = com.ibm.team.dashboard.web.client.internal.PersonalDashboardClient;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ProjectStep = com.ibm.team.dashboard.web.ui.internal.wizard.ProjectStep;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var Wizard = jazz.ui.Wizard;

var DASHBOARD_MSG_GLOBAL = "com.ibm.team.dashboard.web.ui.internal.dashboardMsg";
var STRONG = "strong";
var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.wizard.CreateDashboardWizard",
		[Wizard], {
	
	dashboardUrl: null,
	
	onFinish: function() {},
	
	"-chains-": {
		constructor: "manual"
	},
	constructor: function() {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "WizardMessages");
		}
		this.data = {};
		var appStep = new ApplicationStep({wizard: this});
		var projectStep = new ProjectStep({wizard: this});
		
		arguments[0] = {
			primaryTitle: messages.wizardTitle,
			showStepSummary: true,
			steps: [appStep, projectStep],
			processFinish: dojo.hitch(this, this.processFinish),
			width: "925px"
		};
		arguments.length = 1;
		this.inherited(arguments);
	},
	
	processFinish: function() {
		var currentStep = this.contentWidget.getSelectedStep();
		currentStep.taskStart(messages.wizardMsgCreating);
		
		var templateUrl = this.data.templateUrl;
		var template;
		if (templateUrl) {
			var msgDetail;
			if (this.data.projectScoped) {
				templateUrl += "?projectAreaName=" + encodeURIComponent(this.data.projectAreaName);
				msgDetail = dojo.string.substitute(messages.wizardCongrats2c, [this.data.applicationName, this.data.projectAreaName]);
			}
			else {
				msgDetail = dojo.string.substitute(messages.wizardCongrats2b, [this.data.applicationName]);
			}
			var dfd = jazz.client.xhrGet({
				url: templateUrl,
				handleAs: "text",
				headers: { Accept: "application/rdf+xml" }
			});
			dfd.addCallback(dojo.hitch(this, this._processFinish2, currentStep, msgDetail));
			dfd.addErrback(dojo.hitch(this, function(/* Error */ e) {
				if (e.status === 404) {
					e.log = false;
					if (confirm(messages.wizardConfirmNotFound)) {
						this._processFinish2(currentStep, messages.wizardCongrats2a);
					}
					else {
						// stay in the wizard
						currentStep.taskDone();
					}
				}
				else {
					currentStep.taskFailed(messages.wizardErrorCreating, e);
				}
			}));
		}
		else {
			this._processFinish2(currentStep, messages.wizardCongrats2a);
		}
	},
	
	_processFinish2: function(/* AbstractDashboardStep */ currentStep, /* string */ msgDetail, /* string? */ template) {
		var dfd = PersonalDashboardClient.createDashboard(template, this.dashboardUrl);
		dfd.addCallback(dojo.hitch(this, function(/* string */ url) {
			currentStep.taskDone();
			this.close();
			// don't check for embedded mode because it doesn't register application path with the workbench
			if (Util.getView() !== "embedded" && !PlatformUI.getWorkbench().shouldVetoNavigation(url)) {
				location.href = url;
			}
			var msgParts = jazz.util.i18n.split(msgDetail);
			var span = dojo.create("span");
			span.appendChild(document.createTextNode(messages.wizardCongrats1 + " "));
			var len = msgParts.length;
			for (var i=0;i<len;++i) {
				var part = msgParts[i];
				var textNode = document.createTextNode(part.text);
				if (part.type === STRONG) {
					var b = dojo.create("b", null, span);
					b.appendChild(textNode);
				}
				else {
					span.appendChild(textNode);
				}
			}
			span.appendChild(document.createTextNode(" " + messages.wizardCongrats3));
			dojo.setObject(DASHBOARD_MSG_GLOBAL, {
				type: MessageBox.STATUS_TYPES.OK,
				summary: span,
				closable: true
			});
			this.onFinish();
		}));
		dfd.addErrback(dojo.hitch(this, function(/* Error */ e) {
			var msg;
			switch (e.status) {
				case 403:
					msg = messages.wizardErrorPermission;
					break;
				case 503:
					msg = messages.wizardErrorReadOnly;
					break;
				default:
					msg = messages.wizardErrorCreating;
					break;
			}
			currentStep.taskFailed(msg, e);
		}));
	}
});

})();
