/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ExpressWizardApplication");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.Wizard");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.CreateAdminUser");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ExpressSetupProgress");
dojo.require("jazz.util.browserSupport");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.UnsupportedBrowserWarning");
dojo.require("net.jazz.ajax.contextRoot");

dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Wizard = jazz.ui.Wizard;
var WizardStep = jazz.ui.wizard.WizardStep;
var NotAuthorizedView = com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView;
var CreateAdminUser = com.ibm.team.repository.web.ui.internal.wizard.CreateAdminUser;
var ExpressSetupProgress = com.ibm.team.repository.web.ui.internal.wizard.ExpressSetupProgress;
var ConfigurePublicURI = com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI;

var LOCAL_STORAGE_KEY = "com-ibm-team-repository-web-SetupWizardChoice";
var CREATE_USER_LOCAL_STORAGE_KEY = "com-ibm-team-repository-web-CreateAdminUser";

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ExpressWizardApplication", dijit._Widget, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	},
	
	postCreate: function() {
		this.getUserRoles();
		OAuthAdminClient.postWarmUpLocallyInstalledApps(new ServiceResponseHandler(this, "doNothing", "doNothing"));
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			this.loggedInUser = response.userId;
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this._startApp();
	},
	
	authRequestFailed: function() {
		this._startApp();
	},
	
	_startApp: function () {
		//Need to call a protected service to see if we get a 403 back (which means we are a deactivated ADMIN)
		if(this.authorized) {
			AdminClient.getAllRoles(new ServiceResponseHandler(this, "_checkBrowser", "_handleAuthCheckFailure")); //random service call
		} else {
			this._checkBrowser();
		}
	},
	
	_handleAuthCheckFailure: function(errObject) {
		// Forbidden users see unauthorized message
		if (errObject.status === 403) {
			// this is probably an archived admin user
			this.authorized = false;
		}
		this._checkBrowser();
	},
	
	_checkBrowser: function() {
		//Check to see if we are using a supported Browser
		var support = jazz.util.browserSupport.getSupportLevel();
		if (support && (support == jazz.util.browserSupport.NOT_SUPPORTED)) {
			var warn = new jazz.ui.UnsupportedBrowserWarning();
			this.connect(warn, "onContinue", function() {
				warn.destroy();
				this._createUI();
			});
			warn.placeAt(document.body);
		} else {
			this._createUI()
		}
	},
	
	createLogoutArea: function() {
		var logoutNode = dojo.create("div", { className: "logout-node" });
		this.loggedInUserNode = dojo.create("span", { innerHTML: this.loggedInUser, className: "userName" }, logoutNode);
		dojo.create("span", { innerHTML: "|", style: { color: "#999", padding: "0 4px" } }, logoutNode);
		this.logoutLink = dojo.create("a", { innerHTML: this.messages.logout, href: "#", className: "logoutLink" }, logoutNode);
		this.connect(this.logoutLink, "onclick", function(e) {
			dojo.stopEvent(e);
			var dfd = jazz.app.auth.logout();
			dfd.addCallback(function() {
				window.location.reload();
			});
		});
		return logoutNode;
	},
	
	_createUI: function() {
		document.body.style.background = "none";
		document.title = this.messages.setupName;
		this.content = dojo.create("div", { className: "com-ibm-team-repository-wizard express-setup-wizard" });
		if (this.loggedInUser) {
			this.content.appendChild(this.createLogoutArea());
		}
		dojo.addClass(this.domNode, "com-ibm-team-repository-web-setup-RootNode");
		var setupTitle = this.messages.serverSetupExpress;
		dojo.create("div", { innerHTML: setupTitle.toUpperCase(), className: "setup-header"}, this.content);
		var jazzIcon = dojo.create("img", { className: "jazz-icon" }, this.content);
		jazzIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/jazz-identR.png");
		this.wizardBox = document.createElement("div");
		this.content.appendChild(this.wizardBox);		
		var wizardParams = {
			fullPageContainer: this.wizardBox,
			showStepSummary: false,
			processFinish:  function(){
				try {
					localStorage["com-ibm-team-repository-web-SetupHasFinished"] = dojo.toJson({ "fromSetup": true });
				} catch (e) {}
				document.location.href = net.jazz.ajax.contextRoot() + "/admin";
            },
			customUpdateControls: dojo.hitch(this, "customUpdateControls")
		};
		if (this.authorized) { 
			wizardParams.steps = this.initServerSetupSteps();
		} else {
			var notAuthStep = new NotAuthorizedView({
					applicationName: this.messages.setupName,
					customNoUserMessage: this.messages.userNotRecognizable,
					includeBackLink: false,
					showLogout: true });
			dojo.mixin(notAuthStep, new WizardStep());
			notAuthStep.includeNavigationUI = false;
			wizardParams.stepListLayout = "horizontal";
			wizardParams.steps = [notAuthStep];
			wizardParams.showStepSummary = false;
		}
		this.wizard = new Wizard(wizardParams);
		var outerFooter = new com.ibm.team.repository.web.ui.internal.wizard.LogoFooter();
		this.content.appendChild(outerFooter.domNode);
		this.domNode.appendChild(this.content);
		this.wizard.startup();
	},
	
	initServerSetupSteps: function() {
		var steps = [];
		var data = dojo.fromJson(localStorage[LOCAL_STORAGE_KEY]);
		
		//Save the referrer url from localStorage while we have it...
		this.backButtonURL = (data && data.mainSetupURL) ? data.mainSetupURL : null;
		
		steps.push(new ConfigurePublicURI({ "backURL": this.backButtonURL, "isExpress": true }));
		
		//If the user is not authenticated as ADMIN, we don't need to show the create user step in the Express wizard		
		var userID = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		if(userID && userID === "ADMIN") {
			steps.push(new CreateAdminUser({ myParent: this }));
		} else {
			//We aren't creating a new user, so update the localStorage data accordingly
			localStorage[CREATE_USER_LOCAL_STORAGE_KEY] = dojo.toJson({
				"create.user.page:created.user": false, 
				"create.user.page:skip.create.user": true
			});
		}
		steps.push(new ExpressSetupProgress());
		return steps;
	},
	
	customUpdateControls: function() {
		if(this.wizard) {
			var step = this.wizard.contentWidget.getSelectedStep();
			if(step && this.backButtonURL && step.position == 1) {
				dojo.removeClass(this.wizard.contentWidget.wizardPreviousButton, "noDisplay");
				this.wizard.contentWidget.wizardPreviousButton.disabled = false;
				//Make it hard for the user to invoke this since it will redirect and dirty form will be lost.
				this.wizard.contentWidget.wizardPreviousButton.tabIndex = 10;  
			}	
		}
	},
	
	doNothing: function () {
		// do nothing
	}
	
});
var messages;
dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.LogoFooter", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/LogoFooter.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	}
});

})();