/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.LdapRealm");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.LdapUserValidationDialog");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.LDAPConfigClient");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledBox");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "LdapRealmMessages");


(function() {

var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.wizard.";
var LDAP_SERVICE_NAME = "com.ibm.team.repository.service.jts.internal.userregistry.ldap.LDAPUserRegistryProvider";
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var LDAPConfigClient = com.ibm.team.repository.web.client.internal.LDAPConfigClient;
var messages;

var DISPLAY_PROPS = [];
DISPLAY_PROPS.LDAP1 = [
	"ldap.registryLocation",
	"ldap.registryUserName", 
	"ldap.registryPassword"
];
DISPLAY_PROPS.LDAP2 = [
	"ldap.baseUserDN",
	"ldap.userAttributesMapping"
];
DISPLAY_PROPS.LDAP3 = [
	"ldap.baseGroupDN", 
	"ldap.groupMapping", 
	"ldap.groupNameAttribute",
	"ldap.membersOfGroup"
];

/*          	TEST/VALIDATE/REVERT/NEXT/EDIT
LOADING        		N   N   N   N   N
TESTING        		N   N   N   N   N
PENDING_OK 	      	Y   N   ?   N   Y 
PENDING_ERR 	    N   N   Y   N   Y
PENDING_WARN  	    Y   N   Y   N   Y
CONNECTED_OK        Y   Y   Y   Y   Y
CONNECTED_ERR       Y   N   Y   N   Y
CONNECTED_WARN      Y   Y   Y   Y   Y
USER_OK        		Y   Y   Y   Y   Y
USER_ERR       		Y   N   Y   N   Y
TOMCAT_CONF_CREATED Y   Y   Y   Y   Y  
*/
var STATUS = {
	LOADING: {test: false, validateUser: false, revert: false, next: false, edit: false, code: "INFO",
		message: 'loading'},  
	TESTING: {test: false, validateUser: false, revert: false, next: false, edit: false, code: "INFO", 
		message: 'testing'},  
	PENDING_OK: {test: true, validateUser: false, revert: true, next: false, edit: true, code: "INFO", 
		message: 'pendingOk'},  
	PENDING_ERR: {test: false, validateUser: false, revert: true, next: false, edit: true, code: "ERROR",
		message: 'pendingErr'}, 
	PENDING_WARN: {test: true, validateUser: false, revert: true, next: false, edit: true, code: "WARNING",
		message: 'pendingWarn'}, 
	CONNECTED_OK: {test: true, validateUser: true, revert: true, next: true, edit: true, code: "OK",  
		message: 'connectedOk'},
	CONNECTED_ERR: {test: true, validateUser: false, revert: true, next: false, edit: true, code: "ERROR",
		message: 'connectedErr'}, 
	CONNECTED_WARN: {test: true, validateUser: true, revert: true, next: true, edit: true, code: "WARNING",
		message: 'connectedWarn'}, 
	USER_OK: {test: true, validateUser: true, revert: true, next: true, edit: true, code: "OK",
		message: 'userOk'}, 
	USER_WARN: {test: true, validateUser: true, revert: true, next: true, edit: true, code: "WARNING",
		message: 'userWarn'}, 
	USER_ERR: {test: true, validateUser: true, revert: true, next: false, edit: true, code: "ERROR",
		message: 'userErr'}, 
	TOMCAT_CONF_CREATED: {test: true, validateUser: false, revert: true, next: true, edit: true, code: "INFO",  
		message: 'tomcatConfCreated'},
	TOMCAT_CONF_CREATED_WITH_WARNING: {test: true, validateUser: false, revert: true, next: true, edit: true, code: "WARNING",  
		message: 'tomcatConfCreated'},
	SAVE_ERR: {test: true, validateUser: false, revert: true, next: false, edit: true, code: "ERROR",
		message: 'saveErr'}  
};

dojo.declare(PACKAGE_PREFIX + "LdapRealm",  [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SetupLdapDetail.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	computeLdapQueriesHints: {},

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "LdapRealmMessages");
		var helpLink = "<a href='http://www.ibm.com/support/docview.wss?uid=swg21445366' target=\"_blank\">";
		this.helpLink = dojo.string.substitute(messages.helpLink, [helpLink, "</a>"]); 
	},

	postCreate: function() {
		this._saveTomcatConfigButton.style.display = "none";
		this.setStatusCallback({type: "INFO", message: messages.loading});
		this._initializeContent();
		
		//Create a MessageBox for statusIndicationArea for secondary messages
		this.statusMsgHandler = new MessageBox();
		this.statusIndicationArea.appendChild(this.statusMsgHandler.domNode);
		
		//Add header to the StyledBox in the template
		this.step2_header.innerHTML = "<span class=\"step\">" + this.messages.step2Label + "</span><span>" + this.messages.step2Title + "</span>";

		LDAPConfigClient.getContainerType(new SRH(this, "_getContainerTypeCallback", "_getContainerTypeError"));
	},

	_getContainerTypeCallback: function(msg) {
		if(msg && msg.toLowerCase().indexOf("tomcat")>=0){
			this._saveTomcatConfigButton.style.display = "";
		}
	},
	
	_getContainerTypeError: function(msg) {
	},
	
	// INITAL LOAD METHODS
	_initializeContent: function () {
		var serviceConfigProps = {
			serviceInfo: {
				implementationClassName: LDAP_SERVICE_NAME
			},
			enableEdits: true,
			displayIn: this._hiddenTableBody,
			descriptionMode: "table", 
			initialMode: "edit"
		}
		this.ldapConfig = new ServerPropertyService(serviceConfigProps);
		this.connect(this.ldapConfig, "_markChanged", function() {
			this._setState(STATUS.PENDING_OK);
		});
		this.connect(this.ldapConfig, "_checkUnchanged", function () {
				if(this.ldapConfig._changed) {
					this._setState(STATUS.PENDING_OK);
				} else {
					this._setState(this._initialLoadedState);
				}
			});
		this.connect(this.ldapConfig, "_markInvalid", "_handleInvalidInput");
		this.connect(this.ldapConfig, "_show", "_initializePropertiesAfterLoaded");
	},
	
	_initializePropertiesAfterLoaded: function () {
		this._initializePropertyDisplay(this._ldapEditPropertiesBody, DISPLAY_PROPS.LDAP1);
		this._initializePropertyDisplay(this._ldapUserEditPropertiesBody, DISPLAY_PROPS.LDAP2);
		this._initializePropertyDisplay(this._ldapGroupEditPropertiesBody, DISPLAY_PROPS.LDAP3);

		this._testConnection();
	},	
	
	_initializePropertyDisplay: function (attachPoint, displayProps) {	
		var targetPropertyServiceRow = new ServerPropertyServiceRow({
			_enableToggle: false,
			rowClass: ["indentNone editProperties"],
			_descriptionMode: "table"
		});
		attachPoint.appendChild(targetPropertyServiceRow.domNode);
				
		var longExists = false;
		var shortExists = false;
		for (var i=0; i < displayProps.length; i++) {
			var propertyElement = this.ldapConfig.getServerPropertyElement(displayProps[i]);
			if (propertyElement.elementInfo.propertyType === "STRING") {
				longExists = true;
				var table = targetPropertyServiceRow._longPropertiesTableBody;
			} else {
				shortExists = true;
				var table = targetPropertyServiceRow._shortPropertiesTableBody;
			}
			table.appendChild(propertyElement.domNode);
		};
		if (!longExists) {
			targetPropertyServiceRow._longPropertiesTableHeader.style.display = "none";
		}
		if (!shortExists) {
			targetPropertyServiceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
		
	// HANDLE USER INTERACTION METHODS
	 onInnerButtonAreaClick: function(e) {
		var button = e.target;
		if (button === this._revertButton) {
			this._revert();
		} else if (button === this._testConnectionButton){
			this._testConnection();
		} else if (button === this._saveTomcatConfigButton){
			this._saveTomcatConfig();
		}
	}, 
		
	_handleInvalidInput: function (msg){
		if (msg.code === "ERROR") {
			this._setState(STATUS.PENDING_ERR);
		} else {
			this._setState(STATUS.PENDING_WARN);
		}
	}, 
	
	// TEST CONNECTION METHODS
	_testConnection: function() {
		this._setState(STATUS.TESTING);
		this.ldapConfig.testRobustValidity(new SRH(this, "_testConnectionCallback", "_testConnectionError"));
	},
	
	_testConnectionCallback: function(msg) {
		var connected = null;
		if (msg != null) {
			connected = STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = STATUS.CONNECTED_ERR;
				}
			}
		} else {
			connected = STATUS.CONNECTED_OK;
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		this._setState(connected, msg);
		
		if (connected == STATUS.CONNECTED_OK) {
			this._validateUser();
		}
	},
	
	_testConnectionError: function(errorObj) {
		// adding fromErrorCallback to errorObj so non-error callback test connection errors display with details
		errorObj.fromErrorCallback = true;
		this._setState(STATUS.CONNECTED_ERR, errorObj, messages.connectionError);
	},
	
	_revert: function() {
		this.ldapConfig.restoreSaved();
		this._setState(this._initialLoadedState);
	},

	// VALIDATE LDAP USER METHODS
	_validateUser: function() {
		var currentUser = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		this._doValidateUser(currentUser, "_validateUserCallback", "_validateUserError")
	},
	
	_doValidateUser: function(userid, okCallback, errCallback){
		var properties = this._getProperties();
		if(!properties){
			return false;
		}
		this._setState(STATUS.TESTING);
		properties.validateUser = {userid: userid};
		var handler = new SRH(this, okCallback, errCallback);
		LDAPConfigClient.getUserInfo(handler, {basicProperties: dojo.toJson(properties)});			
	},
	
	_validateUserCallback: function(msg) {
		var needDialog = msg && dojo.some(msg, function(m){
			return m.code === "ERROR";
		});
		if(needDialog){
			var dialog = new com.ibm.team.repository.web.ui.internal.wizard.LdapUserValidationDialog(
					{userid: this._validatedUser || dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID")});
			var self = this;
			this.connect(dialog, "onOk", function(userInfo){
				self._validatedUser = userInfo.userid;
				self._doValidateUser(userInfo.userid, "_validateUserCallback2", "_validateUserError")
			});
			this.connect(dialog, "onCancel", function(userInfo){
				this._setState(STATUS.CONNECTED_OK, msg);				
			});
		}else{
			this._getComputeLdapQueriesHint(msg);
			this._setState(STATUS.CONNECTED_OK);
		}
	},
	
	_validateUserCallback2: function(msg) {
		var connected = null;
		var ok = !msg || dojo.every(msg, function(m){
			return m.code === "OK";
		});
		if (!ok) {
			connected = STATUS.USER_ERR;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = STATUS.CONNECTED_ERR;
				}
			}
		} else {
			this._getComputeLdapQueriesHint(msg);
			connected = STATUS.CONNECTED_OK;
		}
		this._setState(connected, msg);
	},
	
	_getComputeLdapQueriesHint: function(msg){
		this.computeLdapQueriesHints = {};
		dojo.forEach(msg, function(m){
			if(m.code === "OK"){
				dojo.mixin(this.computeLdapQueriesHints, dojo.fromJson(m.message)); 
			}
		}, this);		
	},
	
	_validateUserError: function(msg) {
		this._setState(STATUS.USER_ERR, msg);
	},
	
	_getProperties: function() {
		var pendingUpdates = this.ldapConfig._getAllValuesExceptSensitiveReplacements();
		if (pendingUpdates.length === 0) {
			return null;
		}
		var properties = {};
		for (var i=0; i<pendingUpdates.length; i++) {
			var name = pendingUpdates[i].name;
			var value = pendingUpdates[i].value;
			properties[name] = value;
		};
		return properties		
	},
	
	// SAVE TOMCAT CONFIG METHODS
	_saveTomcatConfig: function() {
		var properties = this._getProperties();
		if(!properties){
			return false;
		}
		dojo.mixin(properties, this.computeLdapQueriesHints);
		var handler = new SRH(this, "_saveTomcatConfigCallback", "_saveTomcatConfigError");
		LDAPConfigClient.postProperties(handler, {basicProperties: dojo.toJson(properties)});
	},
	
	_saveTomcatConfigCallback: function(msg) {
		var status = STATUS.TOMCAT_CONF_CREATED;
		if (msg != null) {
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "WARNING") {
					status = STATUS.TOMCAT_CONF_CREATED_WITH_WARNING;
				}
			}
		}
		this._setState(status, msg);
	},
	
	_saveTomcatConfigError: function(msg){
		this._setState(STATUS.CONNECTED_WARN, msg);
	},

	
	// HANDLE STATE METHODS
	_setState: function(state, errorObj, optionalMsg) {
		// check required for initial load before service properties are loaded
		if (this.ldapConfig != null) {
			this._allowPropertyEdit(state.edit); 
		}	
		var enableRevert = state.revert;
		if (enableRevert && !this.ldapConfig._changed) {
			enableRevert = false;
		}
		this._revertButton.disabled = (!enableRevert);
		this._testConnectionButton.disabled = (!state.test);
		this._saveTomcatConfigButton.disabled = (!state.next);

		var statusMessage = optionalMsg ? optionalMsg  : messages[state.message];
		// errorObj is nested in the status to fit how SetupUsers expects errorCallback path
		var statusCallback = {type: state.code, message: statusMessage, overrideNext: state.next, errorObj: errorObj};
		this._lastStatusCallback = statusCallback;
		this.setStatusCallback(statusCallback,null,true);
		this._setStatusIndication(state.code, messages[state.message], errorObj)
	},
	
	_setStatusIndication: function (code, message, details) {
		dojo.toggleClass(this.statusIndicationArea, "full-width", details);
		this.statusMsgHandler.showMessage({type: code, summary: message, details: details});
	},

	validate: function () {
		if (this._lastStatusCallback) {
			this.setStatusCallback(this._lastStatusCallback);
		}
	},
				
	// UPDATE VIEW METHODS		
	_allowPropertyEdit: function (allowEdits){
		var props = this.ldapConfig._properties;
		for (var i=0; i < props.length; i++) {
			props[i]._editValueNode.disabled = !allowEdits;
		};	 
	},
	
	// POST CONTRIBUTOR UPDATES
	runAction: function(itemId, stateId, licenses, caller) {
		var args = {
			stateId: stateId,
			itemId: itemId,
			repoOnly: true, 
			jsonLicenses: dojo.toJson(licenses)
		}
		AdminClient.postContributor(args, new SRH(caller, "disableAdminAccess", "postAssignLicensesError"));
	}

});

})();