/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.wizard.ProjectStep");

dojo.require("com.ibm.team.dashboard.web.ui.internal.wizard.AbstractDashboardStep");

dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ProcessRestClient");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "WizardMessages");

var AbstractDashboardStep = com.ibm.team.dashboard.web.ui.internal.wizard.AbstractDashboardStep;
var ProcessRestClient = com.ibm.team.dashboard.web.ui.internal.util.ProcessRestClient;
var StyledTable = jazz.ui.StyledTable;

var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.wizard.ProjectStep",
		[AbstractDashboardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/wizard/templates/ProjectStep.html"),
	widgetsInTemplate: true,
	
	_rootServicesUrl: null,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "WizardMessages");
		this.stepTitle = messages.projectTitle;
	},
	
	postCreate: function() {
		var styledTable = new StyledTable({
			columns: [
				{ label: "", width: "16px" },
				{ label: messages.projectHeadingLabel, width: "170px" },
				{ label: messages.projectHeadingSummary }
			]
		});
		var table = styledTable.domNode;
		var tBody = this._tBody = table.tBodies[0];
		// loading row
		var tr = this.messageRow = dojo.create("tr", {"class": "loadingMsg lastRow"}, tBody);
		dojo.create("td", null, tr);
		var td = this._messageCell = dojo.create("td", {colspan: "2"}, tr);
		td.appendChild(document.createTextNode(messages.loading));
		this.tableContent.appendChild(table);
	},
	
	canFinish: function() {
		return this.data.projectAreaName && this.inherited(arguments);
	},
	
	focusStep: function() {
		this.inherited(arguments);
		var rootServicesUrl = this.data.rootServicesUrl;
		if (rootServicesUrl !== this._rootServicesUrl) {
			this._load(rootServicesUrl);
		}
	},
	
	_clearMsg: function() {
		this.messageRow.style.display = "none";
	},

	_clearTable: function() {
		var tableBody = this._tBody;
		var node = this.messageRow.nextSibling;
		while (node) {
			var next = node.nextSibling;
			tableBody.removeChild(node);
			node = next;
		}
	},
	
	_load: function(/* string */ rootServicesUrl) {
		this._showMsg(messages.loading, "loadingMsg");
		this._clearTable();
		
		this._rootServicesUrl = rootServicesUrl;
		var processClient = new ProcessRestClient(rootServicesUrl);
		var dfd = processClient.getProjectAreas();
		dfd.addBoth(dojo.hitch(this, this._clearMsg));
		dfd.addCallback(dojo.hitch(this, function(/* object[] */ projectAreas) {
			if (projectAreas.length === 0) {
				this._showNoProjects();
			}
			else {
				this._populateTable(projectAreas);
			}
			return projectAreas;
		}));
		dfd.addErrback(dojo.hitch(this, function(e) {
			this._showNoProjects();
			this.showError(messages.projectErrorFetching, e);
		}));
	},
	
	onEntrySelect: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			var value = e.target.value;
			this.setData("projectAreaName", value);
		}
	},
	
	_populateTable: function(/* object[] */ projectAreas) {
		projectAreas.sort(_projectComparator);
		var len = projectAreas.length;
		for (var i=0;i<len;++i) {
			var projectArea = projectAreas[i];
			var name = projectArea.name;
			var tr = dojo.create("tr", (i == len-1) ? { "class": "lastRow" } : null);
			var td = dojo.create("td", {"class": "radioCol"}, tr);
			var input = dojo.create("input", {type: "radio", name: "project", value: name, checked: i === 0, id: name}, td);
			if (i === 0) {
				this.setData("projectAreaName", name);
			}
			this.connect(input, "onclick", this.onEntrySelect);
			td = dojo.create("td", {"class": "labelCol"}, tr);
			var label = dojo.create("label", {"for": name}, td);
			dojo.create("span", {"class": "icon projectIcon"}, label);
			var span = dojo.create("span", {"class": "labelText"}, label);
			span.appendChild(document.createTextNode(projectArea.name));
			td = dojo.create("td", {"class": "descCol"}, tr);
			td.appendChild(document.createTextNode(projectArea.summary));
			this._tBody.appendChild(tr);
		}
	},
	
	_showMsg: function(/* string */ msg, /* string */ className) {
		this._messageCell.innerHTML = "";
		this._messageCell.appendChild(document.createTextNode(msg));
		this.messageRow.className = "lastRow" + (className ? " " + className : "" );
		this.messageRow.style.display = "";
	},
	
	_showNoProjects: function() {
		this._showMsg(messages.projectMsgNone, null);
	}
});

/*
 * Compares two project entry objects for alphabetical sorting.
 */
function _projectComparator(/* object */ a, /* object */ b) {
	return a.name > b.name ? 1 : -1;
}

})();
