/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.util.DashboardUtil");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardConfiguration");
dojo.require("net.jazz.ajax.contextRoot");

dojo.require("jazz.app.context");

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardConfiguration = com.ibm.team.dashboard.web.ui.internal.DashboardConfiguration;

var VAR_PROJECT_NAME = "${projectName}";
var VAR_TEAM = "${team}";

/**
 * Dashboard utility methods.
 * 
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
com.ibm.team.dashboard.web.util.DashboardUtil = {
	
	/**
	 * Returns an HREF that can be used as a link to the given dashboard info. Tab is
	 * optional, and if specified, will take you to that specific tab. The scopeItem
	 * param indicates the target dashboard depending on the indicated scope. For
	 * personal dashboards, scopeItem is the id of the dashboard, for team dashboards
	 * it's an array of segments in the team area path and for project dashboards it's
	 * the project area name.
	 * 
	 * If projectName is specified, it will be shown in the context of that project,
	 * otherwise it will be assumed to be in the current project context.
	 * 
	 * @return {string} a dashboard href
	 */
	getDashboardHref: function(/* string */ scope, /* string */ scopeItem, /* string */ tabId, /* string */ projectName, /* string */ webURL, /* string */ noDebug) {
		var href;
		projectName = projectName || jazz.app.context.get().name;
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				href = "/dashboards/" + scopeItem;
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				href = DashboardConfiguration.hrefs.projectDashboard;
				href = href.replace(VAR_PROJECT_NAME, encodeURIComponent(projectName));
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				var team = [];
				var len = scopeItem.length;
				for (var i=0;i<len;++i) {
					team.push(encodeURIComponent(scopeItem[i]));
				}
				href = DashboardConfiguration.hrefs.teamDashboard;
				href = href.replace(VAR_PROJECT_NAME, encodeURIComponent(projectName));
				href = href.replace(VAR_TEAM, team.join("/"));
				break;
		}
		if (tabId) {
			// add tab param to hash
			var index = href.indexOf("#");
			href += ((index !== -1 && index < href.length - 1) ? "&" : "#") + DashboardConstants.URL_PARAM_TAB + "=" + tabId;
		}
		if (!noDebug && djConfig.isDebug) {
			// add debug param if we're in debug mode
			var parts = href.split("#");
			parts[0] += "?debug=" + djConfig.isDebug;
			href = parts.join("#");
		}
		return (webURL || net.jazz.ajax.contextRoot()) + href;
	}
};


})();