#!/bin/sh
# *******************************************************************************
#  Licensed Materials - Property of IBM
#  (c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
#
#  Note to U.S. Government Users Restricted Rights:
#  Use, duplication or disclosure restricted by GSA ADP Schedule
#  Contract with IBM Corp.
#  *******************************************************************************
# NLS_MESSAGEFORMAT_NONE
# NLS_ENCODING=UTF-8


# #############################################################
# #
# # Define the environment variables corresponding to the
# # rm_upgrade.sh script parameters and their default values
# #
# #===========================================================
# #
# # This file is read at the beginning of the rm_upgrade.sh
# # script execution. It sets one environment variable per
# # valid parameter with a predefined default value.
# #
# # Users may want to modify the default value in order to
# # launch the rm_upgrade.sh script with their own values
# # without having to enter the parameter and its
# # corresponding value manually at the command line.
# #
# # WARNING:
# # Not all variables are required to be set for every
# # application.  Read each comment carefully before modifying
# # the value.
# #
# #############################################################

# Parameter: -oldApplicationHome
# Usage: -oldApplicationHome <dir path>
# Description:
# 	Full path to the application home directory in old installation.
# Default:
# 	<none>
# Constraints:
#  - this is a mandatory parameter
#  - <dir path> must be an existing directory ended by 'server/conf'
#    e.g. /Jazz/IBM/RRC_3.x/jazz/server/conf
#
OLD_APPLICATION_HOME=

# Parameter: -applicationContextRoot
# Usage: -applicationContextRoot <root>
# Description:
# 	Application context root.
# Default:
# 	rm
# Constraints:
#  - <root> must be a valid directory single name (i.e. no leading slash)
#
APPLICATION_CONTEXT_ROOT=

# Parameter: -oldJTSHome
# Usage: -oldJTSHome <dir path>
# Description:
# 	Full path to the JTS home directory in old installation.
# Default:
# 	<none>
# Constraints:
#  - <dir path> must be an existing directory ended by 'server/conf'
#  - Only valid for RM and JTS upgrades
#
OLD_JTS_HOME=

# Parameter: -oldJTSContextRoot
# Usage: -oldJTSContextRoot <root>
# Description:
# 	JTS Context root of the old installation.
# Default:
# 	jts
# Constraints:
#  - Only accepted for RM upgrade
#  - <root> must be a valid directory single name (i.e. no leading slash)
#
OLD_JTS_CONTEXT_ROOT=

# Parameter: -updateTomcatFiles
# Usage: -updateTomcatFiles <yes | no>
# Description:
# 	Whether to update the tomcat configuration files.
# Default:
# 	yes
# Constraints:
#  - The only valid values are 'yes' and 'no'
#
UPDATE_TOMCAT_FILES=yes

# Parameter: -oldApplicationTomcatHome
# Usage: -oldApplicationTomcatHome <dir path>
# Description:
# 	Full path to the Application Tomcat root directory in old installation.
# Default:
# 	<none>
# Constraints:
#  - Only accepted for RM upgrade
#  - Must be an existing directory
#
OLD_TOMCAT_HOME=

# Parameter: -oldJTSTomcatHome
# Usage: -oldJTSTomcatHome <dir path>
# Description:
# 	Full path to the JTS Tomcat root directory in the old installation.
# Default:
# 	<none>
# Constraints:
#  - Only accepted for RM upgrade
#  - Must be an existing directory
#
OLD_JTS_TOMCAT_HOME=

# Parameter: -newApplicationHome
# Usage: -newApplicationHome <dir path>
# Description:
# 	Path to the Application home directory in the new installation.
# Default:
# 	conf
# Constraints:
#  - Only accepted for RTC, QM and RM upgrade
#  - When accepted, this is a mandatory parameter
#  - <dir path> must be an existing directory ended by 'server/conf'
#    e.g. /Jazz/IBM/RRC_3.0/JazzTeamServer/server/conf
#
NEW_APPLICATION_HOME="$START_DIR/conf"

# Parameter: -newJTSHome
# Usage: -newJTSHome <dir path>
# Description:
# 	Full path to the JTS home directory in the JTS that the new installation will use.
# Default:
# 	The new application home directory
# Constraints:
#  - <dir path> must be an existing directory ended by 'server/conf'
#
NEW_JTS_HOME=

# Parameter: -newJTSContextRoot
# Usage: -newJTSContextRoot <root>
# Description:
# 	Context root of the new JTS server.
# Default:
# 	jts
# Constraints:
#  - <root> must be a valid directory single name (i.e. no leading slash)
#
JTS_CONTEXT_ROOT=jts

# Parameter: -newApplicationTomcatHome
# Usage: -newApplicationTomcatHome <dir path>
# Description:
# 	Path to the Application Tomcat root directory in the new installation.
# Default:
# 	<none>
#
NEW_TOMCAT_HOME=

# Parameter: -newJTSTomcatHome
# Usage: -newJTSTomcatHome <dir path>
# Description:
# 	Path to the JTS Tomcat root directory in the JTS that the new installation will use.
# Default:
# 	<none>
#
NEW_JTS_TOMCAT_HOME=

# Parameter: -noPrompt
# Usage: -noPrompt
# Description:
# 	Do not prompt any confirmation (i.e. always use noPrompt) while executing any repotools commands
# Default:
# 	<none>
# Constraints:
#  - The only valid values are empty or 'noPrompt'
#
# NO_PROMPT=noPrompt
NO_PROMPT=

# Parameter: -overwrite
# Usage: -overwrite
# Description:
# 	Force overwrite (i.e. always use overwrite=yes) while executing any repotools commands
# Default:
# 	<none>
# Constraints:
#  - The only valid values are empty or 'overwrite=yes'
#
# OVERWRITE=overwrite=yes
OVERWRITE=

# Parameter: -noVerify
# Usage: -noVerify
# Description:
# 	Do not perform any verification (i.e. always use noVerify) while executing any repotools commands
# Default:
# 	<none>
# Constraints:
#  - The only valid values are empty or 'noVerify'
#
# NO_VERIFY=noVerify
NO_VERIFY=

# Parameter: -noStepPrompt
# Usage: -noStepPrompt
# Description:
# 	Do not prompt before each upgrade step
# Default:
# 	false
# Constraints:
#  - The only valid values are 'true' or 'false'
#
# NO_STEP_PROMPT=true
NO_STEP_PROMPT=false

# Parameter: -jtsURL
# Usage: -jtsURL <JTS public URI>
# Description:
# 	Public URI of the JTS
# Default:
# 	<none>
# Constraints:
#  - Only accepted for RM upgrade
#  - Only necessary in a distributed deployment where the JTS is running on a different server instance than the RM application
#
JTS_PUBLIC_URL=
